/*
 * File: $Id: ClinicalSpectroscopyCaseWorklistDataElement.java,v 1.1.1.1 $
 * History
 * Date: 12.12.2016  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2016 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
   
package org.spectrim.clinicalcasehandling;
 
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import org.dcm4che2.data.DicomObject;
import org.spectrim.data.CSIData;
import org.spectrim.data.FileReaderWorker;
import org.spectrim.io.dicom.DicomHandler;
import org.spectrim.io.dicom.DicomHeader;

/**
 * Container class containing elements which are vital for automatic 
 * processing of MRS/MRI data.
 * @author slotboom
 */
public class ClinicalSpectroscopyCaseWorklistDataElement
{
    

    /**
     * Default constructor
     */
    public ClinicalSpectroscopyCaseWorklistDataElement( )
    {

    }
 
    
    /**
     * Creates a new ClinicalSpectroscopyCaseIncomingWorklistDataElement: 
     * all initially required data is present in the DICOM header
     * file which is parsed.
     * @param dcm_spectro_file
     */
    public ClinicalSpectroscopyCaseWorklistDataElement( String dcm_spectro_file )
    {
        DicomObject spc_case = DicomHandler.getDicomObject( dcm_spectro_file );
        if ( spc_case != null )
        {
            DicomHeader header = DicomHandler.getHeader(spc_case);
            DicomHandler.readSiemensData(spc_case, header);
            
            String classUID = DicomHandler.getSOPClassUID(spc_case);
            String imagetype = DicomHandler.getImageType(spc_case);
            
            FileReaderWorker.filetype type;
            
            switch ( classUID ) 
            {    
                case "1.2.840.10008.5.1.4.1.1.4.2": // Spectro std
                    type = FileReaderWorker.filetype.MRS_STD;
                     m_isValidCase = true;
                     m_isSpectrum = true;
                    break;
                case "1.3.12.2.1107.5.9.1": // Spectro/Tensor Siemens
                    if (imagetype.indexOf("TENSOR") >= 0) 
                    {
                        type = FileReaderWorker.filetype.TENSOR;
                    } else 
                    {
                        type = FileReaderWorker.filetype.MRS_SIEMENS;
                         m_isValidCase = true;
                         m_isSpectrum = true;
                    }
                    break;
                case "1.2.840.10008.5.1.4.1.1.7": // Perfusion derived maps
                    type = FileReaderWorker.filetype.PERFUSION;
                    break;
                default: // Unknown format
                    type = FileReaderWorker.filetype.UNKNOWN;                                         
                    break;
            }
            
            // we are sure that this is a valid spectroscopy file
            if ( m_isValidCase == true )
            {
                m_case_UID = DicomHandler.getSOPInstanceUID(spc_case);
                m_patientName = DicomHandler.getPatientName(spc_case);
                m_scannerModel = DicomHandler.getModelName(spc_case);
                
                CSIData csidata;
                csidata = new CSIData( header, null );
                if ( ( csidata.nrVoxelsDir1 == 1 ) && 
                     ( csidata.nrVoxelsDir2 == 1 ) &&
                     ( csidata.nrVoxelsDir3 == 1 ) )
                {
                    m_spectroscopyDatatype = SVS_DATA_TYPE_STRING;
                }
                else
                {
                    m_spectroscopyDatatype = CSI_DATA_TYPE_STRING;
                }
                
            }
            
            m_studyDate = DicomHandler.getStudyDate(spc_case);
            m_studyDate = this.FormatDicomDOBString(m_studyDate);
            m_seriesTime = DicomHandler.getSeriesTime(spc_case);
            
            m_seriesTime = FormatDicomSeriesTimeString(m_seriesTime );
            
            m_studyDate = m_studyDate + " " + m_seriesTime;
            
            m_scannerModel = DicomHandler.getModelName(spc_case);
            m_patientName = DicomHandler.getPatientName(spc_case);
            
            
            int index = m_patientName.lastIndexOf( "^" );
            m_patientName = m_patientName.substring(0, index) + "," + m_patientName.substring(index + 1, m_patientName.length( ) );  
            m_patientName.replaceAll("^", " ");
            m_patientName = m_patientName.replace( "^",  ", " );
            
            m_patientDateOfBirth = DicomHandler.getPatientBirthDate(spc_case);
            
            m_patientDateOfBirth = FormatDicomDOBString( m_patientDateOfBirth );
            
            DateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
            Date date = new Date();
            m_processingDateAndTime = dateFormat.format(date);            
             
            m_seriesNumber = DicomHandler.getSeriesNumber(spc_case) + "";
            m_seriesDescription = DicomHandler.getSeriesDescription(spc_case);
            m_instanceNumber = DicomHandler.getInstanceNr(spc_case) + "";
            m_dicomFileReference = dcm_spectro_file;
            m_case_UID = DicomHandler.getSOPInstanceUID(spc_case);
         
        }
        
    }
    
    /**
     * Formats a DICOM date string (YYYYMMDD) to a more natural format.
     * @param birthdate
     * @return 
     */
    private String FormatDicomDOBString( String birthdate )
    {
        String year  = birthdate.substring(0, 4);
        String month = birthdate.substring(4, 6);
        String date  = birthdate.substring(6, 8);
        String reformattedDate = date + "/" + month + "/" + year;
        
        return reformattedDate;
    }
    
    /**
     * Formats a DICOM series time string (HH:MM:SS) to a more natural format.
     * @param birthdate
     * @return 
     */
    private String FormatDicomSeriesTimeString( String birthdate )
    {
        
        String hours  = birthdate.substring(0, 2);
        String minutes = birthdate.substring(2, 4);
        String seconds  = birthdate.substring(4, 6);
        
        String reformattedTime = hours + ":" + minutes + ":" + seconds;
        
        return reformattedTime;
    }
    
    
    /**
     * Constructor of a clinical spectroscopy worklist element which 
     * contains information for storage of database, file type and processing 
     * state information  
     */
    public ClinicalSpectroscopyCaseWorklistDataElement( String  dateAndTime,
                                                        String  patientName,
                                                        String  patientDateOfBirth,
                                                        String  seriesNumber,
                                                        String  seriesDescription,
                                                        String  instanceNumber,
                                                        String  dicomFileReference,
                                                        String  dicomFileInstanceUID,
                                                        String  dicomSeriesInstanceUID,
                                                        boolean isSpectrum,
                                                        String  processingStatus
                                                      )
    {
        
        m_processingDateAndTime = dateAndTime;
        m_patientName = patientName;
        m_patientDateOfBirth = patientDateOfBirth;
        m_seriesNumber = seriesNumber;
        m_seriesDescription = seriesDescription;
        m_instanceNumber = instanceNumber;
        m_dicomFileReference = dicomFileReference;
        m_dicomFileInstanceUID = dicomFileInstanceUID;
        m_case_UID = dicomFileInstanceUID;
        m_dicomSeriesInstanceUID = dicomSeriesInstanceUID;
        m_isSpectrum = isSpectrum;
        m_processingStatus = processingStatus;
    }

    /**
     *
     * @return date and time String
     */
    public String getStudyDate( )
    {
        return m_studyDate;
    }

    public String getPatientName( )
    {
        return m_patientName;
    }

    public String getSeriesNumber( )
    {
        return m_seriesNumber;
    }

    public String getSeriesDescription( )
    {
        return m_seriesDescription;
    }


    public String getInstanceNumber( )
    {
        return m_instanceNumber;
    }

    /**
     * Returns the complete filesystem reference of the DICOM file
     * @return
     */
    public String getDicomFileReference( )
    {
        return m_dicomFileReference;
    }

    /**
     * Returs the complete filesystem reference of the DICOM file
     * @return
     */
    public String getDicomFileInstanceUID( )
    {
        return m_dicomFileInstanceUID;
    }

    /**
     * Returs the series instance uid of the DICOM file
     * @return
     */
    public String getDicomSeriesInstanceUID( )
    {
        return m_dicomSeriesInstanceUID;
    }

    /**
     * Returns the scanner model.
     * @return 
     */
    public String getScannerModel( ) 
    {
        
        return m_scannerModel;
    }
    
    /**
     * Sets the scanner model.
     * @return 
     */
    public void getScannerModel( String scannerModel ) 
    {
       m_scannerModel = scannerModel;
    }
    
    
    /**
     * Returns the processing date and time
     * @return 
     */
    public String getProcessingDateAndTime( ) 
    {
        return m_processingDateAndTime;
    }
    
    /**
     * Sets the processing date and time
     * @return 
     */
    public void setProcessingDateAndTime( String processingDateAndTime ) 
    {
        m_processingDateAndTime = processingDateAndTime;
    }
    
    /**
     * Sets the instance study date of the DICOM file
     * @param studyDate
     */
    public void setstudyDate( String studyDate )
    {
        m_studyDate = studyDate;
    }

    /**
     * Sets the patient name to which the file belongs
     * @param patientName
     */
    public void setPatientName( String patientName )
    {
        m_patientName = patientName;
    }

    /**
     * Sets the series number of the DICOM file
     * @param seriesNumber
     */
    public void setSeriesNumber( String seriesNumber )
    {
        m_seriesNumber = seriesNumber;
    }

    /**
     * Sets the series number of the DICOM file
     * @param seriesNumber
     */
    public void setSeriesDescription( String seriesDescription )
    {
        m_seriesDescription = seriesDescription;
    }

    /**
     * Returns the instance number of the DICOM file
     * @param instanceNumber
     */
    public void setInstanceNumber( String instanceNumber )
    {
        m_instanceNumber = instanceNumber;
    }

    /**
     * Sets the complete path and filename of the Dicom file
     * @param dicomFileReference
     */
    public void setDicomFileReference( String dicomFileReference )
    {
        m_dicomFileReference = dicomFileReference;
    }

    /**
     * Sets the complete path and filename of the Dicom file
     * @param dicomFileReference
     */
    public void setDicomFileInstanceUID( String dicomFileInstanceUID )
    {
        m_dicomFileInstanceUID = dicomFileInstanceUID;
    }

    /**
     * Sets the series instance uid of the Dicom file
     * @param seriesInstanceUID
     */
    public void setDicomSeriesInstanceUID( String seriesInstanceUID )
    {
        m_dicomSeriesInstanceUID = seriesInstanceUID;
    }

    /**
     * 
     * Sets the type of spectral data we are dealing with
     * 
     * @param m_spectroscopyDatatype
     */
    public void setSpectroscopyDataType( String spectroscopyDataType )
    {
        m_spectroscopyDatatype = spectroscopyDataType;
    }

    /**
     *
     * Returns what type of spectral data we are dealing with
     *
     * @param isSpectrum
     */
    public String getSpectroscopyDataType( )
    {
        return m_spectroscopyDatatype;
    }

    /**
     *
     * Returns whether element is a spectrum or not
     *
     * @param status
     */
    public void setProcessedStatus( String processing_status )
    {
        m_processingStatus = processing_status;
    }

    /**
     *
     * Returns whether element is a spectrum or not
     *
     * @param status
     */
    public String getProcessingStatus( )
    {
        return m_processingStatus;
    }
    
    /**
     * Returns the patient birth day
     */
    public String getPatientDateOfBirth( ) 
    {
        return m_patientDateOfBirth;
    }

   /**
     * Returns the patient birth day
     */
    public void setPatientDateOfBirth( String dob ) 
    {
        m_patientDateOfBirth = dob;
    }
   
   /**
     * Set the case UID which uniquely defines the case to be processed
     */
    public void setM_case_UID(String m_case_UID) {
        m_case_UID = m_case_UID;
    }
    
    /**
     * Returns the case UID which uniquely defines the case to be processed
     */
    public String getM_case_UID() {
        return m_case_UID;
    }
    
    // Private attributes
    private String  m_studyDate = "01.01.1970";
    private String  m_seriesTime = "00:00:00";
    
    private String  m_scannerModel = "not defined";
    private String  m_patientName = "not defined";
    private String  m_patientDateOfBirth = "01.01.1970";
    private String  m_processingDateAndTime = "not defined";
    private String  m_case_UID = "not defined";
    private String  m_seriesNumber = "not defined";
    private String  m_seriesDescription = "not defined";
    private String  m_spectroscopyDatatype = "not defined";
    private String  m_instanceNumber = "not defined";
    private String  m_dicomFileReference = "not defined";
    private String  m_dicomFileInstanceUID = "not defined";
    private String  m_dicomSeriesInstanceUID = "not defined";
    private boolean  m_isSpectrum = false;

    public final static int CSI_DATA = 1;
    public final static int SVS_DATA = 2;
    public final static String CSI_DATA_TYPE_STRING = "CSI";
    public final static String SVS_DATA_TYPE_STRING = "SVS";
    
    private String m_processingStatus = "<html><font color=\"red\">File received</font></html>";

    // important data fields?
    private boolean m_isValidCase;
    
    public final static int NOT_PROCESSED   = 100;
    public final static int BEING_PROCESSED = 101;
    public final static int IS_PROCESSED    = 102;

}