/*
 * File: $Id: DicomDirectory.java,v 1.2 2006/11/09 10:08:55 hans Exp $
 * History
 * Date: 28. Januar 2003, 11:11     Authors: Hans Slotboom
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
package org.spectrim.dicom;

import org.spectrim.lib.DualProgressBar;
import org.spectrim.lib.DualProgressBarListener;
import java.awt.Dimension;

import java.io.*;

/**
 * This class is a container class that contains the header information of all
 * DICOM files present in a directory dicom_dir; The class stores the dicom
 * header information in a HashTable: keys : DICOM filenames (cryptic) values :
 * DicomDecoder objects.
 *
 * @author slotboom
 */
import javax.swing.*;
import java.awt.event.*;

import java.util.*;
import org.dcm4che2.data.DicomObject;
import org.dcm4che3.data.Tag;
import org.spectrim.signalprocessing.QuickSortFloat;
import org.spectrim.io.dicom.DicomHandler;

public class DicomDirectory implements DualProgressBarListener {

    /**
     * Creates a new instance of DicomDirectory.
     * @param parent
     * @param dicom_dir
     */
    public DicomDirectory( JFrame parent, File dicom_dir ) 
    {
        m_parent = parent;
        m_dicomdir = dicom_dir;
        createHashTableOfDicomDirectory(dicom_dir);
        m_sender = this;
    }
    
    /**
     * Creates a new instance of DicomDirectory.
     * This constructor adds ActionListener to the arguments. The reason is 
     * that the complete directory must be parsed before the directory can be 
     * accessed.
     * @param parent
     * @param dicom_dir
     * @param listener this is a listener to this class waiting which sends 
     *        a notification as soon as the directory has been parsed.
     */
    public DicomDirectory( JFrame parent, File dicom_dir, ActionListener listener ) 
    {
        m_parent = parent;
        m_dicomdir = dicom_dir;
        addActionListener( listener );
        createHashTableOfDicomDirectory( dicom_dir );
        m_sender = this;
    }

    /**
     * Creates a hash table of the selected DICOM directory.
     * @param dicom_dir 
     */
    private synchronized void createHashTableOfDicomDirectory( File dicom_dir ) 
    {
        // Get the list of files in the selected dicom directory
        m_dcm_filelist = m_dicomdir.list();
        if (m_dualProgressBar != null) {
            m_dualProgressBar.dispose();
        }
        if (worker != null) {
            worker.interrupt();
            worker = null;
        }

        m_dualProgressBar = new DualProgressBar( m_parent, 100, 100, new Dimension(900, 150), false );
        m_dualProgressBar.setListener(this);
        m_dualProgressBar.setTitle("Analyzing Dicom Directory");

        m_dualProgressBar.setUpperProgress(0);
        m_dualProgressBar.setUpperLabel("        Total progress ");
        m_dualProgressBar.setUpperProgressVisible(false);
        m_dualProgressBar.setLowerLabel("<HTML><BR>     Analyzing Dicom image : " + m_dcm_filelist[0] + "</HTML>");

        worker = new org.spectrim.lib.SwingWorker() {
            public Object construct() {
                analyzeDirectoryTree();
                return null;
            }

            // runs in the event dispatching thread
            public void finished() {
                // After calculation set enabled again 
                m_dualProgressBar.setVisible(false);

                // Notify all listeners that the dicom directory has been
                // analyzed
                Iterator it = m_listeners.iterator();

                // create a map that maps the protocol names to series numbers
                resolveSeriesNumberToSeriesDescriptionMap();

                // notify registered listeners
                while (it.hasNext()) {
                    ((ActionListener) it.next()).actionPerformed(new ActionEvent(m_sender, 0, ""));
                }
            }
        };
        worker.start();
    }

    /**
     * Analyzes the DICOM directory; ie that all DICOM file headers are read
     * and the file header object (FileInfo) objects are stored in a HashTable.
     */
    private void analyzeDirectoryTree() {

        for (int i = 0; i < m_dcm_filelist.length && worker != null; i++) {
            m_dualProgressBar.setLowerProgress((int) (100.0 * (double) i / m_dcm_filelist.length));
            m_dualProgressBar.setLowerLabel("    Analyzing Dicom image : " + m_dcm_filelist[i]);

            File file = new File(m_dicomdir.getPath() + File.separatorChar + m_dcm_filelist[i]);

            if ((file.isFile() == true) && (file.length() > 512) && (file.getName().endsWith(".dcm") || DicomHandler.isDicom(file))) {
                DicomDecoder dicom_deco = new DicomDecoder(m_dicomdir.getPath() + File.separatorChar, m_dcm_filelist[i]);

                FileInfo dicom_fileinfo = null;
                try {
                    dicom_fileinfo = dicom_deco.getFileInfo();
                    m_image_modality = dicom_fileinfo.getImageModality();
                    m_patientName = dicom_fileinfo.getPatientName();
                    
//                    if ( dicom_fileinfo.isSpectrum( ) == true )
//                    {
                        
                        DicomObject object = DicomHandler.getDicomObject( m_dicomdir.getPath() + File.separatorChar + m_dcm_filelist[i] );
                        double TE = -1.0;
                        String fakeSeriesDescription = null; // In Philips this is called ProtocolName
                        if ( dicom_fileinfo.getManufacturer().contains( "Philips") )
                        {
                            fakeSeriesDescription = object.getString(Tag.ProtocolName);
                        }
                        
                        if ( object != null ) 
                        {
                            TE = object.getDouble(Tag.EffectiveEchoTime); // object.getDouble(Integer.parseInt("00189005", 16))
                            fakeSeriesDescription = object.getString(Tag.ProtocolName);
                            if ( fakeSeriesDescription ==  null )
                            {
                                fakeSeriesDescription = object.getString( Tag.SeriesDescription );
                            }
                            
                            dicom_fileinfo.setEchoTime( "" + TE );
                            dicom_fileinfo.setSeriesDescription( fakeSeriesDescription );           
                        }
//                        System.out.println( "Echo time " + TE );
//                    }
                    
                    UID2FileMap.put(dicom_fileinfo.getMediaStorageSOPInstanceUID(), file.toString());
                    seriesUID2studyUID.put(dicom_fileinfo.getSeriesInstanceUID(), dicom_fileinfo.getStudyInstanceUID());
                    studyUID2patientID.put(dicom_fileinfo.getStudyInstanceUID(), dicom_fileinfo.getPatientID());
                    studyUID2studyDate.put(dicom_fileinfo.getStudyInstanceUID(), dicom_fileinfo.getStudyDate());
                    patientID2patientNameBDate.put(dicom_fileinfo.getPatientID(), dicom_fileinfo.getPatientName() + "-" + dicom_fileinfo.getPatientBirthDate());

                } catch (IOException e) {
                    // non handled exception
                    dicom_deco.closeInputStream();
                    //System.out.println( e.getMessage( ) );
                    //System.out.println( "Non handled exception!" );
                }

                if (dicom_fileinfo != null) {
                    m_dicomFileHeaders.put(m_dcm_filelist[i], dicom_fileinfo);
                }
            }
            // if the File object points to a sub directory this subdirectory has to be parsed too
            if (file.isDirectory() == true) {
                analyzeSubDirectory(file);
            }
        }
    }

    /**
     * Analyzes a sub directory and adds the DICOM file references in the
     * LinkedList.
     */
    private void analyzeSubDirectory(File subDirectory) {

        String[] dcmSubDirFilelist = subDirectory.list();
        if (dcmSubDirFilelist != null) {
            for (int i = 0; i < dcmSubDirFilelist.length && worker != null; i++) {
                m_dualProgressBar.setLowerLabel("    Analyzing file : " + subDirectory.getPath() + File.separatorChar + dcmSubDirFilelist[i]);

                File file = new File(subDirectory.getPath() + File.separatorChar + dcmSubDirFilelist[i]);

                if ((file.isFile() == true) && (file.length() > 512) && (file.getName().endsWith(".dcm") || DicomHandler.isDicom(file))) {

                    DicomDecoder dicom_deco = new DicomDecoder(subDirectory.getPath() + File.separatorChar, dcmSubDirFilelist[i]);

                    FileInfo dicom_fileinfo = null;
                    try {
                        dicom_fileinfo = dicom_deco.getFileInfo();
                        m_image_modality = dicom_fileinfo.getImageModality();
                        m_patientName = dicom_fileinfo.getPatientName();
                        UID2FileMap.put(dicom_fileinfo.getMediaStorageSOPInstanceUID(), subDirectory.getPath() + File.separatorChar + dcmSubDirFilelist[i]);
                        seriesUID2studyUID.put(dicom_fileinfo.getSeriesInstanceUID(), dicom_fileinfo.getStudyInstanceUID());
                        studyUID2patientID.put(dicom_fileinfo.getStudyInstanceUID(), dicom_fileinfo.getPatientID());
                        studyUID2studyDate.put(dicom_fileinfo.getStudyInstanceUID(), dicom_fileinfo.getStudyDate());
                        patientID2patientNameBDate.put(dicom_fileinfo.getPatientID(), dicom_fileinfo.getPatientName() + "-" + dicom_fileinfo.getPatientBirthDate());
                    } catch (IOException e) {
                        // non handled exception
                        dicom_deco.closeInputStream();
                        // System.out.println( e.getMessage( ) );
                        // System.out.println( "> Non handled exception" );
                    }
                    // close the stream
                    dicom_deco.closeInputStream();
                    
                    String partialPath = subDirectory.getPath().substring(this.m_dicomdir.getPath().length(), subDirectory.getPath().length());

                    if (dicom_fileinfo != null) {
                        m_dicomFileHeaders.put(partialPath + File.separatorChar + dcmSubDirFilelist[i], dicom_fileinfo);
                    }
                }
                if (file.isDirectory()) {
                    analyzeSubDirectory(file);
                }
            }
        }
    }

    /**
     * Returns a TreeMap containing a map of series numbers and protocol names.
     */
    public void resolveSeriesNumberToSeriesDescriptionMap() {

        Enumeration e = m_dicomFileHeaders.keys();
        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            String series_description = fi.getSeriesDescription();
            int series_number;
            if (fi.getSeriesNumber().compareTo("N.A.") != 0) {
                series_number = new Integer(fi.getSeriesNumber().trim()).intValue();
            } else {
                series_number = 0;
            }

            m_seriesNumberToSeriesDescriptionMap.put(new Integer(series_number), series_description);
        }

        Collection smap = m_seriesNumberToSeriesDescriptionMap.keySet();
        Iterator it = smap.iterator();

        for (int i = 0; i < m_seriesNumberToSeriesDescriptionMap.size(); i++) {
            Integer intje = (Integer) it.next();
            // System.out.println(intje + "  " + m_seriesNumberToSeriesDescriptionMap.get(intje));
        }
        
    }

    /**
     * Returns the highest series number of the series having protocol name
     * protocol_name. Returns -1 if protocol_name is not found
     */
    public int getHighestSeriesNumber(String protocol_name) {
        Iterator it = m_seriesNumberToSeriesDescriptionMap.keySet().iterator();

        int largestSeriesNumber = -1;

        while (it.hasNext()) {
            Integer testInt = (Integer) it.next();

            String testStr = m_seriesNumberToSeriesDescriptionMap.get(testInt);
            if (testStr.contentEquals(new StringBuffer(protocol_name + " "))) {
                largestSeriesNumber = testInt.intValue();
            }
        }
        return largestSeriesNumber;
    }

    /**
     * Returns a Vector of integers: series number of the series having protocol
     * name 'protocol_name'.
     */
    public Vector getSeriesNumbers(String protocol_name) {
        m_seriesNumbersBelongingToSpecificSeriesProtocolName = new Vector<Integer>();

        Iterator it = m_seriesNumberToSeriesDescriptionMap.keySet().iterator();

        int largestSeriesNumber = -1;

        while (it.hasNext()) {
            Integer testInt = (Integer) it.next();
            String testStr = m_seriesNumberToSeriesDescriptionMap.get(testInt);
            if (testStr.compareTo(protocol_name) == 0) {
                m_seriesNumbersBelongingToSpecificSeriesProtocolName.add(testInt);
            }
        }

        return m_seriesNumbersBelongingToSpecificSeriesProtocolName;
    }

    /**
     * Returns the number of images having protocol name 'protocol_name' and
     * series description 'series_description'.
     */
    public int getTotalNumberOfImages(String protocol_name, String series_description) {
        Enumeration e = m_dicomFileHeaders.keys();
        int cntr = 0;

        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            String prot_name = fi.getProtocolName();
            String seriesDescription = fi.getSeriesDescription();
            if ((prot_name.contentEquals(new StringBuffer(protocol_name + " ")))
                    && (seriesDescription.contentEquals(new StringBuffer(series_description + " ")))) {
                cntr++;
            }
        }

        return cntr;
    }

    /**
     * Returns the number of images having protocol name 'protocol_name' and
     * series description 'series_description'.
     */
    public LinkedList<String> getAllFileRefences() {
        LinkedList<String> fileRefs = new LinkedList<String>();
        Enumeration e = m_dicomFileHeaders.keys();

        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            String fileName = fi.getFileName();

            fileRefs.add(alias);
        }

        return fileRefs;
    }

    /**
     * Returns the extended filename reference, ie the filename AND a part of
     * the path.
     */
    public LinkedList<String> getAllExtendedFileRefences() {
        LinkedList<String> fileRefs = new LinkedList<String>();
        Enumeration e = m_dicomFileHeaders.keys();

        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();

            fileRefs.add(alias);
        }

        return fileRefs;
    }

    /**
     * Determines the TreeSet of all images belonging to a 
     * certain series_number.
     */
    public LinkedList<String> getImageFileReferencesOfSeriesNumber(int series_number) {
        Map<Integer, String> orderedFiles = new TreeMap<Integer, String>();
        String instance_num;
        String fileName;

        m_imageFileReferencesOfSeriesNumber.clear();
        Enumeration e = m_dicomFileHeaders.keys();

        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);

            int ser_number = new Integer(fi.getSeriesNumber().trim()).intValue();

            String prot_name = fi.getProtocolName();
            String sequence_name = fi.getSequenceName();

            if (series_number == ser_number) {
                instance_num = fi.getInstanceNumber();
                fileName = m_dicomdir.getPath() + System.getProperty("file.separator") + alias;
                orderedFiles.put(Integer.valueOf(instance_num.trim()), fileName);
//                m_imageFileReferencesOfSeriesNumber.add( m_dicomdir.getPath( ) + System.getProperty( "file.separator" ) + alias );
//                System.out.println( "File name belonging to series " + ser_number + " " + alias + " ( " + prot_name + " " + sequence_name + ")" );
            }
        }
        m_imageFileReferencesOfSeriesNumber.addAll(orderedFiles.values());
        return m_imageFileReferencesOfSeriesNumber;
    }

    /**
     * Determines the TreeSet of all images belonging to a certain series_number.
     */
    public String[] getDifferentSequenceNamesSeries(int series_number) {

        TreeSet<String> imageFileReferencesOfSeriesNumber = new TreeSet<String>();

        String[] allSequenceNames;

        Vector<String> tempSequenceNames = new Vector<String>();

        Enumeration e = m_dicomFileHeaders.keys();

        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            int ser_number = new Integer(fi.getSeriesNumber().trim()).intValue();
            String prot_name = fi.getProtocolName();
            String sequence_name = fi.getSequenceName();

            boolean alreadyInList = false;
            if (series_number == ser_number) {
                imageFileReferencesOfSeriesNumber.add(m_dicomdir.getPath() + System.getProperty("file.separator") + alias);
                System.out.println("File name belonging to series " + ser_number + " " + alias + " ( " + prot_name + " " + sequence_name + ")");
                for (int i = 0; i < tempSequenceNames.size(); i++) {
                    if ((tempSequenceNames.elementAt(i)).compareTo(sequence_name) == 0) {
                        alreadyInList = true;
                        break;
                    }
                }

                if (alreadyInList == false) {
                    tempSequenceNames.add(sequence_name);
                }
            }

        }

        allSequenceNames = new String[tempSequenceNames.size()];

        for (int i = 0; i < tempSequenceNames.size(); i++) {
            allSequenceNames[i] = tempSequenceNames.elementAt(i);
        }

        return allSequenceNames;
    }

    /**
     * Determines the TreeSet of all images belonging to a certain series_number.
     */
    public int[] getDifferentEchoTimesInSeries(int series_number) {

        TreeSet<String> imageFileReferencesOfSeriesNumber = new TreeSet<String>();

        int[] allTEs;

        Vector<Double> tempTEs = new Vector<Double>();

        Enumeration e = m_dicomFileHeaders.keys();

        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            int ser_number = new Integer(fi.getSeriesNumber().trim()).intValue();

            String prot_name = fi.getProtocolName();
            String sequence_name = fi.getSequenceName();
            double TE = new Double(fi.getEchoTime()).doubleValue();

            boolean alreadyInList = false;
            if (series_number == ser_number) {
                imageFileReferencesOfSeriesNumber.add(m_dicomdir.getPath() + System.getProperty("file.separator") + alias);
                System.out.println("File name belonging to series " + ser_number + " " + alias + " ( " + prot_name + " " + sequence_name + ")");
                for (int i = 0; i < tempTEs.size(); i++) {
                    if (tempTEs.elementAt(i).intValue() == TE) {
                        alreadyInList = true;
                        break;
                    }
                }

                if (alreadyInList == false) {
                    tempTEs.add(new Double(TE));
                }
            }
        }

        allTEs = new int[tempTEs.size()];

        for (int i = 0; i < tempTEs.size(); i++) {
            allTEs[i] = tempTEs.elementAt(i).intValue();
        }

        return allTEs;
    }

    /**
     * Determines the file reference string of all images belonging to a certain
     * seriesNumber and instanceNumber.
     */
    public String getImageFileReference(int seriesNumber, int instanceNumber) {

        m_imageFileReferencesOfSeriesNumber.clear();
        Enumeration e = m_dicomFileHeaders.keys();

        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            int ser_number = new Integer(fi.getSeriesNumber().trim()).intValue();
            String prot_name = fi.getProtocolName();
            String sequence_name = fi.getSequenceName();
            int inst_number = new Integer(fi.getInstanceNumber().trim()).intValue();
            if ((seriesNumber == ser_number) && (instanceNumber == inst_number)) {
                return (m_dicomdir.getPath() + System.getProperty("file.separator") + alias);
            }
        }

        return "";
    }

    /**
     * Returns the file that belongs the MediaStorageSOPInstanceUID This
     * MediaStorageSOPInstanceUID is referred eg the reference images of a
     * spectroscopy.
     */
    public String getFileNameBelongingToMediaStorageSOPInstanceUID(String uid) {

//        Enumeration e = m_dicomFileHeaders.keys( );
//        
//        while ( e.hasMoreElements( ) )
//        {
//            String alias = (String)e.nextElement( );
//            FileInfo fi = m_dicomFileHeaders.get( alias );
//            
//            String  mediaStorageSOPInstanceUID = fi.getMediaStorageSOPInstanceUID( );
//        
//            if ( mediaStorageSOPInstanceUID.compareTo( uid ) == 0 )
//            {
//                return ( m_dicomdir.getPath( ) + System.getProperty( "file.separator" ) + alias );
//                //return ( m_dicomdir.getPath( ) + alias );
//            }
//
//            String  sopInstanceUID = fi.getSOPInstanceUID( );
//
//            if ( sopInstanceUID.compareTo( uid ) == 0 )
//            {
//                return ( m_dicomdir.getPath( ) + System.getProperty( "file.separator" ) + alias );
//            }
//        }
//        
//        return "";
        return UID2FileMap.get(uid);
    }

    /**
     * Returns the series time attribute of a specific series Returns -1 if
     * series number is not found.
     */
    public double getSeriesTR(int series_number) {
        Enumeration e = m_dicomFileHeaders.keys();

        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            int ser_number = new Integer(fi.getSeriesNumber().trim()).intValue();
            if (series_number == ser_number) {
                return new Double(fi.getTR()).doubleValue();
            }
        }
        return -1.0;
    }

    /**
     * Determines the TreeSet of all images belonging to different series but
     * have the same instance_number.
     */
    public LinkedList<String> getAllImageFileReferencesHavingInstanceNumber(Vector seriesNumbers, int instanceNumber) {

        Iterator it = seriesNumbers.iterator();
        int cntr = 0;
        while (it.hasNext()) {
            int seriesNumber = ((Integer) it.next()).intValue();
            String reference = getImageFileReference(seriesNumber, instanceNumber);
            m_allImageFileReferencesOfInstanceNumber.add(reference);
            System.out.println(cntr++ + " File reference : " + reference);
        }

        return m_allImageFileReferencesOfInstanceNumber;
    }

    /**
     * Adds an ActionListener to the list of action listeners who are notified
     * when something happens with the Spinbox.
     */
    public void addActionListener(ActionListener e) {
        m_listeners.add(e);
    }

    /**
     * Returns the TreeMap of series number and series names.
     */
    public TreeMap getSeriesDescriptions() {
        return m_seriesNumberToSeriesDescriptionMap;
    }

    /**
     * Returns the tree set of series number and series names.
     */
    public String[] getSeriesDescriptionsAsStringArray() {
        String[] output = new String[m_seriesNumberToSeriesDescriptionMap.size()];

        Collection col = m_seriesNumberToSeriesDescriptionMap.values();
        Iterator it = col.iterator();
        int cntr = 0;
        while (it.hasNext()) {
            output[cntr] = (String) it.next();
            cntr++;
        }

        return output;
    }

    public HashMap getSeriesUID2studyUID() {
        return seriesUID2studyUID;
    }

    ;
    
    public HashMap getStudyUID2patientID() {
        return studyUID2patientID;
    }

    ;
    
    public HashMap getStudyUID2studyDate() {
        return studyUID2studyDate;
    }

    ;
    
    public HashMap getPatientID2patientNameBDate() {
        return patientID2patientNameBDate;
    }

    ;
    
    
    
    /**
     * Returns the path string of the DicomDirectory.
     */
    public String getPath() {
        return m_dicomdir.getPath();
    }

    /**
     * Returns the image modality of the files contained in the directory.
     */
    public String getImageModality() {
        return m_image_modality;
    }

    /**
     * Returns the image size of the image with instanceNumber in a series with
     * seriesDescription.
     */
    public Dimension getImageDimension(String ser_description, int instanceNumber) {
        Dimension imDim = new Dimension(-1, -1);;
        Enumeration e = m_dicomFileHeaders.keys();
        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            String series_description = fi.getSeriesDescription();
            int instance_number = new Integer(fi.getInstanceNumber().trim()).intValue();
            if (series_description.compareTo(ser_description) == 0) {
                if (instance_number == instanceNumber) {
                    imDim = new Dimension(fi.getImageWidth(), fi.getImageHeight());
                    break;
                }
            }
        }
        return imDim;
    }

    /**
     * Returns the image size of the with seriesDescription.
     */
    public Dimension getImageDimension(String ser_description) {
        Dimension imDim = new Dimension(-1, -1);;
        Enumeration e = m_dicomFileHeaders.keys();
        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            String series_description = fi.getSeriesDescription();
            int instance_number = new Integer(fi.getInstanceNumber().trim()).intValue();
            if (series_description.compareTo(ser_description) == 0) {
                imDim = new Dimension(fi.getImageWidth(), fi.getImageHeight());
                break;
            }
        }
        return imDim;
    }

    /**
     * Returns the vector of file references of files belonging to series
     * description and having instance number.
     */
    public Vector<String> getFileReferences(String seriesDescription, int instanceNumber) {

        Vector<String> fileReferences = new Vector<String>();

        TreeMap<Integer, String> fileRefMap = new TreeMap<Integer, String>();
        int imageSize = 0;
        Enumeration e = m_dicomFileHeaders.keys();
        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            imageSize = fi.getImageHeight() * fi.getImageWidth();
            String series_description = fi.getSeriesDescription();
            int instance_number = new Integer(fi.getInstanceNumber().trim()).intValue();
            if (series_description.compareTo(seriesDescription) == 0) {
                if (instance_number == instanceNumber) {
                    fileRefMap.put(new Integer(fi.getSeriesNumber().trim()), alias);
                }
            }
        }

        Iterator it = fileRefMap.keySet().iterator();

        while (it.hasNext()) {
            fileReferences.add(fileRefMap.get((Integer) it.next()));
        }

        return fileReferences;
    }

    /**
     * Returns the image data of all files that have the same series description
     * and has the same instance numbers: This routine is suited for data that
     * is stored in different series for which the data of the same instance
     * numbers (at different series numbers) belong together.
     */
    public FileInfo getFileInfoOfSeriesDescriptionAndSliceLocation(String ser_description, String sliceLocationToSearch) {
        FileInfo fileInfo = null;
        TreeMap<Integer, String> fileRefMap = new TreeMap<Integer, String>();
        int imageSize = 0;
        Enumeration e = m_dicomFileHeaders.keys();
        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            imageSize = fi.getImageHeight() * fi.getImageWidth();
            String series_description = fi.getSeriesDescription();
            String sliceLocation = fi.getSliceLocation();
            if (series_description.compareTo(ser_description) == 0) {
                if (sliceLocation.compareTo(sliceLocationToSearch) == 0) {
                    fileInfo = fi;
                }
            }
        }

        return fileInfo;
    }

    /**
     * Returns the linked list of image file references and has the same
     * instance numbers: This routine is suited for data that is stored in
     * different series for which the data of the same instance numbers (at
     * different series numbers) belong together.
     */
    public LinkedList<String> getFileImageReferencesOfFilesHavingSameSeriesDesciptionAndInstanceNumber(String ser_description, int instanceNumber) {

        LinkedList<String> fileReferences = new LinkedList<String>();

        TreeMap<Integer, String> fileRefMap = new TreeMap<Integer, String>();
        int imageSize = 0;
        Enumeration e = m_dicomFileHeaders.keys();
        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            imageSize = fi.getImageHeight() * fi.getImageWidth();
            String series_description = fi.getSeriesDescription();
            int instance_number = new Integer(fi.getInstanceNumber().trim()).intValue();
            if (series_description.compareTo(ser_description) == 0) {
                if (instance_number == instanceNumber) {
                    fileRefMap.put(new Integer(fi.getSeriesNumber().trim()), alias);
                    m_lastFileInfo = fi;
                }
            }
        }

        Iterator it = fileRefMap.keySet().iterator();
        int[] allImageData = new int[imageSize * fileRefMap.size()];
        int k = 0;
        while (it.hasNext()) {

            String fileRef = m_dicomdir.getPath() + File.separatorChar + fileRefMap.get((Integer) it.next());
            fileReferences.add(fileRef);
        }

        return fileReferences;

    }

    /**
     * Returns the image data of all files that have the same series description
     * and has the same instance numbers: This routine is suited for data that
     * is stored in different series for which the data of the same instance
     * numbers (at different series numbers) belong together.
     */
    public int[] getFileImageDataOfFilesHavingSameSeriesDesciptionAndInstanceNumber(String ser_description, int instanceNumber) {

        TreeMap<Integer, String> fileRefMap = new TreeMap<Integer, String>();
        int imageSize = 0;
        Enumeration e = m_dicomFileHeaders.keys();
        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            imageSize = fi.getImageHeight() * fi.getImageWidth();
            String series_description = fi.getSeriesDescription();
            int instance_number = new Integer(fi.getInstanceNumber().trim()).intValue();
            if (series_description.compareTo(ser_description) == 0) {
                if (instance_number == instanceNumber) {
                    fileRefMap.put(new Integer(fi.getSeriesNumber().trim()), alias);
                    m_lastFileInfo = fi;
                }
            }
        }

        Iterator it = fileRefMap.keySet().iterator();
        int[] allImageData = new int[imageSize * fileRefMap.size()];
        int k = 0;
        while (it.hasNext()) {
            DicomImageData DicIma = new DicomImageData(m_dicomdir.getPath() + File.separatorChar + fileRefMap.get((Integer) it.next()));
            System.arraycopy(DicIma.getPixelData(), 0, allImageData, k * imageSize, imageSize);
            k++;
        }

        return allImageData;
    }

    /**
     * Returns all slice at the same slice position from one series with series
     * description (added to load ct data).
     */
    public LinkedList<String> getAllFileReferencesOfSameSlicePositionFromOneSeries(String ser_description, int sliceNumber) {

        TreeMap<Integer, Double> sliceNumberToSlicePositionMap = new TreeMap<Integer, Double>();

        TreeMap<Integer, String> fileRefMap = new TreeMap<Integer, String>();
        int imageSize = 0;
        double slicePosition = -999999.999;
        Enumeration e = m_dicomFileHeaders.keys();

        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            double slicePos;

            if (fi.getSliceLocation().compareTo("N.A.") != 0) {
                slicePos = new Double(fi.getSliceLocation()).doubleValue();
            } else {
                slicePos = 0.0;
            }

            if (fi.getSeriesDescription().compareTo(ser_description) == 0) {
                if (!sliceNumberToSlicePositionMap.containsValue(new Double(slicePos))) {
                    sliceNumberToSlicePositionMap.put(new Integer(sliceNumberToSlicePositionMap.size() + 1), new Double(slicePos));
                }
            }
        }

        float[] imagePositions = new float[sliceNumberToSlicePositionMap.size()];

        for (int i = 1; i <= sliceNumberToSlicePositionMap.size(); i++) {
            imagePositions[i - 1] = (float) (sliceNumberToSlicePositionMap.get(new Integer(i))).doubleValue();
        }

        imagePositions = QuickSortFloat.quickSort(imagePositions);

        e = m_dicomFileHeaders.keys();
        int cntr = 0;
        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            imageSize = fi.getImageHeight() * fi.getImageWidth();
            String series_description = fi.getSeriesDescription();
            int sliceNo = new Integer(fi.getInstanceNumber().trim()).intValue();

            if (fi.getSliceLocation().compareTo("N.A.") != 0) {
                slicePosition = new Double(fi.getSliceLocation()).doubleValue();
            } else {
                slicePosition = 0.0;
            }

            // slicePosition = new Double( fi.getSliceLocation( ) ).doubleValue( );
            //if ( slicePosition == ( sliceNumberToSlicePositionMap.get( new Integer( sliceNumber ) ) ).doubleValue() )
            if ((float) slicePosition == (float) imagePositions[sliceNumber - 1]) {
                if (series_description.compareTo(ser_description) == 0) {
                    fileRefMap.put(new Integer(fi.getInstanceNumber().trim()), alias);
                    m_lastFileInfo = fi;
                }
            }
        }

        Iterator it = fileRefMap.keySet().iterator();
        //int[] allImageData = new int[imageSize * fileRefMap.size( )];
        LinkedList<String> fileRefs = new LinkedList<String>();

        int k = 0;
        while (it.hasNext()) {
            String fileRef = m_dicomdir.getPath() + File.separatorChar + fileRefMap.get((Integer) it.next());
            fileRefs.add(fileRef);
            k++;
        }

        return fileRefs;
    }

    /**
     * Returns all slice at the same slice position from one series with series
     * description (added to load ct data).
     */
    public int[] getAllFilesOfSameSlicePositionFromOneSeries(String ser_description, int sliceNumber) {
        TreeMap<Integer, Double> sliceNumberToSlicePositionMap = new TreeMap<Integer, Double>();

        TreeMap<Integer, String> fileRefMap = new TreeMap<Integer, String>();
        int imageSize = 0;
        double slicePosition = -999999.999;
        Enumeration e = m_dicomFileHeaders.keys();

        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            double slicePos;

            if (fi.getSliceLocation().compareTo("N.A.") != 0) {
                slicePos = new Double(fi.getSliceLocation()).doubleValue();
            } else {
                slicePos = 0.0;
            }

            if (fi.getSeriesDescription().compareTo(ser_description) == 0) {
                if (!sliceNumberToSlicePositionMap.containsValue(new Double(slicePos))) {
                    sliceNumberToSlicePositionMap.put(new Integer(sliceNumberToSlicePositionMap.size() + 1), new Double(slicePos));
                }
            }
        }

        float[] imagePositions = new float[sliceNumberToSlicePositionMap.size()];

        for (int i = 1; i <= sliceNumberToSlicePositionMap.size(); i++) {
            imagePositions[i - 1] = (float) (sliceNumberToSlicePositionMap.get(new Integer(i))).doubleValue();
        }

        imagePositions = QuickSortFloat.quickSort(imagePositions);

        e = m_dicomFileHeaders.keys();
        int cntr = 0;
        while (e.hasMoreElements()) {
            String alias = (String) e.nextElement();
            FileInfo fi = m_dicomFileHeaders.get(alias);
            imageSize = fi.getImageHeight() * fi.getImageWidth();
            String series_description = fi.getSeriesDescription();
            int sliceNo = new Integer(fi.getInstanceNumber().trim()).intValue();

            if (fi.getSliceLocation().compareTo("N.A.") != 0) {
                slicePosition = new Double(fi.getSliceLocation()).doubleValue();
            } else {
                slicePosition = 0.0;
            }

            // slicePosition = new Double( fi.getSliceLocation( ) ).doubleValue( );
            //if ( slicePosition == ( sliceNumberToSlicePositionMap.get( new Integer( sliceNumber ) ) ).doubleValue() )
            if ((float) slicePosition == (float) imagePositions[sliceNumber - 1]) {
                if (series_description.compareTo(ser_description) == 0) {
                    fileRefMap.put(new Integer(fi.getInstanceNumber().trim()), alias);
                    m_lastFileInfo = fi;
                }
            }
        }

        Iterator it = fileRefMap.keySet().iterator();
        int[] allImageData = new int[imageSize * fileRefMap.size()];
        int k = 0;
        while (it.hasNext()) {
            DicomImageData DicIma = new DicomImageData(m_dicomdir.getPath() + File.separatorChar + fileRefMap.get((Integer) it.next()));
            System.arraycopy(DicIma.getPixelData(), 0, allImageData, k * imageSize, imageSize);
            k++;
        }

        return allImageData;
    }

    /**
     * Returns the Name attribute of the patient whose data are stored in the
     * DicomDirectory object.
     */
    public String getPatientName() {
        return m_patientName;
    }

    /**
     * Returns the DICOM FileHeader information HashTable object
     */
    public Hashtable<String, FileInfo> getDicomHeaderInformation() {
        return m_dicomFileHeaders;
    }

    /**
     * Returns the last loaded FileInfo object.
     *
     * @return
     */
    public FileInfo getLastFileInfo() {
        return m_lastFileInfo;
    }
    // Members of the DicomDirectory class
    private JFrame m_parent;
    private Hashtable<String, FileInfo> m_dicomFileHeaders = new Hashtable<String, FileInfo>();
    private String[] m_dcm_filelist;
    private File m_dicomdir;
    private DualProgressBar m_dualProgressBar;
    private DualProgressBar m_loadTableProgressBar;
    // Member that maps the "series number" to the "protocol name"
    private TreeMap< Integer, String> m_seriesNumberToSeriesDescriptionMap = new TreeMap< Integer, String>();
    private LinkedList<String> m_imageFileReferencesOfSeriesNumber = new LinkedList<String>();
    private HashMap< String, String> seriesUID2studyUID = new HashMap< String, String>();
    private HashMap< String, String> studyUID2patientID = new HashMap< String, String>();
    private HashMap< String, String> studyUID2studyDate = new HashMap< String, String>();
    private HashMap< String, String> patientID2patientNameBDate = new HashMap< String, String>();

    // The treeset of file reference having the same instance number
    // but a different series number 
    private LinkedList<String> m_allImageFileReferencesOfInstanceNumber = new LinkedList<String>();
    // Member containing the all series numbers having a specific 
    // series description
    private String m_seriesProtocolName;
    private Vector<Integer> m_seriesNumbersBelongingToSpecificSeriesProtocolName;
    // Listeners to textfield changes
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>();
    private DicomDirectory m_sender;
    private String m_image_modality;
    private String m_patientName;
    private FileInfo m_lastFileInfo;
    private HashMap<String, String> UID2FileMap = new HashMap<>();
    private org.spectrim.lib.SwingWorker worker;

    @Override
    public void stopMounting() {
        if (worker != null) {
            worker.interrupt();
            System.out.println("Directory Mounting Cancelled!");
            worker = null;
        }
        if(this.m_dualProgressBar != null){
            m_dualProgressBar.dispose();
        }
    }
}
