/*
 * File: $Id: FileInfo.java,v 1.3 2006/11/09 10:11:46 hans Exp $
 * History
 * Date: 6. Februar 2003, 09:34    Authors: Hans Slotboom and Martin Buerki
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 
package org.spectrim.dicom;


import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import org.spectrim.imagegraphicobjects.Point3D;
import org.spectrim.lib.Convert;

/** 
 * This class consists of public fields that describe an image file. 
 */
public class FileInfo 
{

    /* 8-bit unsigned integer (0-255). */
    public static final int GRAY8 = 0;
	
    /* 16-bit signed integer (-32768-32767). Imported signed images
       with any negative pixel values are converted to unsigned by 
       adding 32768. */
    public static final int GRAY16_SIGNED = 1;
	
    /* 16-bit unsigned integer (0-65535). */
    public static final int GRAY16_UNSIGNED = 2;
	
    /* 32-bit signed integer. Imported 32-bit integer images are
       converted to floating-point. */
    public static final int GRAY32_INT = 3;
	
    /* 32-bit floating-point. */
    public static final int GRAY32_FLOAT = 4;
	
    /* 8-bit unsigned integer with color lookup table. */
    public static final int COLOR8 = 5;
	
    /* 24-bit interleaved RGB. Import/export only. */
    public static final int RGB = 6;	
	
    /* 24-bit planer RGB. Import only. */
    public static final int RGB_PLANAR = 7;
	
    /* 1-bit black and white. Import only. */
    public static final int BITMAP = 8;
	
    /* 32-bit interleaved ARGB. Import only. */
    public static final int ARGB = 9;
	
    /* 24-bit interleaved BGR. Import only. */
    public static final int BGR = 10;
		
    // File formats
    public static final int UNKNOWN = 0;
    public static final int RAW = 1;
    public static final int TIFF = 2;
    public static final int GIF_OR_JPG = 3;
    public static final int FITS = 4;

    private String m_SCANInfo   = "NULL";
    private String m_JMRUI_Info = null;
    
    
    /** 
     * Creates a FileInfo object with all of its fields set to their default value.
     */
    public FileInfo( ) 
    {
        // assign default values
    	m_fileFormat        = UNKNOWN;
    	m_fileType          = GRAY8;
    	m_fileName          = "Untitled";
    	m_directory         = "";
    	m_url               = "";
    }
    
    /**
     * Returns the study instance uid.
     * @return 
     */
    public String getStudyInstanceUID( )
    {
        return m_studyInstanceUID;
    }
    
    /** 
     * Returns the number of bytes used per pixel.
     * @return 
     */
    public int getBytesPerPixel( ) 
    {
        switch ( m_fileType ) 
        {
            case GRAY8: 
            case COLOR8:
            case BITMAP:
                                    return 1;
            case GRAY16_SIGNED:
            case GRAY16_UNSIGNED: 
                                    return 2;
            case GRAY32_INT:
            case GRAY32_FLOAT:
            case ARGB:
                                    return 4;
            case RGB:
            case RGB_PLANAR:
            case BGR:
                                    return 3;
            default:
                                    return 0;
        }
    }

    
    /**
     *
     * Converts major TAGs into a displayable text
     *
     * @return
     */
    @Override
    public String toString( )
    {
    	return
    		"name=" + m_fileName
			+ ", dir=" + m_directory
			+ ", url=" + m_url
			+ ", width=" + m_width
			+ ", height=" + m_height
			+ ", nImages=" + m_numberOfFrames
			+ ", type=" + getType( )
			+ ", offset=" + m_filePointerOffsetPixelDataStart
			+ ", whiteZero=" + ( m_whiteIsZero?"t":"f" )
			+ ", Intel=" + ( m_intelByteOrder?"t":"f" )
			+ ", lutSize=" + m_lutSize;
    }

    /**
     * Set the pixel representation tag.
     * @param pixelRepresentation
     */
    public void setPixelRepresentation( String pixelRepresentation )
    {
        m_pixelRepresentation = pixelRepresentation;
    }

     /**
     * Set the pixel representation tag.
     * @return 
     */
    public String getPixelRepresentation( )
    {
        return m_pixelRepresentation;
    }

    /**
     * Sets the pixel spacing string
     * @param scale
     */
    public void setPixelSpacing( String scale ) 
    {
        m_pixelSpacing = scale;
    }

    /**
     * Returns the pixel spacing string.
     * @return 
     */
    public String getPixelSpacing( ) 
    {
        return m_pixelSpacing;
    }

    /**
     * New parameter JMRUIInfo.
     * @param JMRUIInfo
     */
    public void setJMRUIInfo( String JMRUIInfo )
    {
       m_JMRUI_Info = JMRUIInfo;

       if ( ( m_JMRUI_Info != null ) && ( m_JMRUI_Info.compareTo( "NULL" ) != 0 ) )
       {
           m_isDICOMFileCreatedByJMRUI = true;
       }

       // some code testing
       SVS_JMRUI_DICOM_Tag_FAST tag = new SVS_JMRUI_DICOM_Tag_FAST( );

       tag.setJMRUIDICOMTag( m_JMRUI_Info );

       if ( tag.decodeCompleteJMRUIDICOMTag( ) == true )
       {
           // System.out.println( "Successfully decoded jMRUI tag!" );
       }
       else
       {
           System.out.println( "Not successfully decoded jMRUI tag!" );
       }
    }

    /**
     * Returns whether the file is a by jMRUI created report Dicom file.
     * @return
     */
    public boolean isJMRUICreatedDICOMReportFile( )
    {
        return m_isDICOMFileCreatedByJMRUI;
    }


    /**
     * Returns the string  parameter.
     * @return 
     */
    public String getJMRUIInfo( )
    {
       return m_JMRUI_Info;
    }

    /**
     * New parameter
     * @param SCANInfo string contains all additional image information
     * added by the SCAN-laboratory.
     */
    public void setSCANInfo( String SCANInfo )
    {

        System.out.println( "SCAN Info tag = \n" + SCANInfo );

        m_SCANInfo = SCANInfo;
    }

    /**
     * Returns SCANinfo tag.
     * @return 
     */
    public String getSCANInfo( )
    {
       return m_SCANInfo;
    }

    /**
     * Sets the scanner model name.
     * @param scannerModelName
     */
    public void setScannerModelName( String scannerModelName )
    {
        m_scannerModelName = scannerModelName;
    }

    /**
     * Returns the scanner model name.
     * @return 
     */
    public String getScannerModelName( )
    {
        return m_scannerModelName;
    }
    
    
    /**
     * Returns the top left Point3D.
     * @return
     */
    public Point3D getTopLeftPoint( )
    {
        return m_topLeftPoint;
    }

    /**
     * Writes all patient related tags to a random access output file "fout".
     * @param fout
     * @throws java.io.IOException
     */
    public void writePatientRelatedTags( RandomAccessFile fout ) throws IOException
    {
        LinkedHashMap<String, DicomTagAndValue> patientLinkedHashMap = getPatientRelatedDicomTagsAndValues( );

        Iterator it = patientLinkedHashMap.keySet( ).iterator( );
        while ( it.hasNext( ) )
        {
            
            String tagName = (String)it.next( );
            DicomTagAndValue tag = patientLinkedHashMap.get( tagName );

            if ( ( tag.getTagValueAsString( ) != null ) &&
                 ( tag.getTagValueAsString( ).compareTo( "N.A." ) != 0 ) )
            {
                switch ( tag.getTagEcodingID( ) )
                {
                    case DicomDecoder.AE:
                    case DicomDecoder.AS:
                    case DicomDecoder.AT:
                    case DicomDecoder.CS:
                    case DicomDecoder.DA:
                    case DicomDecoder.DS:
                    case DicomDecoder.DT:
                    case DicomDecoder.IS:
                    case DicomDecoder.LO:
                    case DicomDecoder.LT:
                    case DicomDecoder.PN:
                    case DicomDecoder.SH:
                    case DicomDecoder.ST:
                    case DicomDecoder.TM:
                    case DicomDecoder.UI:
                                            // here a test must be made whether the cc of 0xaabbccdd equals zero
                                            // if not, the rotateLeft must be different -> is to be found out qqqqqqqqqq

                                            byte[] bytes1 = Convert.integer2ByteArray( tag.getTagID( ) );

//                                            for ( int i=0; i < 4; i++ )
//                                            {
//                                                System.out.print( bytes1[i] + "  " );
//                                            }
//
//                                            System.out.println( );

                                            if ( bytes1[2] == 0 )
                                            {
                                                int outp3 = Integer.rotateLeft( tag.getTagID( ), 8 );
                                                fout.writeInt( outp3 );
                                            }
                                            else
                                            {
                                                // System.out.println( "> > Writing bytes seperately!" );
                                                fout.write( bytes1[1] );
                                                fout.write( bytes1[0] );
                                                fout.write( bytes1[3] );
                                                fout.write( bytes1[2] );
                                            }

                                            fout.writeShort( tag.getTagEcodingID( ) );

                                            fout.writeShort( Short.reverseBytes( (short)tag.getTagValueAsString( ).length( ) ) );

                                            byte[] bytes = tag.getTagValueAsString( ).getBytes( );
                                            fout.write( bytes );
                                            break;

                    case DicomDecoder.US:
                    case DicomDecoder.SS:


                                            byte[] bytes2 = Convert.integer2ByteArray( tag.getTagID( ) );

//                                          for ( int i= 0; i < 4; i++ )
//                                          {
//                                              System.out.print( bytes2[i] + "  " );
//                                          }
//
//                                          System.out.println( );

                                            if ( bytes2[2] == 0 )
                                            {
                                                int outp3 = Integer.rotateLeft( tag.getTagID( ), 8 );
                                                fout.writeInt( outp3 );
                                            }
                                            else
                                            {
                                                // System.out.println( ">> Writing bytes seperately!" );
                                                fout.write( bytes2[1] );
                                                fout.write( bytes2[0] );
                                                fout.write( bytes2[3] );
                                                fout.write( bytes2[2] );
                                            }
                                            
                                            fout.writeShort( tag.getTagEcodingID( ) );  // 2 bytes

                                            byte[] sab1= new byte[2];

                                            int valAsInt1 = 2;

                                            Convert.int2wordl( sab1, 0, valAsInt1 );

                                            fout.write( sab1 );     // 2 bytes

                                            byte[] sab2= new byte[2];

                                            int valAsInt2 = new Integer( tag.getTagValueAsString( ).trim( ) );

                                            Convert.int2wordl( sab2, 0, valAsInt2 );

                                            fout.write( sab2 );     // 2 bytes

                                            break;
                    default:
                                            System.out.println( "Non handled Tag Encoding ID ! >> " );
                                            System.out.println( "Description : " + tagName );
                                            System.out.println( "Tag ID      : " + tag.getTagEcodingID( ) );
                                            System.out.println( "Tag value   :" +  tag.getTagValueAsString( ) );
                }
            }
        }
    }


    /**
     * Writes all patient related tags to a random access output file "fout".
     * @param fout
     * @throws java.io.IOException
     */
    public void writeMRRelatedTags( RandomAccessFile fout ) throws IOException
    {
        LinkedHashMap<String, DicomTagAndValue> mrRelatedLinkedHashMap = this.getMRRelatedlDicomTagsAndValues( );

        Iterator it = mrRelatedLinkedHashMap.keySet( ).iterator( );
        while ( it.hasNext( ) )
        {

            String tagName = (String)it.next( );
            DicomTagAndValue tag = mrRelatedLinkedHashMap.get( tagName );

            if ( ( tag.getTagValueAsString( ) != null ) &&
                 ( tag.getTagValueAsString( ).compareTo( "N.A." ) != 0 ) )
            {
                switch ( tag.getTagEcodingID( ) )
                {
                    case DicomDecoder.AE:
                    case DicomDecoder.AS:
                    case DicomDecoder.AT:
                    case DicomDecoder.CS:
                    case DicomDecoder.DA:
                    case DicomDecoder.DS:
                    case DicomDecoder.DT:
                    case DicomDecoder.IS:
                    case DicomDecoder.LO:
                    case DicomDecoder.LT:
                    case DicomDecoder.PN:
                    case DicomDecoder.SH:
                    case DicomDecoder.ST:
                    case DicomDecoder.TM:
                    case DicomDecoder.UI:
                                            // here a test must be made whether the cc of 0xaabbccdd equals zero
                                            // if not, the rotateLeft must be different -> is to be found out qqqqqqqqqq

                                            byte[] bytes1 = Convert.integer2ByteArray( tag.getTagID( ) );

//                                            for ( int i=0; i < 4; i++ )
//                                            {
//                                                System.out.print( bytes1[i] + "  " );
//                                            }
//
//                                            System.out.println( );

                                            if ( bytes1[2] == 0 )
                                            {
                                                int outp3 = Integer.rotateLeft( tag.getTagID( ), 8 );
                                                fout.writeInt( outp3 );
                                            }
                                            else
                                            {
                                                // System.out.println( "> > Writing bytes seperately!" );
                                                fout.write( bytes1[1] );
                                                fout.write( bytes1[0] );
                                                fout.write( bytes1[3] );
                                                fout.write( bytes1[2] );
                                            }

                                            fout.writeShort( tag.getTagEcodingID( ) );

                                            fout.writeShort( Short.reverseBytes( (short)tag.getTagValueAsString( ).length( ) ) );

                                            byte[] bytes = tag.getTagValueAsString( ).getBytes( );
                                            fout.write( bytes );
                                            break;

                    case DicomDecoder.US:
                    case DicomDecoder.SS:


                                            byte[] bytes2 = Convert.integer2ByteArray( tag.getTagID( ) );

//                                            for ( int i= 0; i < 4; i++ )
//                                            {
//                                                System.out.print( bytes2[i] + "  " );
//                                            }
//
//                                            System.out.println( );

                                            if ( bytes2[2] == 0 )
                                            {
                                                int outp3 = Integer.rotateLeft( tag.getTagID( ), 8 );
                                                fout.writeInt( outp3 );
                                            }
                                            else
                                            {
                                                // System.out.println( ">> Writing bytes seperately!" );
                                                fout.write( bytes2[1] );
                                                fout.write( bytes2[0] );
                                                fout.write( bytes2[3] );
                                                fout.write( bytes2[2] );
                                            }

                                            fout.writeShort( tag.getTagEcodingID( ) );  // 2 bytes

                                            byte[] sab1= new byte[2];

                                            int valAsInt1 = 2;

                                            Convert.int2wordl( sab1, 0, valAsInt1 );

                                            fout.write( sab1 );     // 2 bytes

                                            byte[] sab2= new byte[2];

                                            int valAsInt2 = new Integer( tag.getTagValueAsString( ).trim( ) );

                                            Convert.int2wordl( sab2, 0, valAsInt2 );

                                            fout.write( sab2 );     // 2 bytes

                                            break;
                    default:
                                            System.out.println( "Non handled Tag Encoding ID ! >> " );
                                            System.out.println( "Description : " + tagName );
                                            System.out.println( "Tag ID      : " + tag.getTagEcodingID( ) );
                                            System.out.println( "Tag value   :" +  tag.getTagValueAsString( ) );
                }
            }
        }
    }





    /**
     * Writes all patient related tags to a random access output file "fout".
     * @param fout
     * @throws java.io.IOException
     */
    public void writeStudySeriesInstancesRelatedTags( RandomAccessFile fout ) throws IOException
    {
        LinkedHashMap<String, DicomTagAndValue> studiesSeriesInstancesLinkedHashMap = getStudySeriesInstanceTagsAndValues( );

        Iterator it = studiesSeriesInstancesLinkedHashMap.keySet( ).iterator( );
        while ( it.hasNext( ) )
        {

            String tagName = (String)it.next( );
            DicomTagAndValue tag = studiesSeriesInstancesLinkedHashMap.get( tagName );

//            System.out.println( "Study/Series/Instance related tagName           = " + tagName );
//            System.out.println( "Study/Series/Instance related tagID             = " + tag.getTagEcodingID( ) );
//            System.out.println( "Study/Series/Instance related tag description   = " + tag.getTagDescription( ) );
//            System.out.println( "Study/Series/Instance related tag value         = " + tag.getTagValueAsString( ) );
         
            if ( ( tag.getTagValueAsString( ) != null ) &&
                 ( tag.getTagValueAsString( ).compareTo( "N.A." ) != 0 ) )
            {
                switch ( tag.getTagEcodingID( ) )
                {
                    case DicomDecoder.AE:
                    case DicomDecoder.AS:
                    case DicomDecoder.AT:
                    case DicomDecoder.CS:
                    case DicomDecoder.DA:
                    case DicomDecoder.DS:
                    case DicomDecoder.DT:
                    case DicomDecoder.IS:
                    case DicomDecoder.LO:
                    case DicomDecoder.LT:
                    case DicomDecoder.PN:
                    case DicomDecoder.SH:
                    case DicomDecoder.ST:
                    case DicomDecoder.TM:
                    case DicomDecoder.UI:
                                            // here a test must be made whether the cc of 0xaabbccdd equals zero
                                            // if not, the rotateLeft must be different -> is to be found out qqqqqqqqqq

                                            byte[] bytes1 = Convert.integer2ByteArray( tag.getTagID( ) );

//                                            for ( int i=0; i < 4; i++ )
//                                            {
//                                                System.out.print( bytes1[i] + "  " );
//                                            }
//
//                                            System.out.println( );

                                            if ( bytes1[2] == 0 )
                                            {
                                                int outp3 = Integer.rotateLeft( tag.getTagID( ), 8 );
                                                fout.writeInt( outp3 );
                                            }
                                            else
                                            {
//                                                System.out.println( "> > Writing bytes seperately!" );
                                                fout.write( bytes1[1] );
                                                fout.write( bytes1[0] );
                                                fout.write( bytes1[3] );
                                                fout.write( bytes1[2] );
                                            }

                                            fout.writeShort( tag.getTagEcodingID( ) );

                                            fout.writeShort( Short.reverseBytes( (short)tag.getTagValueAsString( ).getBytes( ).length ) );

                                            byte[] bytes = tag.getTagValueAsString( ).getBytes( );

                                            fout.write( bytes );
                                            break;

                    case DicomDecoder.US:
                    case DicomDecoder.SS:



                                            byte[] bytes2 = Convert.integer2ByteArray( tag.getTagID( ) );

//                                            for ( int i= 0; i < 4; i++ )
//                                            {
//                                                System.out.print( bytes2[i] + "  " );
//                                            }
//
//                                            System.out.println( );

                                            if ( bytes2[2] == 0 )
                                            {
                                                int outp3 = Integer.rotateLeft( tag.getTagID( ), 8 );
                                                fout.writeInt( outp3 );
                                            }
                                            else
                                            {
                                                // System.out.println( ">> Writing bytes seperately!" );
                                                fout.write( bytes2[1] );
                                                fout.write( bytes2[0] );
                                                fout.write( bytes2[3] );
                                                fout.write( bytes2[2] );
                                            }

                                            fout.writeShort( tag.getTagEcodingID( ) );  // 2 bytes

                                            byte[] sab1= new byte[2];

                                            int valAsInt1 = 2;

                                            Convert.int2wordl( sab1, 0, valAsInt1 );

                                            fout.write( sab1 );     // 2 bytes

                                            byte[] sab2= new byte[2];

                                            int valAsInt2 = new Integer( tag.getTagValueAsString( ).trim( ) );

                                            Convert.int2wordl( sab2, 0, valAsInt2 );

                                            fout.write( sab2 );     // 2 bytes

                                            break;
                    default:
                                            System.out.println( "Non handled Tag Encoding ID ! >> " );
                                            System.out.println( "Description : " + tagName );
                                            System.out.println( "Tag ID      : " + tag.getTagEcodingID( ) );
                                            System.out.println( "Tag value   :" +  tag.getTagValueAsString( ) );
                }
            }
        }
    }


    /**
     * Writes all patient related tags to a random access output file "fout".
     * @param fout
     * @throws java.io.IOException
     */
    public void writeGeometryRelatedTags( RandomAccessFile fout ) throws IOException
    {
        LinkedHashMap<String, DicomTagAndValue> geometryRelatedLinkedHashMap = this.getGeomtryRelatedDicomTagsAndValues( );

        Iterator it = geometryRelatedLinkedHashMap.keySet( ).iterator( );
        while ( it.hasNext( ) )
        {

            String tagName = (String)it.next( );
            DicomTagAndValue tag = geometryRelatedLinkedHashMap.get( tagName );

            if ( ( tag.getTagValueAsString( ) != null ) &&
                 ( tag.getTagValueAsString( ).compareTo( "N.A." ) != 0 ) )
            {
                if ( tag.getTagValueAsString( ).compareTo( "" ) == 0 )
                {
                    System.out.println( " ##### >>>>> Warning : empty geometry tag!" );
                }

                switch ( tag.getTagEcodingID( ) )
                {
                    case DicomDecoder.AE:
                    case DicomDecoder.AS:
                    case DicomDecoder.AT:
                    case DicomDecoder.CS:
                    case DicomDecoder.DA:
                    case DicomDecoder.DS:
                    case DicomDecoder.DT:
                    case DicomDecoder.IS:
                    case DicomDecoder.LO:
                    case DicomDecoder.LT:
                    case DicomDecoder.PN:
                    case DicomDecoder.SH:
                    case DicomDecoder.ST:
                    case DicomDecoder.TM:
                    case DicomDecoder.UI:
                                            // here a test must be made whether the cc of 0xaabbccdd equals zero
                                            // if not, the rotateLeft must be different -> is to be found out qqqqqqqqqq

                                            byte[] bytes1 = Convert.integer2ByteArray( tag.getTagID( ) );

//                                            for ( int i=0; i < 4; i++ )
//                                            {
//                                                System.out.print( bytes1[i] + "  " );
//                                            }
//
//                                            System.out.println( );

                                            if ( bytes1[2] == 0 )
                                            {
                                                int outp3 = Integer.rotateLeft( tag.getTagID( ), 8 );
                                                fout.writeInt( outp3 );
                                            }
                                            else
                                            {
                                                // System.out.println( "> > Writing bytes seperately!" );
                                                fout.write( bytes1[1] );
                                                fout.write( bytes1[0] );
                                                fout.write( bytes1[3] );
                                                fout.write( bytes1[2] );
                                            }

                                            fout.writeShort( tag.getTagEcodingID( ) );

                                            fout.writeShort( Short.reverseBytes( (short)tag.getTagValueAsString( ).length( ) ) );

                                            byte[] bytes = tag.getTagValueAsString( ).getBytes( );
                                            fout.write( bytes );
                                            break;

                    case DicomDecoder.US:
                    case DicomDecoder.SS:

                                            byte[] bytes2 = Convert.integer2ByteArray( tag.getTagID( ) );

//                                            for ( int i= 0; i < 4; i++ )
//                                            {
//                                                System.out.print( bytes2[i] + "  " );
//                                            }
//
//                                            System.out.println( );

                                            if ( bytes2[2] == 0 )
                                            {
                                                int outp3 = Integer.rotateLeft( tag.getTagID( ), 8 );
                                                fout.writeInt( outp3 );
                                            }
                                            else
                                            {
                                                // System.out.println( ">> Writing bytes seperately!" );
                                                fout.write( bytes2[1] );
                                                fout.write( bytes2[0] );
                                                fout.write( bytes2[3] );
                                                fout.write( bytes2[2] );
                                            }

                                            fout.writeShort( tag.getTagEcodingID( ) );  // 2 bytes

                                            byte[] sab1= new byte[2];

                                            int valAsInt1 = 2;

                                            Convert.int2wordl( sab1, 0, valAsInt1 );

                                            fout.write( sab1 );     // 2 bytes

                                            byte[] sab2= new byte[2];

                                            int valAsInt2 = new Integer( tag.getTagValueAsString( ).trim( ) );

                                            Convert.int2wordl( sab2, 0, valAsInt2 );

                                            fout.write( sab2 );     // 2 bytes

                                            break;
                    default:
                                            System.out.println( "Non handled Tag Encoding ID ! >> " );
                                            System.out.println( "Description : " + tagName );
                                            System.out.println( "Tag ID      : " + tag.getTagEcodingID( ) );
                                            System.out.println( "Tag value   :" +  tag.getTagValueAsString( ) );
                }
            }
        }
    }


    /**
     * Writes all patient related tags to a random access output file "fout".
     * @param fout
     * @throws java.io.IOException
     */
    public void writeOtherTags( RandomAccessFile fout ) throws IOException
    {
        LinkedHashMap<String, DicomTagAndValue> otherTagsLinkedHashMap = getOtherDicomTagsAndValues( );

        Iterator it = otherTagsLinkedHashMap.keySet( ).iterator( );
        while ( it.hasNext( ) )
        {

            String tagName = (String)it.next( );
            DicomTagAndValue tag = otherTagsLinkedHashMap.get( tagName );

            if ( ( tag.getTagValueAsString( ) != null ) &&
                 ( tag.getTagValueAsString( ).compareTo( "N.A." ) != 0 ) )
            {
                switch ( tag.getTagEcodingID( ) )
                {
                    case DicomDecoder.AE:
                    case DicomDecoder.AS:
                    case DicomDecoder.AT:
                    case DicomDecoder.CS:
                    case DicomDecoder.DA:
                    case DicomDecoder.DS:
                    case DicomDecoder.DT:
                    case DicomDecoder.IS:
                    case DicomDecoder.LO:
                    case DicomDecoder.LT:
                    case DicomDecoder.PN:
                    case DicomDecoder.SH:
                    case DicomDecoder.ST:
                    case DicomDecoder.TM:
                    case DicomDecoder.UI:
                                            // here a test must be made whether the cc of 0xaabbccdd equals zero
                                            // if not, the rotateLeft must be different -> is to be found out qqqqqqqqqq

                                            byte[] bytes1 = Convert.integer2ByteArray( tag.getTagID( ) );

//                                            for ( int i=0; i < 4; i++ )
//                                            {
//                                                System.out.print( bytes1[i] + "  " );
//                                            }
//
//                                            System.out.println( );

                                            if ( bytes1[2] == 0 )
                                            {
                                                int outp3 = Integer.rotateLeft( tag.getTagID( ), 8 );
                                                fout.writeInt( outp3 );
                                            }
                                            else
                                            {
//                                                System.out.println( "> > Writing bytes seperately!" );
                                                fout.write( bytes1[1] );
                                                fout.write( bytes1[0] );
                                                fout.write( bytes1[3] );
                                                fout.write( bytes1[2] );
                                            }

                                            fout.writeShort( tag.getTagEcodingID( ) );

                                            fout.writeShort( Short.reverseBytes( (short)tag.getTagValueAsString( ).length( ) ) );

                                            byte[] bytes = tag.getTagValueAsString( ).getBytes( );
                                            fout.write( bytes );
                                            break;

                    case DicomDecoder.US:
                    case DicomDecoder.SS:


                                            byte[] bytes2 = Convert.integer2ByteArray( tag.getTagID( ) );

//                                            for ( int i= 0; i < 4; i++ )
//                                            {
//                                                System.out.print( bytes2[i] + "  " );
//                                            }
//
//                                            System.out.println( );

                                            if ( bytes2[2] == 0 )
                                            {
                                                int outp3 = Integer.rotateLeft( tag.getTagID( ), 8 );
                                                fout.writeInt( outp3 );
                                            }
                                            else
                                            {
//                                              System.out.println( ">> Writing bytes seperately!" );
                                                fout.write( bytes2[1] );
                                                fout.write( bytes2[0] );
                                                fout.write( bytes2[3] );
                                                fout.write( bytes2[2] );
                                            }

                                            fout.writeShort( tag.getTagEcodingID( ) );  // 2 bytes

                                            byte[] sab1= new byte[2];

                                            int valAsInt1 = 2;

                                            Convert.int2wordl( sab1, 0, valAsInt1 );

                                            fout.write( sab1 );     // 2 bytes

                                            byte[] sab2= new byte[2];

                                            int valAsInt2 = new Integer( tag.getTagValueAsString( ).trim( ) );

                                            Convert.int2wordl( sab2, 0, valAsInt2 );

                                            fout.write( sab2 );     // 2 bytes

                                            break;
                    default:
                                            System.out.println( "Non handled Tag Encoding ID ! >> " );
                                            System.out.println( "Description : " + tagName );
                                            System.out.println( "Tag ID      : " + tag.getTagEcodingID( ) );
                                            System.out.println( "Tag value   :" +  tag.getTagValueAsString( ) );
                }
            }
        }
    }


    /**
     * If this object is a JMRUI DICOM report object, then this TAG is written
     * automatically to the File.
     * @param fout
     * @throws java.io.IOException
     */
    public void writeJMRUIDICOMTagsToFile(  RandomAccessFile fout ) throws IOException
    {
        // System.out.println("!!! >>> Writing -> writeJMRUIDICOMTagsToFile" + getJMRUIInfo( ) );

        byte[] jmruiInfoAsBytes = getJMRUIInfo( ).getBytes( );

        fout.writeInt( Integer.rotateLeft( Integer.reverseBytes( DicomDecoder.JMRUI_INFO_TAG ), 16 ) );

        fout.writeShort( DicomDecoder.OB );

        byte[] byteArr = new byte[2];

        fout.write( byteArr );

        fout.writeInt( Integer.reverseBytes( jmruiInfoAsBytes.length ) );

        fout.write( jmruiInfoAsBytes );
    }

    /**
     * Sets the SOP Class UID
     * @param sopClassUID
     */
    public void setSOPClassUID( String sopClassUID )
    {
       m_sopClassUID = sopClassUID;
    }

    /**
     * Returns the SOP Class UID.
     * @return 
     */
    public String getSOPClassUID(  )
    {
       return m_sopClassUID;
    }

    /**
     *
     * Sets the media storage sop class UID.
     *
     * @param mediaStorageSOPClassUID
     */
    public void setMediaStorageSOPClassUID( String mediaStorageSOPClassUID )
    {
        m_mediaStorageSOPClassUID = mediaStorageSOPClassUID;
    }

    /**
     *
     * Sets the media storage sop class UID.
     *
     * @return 
     */
    public String getMediaStorageSOPClassUID( )
    {
        return m_mediaStorageSOPClassUID;
    }

    /**
     * Sets the implementation Class UID.
     * @param implementationClassUID
     */
    public void setImplementationClassUID( String implementationClassUID )
    {
        m_implementationClassUID = implementationClassUID;
    }

     /**
     * Returns the implementation Class UID.
     * @return 
     */
    public String getImplementationClassUID( )
    {
        return m_implementationClassUID;
    }

    /**
     * Sets the used character set string.
     * @param characterSet
     */
    public void setCharacterSet( String characterSet )
    {
        m_characterSet = characterSet;
    }

    /**
     * Returns the used character set string
     * @return 
     */
    public String getCharacterSet( )
    {
        return m_characterSet;
    }

    /**
     * Sets the group length attribute
     * @param groupLength
     */
    public void setGroupLength( String groupLength )
    {
        m_groupLength = groupLength;
    }

    /**
     * Returns the group length attribute
     * @return 
     */
    public String getGroupLength( )
    {
        return m_groupLength;
    }

    /**
     * Sets the data offset pointer value: there the data starts
     * @param dataOffsetPointer
     */
    public void setDataOffsetPointer( int dataOffsetPointer )
    {
        m_dataOffsetPointer = dataOffsetPointer;
    }

    /**
     * Returns the data offset pointer value: there the data starts.
     * @return 
     */
    public int getDataOffsetPointer( )
    {
        return m_dataOffsetPointer;
    }

    /**
     * Returns the type of the image encoding.
     * @return
     */
    private String getType( )
    {
    	switch ( m_fileType )
        {
            case GRAY8:
                            return "byte";
            case GRAY16_SIGNED:
                            return "short";
            case GRAY16_UNSIGNED:
                            return "ushort";
            case GRAY32_INT:
                            return "int";
            case GRAY32_FLOAT:
                            return "float";
            case COLOR8:
                            return "byte+lut";
            case RGB:
                            return "RGB";
            case RGB_PLANAR:
                            return "RGB(p)";
            case BITMAP:
                            return "bitmap";
            case ARGB:
                            return "ARGB";
            case BGR:
                            return "BGR";
            default:
                            return "";
    	}
    }
    
    /**
     * Returns the manufacturer tag of the Dicom header file.
     * @return 
     */
    public String getManufacturer( )
    {
        return m_manufacturer;
    }
    
    /**
     * Returns the image file name.
     * @return 
     */
    public String getFileName( )
    {
        return m_fileName;
    }

    /**
     * Sets the image directory.
     * @return 
     */
    public String getDirectory(  )
    {
        return m_directory;
    }
    
    /**
     * Returns the url.
     * @return 
     */
    public String getURL( )
    {
        return m_url;
    }

    /**
     * Returns the number of images.
     * @return 
     */
    public String getNumberOfFrames( )
    {
        return m_numberOfFrames;
    }
    
    /**
     * Returns the image format.
     * @return 
     */
    public int getFileFormat( )
    {
        return m_fileFormat;
    }

    /**
     * Returns the slice position.
     * @return 
     */
    public int getFileType(  )
    {
        return m_fileType;
    }
    
    /**
     * Returns the image width.
     * @return 
     */
    public int getImageWidth( )
    {
        return m_width;
    }

    /**
     * Returns the image height.
     * @return 
     */
    public int getImageHeight( )
    {
        return m_height;
    }

    /**
     * Returns the slice position.
     * @return 
     */
    public String getSliceLocation( )
    {
        return m_sliceLocation;
    }

    /**
     * Returns the file pointer offset where image pixel data starts.
     * @return 
     */
    public int getFilePointerOffsetOfPixelData(  )
    {
        return m_filePointerOffsetPixelDataStart;
    }

    /**
     * Returns the pixel width.
     * @return 
     */
    public String getPixelWidth( )
    {
        return m_pixelWidth;
    }

    /**
     * Returns the pixel height.
     * @return 
     */
    public String getPixelHeight( )
    {
        return m_pixelWidth;
    }

    /**
     * Returns the pixel depth.
     * @return 
     */
    public String getPixelDepth( )
    {
        return m_pixelDepth;
    }

    /**
     * Returns the whiteisZero attribute.
     * @return 
     */
    public boolean getWhiteIsZero( )
    {
        return m_whiteIsZero;
    }

    /**
     * Returns the intelByteOrder attribute.
     * @return 
     */
    public boolean getIntelByteOrder(  )
    {
        return m_intelByteOrder;
    } 
 
    /**
     * Returns lutSize.
     * @return 
     */
    public int getLutSize( )
    {
        return m_lutSize;
    }    
        
    /**
     * Returns Reds byte array.
     * @return 
     */
    public byte[] getReds( )
    {
        return m_reds;
    }
    
    /**
     * Returns Greens byte array.
     * @return 
     */
    public byte[] getGreens( )
    {
        return m_greens;
    }
    
    /**
     * Returns Blues byte array.
     * @return 
     */
    public byte[] getBlues( )
    {
        return m_blues;
    }
    
    /**
     * Returns the unit string.
     * @return 
     */
    public String getUnit( )
    {
        return m_unit;
    }

    
    /**
     * Returns the slice thickness. 
     * @return 
     */
    public String getSliceThickness( )
    {
        return m_sliceThickness;
    }

    /**
     * Returns the repetition time attribute.
     * @return 
     */
    public String getTR( )
    {
        return m_TR;
    }

    /**
     * Returns the echo time attribute.
     * @return 
     */
    public String getEchoTime( )
    {
        return m_echoTime;
    }

    /**
     * Returns the carrier frequency attribute [MHz].
     * @return 
     */
    public String getImagingFrequency( )
    {
        return m_imagingFrequency; 
    }
    
    /**
     * Returns the number of phase encodes.
     * @return 
     */
    public String getNumberOfPhaseEncodes( )
    {
        return m_numPhaseEncodes;
    }

    /**
     * Returns the echo train length attribute.
     * @return 
     */
    public String getEchoTrainLength(  )
    {
        return m_echoTrainLength;
    }

    /**
     * Returns the percentage sampling attribute.
     * @return 
     */
    public String getPercentSampling( )
    {
        return m_percentSampling;
    }

    /**
     * Returns the percentage sampling attribute.
     * @return 
     */
    public String getPercentPhaseFOV(  )
    {
        return m_percentPhaseFOV;
    }

    /**
     * Returns the pixel bandwidth attribute.
     * @return 
     */
    public String getPixelBandwidth(  )
    {
        return m_pixelBandwidth;
    }
        
    /**
     * Returns the pulse sequence name attribute.
     * @return 
     */
    public String getSequenceName( )
    {
        return m_sequenceName;
    }

    /**
     * Returns the pulse software version attribute.
     * @return 
     */
    public String getSoftwareVersion( )
    {
        return m_softwareVersion;
    }
    
    /**
     * Sets the protocol name attribute.
     * @return 
     */
    public String getProtocolName( )
    {
        return m_protocolName;
    }

    /**
     * Returns the contrast bolus volume attribute [ml].
     * @return 
     */
    public String getContrastBolusVolume( )
    {
        return m_contrastBolusVolume;
    }

    /**
     * Returns the contrast flow duration attribute [seconds].
     * @return 
     */
    public String getContrastFlowDuration( )
    {
        return m_contrastFlowDuration;
    }

    /**
     * Returns the transmit coil name attribute.
     * @return 
     */
    public String getTransmitCoilName( )
    {
        return m_transmitCoilName;
    }


    /**
     * Returns the phase encoding direction attribute.
     * @return 
     */
    public String getPhaseEncodingDirection(  )
    {
        return m_phaseEncodingDirection;
    }
    
    /**
     * Returns the flip angle attribute.
     * @return 
     */
    public String getFlipAngle( )
    {
        return m_flipAngle;
    }

    /**
     * Returns the variable flip angle flag attribute.
     * @return 
     */
    public String getVariableFlipAngleFlag( )
    {
        return m_variableFlipAngleFlag;
    }

    /**
     * Returns the specific absorption rate attribute.
     * @return 
     */
    public String getSAR( )
    {
        return m_SAR;
    }

    /**
     * Returns the dBdt attribute.
     * @return 
     */
    public String getdBdt( )
    {
        return m_dBdt;
    }
    
    /**
     * Returns the patient position attribute.
     * @return 
     */
    public String getPatientPosition( )
    {
        return m_patientPosition;
    }
    
    /**
     * Returns the study ID attribute.
     * @return 
     */
    public String getStudyID( )
    {
        return m_studyID;
    }
    
    /**
     * Returns the series number attribute.
     * @return 
     */
    public String getSeriesNumber( )
    {
        return m_seriesNumber;
    }    
    
    /**
     * Returns the acquisition number attribute.
     * @return 
     */
    public String getAcquisitionNumber( )
    {
        return m_acquisitionNumber;
    }       
    
    /**
     * Returns the instance number attribute.
     * @return 
     */
    public String getInstanceNumber( )
    {
        return m_instanceNumber;
    }

    /**
     * Returns the image position (patient) attribute.
     * @return 
     */
    public String getImagePositionPatient( )
    {
        return m_imagePosition;
    }  

    /**
     * Returns the image position x (patient) attribute.
     * @return 
     */
    public double getImagePositionPatientX( )
    {
        return m_x_position_patient;
    }  

   /**
     * Returns the image position y (patient) attribute.
     * @return 
     */
    public double getImagePositionPatientY( )
    {
        return m_y_position_patient;
    }      
    
   /**
     * Returns the image position z (patient) attribute.
     * @return 
     */
    public double getImagePositionPatientZ( )
    {
        return m_z_position_patient;
    }      
    
    /**
     * Returns the image orientation (patient) attribute.
     * @return 
     */
    public String getImageOrientationPatient( )
    {
        return m_imageOrientation;
    }
    
    /**
     * Returns the samples per pixel attribute.
     * @return 
     */
    public String getSamplesPerPixel( )
    {
        return m_samplesPerPixel;
    }

    /**
     * Returns the photometric interpretation attribute.
     * @return 
     */
    public String getPhotometricInterpretation(  )
    {
        return m_photometricInterpretation;
    }
    
    /**
     * Returns the planar configuration attribute.
     * @return 
     */
    public int getPlanarConfiguration(  )
    {
        return m_planarConfiguration;
    }
    
    /**
     * Returns the window center attribute
     * @return 
     */
    public String getWindowCenter(  )
    {
        return m_windowCenter;
    }

    /**
     * Returns the window center attribute.
     * @return 
     */
    public String getWindowWidth(  )
    {
        return m_windowWidth;
    }

    /**
     * Returns bits stored attribute.
     * @return 
     */
    public String getBitsStored( )
    {
        return m_bitsStored;
    }

    /**
     * Returns the high bit number attribute.
     * @return 
     */
    public String getHighBit( )
    {
        return m_highBit;
    }
    
    /**
     * Returns the smallest pixel value attribute.
     * @return 
     */
    public String getSmallestPixelValue( )
    {
        return m_smallestPixelValue;
    }

    /**
     * Returns the largest pixel value attribute.
     * @return 
     */
    public String getLargestPixelValue( )
    {
        return m_largestPixelValue;
    }

    /**
     * Returns the patient name attribute.
     * @return 
     */
    public String getPatientName(  )
    {
        return m_patientName;
    }

    /**
     * Returns the patient ID attribute.
     * @return 
     */
    public String getPatientID(  )
    {
        return m_patientID;
    }  

    /**
     * Returns the patient id attribute.
     * @return 
     */
    public String getPatientBirthDate( )
    {
        return m_patientBirthDate;
    }

    /**
     * Returns the patient sex attribute.
     * @return 
     */
    public String getPatientSex( )
    {
        return m_patientSex;
    }

    /**
     * Returns the patient age attribute.
     * @return 
     */
    public String getPatientAge( )
    {
        return m_patientAge;
    }
    
    /**
     * Returns the patient weight attribute.
     * @return 
     */
    public String getPatientWeight( )
    {
        return m_patientWeight;
    }
    
    /**
     * Returns the contrast agent attribute.
     * @return 
     */
    public String getContrastAgent( )
    {
        return m_contrastAgent;
    }

    /**
     * Returns the scanning sequence attribute.
     * @return 
     */
    public String getScanningSequence( )
    {
        return m_scanningSequence;
    }
    
    /**
     * Returns the scanning sequence attribute.
     * @return 
     */
    public String getSequenceVariant( )
    {
        return m_sequenceVariant;
    }

    /**
     * Returns the sequence variant attribute.
     * @return 
     */
    public String getScanOptions( )
    {
        return m_scanOptions;
    }

    /**
     * Returns the acquisition type attribute.
     * @return 
     */
    public String getAcquisitionType( )
    {
        return m_acquisitionType;
    }
    
    /**
     * Returns the image type attribute.
     * @return 
     */
    public String getImageType( )
    {
        return m_imageType;
    }
    
    /**
     * Returns the study date attribute.
     * @return 
     */
    public String getStudyDate( )
    {
        return m_studyDate;
    }
    
    /**
     * Returns the series date attribute.
     * @return 
     */
    public String getSeriesDate( )
    {
        return m_seriesDate;
    }
    
    /**
     * Returns the acquisition date attribute.
     * @return 
     */
    public String getAcquisitionDate( )
    {
        return m_acquisitionDate;
    }

    /**
     * Returns the content date attribute.
     * @return 
     */
    public String getContentDate( )
    {
        return m_contentDate;
    }
    
    /**
     * Returns the study time attribute.
     * @return 
     */
    public String getStudyTime( )
    {
        return m_studyTime;
    }
    
    /**
     * Returns the series time attribute.
     * @return 
     */
    public String getSeriesTime( )
    {
        return m_seriesTime;
    }    

    /**
     * Returns the acquisition time attribute.
     * @return 
     */
    public String getAcquisitionTime( )
    {
        return m_acquisitionTime;
    }

    /**
     * Returns the content time attribute.
     * @return 
     */
    public String getContentTime( )
    {
        return m_contentTime;
    }

    /**
     * Returns the accession number attribute.
     * @return 
     */
    public String getAccessionNumber( )
    {
        return m_accessionNumber;
    }
    
    /**
     * Returns the series description attribute.
     * @return 
     */
    public String getSeriesDescription(  )
    {
        return m_seriesDescription;
    }

    /**
     * Returns the non image data data structure (byte array).
     * @return 
     */
    public byte[] getNonImageData( )
    {
        return m_nonImageData;
    }
    
    /**
     * Returns the SOP instance uid.
     * @return 
     */
    public String getSOPInstanceUID( )
    {
        return m_sop_instance_uid;
    }
    
    /**
     * Returns the Series instance uid.
     * @return 
     */
    public String getSeriesInstanceUID( )
    {
        return m_series_instance_uid;
    }
    
    
    /* ******************  SETTERS ********************** */
    
    /**
     * Sets the manufacturer tag of the Dicom header file
     * @param manufacturer
     */
    public void setManufacturer( String manufacturer )
    {
        m_manufacturer = manufacturer;
        // System.out.println( "Manufacturer : " + m_manufacturer );
    }
    
    
    /**
     * Returns the study instance uid.
     * @param studyUID
     */
    public void setStudyInstanceUID( String studyUID )
    {
        m_studyInstanceUID = studyUID;
    }

    /**
     * Returns the study description.
     * @param studyDescription
     */
    public void setStudyDescription( String studyDescription )
    {
        m_studyDescription = studyDescription;
    }

    /**
     * Returns the study description.
     * @return 
     */
    public String getStudyDescription( )
    {
        return m_studyDescription;
    }

    /**
     * Sets the SOP instance uid.
     * @param sopInstanceUID
     */
    public void setSOPInstanceUID( String sopInstanceUID )
    {
        m_sop_instance_uid = sopInstanceUID;
    }
    
    /**
     * Sets the Series instance uid.
     * @param seriesInstanceUID
     */
    public void setSeriesInstanceUID( String seriesInstanceUID )
    {
        m_series_instance_uid = seriesInstanceUID;
    }
    
    
    /**
     * Sets the image file name.
     * @param fname
     */
    public void setFileName( String fname )
    {
        m_fileName = fname;
    }
    
    /**
     * Sets the image directory.
     * @param dir
     */
    public void setDirectory( String dir )
    {
        m_directory = dir;
    }
        
    /**
     * Sets the image format.
     * @param format
     */
    public void setFileFormat( int format )
    {
        m_fileFormat = format;
    }
    
    /**
     * Returns the file type.
     * @param file_type
     */
    public void setFileType( int file_type )
    {
        m_fileType = file_type;
    }
    
    /**
     * Sets the image width.
     * @param width
     */
    public void setImageWidth( int width )
    {
        m_width = width;
    }

    /**
     * Sets the image height.
     * @param height
     */
    public void setImageHeight( int height )
    {
        m_height = height;
    }

    /**
     * Sets the image width.
     * @param width
     */
    public void setInitialImageWidth( int width )
    {
        m_initialImageWidth = width;
    }

    /**
     * Sets the image height.
     * @param height
     */
    public void setInitialImageHeight( int height )
    {
        m_initialImageHeight = height;
    }

    /**
     * Sets the image width.
     * @return 
     */
    public int getInitialImageWidth( )
    {
        return m_initialImageWidth;
    }

    /**
     * Sets the image height.
     * @return 
     */
    public int getInitialImageHeight( )
    {
        return m_initialImageHeight;
    }

    /**
     * Sets the slice position.
     * @param slice_location
     */
    public void setSliceLocation( String slice_location )
    {
        m_sliceLocation = slice_location;
    }
 
    /**
     * Sets the file pointer offset where data block starts...
     * @param offset
     */
    public void setFilePointerOffsetOfPixelData( int offset )
    {
        m_filePointerOffsetPixelDataStart = offset;
    }
    
    /**
     * Sets the url.
     * @param url
     */
    public void setURL( String url )
    {
        m_url = url;
    }
    
    /**
     * Sets number of images.
     * @param images
     */
    public void setNumberOfFrames( String images )
    {
        m_numberOfFrames = images;
    }

    /**
     * Sets the pixel width
     * @param width
     */
    public void setPixelWidth( String width )
    {
        m_pixelWidth = width;
    }

    /**
     * Sets the pixel height
     * @param height
     */
    public void setPixelHeight( String height )
    {
        m_pixelHeight = height;
    }

    /**
     *
     * Sets the transfer syntax uid
     *
     * @param transferSyntaxUID
     */
    public void setTransferSyntaxUID( String transferSyntaxUID )
    {
        if ( transferSyntaxUID.indexOf( "1.2.4" ) > -1 || transferSyntaxUID.indexOf( "1.2.5" ) > -1 )
        {
            // m_bufferedInputStream.close( );

            String msg  = "Warning: cannot display compressed DICOM images!\n \n";
            String msg1 = "Transfer syntax UID : " + transferSyntaxUID;

            System.out.println( "SCANalyze.Dicom.FileInfo : " + m_fileName );
            System.out.println( msg );
            System.out.println( msg1 );
            m_canDisplayImage = false;
        }


       m_transferSyntaxUID = transferSyntaxUID;
    }

    /**
     * Returns the transfer syntax uid
     * @return
     */
    public String getTransferSyntaxUID( )
    {
        return m_transferSyntaxUID;
    }


    /**
     * Sets the pixel depth.
     * @param depth
     */
    public void setPixelDepth( String depth )
    {
        m_pixelDepth = depth;
    }
    
    /**
     * Sets the whiteisZero attribute.
     * @param white_is_zero
     */
    public void setWhiteIsZero( boolean white_is_zero )
    {
        m_whiteIsZero = white_is_zero;
    }
    
    /**
     * Sets the intelByteOrder attribute.
     * @param intelByteOrder
     */
    public void setIntelByteOrder( boolean intelByteOrder )
    {
        m_intelByteOrder = intelByteOrder;
    } 
    
    /**
     * Sets lutSize.
     * @param lut_size
     */
    public void setLutSize( int lut_size )
    {
        m_lutSize = lut_size;
    }
    
    /**
     * Sets Reds byte array.
     * @param reds
     */
    public void setReds( byte[] reds )
    {
        m_reds = new byte[reds.length];
        for ( int i = 0; i < reds.length; i++ )
        {
            m_reds[i] = reds[i];
        }
    }
    
    /**
     * Sets Greens byte array.
     * @param greens
     */
    public void setGreens( byte[] greens )
    {
        m_greens = new byte[greens.length];
        for ( int i = 0; i < greens.length; i++ )
        {
            m_greens[i] = greens[i];
        }
    }
    
    /**
     * Sets Blues byte array.
     * @param blues
     */
    public void setBlues( byte[] blues )
    {
        m_blues = new byte[blues.length];
        for ( int i = 0; i < blues.length; i++ )
        {
            m_blues[i] = blues[i];
        }
    }

    /**
     * Sets the unit string.
     * @param unit
     */
    public void setUnit( String unit )
    {
        m_unit = unit;
    }

    /**
     * Sets the slice thickness. 
     * @param thickness
     */
    public void setSliceThickness( String thickness )
    {
        m_sliceThickness = thickness;
    }

    /**
     * Sets the repetition time attribute.
     * @param tr
     */
    public void setTR( String tr )
    {
        m_TR = tr;
    }

    /**
     * Sets the echo time attribute.
     * @param echo_time
     */
    public void setEchoTime( String echo_time )
    {
        m_echoTime = echo_time;
    }
    
    /**
     * Sets the carrier frequency attribute [MHz].
     * @param carrier_frequency
     */
    public void setImagingFrequency( String carrier_frequency )
    {
        m_imagingFrequency = carrier_frequency; 
    }
    
    /**
     * Sets the number of phase encodes.
     * @param num_encodes
     */
    public void setNumberOfPhaseEncodes( String num_encodes )
    {
        m_numPhaseEncodes = num_encodes;
    }

    /**
     * Sets the echo train length attribute.
     * @param echo_train_length
     */
    public void setEchoTrainLength( String echo_train_length )
    {
        m_echoTrainLength = echo_train_length;
    }
    
    /**
     * Sets the percentage sampling attribute.
     * @param percent_sampling
     */
    public void setPercentSampling( String percent_sampling )
    {
        m_percentSampling = percent_sampling;
    }
    
    /**
     * Sets the percentage sampling attribute.
     * @param percent_phase_fov
     */
    public void setPercentPhaseFOV( String percent_phase_fov )
    {
        m_percentPhaseFOV = percent_phase_fov;
    }

    
    /**
     * Sets the pixel bandwidth attribute.
     * @param pixel_bandwidth
     */
    public void setPixelBandwidth( String pixel_bandwidth )
    {
        m_pixelBandwidth = pixel_bandwidth;
    }

    /**
     * Sets the pulse sequence name attribute.
     * @param seq_name
     */
    public void setSequenceName( String seq_name )
    {
        m_sequenceName = seq_name;
    }

    /**
     * Sets the software version attribute.
     * @param software_version
     */
    public void setSoftwareVersion( String software_version )
    {
        m_softwareVersion = software_version;
    }

    /**
     * Sets the protocol name attribute.
     * @param protocol_name
     */
    public void setProtocolName( String protocol_name )
    {
        m_protocolName = protocol_name;
    }
    
    /**
     * Sets the contrast bolus volume attribute [ml].
     * @param bolus_vol
     */
    public void setContrastBolusVolume( String bolus_vol )
    {
        m_contrastBolusVolume = bolus_vol;
    }
    
    /**
     * Sets the contrast flow duration attribute [seconds].
     * @param bolus_flow_duration
     */
    public void setContrastFlowDuration( String bolus_flow_duration )
    {
        m_contrastFlowDuration = bolus_flow_duration;
    }

    /**
     * Sets the transmit coil name attribute.
     * @param transmit_coil_name
     */
    public void setTransmitCoilName( String transmit_coil_name )
    {
        m_transmitCoilName = transmit_coil_name;
    }

    /**
     * Sets the phase encoding direction attribute.
     * @param phase_enc_dir
     */
    public void setPhaseEncodingDirection( String phase_enc_dir )
    {
        m_phaseEncodingDirection = phase_enc_dir;
    }


    /**
     * Sets the flip angle attribute.
     * @param flip_angle
     */
    public void setFlipAngle( String flip_angle )
    {
        m_flipAngle = flip_angle;
    }

    /**
     * Sets the variable flip angle flag attribute.
     * @param var_flip_angle_flag
     */
    public void setVariableFlipAngleFlag( String var_flip_angle_flag )
    {
        m_variableFlipAngleFlag = var_flip_angle_flag;
    }
    
    /**
     * Sets the specific absorption rate attribute.
     * @param sar
     */
    public void setSAR( String sar )
    {
        m_SAR = sar;
    }

    /**
     * Sets the dBdt attribute.
     * @param dbdt
     */
    public void setdBdt( String dbdt )
    {
        m_dBdt = dbdt;
    }    

    /**
     * Sets the patient position attribute.
     * @param pat_pos
     */
    public void setPatientPosition( String pat_pos )
    {
        m_patientPosition = pat_pos;
    }

    /**
     * Sets the study ID attribute.
     * @param stud_id
     */
    public void setStudyID( String stud_id )
    {
        m_studyID = stud_id;
    }

    /**
     * Sets the series number attribute.
     * @param series_num
     */
    public void setSeriesNumber( String series_num )
    {
        m_seriesNumber = series_num;
    }

    /**
     * Sets the acquisition number attribute.
     * @param acq_num
     */
    public void setAcquisitionNumber( String acq_num )
    {
        m_acquisitionNumber = acq_num;
    }       

    /**
     * Sets the instance number attribute.
     * @param inst_num
     */
    public void setInstanceNumber( String inst_num )
    {
        m_instanceNumber = inst_num;
    }       

    /**
     * Sets the image position attribute.
     * @param ima_pos
     */
    public void setImagePositionPatient( String ima_pos )
    {
        m_imagePosition = ima_pos;
        
        setImageXYZPositionPatient( );
        
    }
    
    /**
     * Sets the ( X, Y, Z ) image position attributes.
     */
    private void setImageXYZPositionPatient( )
    {
        
        int i = m_imagePosition.indexOf( '\\' );
        int j = m_imagePosition.lastIndexOf( '\\' );
        
        if ( i > 0 ) 
        {
            m_x_position_patient = Tools.string2double( m_imagePosition.substring( 0, i ) );
            m_y_position_patient = Tools.string2double( m_imagePosition.substring( i + 1, j ) );
        }

        if ( j > 0 ) 
        {
            m_z_position_patient = Tools.string2double( m_imagePosition.substring( j + 1 ) );
        }
    }

    /**
     * Sets the image position (patient) attribute.
     * @param ima_ori
     */
    public void setImageOrientationPatient( String ima_ori )
    {
        m_imageOrientation = ima_ori;
    }
    
    /**
     * Returns the image orientation attribute 1_1.
     * @return 
     */
    public double getImageOrientationPatient_1_1( )
    {   double dbl;
        
        if ( m_imageOrientation != null )
        {
            dbl = (new Double( m_imageOrientation.substring( 0, m_imageOrientation.indexOf( '\\' ) ) ) ).doubleValue( );
        }
        else
        {
            dbl = 0.0;
        }
        return dbl;
    }
    
    /**
     * Returns the image orientation attribute 1_2.
     * @return 
     */
    public double getImageOrientationPatient_1_2( )
    {
        String sub = null;
        if ( m_imageOrientation != null )
        {
            sub = m_imageOrientation.substring( m_imageOrientation.indexOf( '\\' ) + 1,  m_imageOrientation.length( ) );
        }
        if ( sub != null )
        {
            return ( new Double( sub.substring( 0, sub.indexOf( '\\' ) ) ) ).doubleValue( );
        }
        else
        {
            return 0.0;
        }
    }

    /**
     * Returns the image orientation attribute 1_3.
     * @return 
     */
    public double getImageOrientationPatient_1_3( )
    {
        String sub = null;
        if ( m_imageOrientation != null )
        {
            sub = m_imageOrientation.substring( m_imageOrientation.indexOf( '\\' ) + 1,  m_imageOrientation.length( ) );
            if ( sub != null )
            {
                sub = sub.substring( sub.indexOf( '\\' ) + 1,  sub.length( ) );
            }
        }
        
        if ( sub != null )
        {
            return ( new Double( sub.substring( 0, sub.indexOf( '\\' ) ) ) ).doubleValue( );
        }
        else
        {    
            return 0.0;
        }
    }

    /**
     * Returns the image orientation attribute 2_1.
     * @return 
     */
    public double getImageOrientationPatient_2_1( )
    {
        String sub = null;
        if ( m_imageOrientation != null )
        {
            sub = m_imageOrientation.substring( m_imageOrientation.indexOf( '\\' ) + 1,  m_imageOrientation.length( ) );
            
            if ( sub != null )
            {
                sub = sub.substring( m_imageOrientation.indexOf( '\\' ) + 1,  sub.length( ) );
            }
            
            if ( sub != null )
            {
               sub = sub.substring( sub.indexOf( '\\' ) + 1,  sub.length( ) );
            }
            
        }
        if ( sub != null )
        {
            return ( new Double( sub.substring( 0, sub.indexOf( '\\' ) ) ) ).doubleValue( );
        }
        else
        {
            return 0.0;
        }
    }

    /**
     * Returns the image orientation attribute 2_2.
     * @return 
     */
    public double getImageOrientationPatient_2_2( )
    {
        String sub = null;
        if ( m_imageOrientation != null )
        {
            sub = m_imageOrientation.substring( m_imageOrientation.indexOf( '\\' ) + 1,  m_imageOrientation.length( ) );
            if ( sub != null )
            {
                sub = sub.substring( sub.indexOf( '\\' ) + 1,  sub.length( ) );
            }
            if ( sub != null )
            {
                sub = sub.substring( sub.indexOf( '\\' ) + 1,  sub.length( ) );
            }
            if ( sub != null )
            {
                sub = sub.substring( sub.indexOf( '\\' ) + 1,  sub.length( ) );
            }
        }
        if ( sub != null )
        {
            return ( new Double( sub.substring( 0, sub.indexOf( '\\' ) ) ) ).doubleValue( );
        }
        else
        {
            return 0.0;
        }
    }
    
    /**
     * Returns the image orientation attribute 2_3.
     * @return 
     */
    public double getImageOrientationPatient_2_3( )
    {
        String sub = null;
        if ( m_imageOrientation != null )
        {
            sub = m_imageOrientation.substring( m_imageOrientation.indexOf( '\\' ) + 1,  m_imageOrientation.length( ) );
            if ( sub != null )
            {
                sub = sub.substring( sub.indexOf( '\\' ) + 1,  sub.length( ) );
            }
            if ( sub != null )
            {
                sub = sub.substring( sub.indexOf( '\\' ) + 1,  sub.length( ) );
            }
            if ( sub != null )
            {
                sub = sub.substring( sub.indexOf( '\\' ) + 1,  sub.length( ) );
            }
            if ( sub != null )
            {
                sub = sub.substring( sub.indexOf( '\\' ) + 1,  sub.length( ) );
            }
        }
        if ( sub != null )
        {
            return ( new Double( sub ) ).doubleValue( );
        }
        else
        {
            return 0.0;
        }
    }
    
    /**
     * Sets the samples per pixel attribute.
     * @param samples
     */
    public void setSamplesPerPixel( String samples )
    {
        m_samplesPerPixel = samples;
    }    

    /**
     * Sets the photometric interpretation attribute.
     * @param phot_interp
     */
    public void setPhotometricInterpretation( String phot_interp )
    {
        m_photometricInterpretation = phot_interp;
    }

    /**
     * Sets the planar configuration attribute.
     * @param plan_conf
     */
    public void setPlanarConfiguration( int plan_conf )
    {
        m_planarConfiguration = plan_conf;
    }

    /**
     * Sets the planar configuration attribute.
     * @param win_center
     */
    public void setWindowCenter( String win_center )
    {
        m_windowCenter = win_center;
    }

    /**
     * Sets the planar configuration attribute.
     * @param win_width
     */
    public void setWindowWidth( String win_width )
    {
        m_windowWidth = win_width;
    }

    /**
     * Sets the bits stored attribute.
     * @param bits
     */
    public void setBitsStored( String bits )
    {
        m_bitsStored = bits;
    }
    
    /**
     * Sets the high bit number attribute.
     * @param hi_bit
     */
    public void setHighBit( String hi_bit )
    {
        m_highBit = hi_bit;
    }

    /**
     * Sets the smallest pixel value attribute.
     * @param smallest_pixval
     */
    public void setSmallestPixelValue( String smallest_pixval )
    {
        m_smallestPixelValue = smallest_pixval;
    }

    /**
     * Sets the largest pixel value attribute.
     * @param largest_pixval
     */
    public void setLargestPixelValue( String largest_pixval )
    {
        m_largestPixelValue = largest_pixval;
    }

    /**
     * Sets the patient name attribute.
     * @param pat_name
     */
    public void setPatientName( String pat_name )
    {
        m_patientName = pat_name;
    }  
    
    /**
     * Sets the patient id attribute.
     * @param pat_id
     */
    public void setPatientID( String pat_id )
    {
        m_patientID = pat_id;
    }
    
    /**
     * Sets the patient birthday attribute.
     * @param pat_bd
     */
    public void setPatientBirthDate( String pat_bd )
    {
        m_patientBirthDate = pat_bd;
    }

    /**
     * Sets the patient sex attribute.
     * @param patient_sex
     */
    public void setPatientSex( String patient_sex )
    {
        m_patientSex = patient_sex;
    }

    /**
     * Sets the patient age attribute.
     * @param patient_age
     */
    public void setPatientAge( String patient_age )
    {
        m_patientAge = patient_age;
    }
    
    /**
     * Sets the patient weight attribute.
     * @param pat_weight
     */
    public void setPatientWeight( String pat_weight )
    {
        m_patientWeight = pat_weight;
    }
    
    /**
     * Sets the contrast agent attribute.
     * @param contr_agent
     */
    public void setContrastAgent( String contr_agent )
    {
        m_contrastAgent = contr_agent;
    }
    
    /**
     * Sets the scanning sequence attribute.
     * @param sc_seq
     */
    public void setScanningSequence( String sc_seq )
    {
        m_scanningSequence = sc_seq;
    }

    /**
     * Sets the sequence variant attribute.
     * @param seq_var
     */
    public void setSequenceVariant( String seq_var )
    {
        m_sequenceVariant = seq_var;
    }
    
    /**
     * Sets the sequence variant attribute.
     * @param sc_opt
     */
    public void setScanOptions( String sc_opt )
    {
        m_scanOptions = sc_opt;
    }
    
    /**
     * Sets the acquisition type attribute.
     * @param acq_type
     */
    public void setAcquisitionType( String acq_type )
    {
        m_acquisitionType = acq_type;
    }
    
    /**
     * Sets the image type attribute.
     * @param acq_type
     */
    public void setImageType( String acq_type )
    {
        m_imageType = acq_type;
    }

    /**
     * Sets the study date attribute.
     * @param study_date
     */
    public void setStudyDate( String study_date )
    {
        m_studyDate = study_date;
    }
    
    /**
     * Sets the series date attribute.
     * @param series_date
     */
    public void setSeriesDate( String series_date )
    {
        m_seriesDate = series_date;
    }

    /**
     * Sets the acquisition date attribute.
     * @param acq_date
     */
    public void setAcquisitionDate( String acq_date )
    {
        m_acquisitionDate = acq_date;
    }

    /**
     * Sets the content date attribute.
     * @param content_date
     */
    public void setContentDate( String content_date )
    {
        m_contentDate = content_date;
    }

    /**
     * Sets the study time attribute.
     * @param study_time
     */
    public void setStudyTime( String study_time )
    {
        m_studyTime = study_time;
    }

    /**
     * Sets the series time attribute.
     * @param series_time
     */
    public void setSeriesTime( String series_time )
    {
        m_seriesTime = series_time;
    }

    /**
     * Sets the acquisition time attribute.
     * @param acquisition_time
     */
    public void setAcquisitionTime( String acquisition_time )
    {
        m_acquisitionTime = acquisition_time;
    }

    /**
     * Sets the content time attribute.
     * @param content_time
     */
    public void setContentTime( String content_time )
    {
        m_contentTime = content_time;
    }

    /**
     * Sets the accession number attribute.
     * @param accession_number
     */
    public void setAccessionNumber( String accession_number )
    {
        m_accessionNumber = accession_number;
    }
    
    /**
     * Sets the image modality attribute (MR/CT/ etc).
     * @param modality
     */    
    public void setImageModality( String modality )
    {
        m_imageModality = modality;
    }
    
    /**
     * Sets the series description attribute.
     * @param series_description
     */
    public void setSeriesDescription( String series_description )
    {
        m_seriesDescription = series_description;
    }
    
    /**
     * Sets the non image data data structure (byte array)
     * and sets the value of the m_isImageData to false in order.
     * @param data
     */
    public void setNonImageData( byte[] data )
    {
        m_nonImageData = new byte[data.length];
        if ( data.length != 0 )
        {  
            System.arraycopy( data, 0, m_nonImageData, 0, data.length );
            m_isImageData = false;
        }
    }
    
    /**
     * Returns the caller a boolean indicating whether the Dicom file
     * represents normal image data or non image data (eg spectroscopic 
     * data). 
     * @return 
     */
    public boolean isImageData( )
    {
        return m_isImageData;
    }
    
    /**
     * Sets the media storage SOP instance UID attribute.
     * @param uid
     */
    public void setMediaStorageSOPInstanceUID( String uid )
    {
        m_mediaStorageSOPInstanceUID = uid.replace( '"', ' ' ).trim();
    }

    /**
     * Returns the media storage SOP instance UID attribute.
     * @return 
     */
    public String getMediaStorageSOPInstanceUID( )
    {
        return m_mediaStorageSOPInstanceUID;
    }
    
    /**
     * Returns the image modality attribute (MR/CT/ etc).
     * @return 
     */    
    public String getImageModality( )
    {
        return m_imageModality;
    }

    /**
     * Returns the file information in HTML format.
     * @return 
     */
    public String getDicomFileInfoAsHTML( )
    {
        String info = "<HTML>";
        info += "<P>Patient attibutes</P>";
        info += "<P>=================</P>";
        info += "<P>Patient name : " + m_patientName + "</P>";
        info += "<P>Patient ID : " + m_patientID + "</P>";
        info += "<P>Patient birthdate : " + m_patientBirthDate + "</P>";
        info += "<P>Patient sex : " + m_patientSex + "</P>";
        info += "<P>Patient age : " + m_patientAge + "</P>";
        info += "<P>Patient weight : " + m_patientWeight + "</P>";        
        info += "<P>Patient position : " + m_patientPosition + "</P>";
        info += "<P></P>";

        info += "<P>Study information</P>";
        info += "<P>=================</P>";
        info += "<P>Study ID : " + m_studyID + "</P>";
        info += "<P>Study Description : " + m_studyDescription + "</P>";
        info += "<P>Series Number : " + m_seriesNumber + "</P>";
        info += "<P>Acquisition Number : " + m_acquisitionNumber + "</P>";
        info += "<P>Instance Number : " + m_instanceNumber + "</P>";
        info += "<P>Study Time : " + m_studyTime + "</P>";
        info += "<P>Study Date : " + m_studyDate + "</P>";
        info += "<P>Series Date : " + m_seriesDate + "</P>";
        info += "<P>Series Description : " + m_seriesDescription + "</P>";
        info += "<P>Acquisition Date : " + m_acquisitionDate + "</P>";
        info += "<P>Content Date : " + m_contentDate + "</P>";
        info += "<P>Series Time : " + m_seriesTime + "</P>";
        info += "<P>Acquisition Time : " + m_acquisitionTime + "</P>";
        info += "<P>Content Time : " + m_contentTime + "</P>";
        info += "<P>Accession Number : " + m_accessionNumber + "</P>";
        info += "<P></P>";
        
        info += "<P><B> Image properties </B></P>";
        info += "<P><B> ================ </B></P>";
        info += "<P>Image Modality : " + m_imageModality + "</P>";
        info += "<P>Image Type : " + m_imageType + "</P>";
        info += "<P>Image height : " + m_height + " pixels</P>";
        info += "<P>Image width : " + m_width + " pixels</P>";
        info += "<P>Pixel height : " + m_pixelHeight + " mm/pixel</P>";
        info += "<P>Pixel width : " + m_pixelWidth + " mm/pixel</P>";
        info += "<P>Pixel depth : " + m_pixelDepth + " mm/pixel</P>";
        info += "<P>Window center : " + m_windowCenter + "</P>";
        info += "<P>Window width : " + m_windowWidth + "</P>";
        info += "<P>Image position : " + m_imagePosition + "</P>";
        info += "<P>Image orientation : " + m_imageOrientation + "</P>";
        info += "<P>Slice location : " + m_sliceLocation + " mm </P>";
        info += "<P>Slice thickness : " + m_sliceThickness + " mm </P>";
        info += "<P>Smallest pixel value : " + m_smallestPixelValue + "</P>" ;
        info += "<P>Largest pixel value : " + m_largestPixelValue + "</P>";
        info += "<P>SOP Instance UID : " + m_sop_instance_uid + "</P>";
        info += "<P>Series Instance UID : " + m_series_instance_uid + "</P>";
        info += "<P>Media storage SOP Instance UID : " + m_mediaStorageSOPInstanceUID + "</P>";
        info += "<P></P>";
        
        info += "<P> Pulse sequence characteristics </P>";
        info += "<P> ============================== </P>";
        info += "<P>Manufacturer : " + m_manufacturer + "</P>";
        info += "<P>Scanner type : " + m_scannerModelName + "</P>";
        info += "<P>Sequence name : " + m_sequenceName + "</P>";
        info += "<P>Protocol name : " + m_protocolName + "</P>";
        info += "<P>Echo time : " + m_echoTime + " ms </P>";
        info += "<P>TR : " + m_TR + " ms </P>";
        info += "<P>Phase encoding direction : " + m_phaseEncodingDirection + "</P>";
        info += "<P>Number of phase encoding steps : " + m_numPhaseEncodes + "</P>";
        info += "<P>Imaging frequency : " + m_imagingFrequency + "MHz </P>";
        info += "<P>SAR : " + m_SAR + "</P>";
        info += "<P>dB/dt : " + m_dBdt + "</P>";
        info += "<P>Flip angle : " + m_flipAngle + "</P>";
        info += "<P>Variable file angle flag : " + m_variableFlipAngleFlag + "</P>";
        info += "<P>Echo train length : " + m_echoTrainLength + "</P>";
        info += "<P>Percent sampling : " + m_percentSampling + "</P>";
        info += "<P>Percent phase fov : " + m_percentPhaseFOV + "</P>";
        info += "<P>Pixel bandwidth : " + m_pixelBandwidth + "</P>";      
        info += "<P>Software version : " + m_softwareVersion + "</P>";
        info += "<P>Samples per pixel : " + m_samplesPerPixel + "</P>";
        info += "<P>Transmit coil name : " + m_transmitCoilName + "</P>";
        info += "<P>Scanning sequence : " + m_scanningSequence + "</P>";
        info += "<P>Sequence variant : " + m_sequenceVariant + "</P>";
        info += "<P>Scan options : " + m_scanOptions + "</P>";
        info += "<P>Acquisition type : " + m_acquisitionType + "</P>";
        info += "<P></P>";
        
        info += "<P>Contrast agents</P>";
        info += "<P>===============</P>";
        info += "<P>Contrast agent : " + m_contrastAgent + "</P>";
        info += "<P>Contrast/Bolus volume : " + m_contrastBolusVolume + "</P>";
        info += "<P>Contrast flow duration : " + m_contrastFlowDuration + "</P>";
        info += "<P></P>";
        
        info += "<P>File characteristics</P>";
        info += "<P>====================</P>";
        info += "<P>Number of frames : " + m_numberOfFrames + "</P>";
        info += "<P>Photometric Interpretation : " + m_photometricInterpretation + "</P>";
        info += "<P>Planar configuration : " + m_planarConfiguration + "</P>";
        info += "<P>Bits allocated : " + m_fileType + "</P>";
        info += "<P>Data offset : " + m_filePointerOffsetPixelDataStart + " bytes </P>";
        info += "<P>Stored bits : " + m_bitsStored + "</P>";
        info += "<P>High bit : " + m_highBit + "</P>";


        info += "<P>SCANLAB specific TAGS</P>";
        info += "<P>=====================</P>";
        info += "<P>SCANLAB specific info string 1 : " + m_SCANInfo + "</P>";

        info += "</HTML>";
        return info;
    }
    
    /**
     * Sets the spectrum boolean to true indicating that the Dicom data stores
     * spectroscopic data.
     */
    public void setSpectrum( )
    {
        m_isSpectrum = true;
    }
    
    /**
     * Sets the spectrum boolean to true indicating that the Dicom data stores
     * spectroscopic data.
     */
    public void isImage( )
    {
        m_isSpectrum = false;
    }
    
    /**
     * Returns whether the data set is a spectrum or an image.
     * @return 
     */
    public boolean isSpectrum( )
    {
        return m_isSpectrum;
    }

    /**
     * Sets the number of bits allocated for one pixel.
     * @param bitsAllocated
     */
    public void setBitsAllocated( String bitsAllocated )
    {
        m_bitsAllocated = bitsAllocated;
    }

    /**
     * Sets the number of bits allocated for one pixel.
     * @return 
     */
    public String getBitsAllocated( )
    {
        return m_bitsAllocated;
    }
    
    ////////////////////////////////////////////////////////////////////////
    // Routines below are added for making Dicom file creation from scratch
    // possible.

    /**
     *
     * Returns all DICOM tags necessary for storing a DICOM file
     * in a PACS system.
     *
     * @return
     */
    public LinkedList<DicomTagAndValue> getAllDicomTagsAndValues( )
    {
        LinkedList<DicomTagAndValue> allTags = new LinkedList<DicomTagAndValue>( );
        
        // # Patient Module Attributes

        String tagName  = "PATIENT_NAME";

        int tagID       = DicomDecoder.PATIENT_NAME;
        int encodingID  = DicomDecoder.PN;
        String tagValue = this.m_patientName;

        DicomTagAndValue tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );
        
        //---------------------------------------------------------------------------------
        
        tagName    = "PATIENT_ID";
        tagID      = DicomDecoder.PATIENT_ID;
        encodingID = DicomDecoder.LO;
        tagValue   = this.m_patientID;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        //---------------------------------------------------------------------------------

        tagName    = "PATIENT_BIRTHDATE";
        tagID      = DicomDecoder.PATIENT_BIRTHDATE;
        encodingID = DicomDecoder.DA;
        tagValue   = m_patientBirthDate;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        //---------------------------------------------------------------------------------

        tagName    = "PATIENT_SEX";
        tagID      = DicomDecoder.PATIENT_SEX;
        encodingID = DicomDecoder.CS;
        tagValue   = m_patientSex;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        //---------------------------------------------------------------------------------

        tagName    = "PATIENT_AGE";
        tagID      = DicomDecoder.PATIENT_AGE;
        encodingID = DicomDecoder.AS;
        tagValue   = m_patientAge;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        // # General Study Module Attributes

        tagName    = "STUDY_INSTANCE_UID";
        tagID      = DicomDecoder.STUDY_INSTANCE_UID;
        encodingID = DicomDecoder.UI;
        tagValue   = m_studyInstanceUID;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "STUDY_DATE";
        tagID      = DicomDecoder.STUDY_DATE;
        encodingID = DicomDecoder.DA;
        tagValue   = m_studyDate;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "STUDY_TIME";
        tagID      = DicomDecoder.STUDY_TIME;
        encodingID = DicomDecoder.TM;
        tagValue   = m_studyTime;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "ACCESSION_NUMBER";
        tagID      = DicomDecoder.ACCESSION_NUMBER;
        encodingID = DicomDecoder.SH;
        tagValue   = m_accessionNumber;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "STUDY_ID";
        tagID      = DicomDecoder.STUDY_ID;
        encodingID = DicomDecoder.SH;
        tagValue   = m_studyID;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "STUDY_DESCRIPTION";
        tagID      = DicomDecoder.STUDY_DESCRIPTION;
        encodingID = DicomDecoder.LO;
        tagValue   = m_studyDescription;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        // # General Series Module Attributes
        //===================================

        tagName    = "CONTENT_TIME";
        tagID      = DicomDecoder.CONTENT_TIME;
        encodingID = DicomDecoder.TM;
        tagValue   = m_contentTime;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "CONTENT_DATE";
        tagID      = DicomDecoder.CONTENT_DATE;
        encodingID = DicomDecoder.DA;
        tagValue   = m_contentDate;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );


        tagName    = "ACQUISITION_TIME";
        tagID      = DicomDecoder.ACQUISITION_TIME;
        encodingID = DicomDecoder.TM;
        tagValue   = m_acquisitionTime;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );


        tagName    = "ACQUISITION_DATE";
        tagID      = DicomDecoder.ACQUISITION_DATE;
        encodingID = DicomDecoder.DA;
        tagValue   = m_acquisitionDate;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "SERIES_DATE";
        tagID      = DicomDecoder.SERIES_DATE;
        encodingID = DicomDecoder.DA;
        tagValue   = m_seriesDate;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "SERIES_TIME";
        tagID      = DicomDecoder.SERIES_TIME;
        encodingID = DicomDecoder.TM;
        tagValue   = m_seriesTime;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "SERIES_INSTANCE_UID";
        tagID      = DicomDecoder.SERIES_INSTANCE_UID;
        encodingID = DicomDecoder.UI;
        tagValue   = m_series_instance_uid;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "SERIES_DESCRIPTION";
        tagID      = DicomDecoder.SERIES_DESCRIPTION;
        encodingID = DicomDecoder.LO;
        tagValue   = m_seriesDescription;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "SERIES_NUMBER";
        tagID      = DicomDecoder.SERIES_NUMBER;
        encodingID = DicomDecoder.IS;
        tagValue   = m_seriesNumber;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        // # General Image Module Attributes

        tagName    = "INSTANCE_NUMBER";
        tagID      = DicomDecoder.INSTANCE_NUMBER;
        encodingID = DicomDecoder.IS;
        tagValue   = m_instanceNumber;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "SOP_INSTANCE_UID";
        tagID      = DicomDecoder.SOP_INSTANCE_UID;
        encodingID = DicomDecoder.UI;
        tagValue   = m_sop_instance_uid;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "TRANSFER_SYNTAX_UID";
        tagID      = DicomDecoder.TRANSFER_SYNTAX_UID;
        encodingID = DicomDecoder.UI;
        tagValue   = m_transferSyntaxUID;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "SLICE_LOCATION";
        tagID      = DicomDecoder.SLICE_LOCATION;
        encodingID = DicomDecoder.DS;
        tagValue   = this.m_sliceLocation;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "SLICE_THICKNESS";
        tagID      = DicomDecoder.SLICE_THICKNESS;
        encodingID = DicomDecoder.DS;
        tagValue   = m_sliceThickness;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "ROWS";
        tagID      = DicomDecoder.ROWS;
        encodingID = DicomDecoder.US;
        tagValue   = "" + m_height;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "COLUMNS";
        tagID      = DicomDecoder.COLUMNS;
        encodingID = DicomDecoder.US;
        tagValue   = "" + m_width;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "IMAGE_POSITION";
        tagID      = DicomDecoder.IMAGE_POSITION;
        encodingID = DicomDecoder.DS;
        tagValue   = m_imagePosition;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );


        tagName    = "IMAGE_ORIENTATION";
        tagID      = DicomDecoder.IMAGE_ORIENTATION;
        encodingID = DicomDecoder.DS;
        tagValue   = m_imageOrientation;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

     
        tagName    = "TR";
        tagID      = DicomDecoder.TR;
        encodingID = DicomDecoder.DS;
        tagValue   = m_TR;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );


        tagName    = "ECHO_TIME";
        tagID      = DicomDecoder.ECHO_TIME;
        encodingID = DicomDecoder.DS;
        tagValue   = m_echoTime;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "IMAGE_FREQUENCY";
        tagID      = DicomDecoder.IMAGE_FREQUENCY;
        encodingID = DicomDecoder.DS;
        tagValue   = m_imagingFrequency;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "RED_PALETTE";
        tagID      = DicomDecoder.RED_PALETTE;
        encodingID = DicomDecoder.OW;
        tagValue   = new String( m_reds );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "GREEN_PALETTE";
        tagID      = DicomDecoder.GREEN_PALETTE;
        encodingID = DicomDecoder.OW;
        tagValue   = new String( m_greens );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "BLUE_PALETTE";
        tagID      = DicomDecoder.BLUE_PALETTE;
        encodingID = DicomDecoder.OW;
        tagValue   = new String( m_blues );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );


        tagName    = "SLICE_SPACING";
        tagID      = DicomDecoder.SLICE_SPACING;
        encodingID = DicomDecoder.DS;
        tagValue   = m_pixelDepth;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "PIXEL_SPACING";
        tagID      = DicomDecoder.PIXEL_SPACING;
        encodingID = DicomDecoder.DS;
        tagValue   = m_pixelSpacing;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "SEQUENCE_NAME";
        tagID      = DicomDecoder.SEQUENCE_NAME;
        encodingID = DicomDecoder.SH;
        tagValue   = m_sequenceName;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "PIXEL_REPRESENTATION";
        tagID      = DicomDecoder.PIXEL_REPRESENTATION;
        encodingID = DicomDecoder.US;
        tagValue   = m_pixelRepresentation;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "MEDIA_STORAGE_SOP_INSTANCE_UID";
        tagID      = DicomDecoder.MEDIA_STORAGE_SOP_INSTANCE_UID;
        encodingID = DicomDecoder.UI;
        tagValue   = m_mediaStorageSOPInstanceUID;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "SAMPLES_PER_PIXEL";
        tagID      = DicomDecoder.SAMPLES_PER_PIXEL;
        encodingID = DicomDecoder.US;
        tagValue   = m_samplesPerPixel;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "PHOTOMETRIC_INTERPRETATION";
        tagID      = DicomDecoder.PHOTOMETRIC_INTERPRETATION;
        encodingID = DicomDecoder.CS;
        tagValue   = m_photometricInterpretation;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "NUMBER_OF_FRAMES";
        tagID      = DicomDecoder.NUMBER_OF_FRAMES;
        encodingID = DicomDecoder.US;
        tagValue   = m_numberOfFrames;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );


        tagName    = "BITS_ALLOCATED";
        tagID      = DicomDecoder.BITS_ALLOCATED;
        encodingID = DicomDecoder.US;
        tagValue   = m_bitsAllocated;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );


        tagName    = "WINDOW_CENTER";
        tagID      = DicomDecoder.WINDOW_CENTER;
        encodingID = DicomDecoder.DS;
        tagValue   = m_windowCenter;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );


        tagName    = "WINDOW_WIDTH";
        tagID      = DicomDecoder.WINDOW_WIDTH;
        encodingID = DicomDecoder.DS;
        tagValue   = m_windowWidth;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "NUM_PHASE_ENCODES";
        tagID      = DicomDecoder.NUM_PHASE_ENCODES;
        encodingID = DicomDecoder.IS;
        tagValue   = m_numPhaseEncodes;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "ECHO_TRAIN_LENGTH";
        tagID      = DicomDecoder.ECHO_TRAIN_LENGTH;
        encodingID = DicomDecoder.IS;
        tagValue   = m_echoTrainLength;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "PERCENT_SAMPLING";
        tagID      = DicomDecoder.PERCENT_SAMPLING;
        encodingID = DicomDecoder.DS;
        tagValue   = m_percentSampling;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "PERCENT_PHASE_FOV";
        tagID      = DicomDecoder.PERCENT_PHASE_FOV;
        encodingID = DicomDecoder.DS;
        tagValue   = m_percentPhaseFOV;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "PIXEL_BANDWIDTH";
        tagID      = DicomDecoder.PIXEL_BANDWIDTH;
        encodingID = DicomDecoder.DS;
        tagValue   = m_pixelBandwidth;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "SOFTWARE_VERSION";
        tagID      = DicomDecoder.SOFTWARE_VERSION;
        encodingID = DicomDecoder.LO;
        tagValue   = m_softwareVersion;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "PROTOCOL_NAME";
        tagID      = DicomDecoder.PROTOCOL_NAME;
        encodingID = DicomDecoder.LO;
        tagValue   = m_protocolName;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "CONTRAST_BOLUS_VOLUME";
        tagID      = DicomDecoder.CONTRAST_BOLUS_VOLUME;
        encodingID = DicomDecoder.DS;
        tagValue   = m_contrastBolusVolume;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "CONTRAST_FLOW_DURATION";
        tagID      = DicomDecoder.CONTRAST_FLOW_DURATION;
        encodingID = DicomDecoder.DS;
        tagValue   = m_contrastFlowDuration;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "TRANSMIT_COIL_NAME";
        tagID      = DicomDecoder.TRANSMIT_COIL_NAME;
        encodingID = DicomDecoder.DS;
        tagValue   = m_transmitCoilName;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "PHASE_ENCODING_DIR";
        tagID      = DicomDecoder.PHASE_ENCODING_DIR;
        encodingID = DicomDecoder.CS;
        tagValue   = m_phaseEncodingDirection;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "FLIP_ANGLE";
        tagID      = DicomDecoder.FLIP_ANGLE;
        encodingID = DicomDecoder.DS;
        tagValue   = m_flipAngle;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "VARIABLE_FLIP_ANGLE_FLAG";
        tagID      = DicomDecoder.VARIABLE_FLIP_ANGLE_FLAG;
        encodingID = DicomDecoder.CS;
        tagValue   = m_variableFlipAngleFlag;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "SAR";
        tagID      = DicomDecoder.SAR;
        encodingID = DicomDecoder.DS;
        tagValue   = m_SAR;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "DBDT";
        tagID      = DicomDecoder.DBDT;
        encodingID = DicomDecoder.DS;
        tagValue   = m_dBdt;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "PATIENT_POSITION";
        tagID      = DicomDecoder.PATIENT_POSITION;
        encodingID = DicomDecoder.CS;
        tagValue   = m_patientPosition;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "ACQUISITION_NUMBER";
        tagID      = DicomDecoder.ACQUISITION_NUMBER;
        encodingID = DicomDecoder.IS;
        tagValue   = m_acquisitionNumber;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "MANUFACTURER";
        tagID      = DicomDecoder.MANUFACTURER;
        encodingID = DicomDecoder.LO;
        tagValue   = m_manufacturer;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "IMAGE_MODALITY";
        tagID      = DicomDecoder.IMAGE_MODALITY;
        encodingID = DicomDecoder.CS;
        tagValue   = m_imageModality;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "BITS_STORED";
        tagID      = DicomDecoder.BITS_STORED;
        encodingID = DicomDecoder.US;
        tagValue   = m_bitsStored;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "HIGH_BIT";
        tagID      = DicomDecoder.HIGH_BIT;
        encodingID = DicomDecoder.US;
        tagValue   = m_highBit;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "SMALLEST_PIXEL_VALUE";
        tagID      = DicomDecoder.SMALLEST_PIXEL_VALUE;
        encodingID = DicomDecoder.SS;
        tagValue   = m_smallestPixelValue;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "LARGEST_PIXEL_VALUE";
        tagID      = DicomDecoder.LARGEST_PIXEL_VALUE;
        encodingID = DicomDecoder.SS;
        tagValue   = m_largestPixelValue;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "PATIENT_WEIGHT";
        tagID      = DicomDecoder.PATIENT_WEIGHT;
        encodingID = DicomDecoder.DS;
        tagValue   = m_patientWeight;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "CONTRAST_AGENT";
        tagID      = DicomDecoder.CONTRAST_AGENT;
        encodingID = DicomDecoder.LO;
        tagValue   = m_contrastAgent;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "SCANNING_SEQUENCE";
        tagID      = DicomDecoder.SCANNING_SEQUENCE;
        encodingID = DicomDecoder.CS;
        tagValue   = m_scanningSequence;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "SEQUENCE_VARIANT";
        tagID      = DicomDecoder.SEQUENCE_VARIANT;
        encodingID = DicomDecoder.CS;
        tagValue   = m_sequenceVariant;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "SCAN_OPTIONS";
        tagID      = DicomDecoder.SCAN_OPTIONS;
        encodingID = DicomDecoder.CS;
        tagValue   = m_scanOptions;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "ACQUISITION_TYPE";
        tagID      = DicomDecoder.ACQUISITION_TYPE;
        encodingID = DicomDecoder.CS;
        tagValue   = m_acquisitionType;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        tagName    = "IMAGE_TYPE";
        tagID      = DicomDecoder.IMAGE_TYPE;
        encodingID = DicomDecoder.CS;
        tagValue   = m_imageType;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        
        tagName    = "SCANNER_MODEL";
        tagID      = DicomDecoder.SCANNER_MODEL;
        encodingID = DicomDecoder.CS;
        tagValue = m_scannerModelName;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.add( tag );

        //=====================================================================

        return allTags;
    }


    /**
     *
     * Returns all DICOM tags related to study,series instance naming,
     * uids, used in database for storing a DICOM files
     * in a PACS system.
     *
     * @return
     */
    public LinkedHashMap<String, DicomTagAndValue> getStudySeriesInstanceTagsAndValues( )
    {
        LinkedHashMap<String, DicomTagAndValue> studySeriesInstanceTags = new LinkedHashMap<String, DicomTagAndValue>( );

//        String tagName      = "GROUP_LENGTH";
//        int tagID           = DicomDecoder.GROUP_LENGTH;
//        int encodingID      = DicomDecoder.UL;
//        String tagValue     = m_groupLength;

//        DicomTagAndValue tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
//        studySeriesInstanceTags.put( tagName, tag );

//        String tagName    = "MEDIA_STORAGE_SOP_CLASS_UID";
//        int tagID      = DicomDecoder.MEDIA_STORAGE_SOP_CLASS_UID;
//        int encodingID = DicomDecoder.UI;
//        String tagValue;
//
//        if ( m_mediaStorageSOPClassUID != null )
//        {
//            tagValue   = m_mediaStorageSOPClassUID;
//        }
//        else
//        {
//            m_mediaStorageSOPClassUID = "1.2.840.10008.5.1.4.1.1.4";
//            tagValue = m_mediaStorageSOPClassUID;
//        }
//
//        DicomTagAndValue tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
//        studySeriesInstanceTags.put( tagName, tag );
//
//
//        tagName    = "MEDIA_STORAGE_SOP_INSTANCE_UID";
//        tagID      = DicomDecoder.MEDIA_STORAGE_SOP_INSTANCE_UID;
//        encodingID = DicomDecoder.UI;
//        tagValue   = m_mediaStorageSOPInstanceUID;
//
//        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
//        studySeriesInstanceTags.put( tagName, tag );
//
//
//        tagName         = "TRANSFER_SYNTAX_UID";
//        tagID           = DicomDecoder.TRANSFER_SYNTAX_UID;
//        encodingID      = DicomDecoder.UI;
//        tagValue        = m_transferSyntaxUID;
//
//        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
//        studySeriesInstanceTags.put( tagName, tag );
//
//        tagName    = "IMPLEMENTATION_CLASS_UID";
//        tagID      = DicomDecoder.IMPLEMENTATION_CLASS_UID;
//        encodingID = DicomDecoder.UI;
//        tagValue   = m_implementationClassUID;
//
//        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
//        studySeriesInstanceTags.put( tagName, tag );

        String tagName    = "SOP_CLASS_UID";
        int tagID      = DicomDecoder.SOP_CLASS_UID;
        int encodingID = DicomDecoder.UI;
        String tagValue;

        if ( m_sopClassUID != null )
        {
            tagValue   = m_sopClassUID;
        }
        else
        {
            tagValue = "1.2.840.10008.5.1.4.1.1.4";
            m_sopClassUID = tagValue;
        }

        DicomTagAndValue tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );


        tagName    = "SOP_INSTANCE_UID";
        tagID      = DicomDecoder.SOP_INSTANCE_UID;
        encodingID = DicomDecoder.UI;
        tagValue   = m_sop_instance_uid;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );

        tagName    = "ACCESSION_NUMBER";
        tagID      = DicomDecoder.ACCESSION_NUMBER;
        encodingID = DicomDecoder.SH;
        tagValue   = m_accessionNumber;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );

        tagName    = "IMPLEMENTATION_CLASS_UID";
        tagID      = DicomDecoder.IMPLEMENTATION_CLASS_UID;
        encodingID = DicomDecoder.UI;
        tagValue   = m_implementationClassUID;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );

        // # General Study Module Attributes

        tagName    = "STUDY_INSTANCE_UID";
        tagID      = DicomDecoder.STUDY_INSTANCE_UID;
        encodingID = DicomDecoder.UI;
        tagValue   = m_studyInstanceUID;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );

        tagName    = "SERIES_INSTANCE_UID";
        tagID      = DicomDecoder.SERIES_INSTANCE_UID;
        encodingID = DicomDecoder.UI;
        tagValue   = m_series_instance_uid;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );       

        tagName    = "STUDY_DATE";
        tagID      = DicomDecoder.STUDY_DATE;
        encodingID = DicomDecoder.DA;
        tagValue   = m_studyDate;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );

        tagName    = "STUDY_TIME";
        tagID      = DicomDecoder.STUDY_TIME;
        encodingID = DicomDecoder.TM;
        tagValue   = m_studyTime;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );

        tagName    = "STUDY_ID";
        tagID      = DicomDecoder.STUDY_ID;
        encodingID = DicomDecoder.SH;
        tagValue   = m_studyID;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );

        tagName    = "STUDY_DESCRIPTION";
        tagID      = DicomDecoder.STUDY_DESCRIPTION;
        encodingID = DicomDecoder.LO;
        tagValue   = m_studyDescription;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );

        tagName    = "CONTENT_TIME";
        tagID      = DicomDecoder.CONTENT_TIME;
        encodingID = DicomDecoder.TM;
        tagValue   = m_contentTime;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );

        tagName    = "CONTENT_DATE";
        tagID      = DicomDecoder.CONTENT_DATE;
        encodingID = DicomDecoder.DA;
        tagValue   = m_contentDate;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );

        tagName    = "ACQUISITION_TIME";
        tagID      = DicomDecoder.ACQUISITION_TIME;
        encodingID = DicomDecoder.TM;
        tagValue   = m_acquisitionTime;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );

        tagName    = "ACQUISITION_DATE";
        tagID      = DicomDecoder.ACQUISITION_DATE;
        encodingID = DicomDecoder.DA;
        tagValue   = m_acquisitionDate;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );

        // # General Series Module Attributes
        //===================================

        tagName    = "SERIES_NUMBER";
        tagID      = DicomDecoder.SERIES_NUMBER;
        encodingID = DicomDecoder.IS;
        tagValue   = m_seriesNumber;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );

        tagName    = "SERIES_DESCRIPTION";
        tagID      = DicomDecoder.SERIES_DESCRIPTION;
        encodingID = DicomDecoder.LO;
        tagValue   = m_seriesDescription;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );

        tagName    = "SERIES_DATE";
        tagID      = DicomDecoder.SERIES_DATE;
        encodingID = DicomDecoder.DA;
        tagValue   = m_seriesDate;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );

        tagName    = "SERIES_TIME";
        tagID      = DicomDecoder.SERIES_TIME;
        encodingID = DicomDecoder.TM;
        tagValue   = m_seriesTime;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );


        // # General Image Module Attributes

        tagName    = "INSTANCE_NUMBER";
        tagID      = DicomDecoder.INSTANCE_NUMBER;
        encodingID = DicomDecoder.IS;
        tagValue   = m_instanceNumber;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        studySeriesInstanceTags.put( tagName, tag );

        
        return studySeriesInstanceTags;

    }


    /**
     *
     * Returns all DICOM tags necessary for storing a DICOM file
     * in a PACS system.
     *
     * @return
     */
    public LinkedHashMap<String, DicomTagAndValue> getGeomtryRelatedDicomTagsAndValues( )
    {

        LinkedHashMap<String, DicomTagAndValue> geometryRelatedTags = new LinkedHashMap<String, DicomTagAndValue>( );

        String tagName    = "SLICE_LOCATION";
        int tagID         = DicomDecoder.SLICE_LOCATION;
        int encodingID    = DicomDecoder.DS;
        String tagValue   = this.m_sliceLocation;

        DicomTagAndValue tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        geometryRelatedTags.put( tagName, tag );

        tagName           = "SLICE_THICKNESS";
        tagID             = DicomDecoder.SLICE_THICKNESS;
        encodingID        = DicomDecoder.DS;
        tagValue          = m_sliceThickness;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        geometryRelatedTags.put( tagName, tag );

        tagName           = "IMAGE_POSITION";
        tagID             = DicomDecoder.IMAGE_POSITION;
        encodingID        = DicomDecoder.DS;
        tagValue          = m_imagePosition;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        geometryRelatedTags.put( tagName, tag );

        tagName          = "IMAGE_ORIENTATION";
        tagID            = DicomDecoder.IMAGE_ORIENTATION;
        encodingID       = DicomDecoder.DS;
        tagValue         = m_imageOrientation;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        geometryRelatedTags.put( tagName, tag );

        tagName          = "SLICE_SPACING";
        tagID            = DicomDecoder.SLICE_SPACING;
        encodingID       = DicomDecoder.DS;
        tagValue         = m_pixelDepth;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        geometryRelatedTags.put( tagName, tag );

        tagName          = "PIXEL_SPACING";
        tagID            = DicomDecoder.PIXEL_SPACING;
        encodingID       = DicomDecoder.DS;
        tagValue         = m_pixelSpacing;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        geometryRelatedTags.put( tagName, tag );

        //=====================================================================

        tagName           = "ROWS";
        tagID             = DicomDecoder.ROWS;
        encodingID        = DicomDecoder.US;
        tagValue          = "" + m_height;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        geometryRelatedTags.put( tagName, tag );

        tagName           = "COLUMNS";
        tagID             = DicomDecoder.COLUMNS;
        encodingID        = DicomDecoder.US;
        tagValue          = "" + m_width;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        geometryRelatedTags.put( tagName, tag );

        return geometryRelatedTags;

    }

    /**
     *
     * Returns all DICOM tags necessary for storing a DICOM file
     * in a PACS system.
     *
     * @return
     */ 
    public LinkedHashMap<String, DicomTagAndValue> getOtherDicomTagsAndValues( )
    {

        LinkedHashMap<String, DicomTagAndValue> otherTags = new LinkedHashMap<String, DicomTagAndValue>( );


//        String tagName      = "GROUP_LENGTH";
//        int tagID           = DicomDecoder.GROUP_LENGTH;
//        int encodingID      = DicomDecoder.UL;
//        String tagValue     = m_groupLength;
//
//        DicomTagAndValue tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
//        otherTags.put( tagName, tag );


        String tagName      = "PIXEL_REPRESENTATION";
        int tagID           = DicomDecoder.PIXEL_REPRESENTATION;
        int encodingID      = DicomDecoder.US;
        String tagValue     = m_pixelRepresentation;

        DicomTagAndValue tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );

        if ( m_reds != null )
        {
            tagName    = "RED_PALETTE";
            tagID      = DicomDecoder.RED_PALETTE;
            encodingID = DicomDecoder.OW;
            tagValue   = new String( m_reds );

            tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
            otherTags.put( tagName, tag );
        }

        if ( m_greens != null )
        {
            tagName    = "GREEN_PALETTE";
            tagID      = DicomDecoder.GREEN_PALETTE;
            encodingID = DicomDecoder.OW;
            tagValue   = new String( m_greens );

            tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
            otherTags.put( tagName, tag );
        }

        if ( m_blues != null )
        {
            tagName    = "BLUE_PALETTE";
            tagID      = DicomDecoder.BLUE_PALETTE;
            encodingID = DicomDecoder.OW;
            tagValue   = new String( m_blues );

            tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
            otherTags.put( tagName, tag );
        }

        tagName    = "CHARACTER_SET";
        tagID      = DicomDecoder.CHARACTER_SET;
        encodingID = DicomDecoder.CS;
        tagValue   = m_characterSet;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );


        tagName    = "SAMPLES_PER_PIXEL";
        tagID      = DicomDecoder.SAMPLES_PER_PIXEL;
        encodingID = DicomDecoder.US;
        tagValue   = m_samplesPerPixel;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );

        tagName    = "PHOTOMETRIC_INTERPRETATION";
        tagID      = DicomDecoder.PHOTOMETRIC_INTERPRETATION;
        encodingID = DicomDecoder.CS;
        tagValue   = m_photometricInterpretation;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );

        tagName    = "NUMBER_OF_FRAMES";
        tagID      = DicomDecoder.NUMBER_OF_FRAMES;
        encodingID = DicomDecoder.US;
        tagValue   = m_numberOfFrames;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );

        tagName    = "BITS_ALLOCATED";
        tagID      = DicomDecoder.BITS_ALLOCATED;
        encodingID = DicomDecoder.US;
        tagValue   = m_bitsAllocated;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );

        tagName    = "WINDOW_CENTER";
        tagID      = DicomDecoder.WINDOW_CENTER;
        encodingID = DicomDecoder.DS;
        tagValue   = m_windowCenter;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );

        tagName    = "WINDOW_WIDTH";
        tagID      = DicomDecoder.WINDOW_WIDTH;
        encodingID = DicomDecoder.DS;
        tagValue   = m_windowWidth;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );

        tagName    = "SOFTWARE_VERSION";
        tagID      = DicomDecoder.SOFTWARE_VERSION;
        encodingID = DicomDecoder.LO;
        tagValue   = m_softwareVersion;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );

        tagName    = "ACQUISITION_NUMBER";
        tagID      = DicomDecoder.ACQUISITION_NUMBER;
        encodingID = DicomDecoder.IS;
        tagValue   = m_acquisitionNumber;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );

        tagName    = "MANUFACTURER";
        tagID      = DicomDecoder.MANUFACTURER;
        encodingID = DicomDecoder.LO;
        tagValue   = m_manufacturer;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );

        tagName    = "IMAGE_MODALITY";
        tagID      = DicomDecoder.IMAGE_MODALITY;
        encodingID = DicomDecoder.CS;
        tagValue   = m_imageModality;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );

        tagName    = "BITS_STORED";
        tagID      = DicomDecoder.BITS_STORED;
        encodingID = DicomDecoder.US;
        tagValue   = m_bitsStored;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );

        tagName    = "HIGH_BIT";
        tagID      = DicomDecoder.HIGH_BIT;
        encodingID = DicomDecoder.US;
        tagValue   = m_highBit;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );

        tagName    = "SMALLEST_PIXEL_VALUE";
        tagID      = DicomDecoder.SMALLEST_PIXEL_VALUE;
        encodingID = DicomDecoder.SS;
        tagValue   = m_smallestPixelValue;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );

        tagName    = "LARGEST_PIXEL_VALUE";
        tagID      = DicomDecoder.LARGEST_PIXEL_VALUE;
        encodingID = DicomDecoder.SS;
        tagValue   = m_largestPixelValue;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );

        tagName    = "SCANLAB_INFO_TAG";
        tagID      = DicomDecoder.SCANLAB_INFO_TAG;
        encodingID = DicomDecoder.CS;
        tagValue   = m_SCANInfo;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        otherTags.put( tagName, tag );

//        tagName    = "JMRUI_INFO_TAG";
//        tagID      = DicomDecoder.JMRUI_INFO_TAG;
//        encodingID = DicomDecoder.CS;
//        tagValue   = m_JMRUI_Info;

//        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
//        otherTags.put( tagName, tag );

        //=====================================================================

        return otherTags;
    }

    /**
     *
     * Returns the DICOM tags related to patient properties
     *
     * in a PACS system.
     *
     * @return
     */
    public LinkedHashMap<String, DicomTagAndValue> getPatientRelatedDicomTagsAndValues( )
    {

        LinkedHashMap<String, DicomTagAndValue> patientRelatedTags = new LinkedHashMap<String, DicomTagAndValue>( );

        // # Patient Module Attributes

        String tagName  = "PATIENT_NAME";

        int tagID       = DicomDecoder.PATIENT_NAME;
        int encodingID  = DicomDecoder.PN;
        String tagValue = this.m_patientName;

        DicomTagAndValue tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        patientRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "PATIENT_ID";
        tagID      = DicomDecoder.PATIENT_ID;
        encodingID = DicomDecoder.LO;
        tagValue   = this.m_patientID;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        patientRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "PATIENT_BIRTHDATE";
        tagID      = DicomDecoder.PATIENT_BIRTHDATE;
        encodingID = DicomDecoder.DA;
        tagValue   = m_patientBirthDate;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        patientRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "PATIENT_SEX";
        tagID      = DicomDecoder.PATIENT_SEX;
        encodingID = DicomDecoder.CS;
        tagValue   = m_patientSex;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        patientRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "PATIENT_AGE";
        tagID      = DicomDecoder.PATIENT_AGE;
        encodingID = DicomDecoder.AS;
        tagValue   = m_patientAge;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        patientRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "PATIENT_POSITION";
        tagID      = DicomDecoder.PATIENT_POSITION;
        encodingID = DicomDecoder.CS;
        tagValue   = m_patientPosition;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        patientRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "PATIENT_WEIGHT";
        tagID      = DicomDecoder.PATIENT_WEIGHT;
        encodingID = DicomDecoder.DS;
        tagValue   = m_patientWeight;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        patientRelatedTags.put( tagName, tag );


        //=====================================================================

        return patientRelatedTags;
    }



    /**
     *
     * Returns the most essential DICOM tags necessary for storing a DICOM file
     * in a PACS system.
     *
     * @return
     */
    public LinkedHashMap<String, DicomTagAndValue> getMRRelatedlDicomTagsAndValues( )
    {
        
        LinkedHashMap<String, DicomTagAndValue> mrRelatedTags = new LinkedHashMap<String, DicomTagAndValue>( );

        // # Patient Module Attributes

        String tagName    = "SEQUENCE_NAME";
        int tagID      = DicomDecoder.SEQUENCE_NAME;
        int encodingID = DicomDecoder.SH;
        String tagValue   = m_sequenceName;

        DicomTagAndValue tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );


        tagName    = "PROTOCOL_NAME";
        tagID      = DicomDecoder.PROTOCOL_NAME;
        encodingID = DicomDecoder.LO;
        tagValue   = m_protocolName;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        tagName    = "TR";
        tagID      = DicomDecoder.TR;
        encodingID = DicomDecoder.DS;
        tagValue   = m_TR;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "ECHO_TIME";
        tagID      = DicomDecoder.ECHO_TIME;
        encodingID = DicomDecoder.DS;
        tagValue   = m_echoTime;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );
        
        //---------------------------------------------------------------------------------

        tagName    = "IMAGE_FREQUENCY";
        tagID      = DicomDecoder.IMAGE_FREQUENCY;
        encodingID = DicomDecoder.DS;
        tagValue   = m_imagingFrequency;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------
        
        tagName    = "NUM_PHASE_ENCODES";
        tagID      = DicomDecoder.NUM_PHASE_ENCODES;
        encodingID = DicomDecoder.IS;
        tagValue   = m_numPhaseEncodes;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "ECHO_TRAIN_LENGTH";
        tagID      = DicomDecoder.ECHO_TRAIN_LENGTH;
        encodingID = DicomDecoder.IS;
        tagValue   = m_echoTrainLength;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "PERCENT_SAMPLING";
        tagID      = DicomDecoder.PERCENT_SAMPLING;
        encodingID = DicomDecoder.DS;
        tagValue   = m_percentSampling;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "PERCENT_PHASE_FOV";
        tagID      = DicomDecoder.PERCENT_PHASE_FOV;
        encodingID = DicomDecoder.DS;
        tagValue   = m_percentPhaseFOV;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "PIXEL_BANDWIDTH";
        tagID      = DicomDecoder.PIXEL_BANDWIDTH;
        encodingID = DicomDecoder.DS;
        tagValue   = m_pixelBandwidth;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );
        
        //---------------------------------------------------------------------------------

        tagName    = "CONTRAST_BOLUS_VOLUME";
        tagID      = DicomDecoder.CONTRAST_BOLUS_VOLUME;
        encodingID = DicomDecoder.DS;
        tagValue   = m_contrastBolusVolume;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "CONTRAST_FLOW_DURATION";
        tagID      = DicomDecoder.CONTRAST_FLOW_DURATION;
        encodingID = DicomDecoder.DS;
        tagValue   = m_contrastFlowDuration;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        tagName    = "TRANSMIT_COIL_NAME";
        tagID      = DicomDecoder.TRANSMIT_COIL_NAME;
        encodingID = DicomDecoder.DS;
        tagValue   = m_transmitCoilName;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "PHASE_ENCODING_DIR";
        tagID      = DicomDecoder.PHASE_ENCODING_DIR;
        encodingID = DicomDecoder.CS;
        tagValue   = m_phaseEncodingDirection;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "FLIP_ANGLE";
        tagID      = DicomDecoder.FLIP_ANGLE;
        encodingID = DicomDecoder.DS;
        tagValue   = m_flipAngle;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "VARIABLE_FLIP_ANGLE_FLAG";
        tagID      = DicomDecoder.VARIABLE_FLIP_ANGLE_FLAG;
        encodingID = DicomDecoder.CS;
        tagValue   = m_variableFlipAngleFlag;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "SAR";
        tagID      = DicomDecoder.SAR;
        encodingID = DicomDecoder.DS;
        tagValue   = m_SAR;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "DBDT";
        tagID      = DicomDecoder.DBDT;
        encodingID = DicomDecoder.DS;
        tagValue   = m_dBdt;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "CONTRAST_AGENT";
        tagID      = DicomDecoder.CONTRAST_AGENT;
        encodingID = DicomDecoder.LO;
        tagValue   = m_contrastAgent;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );
        
        //---------------------------------------------------------------------------------

        tagName    = "SCANNING_SEQUENCE";
        tagID      = DicomDecoder.SCANNING_SEQUENCE;
        encodingID = DicomDecoder.CS;
        tagValue   = m_scanningSequence;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "SEQUENCE_VARIANT";
        tagID      = DicomDecoder.SEQUENCE_VARIANT;
        encodingID = DicomDecoder.CS;
        tagValue   = m_sequenceVariant;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "SCAN_OPTIONS";
        tagID      = DicomDecoder.SCAN_OPTIONS;
        encodingID = DicomDecoder.CS;
        tagValue   = m_scanOptions;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "ACQUISITION_TYPE";
        tagID      = DicomDecoder.ACQUISITION_TYPE;
        encodingID = DicomDecoder.CS;
        tagValue   = m_acquisitionType;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //---------------------------------------------------------------------------------

        tagName    = "IMAGE_TYPE";
        tagID      = DicomDecoder.IMAGE_TYPE;
        encodingID = DicomDecoder.CS;
        tagValue   = m_imageType;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        tagName    = "SCANNER_MODEL";
        tagID      = DicomDecoder.SCANNER_MODEL;
        encodingID = DicomDecoder.CS;
        tagValue = m_scannerModelName;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        mrRelatedTags.put( tagName, tag );

        //=====================================================================

        return mrRelatedTags;
    }

    /**
     * If data contains compressed image data, in this version we cannot display the image!
     * @return
     */
    public boolean canDisplayImagePixelData( )
    {
        return m_canDisplayImage;
    }

    /**
     * Forces the setting of geometric attributes
     */
    public void setGeometricSliceAttributes( )
    {

        String str = getImageOrientationPatient( );

        if ( ( str != null ) &&
             ( str.compareTo( "N.A." ) != 0 ) &&
             ( str.compareTo( "" ) != 0 ) )
        {
            // for invalid headers this string is sometimes not set! therefore if
            m_dirVecX = new Point3D( );
            m_dirVecY = new Point3D( );

            ///////////////////
            int seperatorPos = str.lastIndexOf( '\\' );
            String subStr = str.substring( seperatorPos + 1, str.length( ) );

            m_dirVecY.m_x3 = (new Double( subStr )).doubleValue( );

            str = str.substring( 0, seperatorPos );
            ///////////////////
            seperatorPos = str.lastIndexOf( '\\' );
            subStr = str.substring( seperatorPos + 1, str.length( ) );

            m_dirVecY.m_x2 = (new Double( subStr )).doubleValue( );

            str = str.substring( 0, seperatorPos );
            ///////////////////
            seperatorPos = str.lastIndexOf( '\\' );
            subStr = str.substring( seperatorPos + 1, str.length( ) );

            m_dirVecY.m_x1 = (new Double( subStr )).doubleValue( );

            str = str.substring( 0, seperatorPos );
            ///////////////////

            seperatorPos = str.lastIndexOf( '\\' );
            subStr = str.substring( seperatorPos + 1, str.length( ) );

            m_dirVecX.m_x3 = ( new Double( subStr ) ).doubleValue( );

            str = str.substring( 0, seperatorPos );

            ///////////////////

            seperatorPos = str.lastIndexOf( '\\' );
            subStr = str.substring( seperatorPos + 1, str.length( ) );

            m_dirVecX.m_x2 = ( new Double( subStr ) ).doubleValue( );

            str = str.substring( 0, seperatorPos );

            ///////////////////

            m_dirVecX.m_x1 = ( new Double( str ) ).doubleValue( );

            ///////////////////
            // Define the spatial 3D coordinates of the topleft point in the image

            m_topLeftPoint = new Point3D( getImagePositionPatientX( ),
                                          getImagePositionPatientY( ),
                                          getImagePositionPatientZ( )
                                        );

            // Compute the FOV sizes width and height of the current slice
            if ( ( getPixelWidth( ).compareTo( "N.A." ) != 0 )  &&
                 ( getPixelHeight( ).compareTo( "N.A." ) != 0 ) &&
                 ( getImageWidth( ) != 0 ) &&
                 ( getImageHeight( ) != 0 ) )
            {
                m_FOVX = new Double( getPixelWidth( ) ).doubleValue( ) * new Double( getImageWidth( ) ).doubleValue( );
                m_FOVY = new Double( getPixelHeight( ) ).doubleValue( ) * new Double( getImageHeight( ) ).doubleValue( );
            }

            if ( ( getPixelWidth( ).compareTo( "N.A." ) == 0 )  &&
                 ( getPixelHeight( ).compareTo( "N.A." ) == 0 ) )
            {
                setPixelWidth( "0.0" );
                setPixelHeight( "0.0" );
            }

            // if the image matrix size has been changed, the top left point must be translated
            // in order to get correct coordinates

            if ( ( getInitialImageHeight( ) != getImageHeight( ) ) ||
                 ( getInitialImageWidth( )  != getImageWidth( ) ) )
            {
                int X = ( getImageWidth( ) - getInitialImageWidth( ) ) / 2;
                int Y = ( getImageHeight( ) - getInitialImageHeight( ) ) / 2;

                double factorX = -( X / ( new Double( getInitialImageWidth( ) ).doubleValue( ) ) ) * new Double( getPixelWidth( ) ).doubleValue( ) * getInitialImageWidth( );
                double factorY = -( Y / ( new Double( getInitialImageHeight( ) ).doubleValue( ) ) ) * new Double( getPixelHeight( ) ).doubleValue( ) * getInitialImageHeight( );

                Point3D hori = new Point3D( this.m_dirVecX );
                hori.scalarMultiply( factorX );

                Point3D verti = new Point3D( this.m_dirVecY );
                verti.scalarMultiply( factorY );

                m_topLeftPoint = m_topLeftPoint.translate( hori );
                m_topLeftPoint = m_topLeftPoint.translate( verti );

            }
        }
    }


    /**
     * Sets the correct Nordic pixel scaling factor 
     * Nordic scales the pixels by a factor before storing them
     * since this factor is not the same for all patients/series/images
     * one has to multiply by this factor to get the Nordic amplitudes
     * back in the controls based on this code here
     * 
     * @param s 
     */
    public void setNordicPixelScaling( float f ) 
    {
        this.m_nordicPixelScaling = f;
        System.out.println( "NORDIC SCALING FACTOR FOUND !!! : " + f );
    }
    
    /**
     * Returns the correct Nordic pixel scaling factor 
     * Nordic scales the pixels by a factor before storing them
     * since this factor is not the same for all patients/series/images
     * one has to multiply by this factor to get the Nordic amplitudes
     * back in the controls based on this code here
     * 
     * @param s 
     */
    public float getNordicPixelScaling( ) 
    {
        return m_nordicPixelScaling;
    }
    
    
    // All private attributes
    private boolean m_isSpectrum = false;
    private String  m_manufacturer = "N.A.";
    private String  m_patientName = "N.A.";
    private String  m_patientID = "N.A.";
    private String  m_patientBirthDate = "N.A.";
    private String  m_patientSex = "N.A.";
    private String  m_patientAge = "N.A.";
    private String  m_patientWeight = "N.A.";
    private String  m_contrastAgent = "N.A.";
    private String  m_scanningSequence = "N.A.";
    private String  m_sequenceVariant = "N.A.";
    private String  m_scanOptions = "N.A.";
    private String  m_acquisitionType = "N.A.";
    private String  m_sop_instance_uid = "N.A.";
    private String  m_series_instance_uid = "N.A.";
    private String  m_transferSyntaxUID = "N.A.";
    private String  m_referenced_SOP_Class_UID = "N.A.";
    private String  m_imageType = "N.A.";
    private String  m_studyDate = "N.A.";
    private String  m_seriesDate = "N.A.";
    private String  m_acquisitionDate = "N.A.";
    private String  m_contentDate = "N.A.";
    private String  m_mediaStorageSOPInstanceUID = "N.A.";
    
    private String  m_spectroscopicVoxelDimensions = "N.A.";
    
    private String  m_imageModality = "N.A.";
    
    private String  m_studyTime = "N.A.";
    private String  m_seriesTime = "N.A.";
    private String  m_acquisitionTime = "N.A.";
    private String  m_contentTime = "N.A.";
    
    private String  m_accessionNumber = "N.A.";
    
    private int     m_fileFormat = -1;
    private int     m_fileType = -1;
    private String  m_fileName = "N.A.";
    private String  m_directory = "N.A.";
    private String  m_url = "N.A.";
    
    private String  m_seriesDescription;
    
    private String  m_sequenceName = "N.A.";
    private String  m_softwareVersion = "N.A.";
    private String  m_protocolName = "N.A.";
    private String  m_contrastBolusVolume = "N.A.";
    private String  m_contrastFlowDuration = "N.A.";
    private String  m_transmitCoilName = "N.A.";
    private String  m_phaseEncodingDirection = "N.A.";
    private String  m_flipAngle = "N.A.";
    private String  m_variableFlipAngleFlag = "N.A.";
    private String  m_photometricInterpretation = "N.A.";

     
    // Two representations
    private String  m_imagePosition = "N.A.";
    private String  m_imageOrientation = "N.A.";
    
    private double  m_x_position_patient = 0.0;
    private double  m_y_position_patient = 0.0;
    private double  m_z_position_patient = 0.0;
    
    private int     m_width = -1;
    private int     m_height = -1;

    private String  m_studyID = "N.A.";
    private String  m_seriesNumber = "N.A.";
    private String  m_acquisitionNumber = "N.A.";
    private String  m_instanceNumber = "N.A.";
    private String  m_samplesPerPixel = "N.A.";
    private int     m_planarConfiguration;
    private String  m_bitsStored = "N.A.";
    private String  m_highBit = "N.A.";
    
    private String  m_sliceLocation = "N.A.";
    private int     m_filePointerOffsetPixelDataStart = 0;
    
    private String   m_SAR = "N.A.";
    private String   m_dBdt = "N.A.";
    private String   m_patientPosition = "N.A.";
    
    private String   m_windowCenter = "N.A.";
    private String   m_windowWidth = "N.A.";
    
    private String   m_numberOfFrames = "N.A.";
    private int      m_gapBetweenImages = -1;
    private boolean  m_whiteIsZero;
    private boolean  m_intelByteOrder;
    private int      m_lutSize = -1;
    private byte[]   m_reds;
    private byte[]   m_greens;
    private byte[]   m_blues;
    
    private String   m_info = "N.A.";
    
    private String   m_pixelWidth = "N.A.";
    private String   m_pixelHeight = "N.A.";

    private int      m_initialImageWidth  = -1;
    private int      m_initialImageHeight = -1;

    private String   m_pixelDepth = "N.A.";
    private String   m_pixelBandwidth = "N.A.";
    
    private String   m_sliceThickness = "N.A.";

    private String   m_smallestPixelValue = "N.A.";
    private String   m_largestPixelValue = "N.A.";
    
    // Pulse sequence repetition time
    private String   m_TR = "N.A.";
    private String   m_echoTime = "N.A.";

    private String   m_imagingFrequency = "N.A.";

    private String   m_numPhaseEncodes = "N.A.";
    private String   m_echoTrainLength = "N.A.";
    private String   m_percentSampling = "N.A.";
    private String   m_percentPhaseFOV = "N.A.";
    private String   m_unit = "N.A.";
    private int      m_calibrationFunction;
    private double[] m_coefficients;
    private String   m_valueUnit = "N.A.";
    private double   m_frameInterval;
    private String   m_description = "N.A.";
    
    private String   m_studyInstanceUID = "N.A.";
    private String   m_studyDescription = "N.A.";

    // Attributes added to handle non-image data dicom files
    // like spectroscopic datafiles
    boolean          m_isImageData     = true;
    boolean          m_canDisplayImage = true;

    // Array that contains the non-image data
    byte[]           m_nonImageData;

    private String   m_pixelRepresentation = "N.A.";
    private String   m_pixelSpacing     = "N.A.";
    private String   m_bitsAllocated    = "N.A.";

    private String   m_scannerModelName = "N.A.";
    private String   m_sopClassUID      = "N.A.";

    private String   m_mediaStorageSOPClassUID  = "N.A.";
    private String   m_implementationClassUID   = "N.A.";
    private String   m_characterSet = "N.A.";
    private String   m_groupLength  = "178";

    private boolean  m_isDICOMFileCreatedByJMRUI = false;
    private int      m_dataOffsetPointer;

    Point3D m_dirVecX;
    Point3D m_dirVecY;
    Point3D m_topLeftPoint;
    double  m_FOVX;
    double  m_FOVY;
   
    private float m_nordicPixelScaling = 1.0f;
    
    
    // Fluoroscopy tags
        
    private float  m_imageAndFluoroscopyAreaDoseProduct         = -1f;
    private float  m_fluoroscopyExposureTime                    =-1f;
    private float  m_fluoroscopyKVP                             =-1f;
    private float  m_ct_KVP                                     =-1f;
    private float  m_fluoroscopyXRayTubeCurrent                 = -1f;
    private float  m_fluoroscopyEstimatedRadiographicMagnificationFactor = -1f;
    private float  m_fluoroscopyDose                            = -1f;
    private float  m_fluoroscopySkinDoseAccumulation            = -1f;
    
    private String m_fluoroscopyStationName                     = "-1";
    private String m_contrastBolusAgentName                     = "-1";
    private String m_bodyPartExamined                           = "-1";
    private float  m_fluoroscopyAveragePulseWidth               = -1f;
    private float  m_fluoroscopySourceToIsoCenter               = -1f;
    private int    m_fluoroscopyCopperFilter                    = -1;
    private float  m_fluoroscopyCurrentTimeProduct              = -1f;
    
    private int    m_fluoroscopyAcquisitionMode                 = -1;
    private int    m_fluoroscopyAcquisitionType                 = -1;
    private int    m_fluoroscopyFootSwitchIndex                 = -1;
    private String m_fluoroscopyRadiationSetting                = "-1";
    private String m_fluoroscopyRadiationMode                   = "-1";
    private float  m_fluoroscopyIntensifierSize                 = -1f;
    private float  m_fluoroscopyFocalSpot                       = -1f;
    private String m_fluoroscopyImagePixelSpacing               = "-1";
    
    private float  m_fluoroscopyPositionerPrimaryAngle          = -1f;
    private float  m_fluoroscopyPositionerSecondaryAngle        = -1f;
    
    private float  m_fluoroscopySkinDosePercent                 = -1f;
    
    // CT related parameters
    private float   m_CT_DataCollectionDiameter                  = -1f;
    
    private String  m_CT_ContrastBolusStartTime                  = "N.A.";
    private String  m_CT_ContrastBolusStopTime                   = "N.A.";
    private float   m_CT_ContrastBolusTotalDose                  = -1f;
    private float   m_CT_ContrastFlowRate                        = -1f;
    private float   m_CT_ContrastFlowDuration                    = -1f;
    
    private float   m_CT_ExposureTime                            = -1f;
    private float   m_CT_XRayTubeCurrent                         = -1f;
    private float   m_CT_FocalSpot                               = -1f;
    
    private float   m_fluoroscopySkinDoseRate                    = -1f;
    private float   m_CT_ContrastBolusIngredientConcentration    = -1f;
    private float   m_CT_ReconstructionDiameter                  = -1f;
    private float   m_CT_DistanceSourceToDetector                = -1f;
    private float   m_CT_DistanceSourceToPatient                 = -1f;
    private float   m_CT_GantryDetectorTilt                      = -1f;
    private double  m_CT_TableHeight                             = -1.0;
    private String  m_CT_RotationDirection                       = "N.A.";
    private int     m_set_CT_Exposure                            = -1;
    private String  m_CT_FilterType                              = "N.A.";
    private int     m_CT_GeneratorPower                          = -1;
    private String  m_CT_DateOfLastCalibration                   = "N.A.";
    private String  m_CT_TimeOfLastCalibration                   = "N.A.";
    private String  m_CT_ConvolutionKernel                       = "N.A.";
    
    private double  m_CT_SingleCollimationWidth                  = -1.0;
    private double  m_CT_TotalCollimationWidth                   = -1.0;
    private double  m_CT_TableSpeed                              = -1.0;
    private double  m_CT_TableFeedPerRotation                    = -1.0;
    
    private double  m_CT_SpiralPitchFactor                       = -1.0;
    private double   m_CT_EstimatedDoseSaving                    = -1.0;
    
    private String  m_CT_DataCollectionCenterPatient             = "N.A.";
    private String  m_CT_ReconstructionTargetCenterPatient       = "N.A.";
    private String  m_CT_ExposureModulationType                  = "N.A.";
    private double  m_CT_CTDIvol                                 = -1.0;
    private String  m_CT_CodeValue                               = "N.A.";
    private String  m_CT_CodingSchemeDesignator                  = "N.A.";
    private String  m_CT_CodeMeaning                             = "N.A.";
    private float   m_CT_FeedPerRotation                         = -1f;
    
    // Determines whether the Dicom is a structured report.
    private boolean m_is_SR_Data = false;
    
    // New tags from the Vida implementation
    public LinkedHashMap<String,String> m_uninterpretedTags = new LinkedHashMap<>();
    
    // Setters
    public void setImageAndFluoroscopyAreaDoseProduct( String s ) 
    {
        m_imageAndFluoroscopyAreaDoseProduct = new Float( s ).floatValue( );
    }

    public void setFluoroscopyExposureTime( String s ) 
    {
        if ( s.compareTo("") != 0 )
        {
            m_fluoroscopyExposureTime = new Float( s).floatValue( );
        }
        else
        {
            m_fluoroscopyExposureTime = 0f;
        }
    }

    public void setFluoroscopyKVP( String s ) 
    {
        if ( s.compareTo("") != 0 )
        {
            m_fluoroscopyKVP    = new Float( s ).floatValue( );
            m_ct_KVP            = m_fluoroscopyKVP;
        }
        else
        {
            m_fluoroscopyKVP    = 0f;
            m_ct_KVP            = 0f;
        }
    }

    public void setFluoroscopyXRayTubeCurrent( String s ) 
    {
        if ( s.compareTo("") != 0 )
        {
            m_fluoroscopyXRayTubeCurrent = new Float( s ).floatValue( );
        }
        else
        {
            m_fluoroscopyXRayTubeCurrent = 0f;
        }
    }

    public void setFluoroscopyEstimatedRadiographicMagnificationFactor( String s ) 
    {
        m_fluoroscopyEstimatedRadiographicMagnificationFactor = new Float( s ).floatValue( ); 
    }

    public void setFluoroscopyDose( String s ) 
    {
        m_fluoroscopyDose = new Float( s ).floatValue( );
    }

    public void setFluoroscopySkinDoseAccumulation( String s ) 
    {
        m_fluoroscopySkinDoseAccumulation = new Float( s ).floatValue( );
    }

    public void setFluoroscopySkinDoseRate( String s ) 
    {
        m_fluoroscopySkinDoseRate = new Float( s ).floatValue( ); 
    }

    public void setFluoroscopyStationName( String s ) 
    {
        m_fluoroscopyStationName = s;
    }

    public void setContrastBolusAgentName( String s ) 
    {
        m_contrastBolusAgentName = s;
    }

    public void setBodyPartExamined( String s ) 
    {
        m_bodyPartExamined = s;
    }

    public void setFluoroscopyAveragePulseWidth( String s ) 
    {
        m_fluoroscopyAveragePulseWidth = new Float( s.trim( ) ).floatValue( ); 
    }

    public void setFluoroscopyRadiationSetting( String s ) 
    {
        m_fluoroscopyRadiationSetting = s;
    }

    public void setFluoroscopyRadiationMode( String s ) 
    {
        m_fluoroscopyRadiationMode = s;
    }

    public void setFluoroscopyIntensifierSize( String s ) 
    {
        m_fluoroscopyIntensifierSize = new Float( s.trim( ) ).floatValue( ); 
    }

    public void setFluoroscopyImagePixelSpacing( String s ) 
    {
        m_fluoroscopyImagePixelSpacing = s;
    }

    public void setFluoroscopyFocalSpot(String s) 
    {
        m_fluoroscopyFocalSpot =  new Float( s.trim( ) ).floatValue( ); 
    }

    public void setFluoroscopyPositionerPrimaryAngle( String s ) 
    {
        if ( s.compareTo( "" ) != 0 )
        {
            m_fluoroscopyPositionerPrimaryAngle =  new Float( s.trim( ) ).floatValue( );  
        }
        else
        {
            m_fluoroscopyPositionerPrimaryAngle = 0f;
        }
    }

    public void setFluoroscopyPositionerSecondaryAngle( String s ) 
    {
        if ( s.compareTo("") != 0 )
        {
            m_fluoroscopyPositionerSecondaryAngle = new Float( s.trim( ) ).floatValue( );  
        }
        else
        {
            m_fluoroscopyPositionerSecondaryAngle = 0f;
        }
    }

    public void setFluoroscopyAcquisitionType( String s ) 
    {
        m_fluoroscopyAcquisitionType  = new Integer( s.trim( ) ).intValue( ); 
    }

    public void setFluoroscopyAcquisitionMode( String s ) 
    {
        m_fluoroscopyAcquisitionMode  = new Integer( s.trim( ) ).intValue( ); 
    }

    public void setFluoroscopyFootSwitchIndex( String s ) 
    {
        m_fluoroscopyFootSwitchIndex  = new Integer( s.trim( ) ).intValue( ); 
    }

    public void setFluoroscopyCurrentTimeProduct( String s ) 
    {
        m_fluoroscopyCurrentTimeProduct = new Float( s.trim( ) ).floatValue( ); 
    }

    public void setFluoroscopyCopperFilter( String s ) 
    {
        m_fluoroscopyCopperFilter =  ( new Integer( s.trim( ) ) ).intValue( ); 
    }

    void setFluoroscopySourceToIsoCenter( String s ) 
    {
        m_fluoroscopySourceToIsoCenter = ( new Float( s.trim( ) ) ).floatValue( ); 
    }

    public void setFluoroscopySkinDosePercent( String s ) 
    {
        m_fluoroscopySkinDosePercent = ( new Float( s.trim( ) ) ).floatValue( ); 
    }

    public void set_CT_DataCollectionDiameter( String s ) 
    {
        m_CT_DataCollectionDiameter = ( new Float( s.trim( ) ) ).floatValue( );
    }

    public void set_CT_ContrastBolusStartTime( String s ) 
    {
        m_CT_ContrastBolusStartTime = s;
    }
    
    public void set_CT_ContrastBolusStopTime( String s ) 
    {
        m_CT_ContrastBolusStopTime = s;
    }

    public void set_CT_ContrastBolusTotalDose( String s ) 
    {
        m_CT_ContrastBolusTotalDose = ( new Float( s.trim( ) ) ).floatValue( );
    }

    public void set_CT_ContrastFlowRate( String s )
    {
        m_CT_ContrastFlowRate = ( new Float( s.trim( ) ) ).floatValue( );
    }

    public void set_CT_ContrastFlowDuration(String s) 
    {
        m_CT_ContrastFlowDuration = ( new Float( s.trim( ) ) ).floatValue( );
    }

    public void set_CT_ExposureTime( String s ) 
    {                
        if ( s.compareTo("") != 0 )
        {
            m_CT_ExposureTime = new Float( s.trim( ) ).floatValue( );
        }
        else
        {
            m_CT_ExposureTime = 0f;
        }
    }

    public void set_CT_XRayTubeCurrent( String s ) 
    {
        if ( s.compareTo("") != 0 )
        {
            m_CT_XRayTubeCurrent = new Float( s.trim( ) ).floatValue( );
        }
        else
        {
            m_CT_XRayTubeCurrent = 0f;
        }
    }
    
    
    public void set_CT_FocalSpot( String s ) 
    {
        m_CT_FocalSpot =  new Float( s.trim( ) ).floatValue( ); 
    }

    public void set_CT_ContrastBolusIngredientConcentration( String s ) 
    {
        if ( s.length( ) > 0 )
        {
            m_CT_ContrastBolusIngredientConcentration = new Float( s ).floatValue( ); 
        }
    }

    public void set_CT_ReconstructionDiameter( String s ) 
    {
        m_CT_ReconstructionDiameter = new Float( s.trim( ) ).floatValue( ); 
    }

    public void set_CT_DistanceSourceToDetector( String s ) 
    {
        m_CT_DistanceSourceToDetector = new Float( s.trim( ) ).floatValue( );
    }

    public void set_CT_DistanceSourceToPatient( String s ) 
    {
        m_CT_DistanceSourceToPatient = new Float( s.trim( ) ).floatValue( );
    }

    public void set_CT_GantryDetectorTilt( String s ) 
    {
        m_CT_GantryDetectorTilt = new Float( s.trim( ) ).floatValue( );
    }

    public void set_CT_TableHeight( String height ) 
    {
        m_CT_TableHeight = new Double( height.trim() ).doubleValue( );
    }

    public void set_CT_RotationDirection( String s ) 
    {
        m_CT_RotationDirection = s;
    }

    public void set_CT_Exposure( String s )
    {
        m_set_CT_Exposure = ( new Integer( s.trim( ) ) ).intValue( );
    }

    public void set_CT_FilterType( String s ) 
    {
        m_CT_FilterType = s;
    }

    public void set_CT_GeneratorPower( String s )
    {
        m_CT_GeneratorPower = ( new Integer( s.trim( ) ) ).intValue( );
    }

    public void set_CT_DateOfLastCalibration( String s ) 
    {
        m_CT_DateOfLastCalibration = s;
    }

    public void set_CT_TimeOfLastCalibration( String s ) 
    {
        m_CT_TimeOfLastCalibration = s;
    }

    public void set_CT_ConvolutionKernel( String s ) 
    {
        m_CT_ConvolutionKernel = s;
    }

    public void set_CT_SingleCollimationWidth( double s_collimation ) 
    {
        m_CT_SingleCollimationWidth = s_collimation;
    }

    public void set_CT_TotalCollimationWidth( double t_collimation ) 
    {
        m_CT_TotalCollimationWidth = t_collimation;
    }

    public void set_CT_TableSpeed( double t_speed ) 
    {
        m_CT_TableSpeed = t_speed;
    }

    public void set_CT_TableFeedPerRotation( double feed ) 
    {
        m_CT_TableFeedPerRotation = feed;
    }

    public void set_CT_SpiralPitchFactor( double pitch ) 
    {
        m_CT_SpiralPitchFactor = pitch;
    }

    public void set_CT_DataCollectionCenterPatient( String s ) 
    {
        m_CT_DataCollectionCenterPatient = s;
    }

    public void set_CT_ReconstructionTargetCenterPatient( String s ) 
    {
        m_CT_ReconstructionTargetCenterPatient = s;
    }

    public void set_CT_ExposureModulationType( String s ) 
    {
        m_CT_ExposureModulationType = s;
    }

    public void set_CT_EstimatedDoseSaving( double dose_saving ) 
    {
        m_CT_EstimatedDoseSaving = dose_saving;
    }

    public void set_CT_CTDIvol( double CTDIvol ) 
    {
        m_CT_CTDIvol = CTDIvol;
    }

    public void set_CT_CodeValue( String s ) 
    {
        m_CT_CodeValue = s;
    }

    public void set_CT_CodingSchemeDesignator( String s ) 
    {
       m_CT_CodingSchemeDesignator = s;
    }

    public void set_CT_CodeMeaning( String s ) 
    {
        m_CT_CodeMeaning = s;
    }

    public void set_CT_FeedPerRotation( String s ) 
    {
        m_CT_FeedPerRotation = new Float( s.trim( ) ).floatValue( );
    }

    public void set_CTXRayTubeCurrent( String s ) 
    {
        m_CT_XRayTubeCurrent = new Float( s.trim( ) ).floatValue( );
    }

    public void setCTExposureTime(String s) 
    {
        m_CT_ExposureTime = new Float( s.trim( ) ).floatValue( );
    }

    // The corresponding getters...
      // Setters
    public float getImageAndFluoroscopyAreaDoseProduct( ) 
    {
        return m_imageAndFluoroscopyAreaDoseProduct;
    }

    public float getFluoroscopyExposureTime( ) 
    {
        return m_fluoroscopyExposureTime;
    }

    public float getFluoroscopy_KVP( ) 
    {
        return m_fluoroscopyKVP;
    }

    public float get_CT_KVP( ) 
    {
        return m_ct_KVP;
    }
    
    public float getFluoroscopyXRayTubeCurrent( ) 
    {
        return m_fluoroscopyXRayTubeCurrent;
    }

    public float getFluoroscopyEstimatedRadiographicMagnificationFactor( ) 
    {
        return m_fluoroscopyEstimatedRadiographicMagnificationFactor; 
    }

    public float getFluoroscopyDose( ) 
    {
        return m_fluoroscopyDose;
    }

    public float getFluoroscopySkinDoseAccumulation( ) 
    {
        return m_fluoroscopySkinDoseAccumulation;
    }

    public float getFluoroscopySkinDoseRate( ) 
    {
        return m_fluoroscopySkinDoseRate; 
    }

    public String getFluoroscopyStationName( ) 
    {
        return m_fluoroscopyStationName;
    }

    public String getContrastBolusAgentName( ) 
    {
        return m_contrastBolusAgentName;
    }

    public String getBodyPartExamined( ) 
    {
        return m_bodyPartExamined;
    }

    public float getFluoroscopyAveragePulseWidth( ) 
    {
        return m_fluoroscopyAveragePulseWidth; 
    }

    public String  getFluoroscopyRadiationSetting( ) 
    {
        return m_fluoroscopyRadiationSetting;
    }

    public String  getFluoroscopyRadiationMode( ) 
    {
        return m_fluoroscopyRadiationMode;
    }

    public float  getFluoroscopyIntensifierSize( ) 
    {
        return m_fluoroscopyIntensifierSize; 
    }

    public String  getFluoroscopyImagePixelSpacing( ) 
    {
        return m_fluoroscopyImagePixelSpacing;
    }

    public float getFluoroscopyFocalSpot( ) 
    {
        return m_fluoroscopyFocalSpot; 
    }

    public float getFluoroscopyPositionerPrimaryAngle( ) 
    {
        return m_fluoroscopyPositionerPrimaryAngle;
    }

    public float getFluoroscopyPositionerSecondaryAngle( ) 
    {
        return m_fluoroscopyPositionerSecondaryAngle;
    }

    public int getFluoroscopyAcquisitionType( ) 
    {
        return m_fluoroscopyAcquisitionType; 
    }

    public int getFluoroscopyAcquisitionMode( ) 
    {
        return m_fluoroscopyAcquisitionMode; 
    }

    public int getFluoroscopyFootSwitchIndex( ) 
    {
        return m_fluoroscopyFootSwitchIndex; 
    }

    public float getFluoroscopyCurrentTimeProduct( ) 
    {
        return m_fluoroscopyCurrentTimeProduct; 
    }

    public int getFluoroscopyCopperFilter( ) 
    {
        return m_fluoroscopyCopperFilter; 
    }

    public float getFluoroscopySourceToIsoCenter( ) 
    {
        return m_fluoroscopySourceToIsoCenter; 
    }

    public float getFluoroscopySkinDosePercent( ) 
    {
        return m_fluoroscopySkinDosePercent; 
    }

    public float get_CT_DataCollectionDiameter( ) 
    {
        return m_CT_DataCollectionDiameter;
    }

    public String get_CT_ContrastBolusStartTime( ) 
    {
        return m_CT_ContrastBolusStartTime;
    }
    
    public String get_CT_ContrastBolusStopTime( ) 
    {
        return m_CT_ContrastBolusStopTime;
    }

    public float get_CT_ContrastBolusTotalDose( )
    {
        return m_CT_ContrastBolusTotalDose;
    }

    public float get_CT_ContrastFlowRate( )
    {
        return m_CT_ContrastFlowRate;
    }

    public float get_CT_ContrastFlowDuration( ) 
    {
        return m_CT_ContrastFlowDuration;
    }

    public float get_CT_ExposureTime( ) 
    {                
        return m_CT_ExposureTime;
    }

    public float get_CT_XRayTubeCurrent( ) 
    {
        return m_CT_XRayTubeCurrent;
    }
    
    public float get_CT_FocalSpot( ) 
    {
        return m_CT_FocalSpot; 
    }

    public float get_CT_ContrastBolusIngredientConcentration( ) 
    {
        return m_CT_ContrastBolusIngredientConcentration; 
    }

    public float get_CT_ReconstructionDiameter( ) 
    {
        return m_CT_ReconstructionDiameter; 
    }

    public float get_CT_DistanceSourceToDetector( ) 
    {
        return m_CT_DistanceSourceToDetector;
    }

    public float get_CT_DistanceSourceToPatient( ) 
    {
        return m_CT_DistanceSourceToPatient;
    }

    public float get_CT_GantryDetectorTilt( ) 
    {
        return m_CT_GantryDetectorTilt;
    }

    public double get_CT_TableHeight( ) 
    {
        return m_CT_TableHeight;
    }

    public String get_CT_RotationDirection( ) 
    {
        return m_CT_RotationDirection;
    }

    public int get_CT_Exposure( )
    {
        return m_set_CT_Exposure;
    }

    public String get_CT_FilterType( ) 
    {
        return m_CT_FilterType;
    }

    public int get_CT_GeneratorPower( )
    {
        return m_CT_GeneratorPower;
    }

    public String get_CT_DateOfLastCalibration( ) 
    {
        return m_CT_DateOfLastCalibration;
    }

    public String get_CT_TimeOfLastCalibration( ) 
    {
        return m_CT_TimeOfLastCalibration;
    }

    public String get_CT_ConvolutionKernel( ) 
    {
        return m_CT_ConvolutionKernel;
    }

    public double get_CT_SingleCollimationWidth( ) 
    {
        return m_CT_SingleCollimationWidth;
    }

    public double get_CT_TotalCollimationWidth( ) 
    {
        return m_CT_TotalCollimationWidth;
    }

    public double get_CT_TableSpeed( ) 
    {
        return m_CT_TableSpeed;
    }

    public double get_CT_TableFeedPerRotation( ) 
    {
        return m_CT_TableFeedPerRotation;
    }

    public double get_CT_SpiralPitchFactor( ) 
    {
        return m_CT_SpiralPitchFactor;
    }

    public String get_CT_DataCollectionCenterPatient( ) 
    {
        return m_CT_DataCollectionCenterPatient;
    }

    public String get_CT_ReconstructionTargetCenterPatient( ) 
    {
        return m_CT_ReconstructionTargetCenterPatient;
    }

    public String get_CT_ExposureModulationType( ) 
    {
        return m_CT_ExposureModulationType;
    }

    public double get_CT_EstimatedDoseSaving( ) 
    {
        return m_CT_SpiralPitchFactor;
    }

    public double get_CT_CTDIvol( ) 
    {
        return m_CT_CTDIvol;
    }

    public String get_CT_CodeValue( ) 
    {
        return m_CT_CodeValue;
    }

    public String get_CT_CodingSchemeDesignator( ) 
    {
       return m_CT_CodingSchemeDesignator;
    }

    public String get_CT_CodeMeaning( ) 
    { 
        return m_CT_CodeMeaning;
    }

    public float get_CT_FeedPerRotation( ) 
    {
        return m_CT_FeedPerRotation ;
    }

    public float get_CTXRayTubeCurrent( )
    {
        return m_CT_XRayTubeCurrent;
    }

    public float getCTExposureTime( )
    {
        return m_CT_ExposureTime;
    }

    /**
     * 
     * Sets the list of all tags which are read from the Dicom file but who are
     * not interpreted.
     * 
     * @param tagAsString
     * @param value 
     */
    void addUnknownSiemensProprietoryDicomTag( String tagAsString, String value ) 
    {
        m_uninterpretedTags.put( tagAsString, value );
    }
    
    /**
     * Returns all tags who were read but of which no meaning is known.
     * @return 
     */
    public LinkedHashMap<String,String> getUninterpretedTags()
    {
        return m_uninterpretedTags;
    }

    /**
     * Enables the identification structured reports.
     * @param b 
     */
    void set_Is_SR_Data( boolean b ) 
    {
       m_is_SR_Data = b;
    }
    
    /**
     * Returns whether Dicom is structured report.
     * @return 
     */
    public boolean isDicom_Structured_Report( )
    {
        return m_is_SR_Data;
    }
    
    /**
     * Sets the Dicom files' referenced SOP class uid.
     * @param s 
     */
    public void setReferencedSOPClassUID(String s) 
    {
        m_referenced_SOP_Class_UID = s;
    }
    
    /**
     * Returns the Dicom files' referenced SOP class uid.
     * @return 
     */
    public String getReferencedSOPClassUID( ) 
    {
        return m_referenced_SOP_Class_UID;
    }

    public void setSpectroscopicVoxelDimensions( String dimensionsString ) 
    {
        m_spectroscopicVoxelDimensions = dimensionsString;
    }
    
    /**
     * Returns the voxel dimensions.
     * @return
     */
    public String getSpectroscopicVoxelDimensions( ) 
    {
        return m_spectroscopicVoxelDimensions;
    }
}