/*
 * File: $Id: RegioPolygonPixelDistributionHistogram.java,v 1.1.1.1 2006/04/19 09:38:47 hans Exp $
 * History
 * Date: 22.01.2004  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 - 2004 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.imagegraphicobjects;

import java.awt.BorderLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.Polygon;
import java.awt.Rectangle;

import javax.swing.JPanel;
import javax.swing.JFrame;

import org.spectrim.dicom.DicomImageData;
import org.spectrim.xyplot.XYPlot;
import org.spectrim.xyplot.Curve;

/**
 *    This class creates a histogram with pixel intensity distribution and displays
 *    as well the major statistical information of the image part enclosed by the 
 *    RegioPolygon
 * @author  hans
 */
public class RegioPolygonPixelDistributionHistogram extends JFrame
{
    
    
    /** 
     * Creates a new instance of RegioPolygonPixelDistributionHistogram 
     */
    public RegioPolygonPixelDistributionHistogram( JFrame parent, RegioPolygon regPol, float zoom, 
                                                   int[] imag, int imageWidth, int imageHeight, 
                                                   float nordicScalingFactor ) 
    {
        
        m_parent = parent;
        m_zoom = zoom;
        m_imag = imag.clone( );
        m_regPol = new RegioPolygon( regPol );
        
        m_imageHeight = imageWidth;
        m_imageWidth = imageHeight;
        
        m_nordicScalingFactor = nordicScalingFactor;
        
        m_collectionPanel = new JPanel( );
        m_collectionPanel.setLayout( new BorderLayout( ) );
        
        m_xyplot = new XYPlot( "histogram", this );
        
        m_collectionPanel.add( m_xyplot, BorderLayout.CENTER );
        
        addWindowListener( 
            new WindowAdapter( )
            {
                public void windowClosing( WindowEvent event )
                {
                   dispose( );
                }
            }
        );
        
        this.getContentPane( ).add( m_collectionPanel );
        
        pack( );
	
        calculateStatistics( );
        calculateHistogram( );
        
        setVisible( true );
    }
    
    /**
     * Calculates the histogram of the intensity distribution of the pixels
     * enclosed. 
     */
    private void calculateHistogram( )
    {
        

        m_rangeWidth = ( m_maxPixelValueAsInt - m_minPixelValueAsInt );
        
        if ( m_rangeWidth != 0 )
        {
            m_numberOfClasses = ( m_roiSize / 10 );
        
            if ( m_numberOfClasses < 10 )
            {
                m_numberOfClasses = 10;
            }
        
            if ( m_numberOfClasses > 256 )
            {
                m_numberOfClasses = 256;
            }
        
            m_classWidth = (float)m_rangeWidth / (float)m_numberOfClasses;
            
            m_occurences = new int[ m_numberOfClasses + 1];
            
            Polygon roi = (Polygon)m_regPol.scaled( 1f / m_zoom );

            m_pixelValues = new float[m_regPol.getInnerVolume( )];

            if ( roi != null )
            {
                Rectangle bb = roi.getBounds( );
                if ( bb.x < 0 )
                {    
                    bb.x = 0;
                }
                if ( bb.x + bb.width >= m_imageWidth ) 
                {
                    bb.width = m_imageWidth - bb.x - 1;
                }
                if ( bb.y < 0 )
                {
                    bb.y = 0;
                }
        
                if ( bb.y + bb.height >= m_imageHeight ) 
                {
                    bb.height = m_imageHeight - bb.y - 1;
                }
        
                int imageSize = m_imageWidth * m_imageHeight;

                int cntr = 0;

                // Calculate the average of the ROI
                for ( int y = bb.y; y <= bb.y + bb.height; y++ ) 
                {
                    int yoffset = y * m_imageWidth;
                    for ( int x = bb.x; x <= bb.x + bb.width; x++ ) 
                    {
                        if ( roi.contains( x, y ) ) 
                        {
                            float pixelValue = m_imag[ x + m_imageWidth * y ];
                            m_pixelValues[cntr] = m_imag[ x + m_imageWidth * y ];
                            int xVal = (int)( ( pixelValue - m_minPixelValueAsInt )  /  m_classWidth ); 
                            m_occurences[ xVal ]++;
                            cntr++;
                        }
                    }
                }
            }
            
            float[] xVals = new float[ 2 * m_occurences.length ];
            float[] yVals = new float[ 2 * m_occurences.length ];
            
            for ( int i = 0; i < m_occurences.length; i++ )
            {
                xVals[2*i] = ( m_minPixelValue + (float)i * m_classWidth ) * m_nordicScalingFactor;
                yVals[2*i] = m_occurences[i];
                xVals[2*i + 1] = ( m_minPixelValue + (float)(i+1) * m_classWidth ) * m_nordicScalingFactor;
                yVals[2*i + 1] = m_occurences[i];
            }
            
            m_xyplot.addCurve( new Curve( xVals, yVals ), true );
            m_xyplot.repaint( );
            
        }
    }

    /**
     *  Calculates the average pixel value and standard deviation of the
     *  currently active polygon (which encloses a ROI).
     *  @param Polygon roi  the currently selected polygon
     */
    private void calculateStatistics( )
    {
        m_roiSize = 0;
        
        float zoom = 1f / m_zoom;
        Polygon roi = (Polygon)m_regPol.scaled( zoom );
        
        if ( roi != null )
        {
            Rectangle bb = roi.getBounds( );
            if ( bb.x < 0 )
            {    
                bb.x = 0;
            }
            if ( bb.x + bb.width >= m_imageWidth ) 
            {
                bb.width = m_imageWidth - bb.x - 1;
            }
            if ( bb.y < 0 )
            {
                bb.y = 0;
            }
        
            if ( bb.y + bb.height >= m_imageHeight ) 
            {
                bb.height = m_imageHeight - bb.y - 1;
            }
        
            int imageSize = m_imageWidth * m_imageHeight;
            
            // Calculate the average of the ROI
            for ( int y = bb.y; y <= bb.y + bb.height; y++ ) 
            {
                int yoffset = y * m_imageWidth;
                for ( int x = bb.x; x <= bb.x + bb.width; x++ ) 
                {
                    if ( roi.contains( x, y ) ) 
                    {
                        m_roiSize++;
                        int xyoffset = yoffset;
                        m_average += m_imag[ x + m_imageWidth * y ];
                        
                        if ( m_imag[ x + m_imageWidth * y ] < m_minPixelValueAsInt )
                        {
                            m_minPixelValueAsInt = m_imag[ x + m_imageWidth * y ];
                            m_minPixelValue = m_imag[ x + m_imageWidth * y ] * m_nordicScalingFactor;
                        }
                        
                        if ( m_imag[ x + m_imageWidth * y ] > m_maxPixelValueAsInt )
                        {
                            m_maxPixelValueAsInt = m_imag[ x + m_imageWidth * y ];
                            m_maxPixelValue = m_imag[ x + m_imageWidth * y ] * m_nordicScalingFactor;
                        }
                    }
                }
            }

            m_average = m_average / m_roiSize;

            // Calculate the standard deviation of the current ROI
            float diff = 0.0f;
            // reset the roi size
            m_roiSize = 0;
        
            for ( int y = bb.y; y <= bb.y + bb.height; y++ )
            {
                int yoffset = y * m_imageWidth;
                for ( int x = bb.x; x <= bb.x + bb.width; x++ )
                {
                    if ( roi.contains( x, y ) )
                    {
                        m_roiSize++;
                        diff = m_average - m_imag[ x + m_imageWidth * y ];
                        m_stdev += diff * diff;
                    }
                }
            }

            m_stdev = (float)Math.sqrt( (double)m_stdev / (float)m_roiSize) ;

            System.out.println( "ROI size (pixel) = " + m_roiSize );
            System.out.println( "Average = " + m_average );
            System.out.println( "Standard deviation = " + m_stdev );
        }
    }
    
    
    /**
     * 
     * Returns the array of image pixels with the selected RegioPolygon.
     * 
     * @return
     */
    public float[] getImagePixels( )
    {
       return m_pixelValues;
    }


    // Attributes
    private final JFrame          m_parent;
    private final JPanel          m_collectionPanel;
    private final XYPlot          m_xyplot;
    private final RegioPolygon    m_regPol;
    private final int[]           m_imag;
    private final float           m_zoom;
    private final int             m_imageWidth;
    private final int             m_imageHeight;
    
    // Statistical parameters
    private float           m_minPixelValue = Integer.MAX_VALUE;
    private float           m_maxPixelValue = Integer.MIN_VALUE;
    private int             m_minPixelValueAsInt = Integer.MAX_VALUE;
    private int             m_maxPixelValueAsInt = Integer.MIN_VALUE;
    
    
    private float           m_average = 0f;
    private float           m_stdev = 0f;
    private int             m_roiSize = 0;
    
    // Histogram related attributes
    private float m_rangeWidth;
    private float m_classWidth;
    private int   m_numberOfClasses;
    private int[] m_occurences;
    
    private float[] m_pixelValues;

    // use nordic scaling if appropriate! default nordic scaling is 1.0f
    private float m_nordicScalingFactor = 1.0f;
    
    // For proper serialization
    private static final long serialVersionUID = 1431295898129393L;

    
}
