/*
 * File: $Id: ProbabilityBasedSegmentationCalculator.java,v 1.1.1.1 2008/04/18 09:38:48 hans Exp $
 * History
 * Date: 18.04.2008  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2008 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * ProbabilityBasedSegmentationCalculator.java
 *
 * Created on April 18, 2008, 1:59 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.spectrim.imageprocessing;

import org.spectrim.dicom.DicomSliceCanvasPolyDraw;
import org.spectrim.imagegraphicobjects.RegioPolygon;
import org.spectrim.lib.Dialogs;
import org.spectrim.lib.SwingWorker;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.JFrame;

/**
 * Class that handles the segmentation of images or square image regions based
 * on probability computations. 
 * @author hans
 */
public class ProbabilityBasedSegmentationCalculator 
{
  
  
    /** 
     * Creates a new instance of ProbabilityBasedSegmentationCalculator 
     * @param parent
     * @param image
     * @param lowThreshold
     * @param highThresshold
     * @param minROISize
     * @param maxROISize
     * @param performThresholdingBeforeSegmentation
     * @param performAdvancedSegmentation
     * @param colors
     * @param probeSizeX
     * @param probeSizeY
     * @param permutations
     * @param numberOfProbingPixels
     * @param pixelRangeDeterminationType
     * @param pixelRangeMultiplier
     * @param minimumImageNumberToFocus
     * @param maximumImageNumberToFocus
     * @param nordicScalingFactor
     */
    public ProbabilityBasedSegmentationCalculator( JFrame parent, DicomSliceCanvasPolyDraw image, 
                                                   float lowThreshold, float highThresshold, 
                                                   int minROISize, int maxROISize,
                                                   boolean performThresholdingBeforeSegmentation,
                                                   boolean performAdvancedSegmentation,
                                                   Color[] colors, 
                                                   int probeSizeX, int probeSizeY, 
                                                   int permutations, int numberOfProbingPixels,
                                                   int pixelRangeDeterminationType,
                                                   float pixelRangeMultiplier,
                                                   int minimumImageNumberToFocus,
                                                   int maximumImageNumberToFocus,
                                                   float nordicScalingFactor )
    {
        m_parent = parent;
        m_currentView = image;
        m_nordicScalingFactor = nordicScalingFactor;
        m_lowThreshold   = (int)( lowThreshold   / image.getNordicScaleFactor( ) );
        m_highThresshold = (int)( highThresshold / image.getNordicScaleFactor( ) );
        
        m_minROISize = minROISize; 
        m_maxROISize = maxROISize;

        m_minimumImageNumberToFocus = minimumImageNumberToFocus;
        m_maximumImageNumberToFocus = maximumImageNumberToFocus;

        if ( image.getRegionToSegment( ) != null )
        {
            m_segmentationWindow = image.getRegionToSegment( );
        }
        
        m_performThresholdingBeforeSegmentation = performThresholdingBeforeSegmentation;
        m_performAdvancedSegmentation = performAdvancedSegmentation;
        
        if ( m_performThresholdingBeforeSegmentation == false )
        {
            int w = m_currentView.getImageWidth( );
            int h = m_currentView.getImageHeight( );
            int ni = m_currentView.getNumberOfImages( );
            
            m_binaryPixelIntensityThressholdOverlay = new boolean[ni][h][w];
            
            for ( int i = 0; i < ni - 1; i++ )
            {
                for ( int k = 0; k < w - 1; k++ )
                {
                    for ( int l = 0; l < h - 1; l++ )
                    {
                        m_binaryPixelIntensityThressholdOverlay[i][k][l] = true;
                    }
                }
            }
        }
        m_segmentationColors = colors;
        
        m_probeSizeX = probeSizeX;
        m_probeSizeY = probeSizeY;
        m_permutations = permutations;
        m_numberOfProbingPixels = numberOfProbingPixels;
        
        m_zoom = m_currentView.getZoom( );
        m_imageWidth = m_currentView.getImageWidth( );
        m_imageHeight = m_currentView.getImageHeight( );
        m_numberOfImages = m_currentView.getNumberOfImages( );
        
        m_pixelRangeDeterminationType = pixelRangeDeterminationType;
        m_pixelRangeMultiplier = pixelRangeMultiplier;
    }

    
    /**
     * Segments the image in separate thread.
     */
    public void performSegmentation( )
    {
       
       final SwingWorker worker = new SwingWorker( )
       {
            public Object construct( )
            {
                try
                {
                    // segment the image
                    segmentImage( );   
                }
                catch ( OutOfMemoryError e )
                {
                    Dialogs d = new Dialogs( );
                    d.messageBox( m_parent, "<HTML> Too much ROI's are found to be handled <BR> Change pixel thressholds ranges & roi size ranges </HTML>" );
                }
                return null;
            }
                
            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                m_parent.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) ); 
                
                notifyActionListeners( PROBABILISTIC_SEGMENTATION_PERFORMED_EVENT );
            }
 
        };
        
        worker.start( );
    }
    
    /**
     * Computes and returns the binary pixel intensity overlay which
     * is the result of (a.) the user defined region of interest to segment
     * and (b.) the pixel intensity thresholding (min, max) pixel value.
     * @return 
     */ 
    public boolean[][][] computeBinaryPixelIntensityThressholdOverlay( )
    {
        
        m_binaryPixelIntensityThressholdOverlay = new boolean[m_numberOfImages][m_imageHeight][m_imageWidth];
	
        int[] pix = m_currentView.getImagePixelData( );
        
        if ( m_binaryPixelIntensityThressholdOverlay == null ) 
        {
            Dialogs.messageBox( m_parent, "Not enough memory for operation computeBinaryPixelIntensityThressholdOverlay( )" );
            return null;
        }
        
        int counter = 0;
        
        if ( m_segmentationWindow != null )
        {
            // The user defined a limited region to segment ..
            RegioPolygon segMentWin = new RegioPolygon( m_segmentationWindow );
            segMentWin.setScalingFactor( 1f / m_zoom );
            
            //for ( int i = 0; i < m_numberOfImages; i++ )
            for ( int i = m_minimumImageNumberToFocus; i < m_maximumImageNumberToFocus; i++ )
            {
                counter++;
                for ( int k = 0; k < m_imageHeight - 1; k++ ) 
                {
                    //int off = k * m_imageHeight;
                    int off = i * m_imageWidth * m_imageHeight + k * m_imageHeight;
                    for ( int l = 0; l < m_imageWidth - 1; l++ )
                    {
                        
                        int pixVal = pix[ off + l];
                        if ( ( segMentWin.contains( l, k )== true ) && ( pixVal >= m_lowThreshold && pixVal <= m_highThresshold ) )
                        {
                            m_binaryPixelIntensityThressholdOverlay[i][l][k] = true;
                        }
                        else
                        {
                            m_binaryPixelIntensityThressholdOverlay[i][l][k] = false;
                        }
                    }
                
                    m_progress = (int)( (float)counter * 100.0 / ((float)( m_numberOfImages ) ) );
                    notifyActionListeners( NOTIFY_PROGRESS_EVENT );
                }
            }
        }
        else
        {
            // The user wants to segment the whole image

            //for ( int i = 0; i < m_numberOfImages; i++ )
            for ( int i = m_minimumImageNumberToFocus; i < m_maximumImageNumberToFocus; i++ )
            {
                counter++;
                for ( int k = 0; k < m_imageHeight - 1; k++ ) 
                {
                    int off = i * m_imageWidth * m_imageHeight + k * m_imageHeight;
                    
                    for ( int l = 0; l < m_imageWidth - 1; l++ )
                    {
                        int pixVal = pix[off + l];
                        if ( ( pixVal >= m_lowThreshold && pixVal <= m_highThresshold ) )
                        {
                            m_binaryPixelIntensityThressholdOverlay[i][l][k] = true;
                        }
                        else
                        {
                            m_binaryPixelIntensityThressholdOverlay[i][l][k] = false;
                        }
                    }
                    m_progress = (int)( (float)counter * 100.0 / ( (float)( m_numberOfImages ) ) );
                    notifyActionListeners( NOTIFY_PROGRESS_EVENT );
                }
            }
        }
        
        return m_binaryPixelIntensityThressholdOverlay;
    }

     /**
      *  Computes the map with conflicting pixels for which an additional 
      *  decision must me made 
      */
    private void computeConflictingPixelMap( ) 
    {
     
        int[][][] multiplicityOverlay = new int[m_numberOfImages][m_imageHeight][m_imageWidth];
        
        m_conflictingClassificationPixelOverlay = new boolean[m_numberOfImages][m_imageHeight][m_imageWidth];
        
        // first assume that there are no conflicts
        for ( int k = 0; k < m_numberOfImages; k++ )
        {
            for ( int i = 0 ; i < m_imageHeight; i++ )
            {
                for ( int j = 0 ; j < m_imageWidth; j++ )
                {
                    m_conflictingClassificationPixelOverlay[k][i][j] = false;
                }
            }
        }
        
        //for ( int q = 0; q < m_numberOfImages; q++ )
        for ( int q = m_minimumImageNumberToFocus; q < m_maximumImageNumberToFocus; q++ )
        {
            LinkedList<boolean[][]> linkedListOfSliceNumber = m_imageSegmentsLinkedList.get( q );
            
            for ( int k = 0; k < linkedListOfSliceNumber.size( ); k++ ) 
            {
                boolean[][] map = linkedListOfSliceNumber.get( k );
            
                for ( int i = 0; i < m_imageHeight; i++ )
                {
                    for ( int j = 0; j < m_imageWidth ; j++ )
                    {
                        // compute only if pixel is within the user defined range
                        if ( m_segmentationWindow != null )
                        {
                            // The user defined a limited region to segment ..
                            RegioPolygon segMentWin = new RegioPolygon( m_segmentationWindow );
                            segMentWin.setScalingFactor( 1f / m_zoom );
                        
                            if ( segMentWin.contains( j, i ) == true )
                            {    
                                if  ( map[i][j]== true ) 
                                {
                                    multiplicityOverlay[q][i][j] = multiplicityOverlay[q][i][j] + 1;
                                }
                            }
                        }
                        else
                        {
                            // compute all pixels
                            if ( map[i][j] == true  )
                            {
                                multiplicityOverlay[q][i][j] = multiplicityOverlay[q][i][j] + 1;
                            }
                        }
                    }
                }
            }
        }
        
        int cntr = 0;
        
        // for all pixels with multiplicity > 1 there is a conflict!
        //for ( int q = 0; q < m_numberOfImages; q++ )
        for ( int q = m_minimumImageNumberToFocus; q < m_maximumImageNumberToFocus; q++ )
        {
            for ( int i = 0 ; i < m_imageHeight; i++ )
            {
                for ( int j = 0 ; j < m_imageWidth; j++ )
                {
                    if ( multiplicityOverlay[q][i][j] > 1 )
                    {
                        m_conflictingClassificationPixelOverlay[q][i][j] = true;
                    }
                }
            }
        }
    }
    
    /**
     * Does the actual segmentation: whole image.
     */
    private void segmentImage( )
    {
        if ( m_performThresholdingBeforeSegmentation == true )
        {
            m_binaryPixelIntensityThressholdOverlay = computeBinaryPixelIntensityThressholdOverlay( );
        }
        
        performSimpleProbabilisticImageSegmentation( );
        
        if ( m_performAdvancedSegmentation == true )
        {
            // Perform additional segmentation on resulting image part
            performAdvancedSegmentation( );
        }
    }
    
    /**
     * Performs simple probabilistic image segmentation.
     */
    public void performSimpleProbabilisticImageSegmentation( )
    {
        
        m_simpleProbCalc = new  SimpleProbabilisticImageSegmentationCalculator( m_binaryPixelIntensityThressholdOverlay,
                                                                                m_currentView,
                                                                                m_probeSizeX, m_probeSizeY, 
                                                                                m_permutations, m_numberOfProbingPixels,
                                                                                m_pixelRangeDeterminationType, m_pixelRangeMultiplier,
                                                                                m_minimumImageNumberToFocus, m_maximumImageNumberToFocus, 
                                                                                m_nordicScalingFactor );
        
        m_imageSegmentsLinkedList = new ArrayList< LinkedList< boolean[][] > >( );
        
        //m_imagePMapsLinkedList = new LinkedList<float[][]>( );
        //m_imageTMapsLinkedList = new LinkedList<float[][]>( );
        
        // first do the crude segmentation based on means and variances
        
        LinkedList<ImageParametersWithinARegioPolygon> allPolygonRegionInformation = m_currentView.getAllTissueTypeParametersAsLinkedList( );
        
        // use m_segmentationColors !!
        for ( int k = 0; k < m_numberOfImages; k++ )
        {
            LinkedList< boolean[][] > overlays = new LinkedList< boolean[][] >();
            
            // iterate over all tissue types
            for ( int i = 0; i < allPolygonRegionInformation.size( ); i++ )
            {
                // if the image number is in the focus range than compute
                if ( ( k >= m_minimumImageNumberToFocus ) && ( k <=m_maximumImageNumberToFocus ) )
                {
                    ImageParametersWithinARegioPolygon tissueInfo = allPolygonRegionInformation.get( i );
                
                    boolean[][] segment = m_simpleProbCalc.getSegmentedTissueMapWithSimpleClassifier( k, tissueInfo );
                
                    overlays.add( segment );
                }
                else
                {
                    // set default values for this case
                     boolean[][] segment = new boolean[m_imageWidth][m_imageHeight];
                     overlays.add( segment );
                }
                
            }

            m_imageSegmentsLinkedList.add( overlays );

        }
    }
    
    /**
     * 
     * Returns the computed overlays for each image and each tissue type
     * 
     * @return
     */
    public ArrayList< LinkedList< boolean[][] > > getProbabilisticSegmentedTissueOverLays( )
    {
        return m_imageSegmentsLinkedList;
    }
    
    
    /**
     *  This method handles all advanced b
     */
    private void performAdvancedSegmentation( )
    {
        // find out for which pixels there is a conflict
        
        computeConflictingPixelMap( );
        
        int tissueType;
        for ( int q = 0; q < m_numberOfImages; q++ )
        {
            for ( int i = 1; i < m_imageWidth - 1; i++ )
            {
                for ( int j = 1; j < m_imageHeight - 1; j++ )
                {
                    // pixel is classified to more than one pixel         
                    // find out what are x-es and what are the y-es??
                    
                    if ( m_conflictingClassificationPixelOverlay[q][i][j] == true )
                    {
                        tissueType = decideMostProbableTissueClass( i, j );
                        updateTissueImageList( q, i, j, tissueType );
                    }
                }
            }
        }
    }
    
    
    /**
     *  After a probabilistic decision has been made to which type the pixel 
     *  belongs, this method updates the lists setting the boolean true to 
     *  the appropriate tissue type to "true" and setting the others to "false" 
     */
    private void updateTissueImageList( int imageNumber, int Xpos, int Ypos, int tissueType )
    {       
        BufferedImage[] images = new BufferedImage[ m_imageSegmentsLinkedList.size( ) ];
        
        LinkedList<boolean[][]> segmentList = m_imageSegmentsLinkedList.get( imageNumber );
        
        for ( int i = 0 ; i < segmentList.size( ) ; i++ )
        {
            boolean[][] segment = segmentList.get( i );
            if ( i == tissueType )
            {
                segment[Xpos][Ypos] = true;
            }
            else
            {
                segment[Xpos][Ypos] = false;
            }
        }
    }
    
   
    /**
     * Determines to which tissue class this pixel belongs most likely
     */ 
     private int decideMostProbableTissueClass( int posX, int posY )
     {
         // we ask our oracle to return to use to which type pixel(i,j) belongs
         float[] testArr = m_simpleProbCalc.computeTestArray( posX, posY );
         
         float maximum = -Float.MAX_VALUE;
         int mostLikelyTissueType = -1;
         
         for ( int i = 0; i < m_currentView.getPolygonList( ).size( ); i++ )
         {
             float probability = m_simpleProbCalc.getProbabilityForTissueType( i, testArr );
             
             if ( probability > maximum )
             {
                 maximum = probability;
                 mostLikelyTissueType = i;
             }
         }
         
         return mostLikelyTissueType;
     }
    
    
    
    
    /**
     * Adds an actionlistener to the list of action listeners who are notified
     * when something happens with the spinbox
     */
    public void addActionListener( ActionListener e )
    {
        m_listeners.add( e );
    }
    
    /**
     * Notify all actionListeners
     */
    private void notifyActionListeners( int eventType )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
        }
    }
    
    /**
     * Returns the progress of the segmentation process
     */
    public int getProgress( )
    {
        return m_progress;
    }
    
    /**
     * Returns the boolean map indicating whether the pixel is inside or
     * outside the user defined pixel intensity range
     */
    public boolean[][][] getBinaryPixelIntensityThressholdOverlays( )
    {        
        return m_binaryPixelIntensityThressholdOverlay;
    }
    
    /**
    * Creates a buffered image of a boolean[][] array
    */ 
    public BufferedImage getBinaryPixelIntensityThressholdOverlayAsBufferedImage( int imageNumber, Color overlayColor, float alpha )
    {
        //int width = m_currentView.getImageWidth( );
        //int height = m_currentView.getImageHeight( );
        //float zoom = m_currentView.getZoom( );
        
        overlayColor = new Color( overlayColor.getRed( )/255f, overlayColor.getGreen( )/255f, overlayColor.getBlue( )/255f, alpha ); 
        
        BufferedImage image = new BufferedImage( (int)m_zoom * m_imageWidth, (int)m_zoom * m_imageHeight, BufferedImage.TYPE_INT_ARGB );
        
        Graphics g = image.getGraphics( );
        
        if ( m_binaryPixelIntensityThressholdOverlay != null )
        {
            // store current color
            
            g.setColor( overlayColor );
            
            for ( int i = 0; i < m_imageWidth ; i++ )
            {
                for ( int j = 0; j < m_imageHeight; j++ )
                {
                    if ( m_binaryPixelIntensityThressholdOverlay[imageNumber][i][j] == true )
                    {
                        g.fillRect( (int)(m_zoom * i), (int)(m_zoom * j), (int)m_zoom , (int)m_zoom );
                    }
                }
            } 
        }
        
        return image;
    }
    
    /**
    * Creates a buffered image of a boolean[][] array
    */ 
    public BufferedImage convertMapToBufferedImage( boolean[][] segment, Color overlayColor, float alpha )
    {
//        int width = m_currentView.getImageWidth( );
//        int height = m_currentView.getImageHeight( );
//        float zoom = m_currentView.getZoom( );
        
        overlayColor = new Color( overlayColor.getRed( )/255f, overlayColor.getGreen( )/255f, overlayColor.getBlue( )/255f, alpha ); 
        
        BufferedImage image = new BufferedImage( (int)m_zoom * m_imageWidth, (int)m_zoom * m_imageHeight, BufferedImage.TYPE_INT_ARGB );
        
        Graphics g = image.getGraphics( );
        
        if ( m_binaryPixelIntensityThressholdOverlay != null )
        {
            // store current color
            
            g.setColor( overlayColor );
            
            for ( int i = 0; i < m_imageWidth ; i++ )
            {
                for ( int j = 0; j < m_imageHeight; j++ )
                {
                    if ( segment[i][j] == true )
                    {
                        g.fillRect( (int)(m_zoom * i), (int)(m_zoom * j), (int)m_zoom , (int)m_zoom );
                    }
                }
            } 
        }
        
        return image;
    }
        
    /**
     * Returns the number of tissues which have been segmented by the algorithm
     */
    public int getNumberOfDifferentTissues( )
    {
        return m_currentView.getPolygonList( ).size( );
    }
    
    /**
     * Returns the composite image multi color overlay image of all selected tissue types
     * in the GUI which were classified with the simple algorithm
     */ 
    public BufferedImage getCompositeImageOfClassifiedTissueTypes( int imageNumber, Color[] colors, int[] selectedTissues, float alpha )
    { 
                
        Color[][] colormap = new Color[m_imageHeight][m_imageWidth];
        
        LinkedList< boolean[][] > imageSegmentsOfSliceNumber =  m_imageSegmentsLinkedList.get( imageNumber );
        
        for ( int k = 0 ; k < selectedTissues.length ; k++ )
        {
            boolean[][] imSegment = imageSegmentsOfSliceNumber.get( selectedTissues[k] );

            for ( int i = 0; i < m_imageWidth ; i++ )
            {
                for ( int j = 0; j < m_imageHeight; j++ )
                { 
                    if ( m_binaryPixelIntensityThressholdOverlay[imageNumber][i][j] == false )
                    {
                        colormap[i][j] = null;
                    }
                    else
                    {
                        float maxTValue = -Float.MAX_VALUE;
                        
                        // if there are multiple possibilities the last tissue 
                        // type gets the color!    
                        if ( imSegment[i][j] == true )
                        {
                            colormap[i][j] = colors[ selectedTissues[k] ];
                        }
                    }
                }
            }
        }
        
        // now draw the image ofline
        BufferedImage image = new BufferedImage( (int)m_zoom * m_imageWidth, (int)m_zoom * m_imageHeight, BufferedImage.TYPE_INT_ARGB );
        
        Graphics g = image.getGraphics( );
        for ( int i = 0; i < m_imageWidth ; i++ )
        {
            for ( int j = 0; j < m_imageHeight; j++ )
            {
                if ( colormap[i][j] != null )
                {
                    Color col = new Color( colormap[i][j].getRed()/255, colormap[i][j].getGreen()/255f, colormap[i][j].getBlue()/255f, alpha );
                    g.setColor( col );
                    g.fillRect( (int)(m_zoom * i), (int)(m_zoom * j), (int)m_zoom , (int)m_zoom );
                }
            }
        }
        // and return it
        return image;
    }
    
    // Attributes of the class
    private JFrame m_parent;

    private DicomSliceCanvasPolyDraw m_currentView;

    private int m_lowThreshold;

    private int m_highThresshold;

    private int m_minROISize;
    
    private int m_maxROISize;

    private RegioPolygon m_segmentationWindow;
    
    // Listeners to changes in the segmentation gui control
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
    
    public final static int PROBABILISTIC_SEGMENTATION_PERFORMED_EVENT = 0;
    public final static int NOTIFY_PROGRESS_EVENT = 1;
    
    // indicates the progress of the segmentation process
    private int m_progress = 0;
    
    // overlay is computed for the whole image stack
    private boolean[][][] m_binaryPixelIntensityThressholdOverlay;
    
    // Datastructure that indicate which pixels classified to multiple classes
    private boolean[][][] m_conflictingClassificationPixelOverlay; 
    
    private boolean m_performThresholdingBeforeSegmentation;
    
    private Color[] m_segmentationColors;
    
    private SimpleProbabilisticImageSegmentationCalculator m_simpleProbCalc;  
    
    private int m_probeSizeX;
    
    private int m_probeSizeY; 
    
    private ArrayList< LinkedList< boolean[][] > > m_imageSegmentsLinkedList;
    private LinkedList< float[][] > m_imagePMapsLinkedList;
    private LinkedList< float[][] > m_imageTMapsLinkedList;
    
    private boolean m_performAdvancedSegmentation;
    
    private int m_numberOfProbingPixels;

    private int m_permutations;
    
    private float m_zoom;
    private int	  m_imageWidth;
    private int   m_imageHeight;
    private int   m_numberOfImages;
    
    private int   m_pixelRangeDeterminationType;
    private float m_pixelRangeMultiplier;

    private int m_minimumImageNumberToFocus;
    private int m_maximumImageNumberToFocus;
    private float m_nordicScalingFactor;

}