/*
 * File: $Id: OrderStatisticSegmentationCalculator.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 01.03.2010  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2010 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.imageprocessing.orderstatisticsegmentation;

import org.spectrim.dicom.DicomSliceCanvasPolyDraw;
import org.spectrim.imagegraphicobjects.RegioPolygon;
import org.spectrim.imageprocessing.ImageParametersWithinARegioPolygon;
import org.spectrim.imageprocessing.MorphologicFilters;
import org.spectrim.imageprocessing.QuickSort;
import org.spectrim.lib.Dialogs;
import org.spectrim.lib.SwingWorker;
import org.spectrim.signalprocessing.IntegerSignalHistogramCalculator;
import org.spectrim.signalprocessing.RankSpectrumHistogramCalculator;
import org.spectrim.xyplot.Curve;
import org.spectrim.xyplot.XYPlot;
import org.spectrim.xyplot.XYPlotFrame;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;
import javax.swing.JFrame;

/**
 * 
 * Class handling segmentation of grayscale images by order statistics
 * filtering of pixel region.
 * 
 * @author hans
 */
public class OrderStatisticSegmentationCalculator 
{

    /**
     * User instantiates a calculator class with a existing 
     * DicomSliceCanvasPolydraw object 
     * @param parent
     * @param canvas
     * @param pixelRegionSize
     * @param performSimplePixelValueThresholdingBeforeSegmentation
     * @param performOrderStatisticROIThressholdingBeforeSegmentation
     * @param simpleThressholdingMinimumValue
     * @param highThresshold
     * @param performMorphologicalFiltering
     * @param segmentationColors
     * @param probeSizeX
     * @param probeSizeY
     * @param numberOfProbingPixels
     * @param permutations
     * @param pixelRangeComputationType
     * @param pixelRangeMultiplier
     * @param osfSortingType
     * @param nordicScalingFactor
     */
    public OrderStatisticSegmentationCalculator( JFrame parent, DicomSliceCanvasPolyDraw canvas,
                                                 int pixelRegionSize,
                                                 boolean performSimplePixelValueThresholdingBeforeSegmentation,
                                                 boolean performOrderStatisticROIThressholdingBeforeSegmentation,
                                                 int simpleThressholdingMinimumValue,
                                                 int highThresshold,
                                                 boolean performMorphologicalFiltering,
                                                 Color[] segmentationColors,
                                                 int probeSizeX, int probeSizeY,
                                                 int permutations, int numberOfProbingPixels,
                                                 int pixelRangeComputationType,
                                                 int pixelRangeMultiplier,
                                                 int osfSortingType,
                                                 float nordicScalingFactor
                                               )
    {

        m_parent = parent;
        m_nordicScalingFactor = nordicScalingFactor;
        m_dicomCanvas = canvas;
        m_zoom = m_dicomCanvas.getZoom( );
        m_segmentationColors = segmentationColors;

        if ( m_dicomCanvas.getRegionToSegment( ) != null )
        {
            m_segmentationWindow = m_dicomCanvas.getRegionToSegment( );
        }

        m_performPixelValueThresholdingBeforeSegmentation = performSimplePixelValueThresholdingBeforeSegmentation;
        m_performOrderStatisticROIThressholdingSegmentation = performOrderStatisticROIThressholdingBeforeSegmentation;
        setPixelRegionSize( pixelRegionSize );
        
        m_binaryPixelIntensityThressholdOverlay = new boolean[m_numberOfImages][m_imageWidth][m_imageHeight];
          
        m_simpleThressholdingMinimumValue = simpleThressholdingMinimumValue;
        m_simpleThressholdingMaximumValue = highThresshold;

        if ( m_performPixelValueThresholdingBeforeSegmentation == false )
        {
            for ( int i = 0; i < m_numberOfImages; i++ )
            {
                for ( int k = 0; k < m_imageWidth; k++ )
                {
                    for ( int l = 0; l < m_imageHeight; l++ )
                    {
                        m_binaryPixelIntensityThressholdOverlay[i][k][l] = true;
                    }
                }
            }
        }

        if ( m_performOrderStatisticROIThressholdingSegmentation == false )
        {
            for ( int i = 0; i < m_numberOfImages; i++ )
            {
                for ( int k = 0; k < m_imageWidth; k++ )
                {
                    for ( int l = 0; l < m_imageHeight; l++ )
                    {
                        m_binaryPixelIntensityOSFThressholdOverlay[i][k][l] = true;
                    }
                }
            }
        }

        m_probeSizeX = probeSizeX;
        m_probeSizeY = probeSizeY;

        m_numberOfProbingPixels = numberOfProbingPixels;

        m_pixelRangeComputationType = pixelRangeComputationType;
        m_pixelRangeMultiplier = pixelRangeMultiplier;

        m_pixelRangeComputationType = pixelRangeComputationType;

        m_permutations = permutations;

        m_performMorphologicalFiltering = performMorphologicalFiltering;

        m_osfSortingType = osfSortingType;
        
        prepareImagePixelData( );

    }

    /**
     * sets the values of this calculator according the GUI setting values
     * @param pixelRegionSize
     * @param performSimplePixelValueThresholdingBeforeSegmentation
     * @param performOrderStatisticROIThressholdingBeforeSegmentation
     * @param simplePixelThressholdLowValue
     * @param simplePixelThressholdHighValue
     * @param OSFThressholdingMinimumValue
     * @param OSFThressholdingMaximumValue
     * @param performMorphologicalFiltering
     * @param osfSortingType
     */
    public void setUserDefinedOSFSegmentationConditions( int pixelRegionSize,
                                                         boolean performSimplePixelValueThresholdingBeforeSegmentation,
                                                         int simplePixelThressholdLowValue,
                                                         int simplePixelThressholdHighValue,
                                                         boolean performOrderStatisticROIThressholdingBeforeSegmentation,
                                                         int OSFThressholdingMinimumValue,
                                                         int OSFThressholdingMaximumValue,
                                                         boolean performMorphologicalFiltering,
                                                         int osfSortingType
                                                        )
    {

        if ( pixelRegionSize != m_regionSize )
        {
            System.out.println( "Not supported yet, can only perform OSFSEG on 3x3 regions!!" );
        }

        // can be changed and influences the order of computation
        m_performPixelValueThresholdingBeforeSegmentation = performSimplePixelValueThresholdingBeforeSegmentation;      
        
        m_simpleThressholdingMinimumValue = simplePixelThressholdLowValue;
        m_simpleThressholdingMaximumValue = simplePixelThressholdHighValue;

        // perform further OSF thressholding
        m_performOrderStatisticROIThressholdingSegmentation = performOrderStatisticROIThressholdingBeforeSegmentation;

        m_OSFThressholdingMinimumValue = OSFThressholdingMinimumValue;
        m_OSFThressholdingMaximumValue = OSFThressholdingMaximumValue;
        
        // has no function up till now!
        m_performMorphologicalFiltering = performMorphologicalFiltering;

        m_osfSortingType = osfSortingType;

        prepareImagePixelData( );
    }


    /**
     * Does the actual segmentation: whole image.
     */
    private void segmentImage( )
    {
        notifyActionListeners( START_RECALCULATION_OSF_SEGMENTATION_EVENT );

        if ( m_performPixelValueThresholdingBeforeSegmentation == true )
        {
            m_binaryPixelIntensityThressholdOverlay = computeBinaryPixelIntensityThressholdOverlay( );
            
            
            if ( this.m_performMorphologicalFiltering == true )
            {
                for ( int i = 0; i < m_binaryPixelIntensityThressholdOverlay.length; i++ )
                {
                    boolean[][] overlay = m_binaryPixelIntensityThressholdOverlay[i];
                    
//                    Morphologic1DKernalOperator op_hori = new Morphologic1DKernalOperator( Morphologic1DKernalOperator.HORIZONTAL_OPERATOR );
//                    Morphologic1DKernalOperator op_vert = new Morphologic1DKernalOperator( Morphologic1DKernalOperator.VERTICAL_OPERATOR );
                    
                    MorphologicFilters mofi = new MorphologicFilters( );
                    
                    overlay = mofi.erode( overlay, m_imageWidth, m_imageHeight );
                    overlay = mofi.erode( overlay, m_imageWidth, m_imageHeight );
                    overlay = mofi.erode( overlay, m_imageWidth, m_imageHeight );
                    overlay = mofi.erode( overlay, m_imageWidth, m_imageHeight );
                    
                    overlay = mofi.dilate( overlay, m_imageWidth, m_imageHeight );
                    overlay = mofi.dilate( overlay, m_imageWidth, m_imageHeight );
                    overlay = mofi.dilate( overlay, m_imageWidth, m_imageHeight );
                    overlay = mofi.dilate( overlay, m_imageWidth, m_imageHeight );
                    
                    m_binaryPixelIntensityThressholdOverlay[i] = overlay;
                }
            }
        }

        if ( m_performOrderStatisticROIThressholdingSegmentation == true )
        {

            if ( m_binaryPixelIntensityOSFThressholdOverlay == null )
            {
                m_binaryPixelIntensityOSFThressholdOverlay = new boolean[m_numberOfImages][m_imageWidth][m_imageHeight];

                // Initialize overlay as false!
                for ( int i = 0; i < m_numberOfImages; i++ )
                {
                    for ( int k = 0; k < m_imageHeight ; k++ )
                    {
                        for ( int l = 0; l < m_imageWidth; l++ )
                        {
                            m_binaryPixelIntensityOSFThressholdOverlay[i][l][k] = false;
                        }
                    }
                }
                m_binaryPixelIntensityOSFThressholdOverlay = performOSFImageSegmentation( );
            }
            else
            {
                m_binaryPixelIntensityOSFThressholdOverlay = performOSFImageSegmentation( );
            }
        }

        // Compute the final overlay 
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            for ( int k = 0; k < m_imageHeight ; k++ )
            {
                for ( int l = 0; l < m_imageWidth; l++ )
                {
                    m_binaryPixelIntensityOSFThressholdOverlay[i][l][k] = m_binaryPixelIntensityOSFThressholdOverlay[i][l][k] && m_binaryPixelIntensityThressholdOverlay[i][l][k];
                }
            }
        }

        
//        for ( int i = 0; i < m_binaryPixelIntensityOSFThressholdOverlay.length; i++ )
//                {
//                    boolean[][] overlay = m_binaryPixelIntensityOSFThressholdOverlay[i];
//                    
////                    Morphologic1DKernalOperator op_hori = new Morphologic1DKernalOperator( Morphologic1DKernalOperator.HORIZONTAL_OPERATOR );
////                    Morphologic1DKernalOperator op_vert = new Morphologic1DKernalOperator( Morphologic1DKernalOperator.VERTICAL_OPERATOR );
//                    
//                    MorphologicFilters mofi = new MorphologicFilters( );
//                    
//                    overlay = mofi.erode( overlay, m_imageWidth, m_imageHeight );
//                    overlay = mofi.erode( overlay, m_imageWidth, m_imageHeight );
////                    overlay = mofi.erode( overlay, m_imageWidth, m_imageHeight );
////                    overlay = mofi.erode( overlay, m_imageWidth, m_imageHeight );
//                    
//                    overlay = mofi.dilate( overlay, m_imageWidth, m_imageHeight );
//                    overlay = mofi.dilate( overlay, m_imageWidth, m_imageHeight );
////                    overlay = mofi.dilate( overlay, m_imageWidth, m_imageHeight );
////                    overlay = mofi.dilate( overlay, m_imageWidth, m_imageHeight );
//                    
//                    m_binaryPixelIntensityOSFThressholdOverlay[i] = overlay;
//                }
        
        
        
//
//        if ( m_performAdvancedSegmentation == true )
//        {
//            // Perform additional segmentation on resulting image part
//            performAdvancedSegmentation( );
//        }
    }


     /**
     * Performs simple probabilistic image segmentation.
     * @return 
     */
    public boolean[][][] performOSFImageSegmentation( )
    {

        int counter = 0;

        // The user wants to segment the whole image stack
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            counter++;
            for ( int k = 0; k < m_imageHeight; k++ )
            {
                int off = i * m_imageWidth * m_imageHeight + k * m_imageHeight;

                for ( int l = 0; l < m_imageWidth; l++ )
                {
                    // only do something if pixel may be considered, and was not removed

                    if ( ( (  off + l ) >= (int)( ( 1.0 * m_OSFThressholdingMinimumValue ) * m_doubleOrderedIndexArray.length / ( 10000.0 ) ) ) &&
                         ( (  off + l ) <= (int)( ( 1.0 * m_OSFThressholdingMaximumValue ) * m_doubleOrderedIndexArray.length / ( 10000.0 ) ) ) )
                    {
                        // compute which original pixel this was!
                        int index = m_doubleOrderedIndexArray[ off + l ];

                        int imaNum = index / ( m_imageHeight * m_imageWidth );
                        int jj = ( index % ( m_imageHeight * m_imageWidth ) ) / m_imageWidth;
                        int kk = ( index % ( m_imageHeight * m_imageWidth ) ) % m_imageWidth;

                        // also the OSF criterium is fullfilled!
                        m_binaryPixelIntensityOSFThressholdOverlay[imaNum][kk][jj] = true;
                    }
                    else
                    {
                        // also the OSF criterium is NOT fullfilled, so exclude the pixel
                        // from the overlay!
                        int index = m_doubleOrderedIndexArray[ off + l ];

                        int imaNum = index / ( m_imageHeight * m_imageWidth );
                        int jj = ( index % ( m_imageHeight * m_imageWidth ) ) / m_imageWidth;
                        int kk = ( index % ( m_imageHeight * m_imageWidth ) ) % m_imageWidth;

                        m_binaryPixelIntensityOSFThressholdOverlay[imaNum][kk][jj] = false;
                    }

                }

                m_progress = (int)( (float)counter * 100.0 / ( (float)( m_numberOfImages ) ) );

            }
        }
        
        // before returning to the caller code, compute the lines of symmetry
        // valid for the OSF sub band setting
        computeLinearRegressions( m_binaryPixelIntensityThressholdOverlay );
        computeEntropies( );
        
        // now return
        return m_binaryPixelIntensityOSFThressholdOverlay;
    }
   
    /**
     * Return the slopes of the lines of symmetry belonging to a specific
     * OSF-sub band.
     * @return 
     */
    public double[] getSlopes( )
    {
        return m_slope;
    }
    
    /**
     * Return the slopes of the lines of symmetry belonging to a specific
     * OSF-sub band.
     * @return 
     */
    public double[] getIntercepts( )
    {
        return m_intercept;
    }
    
    
    /**
     *  Try to find the symmetry axis for the sub order statistic bands.
     * @param OSFThressholdOverlay
     */
    public void computeLinearRegressions( boolean[][][] OSFThressholdOverlay )
    {
        m_slope = new double[m_numberOfImages];
        m_intercept = new double[m_numberOfImages];
        
        m_leftOverlay = new boolean[m_numberOfImages][m_imageWidth][m_imageHeight];
        m_rightOverlay = new boolean[m_numberOfImages][m_imageWidth][m_imageHeight];
        
        // The user wants to segment the whole image stack
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            double sum_x = 0.0;
            double sum_x2 = 0.0;
            double sum_y = 0.0;
            double sum_xy = 0.0;

            double[] x = new double[m_imageWidth * m_imageHeight * m_numberOfImages];
            double[] y = new double[m_imageWidth * m_imageHeight * m_numberOfImages];

            int counter = 0;
            
            // first determine the mean in x and y direction
            for ( int k = 0; k < m_imageHeight; k++ )
            {
                int off = i * m_imageWidth * m_imageHeight + k * m_imageHeight;

                for ( int l = 0; l < m_imageWidth; l++ )
                {
                    if ( OSFThressholdOverlay[i][l][k] )
                    {
                        x[counter] = (double)k;
                        y[counter] = (double)l;
                        sum_x += k;
                        sum_y += l;
                        sum_x2 += k * k;
                        counter++;
                    }
                }
            }
            
            XYPlot plt = new XYPlot( "Point for regression", null );
            
            plt.addCurve( new Curve( x, y, Color.BLUE ), true );
            plt.setVisible(true);
            
            if ( counter > 0 )
            {
                double x_bar = sum_x / counter;
                double y_bar = sum_y / counter;
                double xx_bar = 0; double yy_bar = 0; double xy_bar = 0;
                for ( int cntr=0; cntr < counter; cntr++)
                {
                    xx_bar += (x[cntr] - x_bar ) * (x[cntr] - x_bar );
                    yy_bar += (y[cntr] - y_bar ) * (y[cntr] - y_bar );
                    xy_bar += (x[cntr] - x_bar ) * (y[cntr] - y_bar );
                }
                
                m_slope[i] = xy_bar/xx_bar;
                m_intercept[i] = y_bar - m_slope[i] * x_bar;
                
            }
            
            System.out.println( "image : " + i + " -- slope : " + m_slope[i] + " -- intercept : " + m_intercept[i] + " -- counter : " + counter );
            
        }
    }
    
    
    /**
     * Compute the entropies.
     */
    public void computeEntropies( )
    {
        
        int counterleft = 0;
        int counterright = 0;
        
        m_leftOverlay = new boolean[m_numberOfImages][m_imageWidth][m_imageHeight];
        m_rightOverlay = new boolean[m_numberOfImages][m_imageWidth][m_imageHeight];
        
        for ( int i = 0 ; i < m_numberOfImages; i++ )
        {
            
            boolean[][] OSFOverlay = m_binaryPixelIntensityOSFThressholdOverlay[i];
            
            for ( int ii = 0; ii < m_imageHeight; ii++)
            {
                for ( int jj = 0; jj < m_imageWidth; jj++)
                {
                    if ( OSFOverlay[ii][jj] == true )
                    {
                        if ( ii - m_intercept[i] - m_slope[i]*jj > 0  )
                        {
                            counterleft++;
                            m_leftOverlay[i][ii][jj]=true;
                            m_rightOverlay[i][ii][jj]=false;
                        }
                        else
                        {
                            m_rightOverlay[i][ii][jj]=true;
                            m_leftOverlay[i][ii][jj]=false;
                            counterright++;
                        }
                    }
                }
            }
            
            System.out.println( "Pixels Left  = " + counterleft );
            System.out.println( "Pixels Right = " + counterright );
            
            double probability1 = Math.abs((double)(counterleft) / (double)( counterleft + counterright ));
            double probability2 = Math.abs((double)(counterright) / (double)( counterleft + counterright ));
            double entropy = 0;
            
            if ( counterleft != 0 )
            {
                entropy = - probability1 * Math.log( probability1 )/Math.log(2.0) - probability2 * Math.log( probability2 )/Math.log(2.0);
            }
            System.out.println( "Probability1 = " + probability1 );
            System.out.println( "Probability2 = " + probability2 );
            System.out.println( "Entropy = " + entropy );
            
            m_binaryPixelIntensityOSFThressholdOverlay[i] = OSFOverlay;
            
        }
    }
    
    /**
     * Returns the computed pixels positions on the right side of the symmetry axis
     * @return 
     */
    public boolean[][][] getRightOverlay( )
    {
        return m_rightOverlay;
    }
    
    /**
     * Returns the computed pixels positions on the right side of the symmetry axis
     * @return 
     */
    public boolean[][][] getLeftOverlay( )
    {
        return m_leftOverlay;
    }
    
    
    /**
     * Segments the image in separate thread.
     */
    public void initializeOSFFilter( )
    {

       final SwingWorker worker = new SwingWorker( )
       {
            @Override
            public Object construct( )
            {
                try
                {
                    // segment the image
                    computeOrderedDataStructures( );
                }
                catch ( OutOfMemoryError e )
                {
                    Dialogs.messageBox( m_parent, "<HTML> Out of memory in initializeOSFFilter </HTML>" );
                }
                return null;
            }

            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                m_parent.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );

                m_progress = 0;
                notifyActionListeners( NOTIFY_OSF_PROGRESS_EVENT );
                notifyActionListeners( OSF_INITIALIZATION_READY_EVENT );
                m_isInitialized = true;
            }

        };

        worker.start( );
    }

    /**
     * Segments the image in separate thread.
     */
    public void performSegmentation( )
    {

       final SwingWorker worker = new SwingWorker( )
       {
            @Override
            public Object construct( )
            {
                try
                {
                    // segment the image
                    segmentImage( );
                }
                catch ( OutOfMemoryError e )
                {
                    Dialogs.messageBox( m_parent, "<HTML> Too much ROI's are found to be handled <BR> Change pixel thressholds ranges & roi size ranges </HTML>" );
                }
                return null;
            }

            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                m_parent.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );

                notifyActionListeners( OSF_SEGMENTATION_PERFORMED_EVENT );
            }

        };

        worker.start( );
    }


    /**
     * Computes and returns the binary pixel intensity overlay which
     * is the result of (a.) the user defined region of interest to segment
     * and (b.) the pixel intensity thresholding (min, max) pixel value.
     * @return 
     */
    public boolean[][][] computeBinaryPixelIntensityThressholdOverlay( )
    {

        m_binaryPixelIntensityThressholdOverlay = new boolean[m_numberOfImages][m_imageHeight][m_imageWidth];

        int[] pix = m_dicomCanvas.getImagePixelData( );

        if ( m_binaryPixelIntensityThressholdOverlay == null )
        {
            Dialogs.messageBox( m_parent, "Not enough memory for operation computeBinaryPixelIntensityThressholdOverlay( )" );
            return null;
        }

        int counter = 0;

        if ( m_segmentationWindow != null )
        {
            // The user defined a limited region to segment ..
            RegioPolygon segMentWin = new RegioPolygon( m_segmentationWindow );
            segMentWin.setScalingFactor( 1f / m_zoom );
            for ( int i = 0; i < m_numberOfImages; i++ )
            {
                counter++;
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    //int off = k * m_imageHeight;
                    int off = i * m_imageWidth * m_imageHeight + k * m_imageHeight;
                    for ( int l = 0; l < m_imageWidth; l++ )
                    {

                        int pixVal = pix[ off + l];
                        if ( ( segMentWin.contains( l, k ) == true ) && ( pixVal >= m_simpleThressholdingMinimumValue && pixVal <= m_simpleThressholdingMaximumValue ) )
                        {
                            m_binaryPixelIntensityThressholdOverlay[i][l][k] = true;
                        }
                        else
                        {
                            m_binaryPixelIntensityThressholdOverlay[i][l][k] = false;
                        }
                    }

                    m_progress = (int)( (float)counter * 100.0 / ((float)( m_numberOfImages ) ) );

                    notifyActionListeners( NOTIFY_OSF_PROGRESS_EVENT );
                }
            }
        }
        else
        {
            // The user wants to segment the whole image
            for ( int i = 0; i < m_numberOfImages; i++ )
            {
                counter++;
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    int off = i * m_imageWidth * m_imageHeight + k * m_imageHeight;

                    for ( int l = 0; l < m_imageWidth; l++ )
                    {
                        int pixVal = pix[off + l];
                        if ( ( pixVal >= m_simpleThressholdingMinimumValue && pixVal <= m_simpleThressholdingMaximumValue ) )
                        {
                            m_binaryPixelIntensityThressholdOverlay[i][l][k] = true;
                        }
                        else
                        {
                            m_binaryPixelIntensityThressholdOverlay[i][l][k] = false;
                        }
                    }
                    m_progress = (int)( (float)counter * 100.0 / ((float)( m_numberOfImages ) ) );
                    notifyActionListeners( NOTIFY_OSF_PROGRESS_EVENT );
                }
            }
        }

        return m_binaryPixelIntensityThressholdOverlay;
    }

    /**
     * Computes the norm and the phases of all images.
     */
    private void computeNormsAndAnglesForAllImages( )
    {
        
        m_allNorms = new float[m_numberOfImages][m_imageWidth][m_imageHeight];
        m_allAnglesWithIdentityVector = new float[m_numberOfImages][m_imageWidth][m_imageHeight];
        
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            for ( int j = 0; j < m_imageWidth; j++ )
            {
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    m_allNorms[i][j][k] = computeNorm( i, j, k );
                    m_allAnglesWithIdentityVector[i][j][k] = computeAngleWithIdentityVector( i, j, k );
                }
            }
        }
    }
    
    /**
     *  Computes the data structures necessary for 4 point metric.
     * 
     * @param m_orderedImageData
     */
    private void compute4PointMetric( ) 
    {
        int[] array = new int[m_regionSize+1];
        
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            for ( int j = 1; j < m_imageWidth - 1; j++ )
            {
                for ( int k = 1; k < m_imageHeight - 1; k++ )
                {
                    // define non sorted array
                    array[0] = m_imageStackPixelData[i][j][k-1];
                    array[1] = m_imageStackPixelData[i][j-1][k];
                    array[2] = m_imageStackPixelData[i][j][k];
                    array[3] = m_imageStackPixelData[i][j+1][k];
                    array[4] = m_imageStackPixelData[i][j][k+1];   
                
                    // sort array
                    array = QuickSort.quickSort( array );
                    
                    // define sorted data structure
                    for ( int m = 0; m < m_regionSize + 1; m++ )
                    {
                        m_orderedImageData[m][i][j][k] = array[m];  
                    }
                }
            }
        }
    }
    
    /**
     *  Computes the data structures necessary for an 8 point metric.
     * 
     * @param m_orderedImageData
     */
    private void compute8PointMetric( ) 
    {
        int[] array = new int[m_regionSize+1];

        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            for ( int j = 0; j < m_imageWidth; j++ )
            {
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    array[0] = m_imageStackPixelData[i][( j - 1 + m_imageWidth) % m_imageWidth][( k - 1 + m_imageHeight ) % m_imageHeight ];
                    array[1] = m_imageStackPixelData[i][( j - 1 + m_imageWidth) % m_imageWidth][ ( k + m_imageHeight ) % m_imageHeight ];
                    array[2] = m_imageStackPixelData[i][( j - 1 + m_imageWidth) % m_imageWidth ][ ( k + 1 + m_imageHeight ) % m_imageHeight];

                    array[3] = m_imageStackPixelData[i][( j + m_imageWidth) % m_imageWidth ][( k - 1 + m_imageHeight ) % m_imageHeight];
                    array[4] = m_imageStackPixelData[i][( j + m_imageWidth) % m_imageWidth][( k + m_imageHeight ) % m_imageHeight];
                    array[5] = m_imageStackPixelData[i][( j + m_imageWidth) % m_imageWidth][( k + 1 + m_imageHeight ) % m_imageHeight];

                    array[6] = m_imageStackPixelData[i][( j + 1 + m_imageWidth) % m_imageWidth][ ( k - 1 + m_imageHeight ) % m_imageHeight];
                    array[7] = m_imageStackPixelData[i][( j + 1 + m_imageWidth) % m_imageWidth][ ( k + m_imageHeight ) % m_imageHeight];
                    array[8] = m_imageStackPixelData[i][( j + 1 + m_imageWidth) % m_imageWidth][( k + 1 + m_imageHeight ) % m_imageHeight];

                    // sort array
                    array = QuickSort.quickSort( array );
                    
                    // define sorted data structure
                    for ( int m = 0; m < m_regionSize + 1; m++ )
                    {
                        m_orderedImageData[m][i][j][k] = array[m];  
                    }
                }
            }

            m_progress = (int)( (float)i * 100.0 / ((float)( m_numberOfImages ) ) );
            notifyActionListeners( NOTIFY_OSF_PROGRESS_EVENT_PASS_2 );

        }
    }

    
    /**
     *  Computes the data structures necessary for 8 point metric.
     *
     * @param m_orderedImageData
     */
    private void compute8PointMetricAsString( )
    {
       
        String st0 = "";
        String st1 = "";
        String st2 = "";
        String st3 = "";
        String st4 = "";
        String st5 = "";
        String st6 = "";
        String st7 = "";
        String st8 = "";

        long start = 0;
        long stop  = 0;

        m_doubleOrderedData = new String[m_numberOfImages*m_imageWidth*m_imageHeight];

        m_doubleOrderedIndexArray = new int[m_numberOfImages*m_imageWidth*m_imageHeight];

        int cntr = 0;

        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            for ( int j = 0; j < m_imageWidth; j++ )
            {
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    if ( ( m_orderedImageData[0][i][j][k] ) >= 0 && ( m_orderedImageData[0][i][j][k] < 10 ) )
                    {
                        st0 = "000" + m_orderedImageData[0][i][j][k];
                    }
                    if ( ( m_orderedImageData[0][i][j][k] ) >= 10 && ( m_orderedImageData[0][i][j][k] < 100 ) )
                    {
                        st0 = "00" + m_orderedImageData[0][i][j][k];
                    }
                    if ( ( m_orderedImageData[0][i][j][k] ) >= 100 && ( m_orderedImageData[0][i][j][k] < 1000 ) )
                    {
                        st0 = "0" + m_orderedImageData[0][i][j][k];
                    }
                    if ( ( m_orderedImageData[0][i][j][k] ) >= 1000 && ( m_orderedImageData[0][i][j][k] < 10000 ) )
                    {
                        st0 = "" + m_orderedImageData[0][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[1][i][j][k] ) >= 0 && ( m_orderedImageData[1][i][j][k] < 10 ) )
                    {
                        st1 = "000" + m_orderedImageData[1][i][j][k];
                    }
                    if ( ( m_orderedImageData[1][i][j][k] ) >= 10 && ( m_orderedImageData[1][i][j][k] < 100 ) )
                    {
                        st1 = "00" + m_orderedImageData[1][i][j][k];
                    }
                    if ( ( m_orderedImageData[1][i][j][k] ) >= 100 && ( m_orderedImageData[1][i][j][k] < 1000 ) )
                    {
                        st1 = "0" + m_orderedImageData[1][i][j][k];
                    }
                    if ( ( m_orderedImageData[1][i][j][k] ) >= 1000 && ( m_orderedImageData[1][i][j][k] < 10000 ) )
                    {
                        st1 = "" + m_orderedImageData[1][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[2][i][j][k] ) >= 0 && ( m_orderedImageData[2][i][j][k] < 10 ) )
                    {
                        st2 = "000" + m_orderedImageData[2][i][j][k];
                    }
                    if ( ( m_orderedImageData[2][i][j][k] ) >= 10 && ( m_orderedImageData[2][i][j][k] < 100 ) )
                    {
                        st2 = "00" + m_orderedImageData[2][i][j][k];
                    }
                    if ( ( m_orderedImageData[2][i][j][k] ) >= 100 && ( m_orderedImageData[2][i][j][k] < 1000 ) )
                    {
                        st2 = "0" + m_orderedImageData[2][i][j][k];
                    }
                    if ( ( m_orderedImageData[2][i][j][k] ) >= 1000 && ( m_orderedImageData[2][i][j][k] < 10000 ) )
                    {
                        st2 = "" + m_orderedImageData[2][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[3][i][j][k] ) >= 0 && ( m_orderedImageData[3][i][j][k] < 10 ) )
                    {
                        st3 = "000" + m_orderedImageData[3][i][j][k];
                    }
                    if ( ( m_orderedImageData[3][i][j][k] ) >= 10 && ( m_orderedImageData[3][i][j][k] < 100 ) )
                    {
                        st3 = "00" + m_orderedImageData[3][i][j][k];
                    }
                    if ( ( m_orderedImageData[3][i][j][k] ) >= 100 && ( m_orderedImageData[3][i][j][k] < 1000 ) )
                    {
                        st3 = "0" + m_orderedImageData[3][i][j][k];
                    }
                    if ( ( m_orderedImageData[3][i][j][k] ) >= 1000 && ( m_orderedImageData[3][i][j][k] < 10000 ) )
                    {
                        st3 = "" + m_orderedImageData[3][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[4][i][j][k] ) >= 0 && ( m_orderedImageData[4][i][j][k] < 10 ) )
                    {
                        st4 = "000" + m_orderedImageData[4][i][j][k];
                    }
                    if ( ( m_orderedImageData[4][i][j][k] ) >= 10 && ( m_orderedImageData[4][i][j][k] < 100 ) )
                    {
                        st4 = "00" + m_orderedImageData[4][i][j][k];
                    }
                    if ( ( m_orderedImageData[4][i][j][k] ) >= 100 && ( m_orderedImageData[4][i][j][k] < 1000 ) )
                    {
                        st4 = "0" + m_orderedImageData[4][i][j][k];
                    }
                    if ( ( m_orderedImageData[4][i][j][k] ) >= 1000 && ( m_orderedImageData[4][i][j][k] < 10000 ) )
                    {
                        st4 = "" + m_orderedImageData[4][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[5][i][j][k] ) >= 0 && ( m_orderedImageData[5][i][j][k] < 10 ) )
                    {
                        st5 = "000" + m_orderedImageData[5][i][j][k];
                    }
                    if ( ( m_orderedImageData[5][i][j][k] ) >= 10 && ( m_orderedImageData[5][i][j][k] < 100 ) )
                    {
                        st5 = "00" + m_orderedImageData[5][i][j][k];
                    }
                    if ( ( m_orderedImageData[5][i][j][k] ) >= 100 && ( m_orderedImageData[5][i][j][k] < 1000 ) )
                    {
                        st5 = "0" + m_orderedImageData[5][i][j][k];
                    }
                    if ( ( m_orderedImageData[5][i][j][k] ) >= 1000 && ( m_orderedImageData[5][i][j][k] < 10000 ) )
                    {
                        st5 = "" + m_orderedImageData[5][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[6][i][j][k] ) >= 0 && ( m_orderedImageData[6][i][j][k] < 10 ) )
                    {
                        st6 = "000" + m_orderedImageData[6][i][j][k];
                    }
                    if ( ( m_orderedImageData[6][i][j][k] ) >= 10 && ( m_orderedImageData[6][i][j][k] < 100 ) )
                    {
                        st6 = "00" + m_orderedImageData[6][i][j][k];
                    }
                    if ( ( m_orderedImageData[6][i][j][k] ) >= 100 && ( m_orderedImageData[6][i][j][k] < 1000 ) )
                    {
                        st6 = "0" + m_orderedImageData[6][i][j][k];
                    }
                    if ( ( m_orderedImageData[6][i][j][k] ) >= 1000 && ( m_orderedImageData[6][i][j][k] < 10000 ) )
                    {
                        st6 = "" + m_orderedImageData[6][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[7][i][j][k] ) >= 0 && ( m_orderedImageData[7][i][j][k] < 10 ) )
                    {
                        st7 = "000" + m_orderedImageData[7][i][j][k];
                    }
                    if ( ( m_orderedImageData[7][i][j][k] ) >= 10 && ( m_orderedImageData[7][i][j][k] < 100 ) )
                    {
                        st7 = "00" + m_orderedImageData[7][i][j][k];
                    }
                    if ( ( m_orderedImageData[7][i][j][k] ) >= 100 && ( m_orderedImageData[7][i][j][k] < 1000 ) )
                    {
                        st7 = "0" + m_orderedImageData[7][i][j][k];
                    }
                    if ( ( m_orderedImageData[7][i][j][k] ) >= 1000 && ( m_orderedImageData[7][i][j][k] < 10000 ) )
                    {
                        st7 = "" + m_orderedImageData[7][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[8][i][j][k] ) >= 0 && ( m_orderedImageData[8][i][j][k] < 10 ) )
                    {
                        st8 = "000" + m_orderedImageData[8][i][j][k];
                    }
                    if ( ( m_orderedImageData[8][i][j][k] ) >= 10 && ( m_orderedImageData[8][i][j][k] < 100 ) )
                    {
                        st8 = "00" + m_orderedImageData[8][i][j][k];
                    }
                    if ( ( m_orderedImageData[8][i][j][k] ) >= 100 && ( m_orderedImageData[8][i][j][k] < 1000 ) )
                    {
                        st8 = "0" + m_orderedImageData[8][i][j][k];
                    }
                    if ( ( m_orderedImageData[8][i][j][k] ) >= 1000 && ( m_orderedImageData[8][i][j][k] < 10000 ) )
                    {
                        st8 = "" + m_orderedImageData[8][i][j][k];
                    }
                    m_doubleOrderedIndexArray[cntr] = cntr;

                    if ( m_osfSortingType == OrderStatisticsProbingRegionPropertiesPanel.MINIMUM_ORDER_MINIMUM_2_MAXIMUM_SORTING )
                    {
                        m_doubleOrderedData[cntr] = st0  + st1  + st2  + st3  + st4  + st5  + st6  + st7  + st8;
                    }

                    if ( m_osfSortingType == OrderStatisticsProbingRegionPropertiesPanel.MAXIMUM_ORDER_MINIMUM_2_MAXIMUM_SORTING )
                    {
                        m_doubleOrderedData[cntr] =  st8 + st0 + st1 + st2 + st3 + st4 + st5 + st6 + st7;
                    }
                    
                    if ( m_osfSortingType == OrderStatisticsProbingRegionPropertiesPanel.MEDIAN_ORDER_MINIMUM_2_MEDIAN_2_MAXIMUM_SORTING )
                    {
                        m_doubleOrderedData[cntr] = st4 + st0 + st1 + st2 + st3 + st5 + st6 + st7 + st8;
                    }
                    
                    cntr++;
                }
            }


            m_progress = (int)( (float)i * 100.0 / ((float)( m_numberOfImages ) ) );

            notifyActionListeners( NOTIFY_OSF_PROGRESS_EVENT_PASS_3 );
        }

        m_progress =  99;
        notifyActionListeners( NOTIFY_OSF_PROGRESS_EVENT_PASS_4 );

        // sort array
        m_doubleOrderedData = QuickSort.quickSort( m_doubleOrderedIndexArray, m_doubleOrderedData );

        // release the memory from largest, non necessary to persist datastructure
        m_doubleOrderedData = null;

        m_progress = 100;
        notifyActionListeners( NOTIFY_OSF_PROGRESS_EVENT_PASS_5 );
    }
    

    /**
     *  Computes the data structures necessary for 20 point metric as string
     *
     * @param m_orderedImageData
     */
    private void compute20PointMetricAsString( )
    {
       
        String st0 = "";
        String st1 = "";
        String st2 = "";
        String st3 = "";
        String st4 = "";
        String st5 = "";
        String st6 = "";
        String st7 = "";
        String st8 = "";
        String st9 = "";
        String st10 = "";
        String st11 = "";
        String st12 = "";
        String st13 = "";
        String st14 = "";
        String st15 = "";
        String st16 = "";
        String st17 = "";
        String st18 = "";
        String st19 = "";
        String st20 = "";
        
        long start = 0;
        long stop  = 0;

        m_doubleOrderedData = new String[m_numberOfImages*m_imageWidth*m_imageHeight];

        m_doubleOrderedIndexArray = new int[m_numberOfImages*m_imageWidth*m_imageHeight];

        int cntr = 0;

 
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            for ( int j = 0; j < m_imageWidth; j++ )
            {
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    if ( ( m_orderedImageData[0][i][j][k] ) >= 0 && ( m_orderedImageData[0][i][j][k] < 10 ) )
                    {
                        st0 = "000" + m_orderedImageData[0][i][j][k];
                    }
                    if ( ( m_orderedImageData[0][i][j][k] ) >= 10 && ( m_orderedImageData[0][i][j][k] < 100 ) )
                    {
                        st0 = "00" + m_orderedImageData[0][i][j][k];
                    }
                    if ( ( m_orderedImageData[0][i][j][k] ) >= 100 && ( m_orderedImageData[0][i][j][k] < 1000 ) )
                    {
                        st0 = "0" + m_orderedImageData[0][i][j][k];
                    }
                    if ( ( m_orderedImageData[0][i][j][k] ) >= 1000 && ( m_orderedImageData[0][i][j][k] < 10000 ) )
                    {
                        st0 = "" + m_orderedImageData[0][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[1][i][j][k] ) >= 0 && ( m_orderedImageData[1][i][j][k] < 10 ) )
                    {
                        st1 = "000" + m_orderedImageData[1][i][j][k];
                    }
                    if ( ( m_orderedImageData[1][i][j][k] ) >= 10 && ( m_orderedImageData[1][i][j][k] < 100 ) )
                    {
                        st1 = "00" + m_orderedImageData[1][i][j][k];
                    }
                    if ( ( m_orderedImageData[1][i][j][k] ) >= 100 && ( m_orderedImageData[1][i][j][k] < 1000 ) )
                    {
                        st1 = "0" + m_orderedImageData[1][i][j][k];
                    }
                    if ( ( m_orderedImageData[1][i][j][k] ) >= 1000 && ( m_orderedImageData[1][i][j][k] < 10000 ) )
                    {
                        st1 = "" + m_orderedImageData[1][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[2][i][j][k] ) >= 0 && ( m_orderedImageData[2][i][j][k] < 10 ) )
                    {
                        st2 = "000" + m_orderedImageData[2][i][j][k];
                    }
                    if ( ( m_orderedImageData[2][i][j][k] ) >= 10 && ( m_orderedImageData[2][i][j][k] < 100 ) )
                    {
                        st2 = "00" + m_orderedImageData[2][i][j][k];
                    }
                    if ( ( m_orderedImageData[2][i][j][k] ) >= 100 && ( m_orderedImageData[2][i][j][k] < 1000 ) )
                    {
                        st2 = "0" + m_orderedImageData[2][i][j][k];
                    }
                    if ( ( m_orderedImageData[2][i][j][k] ) >= 1000 && ( m_orderedImageData[2][i][j][k] < 10000 ) )
                    {
                        st2 = "" + m_orderedImageData[2][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[3][i][j][k] ) >= 0 && ( m_orderedImageData[3][i][j][k] < 10 ) )
                    {
                        st3 = "000" + m_orderedImageData[3][i][j][k];
                    }
                    if ( ( m_orderedImageData[3][i][j][k] ) >= 10 && ( m_orderedImageData[3][i][j][k] < 100 ) )
                    {
                        st3 = "00" + m_orderedImageData[3][i][j][k];
                    }
                    if ( ( m_orderedImageData[3][i][j][k] ) >= 100 && ( m_orderedImageData[3][i][j][k] < 1000 ) )
                    {
                        st3 = "0" + m_orderedImageData[3][i][j][k];
                    }
                    if ( ( m_orderedImageData[3][i][j][k] ) >= 1000 && ( m_orderedImageData[3][i][j][k] < 10000 ) )
                    {
                        st3 = "" + m_orderedImageData[3][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[4][i][j][k] ) >= 0 && ( m_orderedImageData[4][i][j][k] < 10 ) )
                    {
                        st4 = "000" + m_orderedImageData[4][i][j][k];
                    }
                    if ( ( m_orderedImageData[4][i][j][k] ) >= 10 && ( m_orderedImageData[4][i][j][k] < 100 ) )
                    {
                        st4 = "00" + m_orderedImageData[4][i][j][k];
                    }
                    if ( ( m_orderedImageData[4][i][j][k] ) >= 100 && ( m_orderedImageData[4][i][j][k] < 1000 ) )
                    {
                        st4 = "0" + m_orderedImageData[4][i][j][k];
                    }
                    if ( ( m_orderedImageData[4][i][j][k] ) >= 1000 && ( m_orderedImageData[4][i][j][k] < 10000 ) )
                    {
                        st4 = "" + m_orderedImageData[4][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[5][i][j][k] ) >= 0 && ( m_orderedImageData[5][i][j][k] < 10 ) )
                    {
                        st5 = "000" + m_orderedImageData[5][i][j][k];
                    }
                    if ( ( m_orderedImageData[5][i][j][k] ) >= 10 && ( m_orderedImageData[5][i][j][k] < 100 ) )
                    {
                        st5 = "00" + m_orderedImageData[5][i][j][k];
                    }
                    if ( ( m_orderedImageData[5][i][j][k] ) >= 100 && ( m_orderedImageData[5][i][j][k] < 1000 ) )
                    {
                        st5 = "0" + m_orderedImageData[5][i][j][k];
                    }
                    if ( ( m_orderedImageData[5][i][j][k] ) >= 1000 && ( m_orderedImageData[5][i][j][k] < 10000 ) )
                    {
                        st5 = "" + m_orderedImageData[5][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[6][i][j][k] ) >= 0 && ( m_orderedImageData[6][i][j][k] < 10 ) )
                    {
                        st6 = "000" + m_orderedImageData[6][i][j][k];
                    }
                    if ( ( m_orderedImageData[6][i][j][k] ) >= 10 && ( m_orderedImageData[6][i][j][k] < 100 ) )
                    {
                        st6 = "00" + m_orderedImageData[6][i][j][k];
                    }
                    if ( ( m_orderedImageData[6][i][j][k] ) >= 100 && ( m_orderedImageData[6][i][j][k] < 1000 ) )
                    {
                        st6 = "0" + m_orderedImageData[6][i][j][k];
                    }
                    if ( ( m_orderedImageData[6][i][j][k] ) >= 1000 && ( m_orderedImageData[6][i][j][k] < 10000 ) )
                    {
                        st6 = "" + m_orderedImageData[6][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[7][i][j][k] ) >= 0 && ( m_orderedImageData[7][i][j][k] < 10 ) )
                    {
                        st7 = "000" + m_orderedImageData[7][i][j][k];
                    }
                    if ( ( m_orderedImageData[7][i][j][k] ) >= 10 && ( m_orderedImageData[7][i][j][k] < 100 ) )
                    {
                        st7 = "00" + m_orderedImageData[7][i][j][k];
                    }
                    if ( ( m_orderedImageData[7][i][j][k] ) >= 100 && ( m_orderedImageData[7][i][j][k] < 1000 ) )
                    {
                        st7 = "0" + m_orderedImageData[7][i][j][k];
                    }
                    if ( ( m_orderedImageData[7][i][j][k] ) >= 1000 && ( m_orderedImageData[7][i][j][k] < 10000 ) )
                    {
                        st7 = "" + m_orderedImageData[7][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[8][i][j][k] ) >= 0 && ( m_orderedImageData[8][i][j][k] < 10 ) )
                    {
                        st8 = "000" + m_orderedImageData[8][i][j][k];
                    }
                    if ( ( m_orderedImageData[8][i][j][k] ) >= 10 && ( m_orderedImageData[8][i][j][k] < 100 ) )
                    {
                        st8 = "00" + m_orderedImageData[8][i][j][k];
                    }
                    if ( ( m_orderedImageData[8][i][j][k] ) >= 100 && ( m_orderedImageData[8][i][j][k] < 1000 ) )
                    {
                        st8 = "0" + m_orderedImageData[8][i][j][k];
                    }
                    if ( ( m_orderedImageData[8][i][j][k] ) >= 1000 && ( m_orderedImageData[8][i][j][k] < 10000 ) )
                    {
                        st8 = "" + m_orderedImageData[8][i][j][k];
                    }

                    if ( ( m_orderedImageData[9][i][j][k] ) >= 0 && ( m_orderedImageData[9][i][j][k] < 10 ) )
                    {
                        st9 = "000" + m_orderedImageData[9][i][j][k];
                    }
                    if ( ( m_orderedImageData[9][i][j][k] ) >= 10 && ( m_orderedImageData[9][i][j][k] < 100 ) )
                    {
                        st9 = "00" + m_orderedImageData[9][i][j][k];
                    }
                    if ( ( m_orderedImageData[9][i][j][k] ) >= 100 && ( m_orderedImageData[9][i][j][k] < 1000 ) )
                    {
                        st9 = "0" + m_orderedImageData[9][i][j][k];
                    }
                    if ( ( m_orderedImageData[9][i][j][k] ) >= 1000 && ( m_orderedImageData[9][i][j][k] < 10000 ) )
                    {
                        st9 = "" + m_orderedImageData[9][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[10][i][j][k] ) >= 0 && ( m_orderedImageData[10][i][j][k] < 10 ) )
                    {
                        st10 = "000" + m_orderedImageData[10][i][j][k];
                    }
                    if ( ( m_orderedImageData[10][i][j][k] ) >= 10 && ( m_orderedImageData[10][i][j][k] < 100 ) )
                    {
                        st10 = "00" + m_orderedImageData[10][i][j][k];
                    }
                    if ( ( m_orderedImageData[10][i][j][k] ) >= 100 && ( m_orderedImageData[10][i][j][k] < 1000 ) )
                    {
                        st10 = "0" + m_orderedImageData[10][i][j][k];
                    }
                    if ( ( m_orderedImageData[10][i][j][k] ) >= 1000 && ( m_orderedImageData[10][i][j][k] < 10000 ) )
                    {
                        st10 = "" + m_orderedImageData[10][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[11][i][j][k] ) >= 0 && ( m_orderedImageData[11][i][j][k] < 10 ) )
                    {
                        st11 = "000" + m_orderedImageData[11][i][j][k];
                    }
                    if ( ( m_orderedImageData[11][i][j][k] ) >= 10 && ( m_orderedImageData[11][i][j][k] < 100 ) )
                    {
                        st11 = "00" + m_orderedImageData[11][i][j][k];
                    }
                    if ( ( m_orderedImageData[11][i][j][k] ) >= 100 && ( m_orderedImageData[11][i][j][k] < 1000 ) )
                    {
                        st11 = "0" + m_orderedImageData[11][i][j][k];
                    }
                    if ( ( m_orderedImageData[11][i][j][k] ) >= 1000 && ( m_orderedImageData[11][i][j][k] < 10000 ) )
                    {
                        st11 = "" + m_orderedImageData[11][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[12][i][j][k] ) >= 0 && ( m_orderedImageData[12][i][j][k] < 10 ) )
                    {
                        st12 = "000" + m_orderedImageData[3][i][j][k];
                    }
                    if ( ( m_orderedImageData[12][i][j][k] ) >= 10 && ( m_orderedImageData[12][i][j][k] < 100 ) )
                    {
                        st12 = "00" + m_orderedImageData[12][i][j][k];
                    }
                    if ( ( m_orderedImageData[12][i][j][k] ) >= 100 && ( m_orderedImageData[12][i][j][k] < 1000 ) )
                    {
                        st12 = "0" + m_orderedImageData[12][i][j][k];
                    }
                    if ( ( m_orderedImageData[12][i][j][k] ) >= 1000 && ( m_orderedImageData[12][i][j][k] < 10000 ) )
                    {
                        st12 = "" + m_orderedImageData[12][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[13][i][j][k] ) >= 0 && ( m_orderedImageData[13][i][j][k] < 10 ) )
                    {
                        st13 = "000" + m_orderedImageData[13][i][j][k];
                    }
                    if ( ( m_orderedImageData[13][i][j][k] ) >= 10 && ( m_orderedImageData[13][i][j][k] < 100 ) )
                    {
                        st13 = "00" + m_orderedImageData[13][i][j][k];
                    }
                    if ( ( m_orderedImageData[13][i][j][k] ) >= 100 && ( m_orderedImageData[13][i][j][k] < 1000 ) )
                    {
                        st13 = "0" + m_orderedImageData[13][i][j][k];
                    }
                    if ( ( m_orderedImageData[13][i][j][k] ) >= 1000 && ( m_orderedImageData[13][i][j][k] < 10000 ) )
                    {
                        st13 = "" + m_orderedImageData[13][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[14][i][j][k] ) >= 0 && ( m_orderedImageData[14][i][j][k] < 10 ) )
                    {
                        st14 = "000" + m_orderedImageData[14][i][j][k];
                    }
                    if ( ( m_orderedImageData[14][i][j][k] ) >= 10 && ( m_orderedImageData[14][i][j][k] < 100 ) )
                    {
                        st14 = "00" + m_orderedImageData[14][i][j][k];
                    }
                    if ( ( m_orderedImageData[14][i][j][k] ) >= 100 && ( m_orderedImageData[14][i][j][k] < 1000 ) )
                    {
                        st14 = "0" + m_orderedImageData[14][i][j][k];
                    }
                    if ( ( m_orderedImageData[14][i][j][k] ) >= 1000 && ( m_orderedImageData[14][i][j][k] < 10000 ) )
                    {
                        st14 = "" + m_orderedImageData[14][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[15][i][j][k] ) >= 0 && ( m_orderedImageData[15][i][j][k] < 10 ) )
                    {
                        st15 = "000" + m_orderedImageData[15][i][j][k];
                    }
                    if ( ( m_orderedImageData[15][i][j][k] ) >= 10 && ( m_orderedImageData[15][i][j][k] < 100 ) )
                    {
                        st15 = "00" + m_orderedImageData[15][i][j][k];
                    }
                    if ( ( m_orderedImageData[15][i][j][k] ) >= 100 && ( m_orderedImageData[15][i][j][k] < 1000 ) )
                    {
                        st15 = "0" + m_orderedImageData[15][i][j][k];
                    }
                    if ( ( m_orderedImageData[15][i][j][k] ) >= 1000 && ( m_orderedImageData[15][i][j][k] < 10000 ) )
                    {
                        st15 = "" + m_orderedImageData[15][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[16][i][j][k] ) >= 0 && ( m_orderedImageData[16][i][j][k] < 10 ) )
                    {
                        st16 = "000" + m_orderedImageData[16][i][j][k];
                    }
                    if ( ( m_orderedImageData[16][i][j][k] ) >= 10 && ( m_orderedImageData[16][i][j][k] < 100 ) )
                    {
                        st16 = "00" + m_orderedImageData[16][i][j][k];
                    }
                    if ( ( m_orderedImageData[16][i][j][k] ) >= 100 && ( m_orderedImageData[16][i][j][k] < 1000 ) )
                    {
                        st16 = "0" + m_orderedImageData[16][i][j][k];
                    }
                    if ( ( m_orderedImageData[16][i][j][k] ) >= 1000 && ( m_orderedImageData[16][i][j][k] < 10000 ) )
                    {
                        st16 = "" + m_orderedImageData[16][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[17][i][j][k] ) >= 0 && ( m_orderedImageData[17][i][j][k] < 10 ) )
                    {
                        st17 = "000" + m_orderedImageData[17][i][j][k];
                    }
                    if ( ( m_orderedImageData[17][i][j][k] ) >= 10 && ( m_orderedImageData[17][i][j][k] < 100 ) )
                    {
                        st17 = "00" + m_orderedImageData[17][i][j][k];
                    }
                    if ( ( m_orderedImageData[17][i][j][k] ) >= 100 && ( m_orderedImageData[17][i][j][k] < 1000 ) )
                    {
                        st17 = "0" + m_orderedImageData[17][i][j][k];
                    }
                    if ( ( m_orderedImageData[17][i][j][k] ) >= 1000 && ( m_orderedImageData[17][i][j][k] < 10000 ) )
                    {
                        st17 = "" + m_orderedImageData[17][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[18][i][j][k] ) >= 0 && ( m_orderedImageData[18][i][j][k] < 10 ) )
                    {
                        st18 = "000" + m_orderedImageData[18][i][j][k];
                    }
                    if ( ( m_orderedImageData[18][i][j][k] ) >= 10 && ( m_orderedImageData[18][i][j][k] < 100 ) )
                    {
                        st18 = "00" + m_orderedImageData[18][i][j][k];
                    }
                    if ( ( m_orderedImageData[18][i][j][k] ) >= 100 && ( m_orderedImageData[18][i][j][k] < 1000 ) )
                    {
                        st18 = "0" + m_orderedImageData[18][i][j][k];
                    }
                    if ( ( m_orderedImageData[18][i][j][k] ) >= 1000 && ( m_orderedImageData[18][i][j][k] < 10000 ) )
                    {
                        st18 = "" + m_orderedImageData[18][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[19][i][j][k] ) >= 0 && ( m_orderedImageData[19][i][j][k] < 10 ) )
                    {
                        st19 = "000" + m_orderedImageData[19][i][j][k];
                    }
                    if ( ( m_orderedImageData[19][i][j][k] ) >= 10 && ( m_orderedImageData[19][i][j][k] < 100 ) )
                    {
                        st19 = "00" + m_orderedImageData[19][i][j][k];
                    }
                    if ( ( m_orderedImageData[19][i][j][k] ) >= 100 && ( m_orderedImageData[19][i][j][k] < 1000 ) )
                    {
                        st19 = "0" + m_orderedImageData[19][i][j][k];
                    }
                    if ( ( m_orderedImageData[19][i][j][k] ) >= 1000 && ( m_orderedImageData[19][i][j][k] < 10000 ) )
                    {
                        st19 = "" + m_orderedImageData[19][i][j][k];
                    }
                    //====================================================================================================
                    if ( ( m_orderedImageData[20][i][j][k] ) >= 0 && ( m_orderedImageData[20][i][j][k] < 10 ) )
                    {
                        st20 = "000" + m_orderedImageData[20][i][j][k];
                    }
                    if ( ( m_orderedImageData[20][i][j][k] ) >= 10 && ( m_orderedImageData[20][i][j][k] < 100 ) )
                    {
                        st20 = "00" + m_orderedImageData[20][i][j][k];
                    }
                    if ( ( m_orderedImageData[20][i][j][k] ) >= 100 && ( m_orderedImageData[20][i][j][k] < 1000 ) )
                    {
                        st20 = "0" + m_orderedImageData[20][i][j][k];
                    }
                    if ( ( m_orderedImageData[20][i][j][k] ) >= 1000 && ( m_orderedImageData[20][i][j][k] < 10000 ) )
                    {
                        st20 = "" + m_orderedImageData[20][i][j][k];
                    }

                    m_doubleOrderedIndexArray[cntr] = cntr;
                    if ( m_osfSortingType == OrderStatisticsProbingRegionPropertiesPanel.MINIMUM_ORDER_MINIMUM_2_MAXIMUM_SORTING )
                    {
                        m_doubleOrderedData[cntr] = st0  + st1  + st2  + st3  + st4  + st5  + st6  + st7  + st8 + 
                                                    st9  + st10 + st11 + st12 + st13 + st14 + st15 + st16 + 
                                                    st17 + st18 + st19 + st20;
                    }

                    if ( m_osfSortingType == OrderStatisticsProbingRegionPropertiesPanel.MAXIMUM_ORDER_MINIMUM_2_MAXIMUM_SORTING )
                    {
                        m_doubleOrderedData[cntr] =  st20 + st0 + st1 + st2 + st3 + st4 + st5 + st6 + st7 + st8 +
                                                     st9  + st10 + st11 + st12 + st13 + st14 + st15 + st16 + 
                                                     st17 + st18 + st19;
                    }
                    if ( m_osfSortingType == OrderStatisticsProbingRegionPropertiesPanel.MEDIAN_ORDER_MINIMUM_2_MEDIAN_2_MAXIMUM_SORTING )
                    {
                        m_doubleOrderedData[cntr] = st10 + st0 + st1 + st2 + st3 + st5 + st6 + st7 + st8 +
                                                    st9 + st11 + st12 + st13 + st14 + st15 + st16 + 
                                                    st17 + st18 + st19;
                    }

                    cntr++;
                }
            }

            m_progress = (int)( (float)i * 100.0 / ((float)( m_numberOfImages ) ) );

            notifyActionListeners( NOTIFY_OSF_PROGRESS_EVENT_PASS_3 );
        }

        m_progress =  99;
        notifyActionListeners( NOTIFY_OSF_PROGRESS_EVENT_PASS_4 );

        // sort array
        m_doubleOrderedData = QuickSort.quickSort( m_doubleOrderedIndexArray, m_doubleOrderedData );

        // release the memory from largest, non necessary to persist datastructure
        m_doubleOrderedData = null;

        m_progress = 100;
        notifyActionListeners( NOTIFY_OSF_PROGRESS_EVENT_PASS_5 );
    }

    /**
     * Returns the progress of the computation process.
     * @return
     */
    public int getProgress( )
    {
        return m_progress;
    }

    /**
     *  Computes the data structures necessary for 4 point metric.
     * 
     * @param m_orderedImageData
     */
    private void compute20PointMetric( ) 
    {
        int[] array = new int[m_regionSize+1];
        
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            for ( int j = 2; j < m_imageWidth - 2; j++ )
            {
                for ( int k = 2; k < m_imageHeight - 2; k++ )
                {
                    
                    array[0] = m_imageStackPixelData[i][j-2][k-1];
                    array[1] = m_imageStackPixelData[i][j-2][k];
                    array[2] = m_imageStackPixelData[i][j-2][k+1];
                    
                    array[3] = m_imageStackPixelData[i][j-1][k-2];
                    array[4] = m_imageStackPixelData[i][j-1][k-1];
                    array[5] = m_imageStackPixelData[i][j-1][k];
                    array[6] = m_imageStackPixelData[i][j-1][k+1];
                    array[7] = m_imageStackPixelData[i][j+1][k+2];
                    
                    array[8] = m_imageStackPixelData[i][j][k-2];
                    array[9] = m_imageStackPixelData[i][j][k-1];
                    
                    array[10] = m_imageStackPixelData[i][j][k];
                    
                    array[11] = m_imageStackPixelData[i][j][k+1];
                    array[12] = m_imageStackPixelData[i][j][k+2];
                    
                    array[13] = m_imageStackPixelData[i][j+1][k-2];
                    array[14] = m_imageStackPixelData[i][j+1][k-1];
                    array[15] = m_imageStackPixelData[i][j+1][k];
                    array[16] = m_imageStackPixelData[i][j+1][k+1];
                    array[17] = m_imageStackPixelData[i][j+1][k+2];
                    
                    array[18] = m_imageStackPixelData[i][j+2][k-1];
                    array[19] = m_imageStackPixelData[i][j+2][k];
                    array[20] = m_imageStackPixelData[i][j+2][k+1];
                    
                    // sort array
                    array = QuickSort.quickSort( array );
                    
                    // define sorted data structure
                    for ( int m = 0; m < m_regionSize + 1; m++ )
                    {
                        m_orderedImageData[m][i][j][k] = array[m];  
                    }
                }
            }
        }
    }
    
    
    /**
     * 
     * Returns the length of the m_regionSize + 1 - dimensional vector.
     * 
     * @param vector
     *
     * @return norm (length) of vector
     */
    public float computeNorm( int[] vector )
    {
       float norm = 0f;
       
       for ( int i = 0 ; i < m_regionSize + 1; i++ )
       {
           norm += vector[i] * vector[i];
       }
       
       return (float)Math.sqrt( norm );   
    }
    
    
    /**
     * 
     * Returns the length of the m_regionSize + 1 - dimensional vector
     * 
     * @param imageNumber
     * @param imageXPos
     * @param imageYpos
     * @return
     */
    public float computeNorm( int imageNumber, int imageXPos, int imageYpos )
    {
       float norm = 0f;
       
       for ( int i = 0 ; i < m_regionSize + 1; i++ )
       {
           norm += m_orderedImageData[i][imageNumber][imageXPos][imageYpos] * m_orderedImageData[i][imageNumber][imageXPos][imageYpos];
       }
       
       return (float)Math.sqrt( norm );   
    }
    
    
    /**
     * 
     * Computes the dot product of two ordered vectors
     * 
     * indices [imageNumber1][imageXPos1][imageYpos1] 
     * 
     * and
     * 
     * indices [imageNumber2][imageXPos2][imageYpos2]
     *  
     * @param imageNumber1
     * @param imageXPos1
     * @param imageYpos1
     * 
     * @param imageNumber2
     * @param imageXPos2
     * @param imageYpos2
     * @return
     */
    public float computeDotProduct( int imageNumber1, int imageXPos1, int imageYpos1,
                                    int imageNumber2, int imageXPos2, int imageYpos2 )
    {
         int dotProduct = 0;
         
         for ( int i = 0 ; i < m_regionSize + 1; i++ )
         {
            dotProduct += m_orderedImageData[i][imageNumber1][imageXPos1][imageYpos1] * m_orderedImageData[i][imageNumber2][imageXPos2][imageYpos2];
         }
         
         return (float)dotProduct;
    }
    
    /**
     * 
     * Computes the dot product between the ordered vector:
     * 
     * indices [imageNumber1][imageXPos1][imageYpos1] 
     * 
     * and
     * 
     * vector given by  
     *  
     * @param imageNumber
     * @param imageXPos
     * @param imageYpos
     * @param vector
     * @return dot product
     */
    public float computeDotProduct( int imageNumber, int imageXPos, 
                                    int imageYpos, int[] vector )
    {
         int dotProduct = 0;
         
         for ( int i = 0 ; i < m_regionSize + 1; i++ )
         {
            dotProduct += vector[i] * m_orderedImageData[i][imageNumber][imageXPos][imageYpos];
         }
         
         return (float)dotProduct;
    }
    
    /**
     * 
     * Computes the dot product between two vectors:
     * 
     *  
     * @param vector1
     * @param vector2
     * 
     * @return dot product
     */
    public float computeDotProduct( int[] vector1,
                                    int[] vector2 )
    {
         int dotProduct = 0;
         
         for ( int i = 0 ; i < m_regionSize + 1; i++ )
         {
            dotProduct += vector1[i] * vector2[i];
         }
         
         return (float)dotProduct;
    }
    
    /**
     * 
     * Computes the angle between two ordered vectors in points with
     * 
     * indices [imageNumber1][imageXPos1][imageYpos1] 
     * 
     * and
     * 
     * indices [imageNumber2][imageXPos2][imageYpos2]
     * 
     * @param imageNumber1
     * @param imageXPos1
     * @param imageYpos1
     * @param imageNumber2
     * @param imageXPos2
     * @param imageYpos2
     * @return the angle between the two vectors
     */
    public float computeAngle( int imageNumber1, int imageXPos1, int imageYpos1,
                               int imageNumber2, int imageXPos2, int imageYpos2 )
    {
        float norm1 = 0f;
        float norm2 = 0f;
        float dotProduct = 0f;
       
        float angle = 0f;
       
       
        norm1 = computeNorm( imageNumber1, imageXPos1, imageYpos1 );
        norm2 = computeNorm( imageNumber2, imageXPos2, imageYpos2 );
           
        dotProduct = computeDotProduct( imageNumber1, imageXPos1, imageYpos1,
                                        imageNumber2, imageXPos2, imageYpos2 );
           
        if ( ( norm1 != 0f ) && ( norm2 != 0f ) )
        {
            angle = (float)Math.acos( dotProduct / ( norm1 * norm2 ) ); 
        }
        else
        {
           angle = 0f;
        }
       
        return angle;
    }
    
    
    
    /**
     * 
     * Computes the angle between two ordered vectors in points with
     * 
     * indices [imageNumber1][imageXPos1][imageYpos1] 
     * 
     * and
     * 
     * indices [imageNumber2][imageXPos2][imageYpos2]
     * 
     * @param imageNumber1
     * @param imageXPos1
     * @param imageYpos1
     * @param imageNumber2
     * @param imageXPos2
     * @param imageYpos2
     * @return the angle between the two vectors
     */
    public float computeAngle( int[] vector1,
                               int[] vector2 )
    {
        float norm1 = 0f;
        float norm2 = 0f;
        float dotProduct = 0f;
       
        float angle = 0f;
       
        norm1 = computeNorm( vector1 );
        norm2 = computeNorm( vector2 );
           
        dotProduct = computeDotProduct( vector1,
                                        vector2 );
           
        if ( ( norm1 != 0f ) && ( norm2 != 0f ) )
        {
            angle = (float)Math.acos( dotProduct / ( norm1 * norm2 ) ); 
        }
        else
        {
           angle = 0f;
        }
       
        return angle;
    }
    
    /**
     * 
     * Computes the angle between two ordered vectors in points with
     * 
     * indices [imageNumber1][imageXPos1][imageYpos1] 
     * 
     * and
     * 
     * indices [imageNumber2][imageXPos2][imageYpos2]
     * 
     * 
     * @param imageNumber
     * @param imageXPos
     * @param imageYpos
     * @return the angle between the identity vector (1,1,...,1)
     */
    public float computeAngleWithIdentityVector( int imageNumber, int imageXPos, int imageYpos )
    {
        float norm1 = (float)Math.sqrt( m_regionSize + 1 );
        
        int[] identityVector = new int[ m_regionSize + 1 ];
        
        for ( int i = 0; i < m_regionSize + 1; i++ )
        {
            identityVector[i] = 1;
        }
        
        float norm2 = 0f;
        float dotProduct = 0f;
       
        float angle = 0f;
       
        norm2 = computeNorm( imageNumber, imageXPos, imageYpos );
           
        dotProduct = computeDotProduct( imageNumber, imageXPos, imageYpos, identityVector );
        
        if ( ( norm1 != 0f ) && ( norm2 != 0f ) )
        {
            angle = (float)Math.acos( dotProduct / ( norm1 * norm2 ) ); 
        }
        else
        {
           angle = 0f;
        }
       
        return angle;
    }
    
    
    /**
     * Sets the pixel region size. This integer value can have the following 
     * values in order to examine symmetric :
     *    4 point connectivity     |  8 point connectivity : 9
     *             .x.                         xxx
     *             xpx                         xpx
     *             .x.                         xxx  
     *                        
     *                20 point connectivity       
     *          
     *                        .xxx. 
     *                        xxxxx
     *                        xxpxx
     *                        xxxxx
     *                        .xxx. 
     */
    private void setPixelRegionSize( int regionSize )
    {
        if ( ( regionSize == 4 ) || ( regionSize == 8 ) || ( regionSize == 20 ) )
        {
            m_regionSize = regionSize;
        }
        else
        {
            m_regionSize = 4;
        }
    }
    
    /**
     * Algorithm that prepares the image pixel data in format that can be handled
     * in a fast way.
     */
    public void prepareImagePixelData( )
    {

        long start, stop;

        int[] imagePixels = m_dicomCanvas.getImagePixelData( );

        m_numberOfImages = m_dicomCanvas.getNumberOfImages( );
        m_imageWidth = m_dicomCanvas.getImageWidth( );
        m_imageHeight = m_dicomCanvas.getImageHeight( );

        System.out.print( "Number of images : " + m_numberOfImages );
        System.out.print( "     Image width      : " + m_imageWidth );
        System.out.println( "   Image height     : " + m_imageHeight );

        m_imageStackPixelData = new int[m_numberOfImages][m_imageWidth][m_imageHeight];
        
        if ( ( m_regionSize > -1 ) && ( imagePixels  != null ) )
        {
            for ( int i = 0 ; i < m_dicomCanvas.getNumberOfImages( ) ; i++ )
            {
                for ( int j = 0; j < m_dicomCanvas.getImageWidth( ) ; j++ )
                {
                    for ( int k = 0; k < m_dicomCanvas.getImageHeight( ) ; k++ )
                    {
                        m_imageStackPixelData[i][j][k] = imagePixels[ i * m_imageWidth * m_imageHeight + j * m_imageHeight + k ];
                    }
                }

                m_progress = (int)( (float)i * 100.0 / ((float)( m_numberOfImages ) ) );
                notifyActionListeners( NOTIFY_OSF_PROGRESS_EVENT_PASS_1 );       
            }
        }
    }
    
    /**
     * Compute the ordered data structures.
     */
    public void computeOrderedDataStructures( )
    {
        long start, stop;
        
        System.out.println( "System time before data ordering :" + ( start = System.currentTimeMillis( ) ) );

        m_orderedImageData = new int[m_regionSize+1][m_numberOfImages][m_imageWidth][m_imageHeight];
        
        if ( m_regionSize == 4 )
        {
            compute4PointMetric( ); 
        }
        
        if ( m_regionSize == 8 )
        {
            // performs the ordering within one region
            compute8PointMetric( );
            // perform ordering of sort string
            compute8PointMetricAsString( );
        }
        
        if ( m_regionSize == 20 )
        {
            // performs the ordering within one region
            compute20PointMetric( );
            // perform ordering of sort string
            compute20PointMetricAsString( );
        }

        System.out.println( "System time after data ordering :" + ( stop = System.currentTimeMillis( ) ) );
        System.out.println( "Time needed to order pixel data :" + ( stop - start ) );

    }

   /**
     * Adds an action listener to the list of action listeners.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_listeners.add( e );
    }

    /**
     * Notify all actionListeners.
     */
    private void notifyActionListeners( int eventType )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
        }
    }

     /**
     * Returns the boolean map indicating whether the pixel is inside or
     * outside the user defined pixel intensity range.
     * @return 
     */
    public boolean[][][] getBinaryPixelIntensityThressholdOverlays( )
    {
        return m_binaryPixelIntensityThressholdOverlay;
    }


     /**
     * Returns the boolean map indicating whether the pixel is inside or
     * outside the user defined pixel intensity range
     * @return 
     */
    public boolean[][][] getBinaryPixelIntensityOSFThressholdOverlays( )
    {
        return m_binaryPixelIntensityOSFThressholdOverlay;
    }

   /**
    * Creates a buffered image of a boolean[][] array.
     * @param imageNumber
     * @param overlayColor
     * @param alpha
     * @return 
    */
    public BufferedImage getBinaryPixelIntensityThressholdOverlayAsBufferedImage( int imageNumber, Color overlayColor, float alpha )
    {
        //int width = m_currentView.getImageWidth( );
        //int height = m_currentView.getImageHeight( );
        //float zoom = m_currentView.getZoom( );

        overlayColor = new Color( overlayColor.getRed( )/255f, overlayColor.getGreen( )/255f, overlayColor.getBlue( )/255f, alpha );

        BufferedImage image = new BufferedImage( (int)m_zoom * m_imageWidth, (int)m_zoom * m_imageHeight, BufferedImage.TYPE_INT_ARGB );

        Graphics g = image.getGraphics( );

        if ( m_binaryPixelIntensityThressholdOverlay != null )
        {
            // store current color

            g.setColor( overlayColor );

            for ( int i = 0; i < m_imageWidth ; i++ )
            {
                for ( int j = 0; j < m_imageHeight; j++ )
                {
                    if ( m_binaryPixelIntensityThressholdOverlay[imageNumber][i][j] == true )
                    {
                        g.fillRect( (int)(m_zoom * i), (int)(m_zoom * j), (int)m_zoom , (int)m_zoom );
                    }
                }
            }
        }

        return image;
    }

    /**
    * Creates a buffered image of a boolean[][] array.
     * @param imageNumber
     * @param overlayColor
     * @param alpha
     * @return 
    */
    public BufferedImage getBinaryPixelIntensityOSFThressholdOverlayAsBufferedImage( int imageNumber, Color overlayColor, float alpha )
    {
        //int width = m_currentView.getImageWidth( );
        //int height = m_currentView.getImageHeight( );
        //float zoom = m_currentView.getZoom( );

        overlayColor = new Color( overlayColor.getRed( )/255f, overlayColor.getGreen( )/255f, overlayColor.getBlue( )/255f, alpha );

        BufferedImage image = new BufferedImage( (int)m_zoom * m_imageWidth, (int)m_zoom * m_imageHeight, BufferedImage.TYPE_INT_ARGB );

        Graphics g = image.getGraphics( );

        if ( m_binaryPixelIntensityOSFThressholdOverlay != null )
        {
            // store current color

            g.setColor( overlayColor );

            for ( int i = 0; i < m_imageWidth ; i++ )
            {
                for ( int j = 0; j < m_imageHeight; j++ )
                {
                    if ( m_binaryPixelIntensityOSFThressholdOverlay[imageNumber][i][j] == true )
                    {
                        g.fillRect( (int)(m_zoom * i), (int)(m_zoom * j), (int)m_zoom , (int)m_zoom );
                    }
                }
            }
        }

        return image;
    }

    /**
     *
     * Returns the computed overlays for each image and each tissue type.
     *
     * @return
     */
    public ArrayList< LinkedList< boolean[][] > > getProbabilisticSegmentedTissueOverLays( )
    {
        return m_imageSegmentsLinkedList;
    }

    /**
     *  After a probabilistic decision has been made to which type the pixel
     *  belongs, this method updates the lists setting the boolean true to
     *  the appropriate tissue type to "true" and setting the others to "false".
     */
    private void updateTissueImageList( int imageNumber, int Xpos, int Ypos, int tissueType )
    {
        BufferedImage[] images = new BufferedImage[ m_imageSegmentsLinkedList.size( ) ];

        LinkedList<boolean[][]> segmentList = m_imageSegmentsLinkedList.get( imageNumber );

        for ( int i = 0 ; i < segmentList.size( ) ; i++ )
        {
            boolean[][] segment = segmentList.get( i );
            if ( i == tissueType )
            {
                segment[Xpos][Ypos] = true;
            }
            else
            {
                segment[Xpos][Ypos] = false;
            }
        }
    }

    /**
     * Performs simple probabilistic image segmentation
     */
    public void performSimpleProbabilisticImageSegmentation( )
    {


        m_simpleProbCalc = new  SimpleOrderStatisticImageSegmentationCalculator( m_binaryPixelIntensityThressholdOverlay,
                                                                                 m_dicomCanvas,
                                                                                 m_probeSizeX, m_probeSizeY,
                                                                                 m_permutations,
                                                                                 m_numberOfProbingPixels,
                                                                                 m_pixelRangeComputationType,
                                                                                 m_pixelRangeMultiplier, m_nordicScalingFactor );

        m_imageSegmentsLinkedList = new ArrayList< LinkedList< boolean[][] > >( );

        //m_imagePMapsLinkedList = new LinkedList<float[][]>( );
        //m_imageTMapsLinkedList = new LinkedList<float[][]>( );

        // first do the crude segmentation based on means and variances

        LinkedList<ImageParametersWithinARegioPolygon> allPolygonRegionInformation = m_dicomCanvas.getAllTissueTypeParametersAsLinkedList( );

        // use m_segmentationColors !!
        for ( int k = 0; k < m_numberOfImages; k++ )
        {
            LinkedList< boolean[][] > overlays = new LinkedList< boolean[][] >();

            // iterate over all tissue types
            for ( int i = 0; i < allPolygonRegionInformation.size( ); i++ )
            {

                ImageParametersWithinARegioPolygon tissueInfo = allPolygonRegionInformation.get( i );


                boolean[][] segment = m_simpleProbCalc.getSegmentedTissueMapWithSimpleClassifier( k, tissueInfo );

                overlays.add( segment );

            }
            m_imageSegmentsLinkedList.add( overlays );
        }
    }

    /**
     * Displays the histograms of all OSF statistical orders
     */
    public void displayAllHistograms( )
    {

        int[] xaxis =  new int[ m_numberOfImages * m_imageWidth * m_imageHeight ];
        int[] histogram = new int[ m_numberOfImages * m_imageWidth * m_imageHeight ];


        XYPlotFrame plotHistogram = new XYPlotFrame( "Histogram Orders", null );
        //XYPlotFrame plotInputCurve = new XYPlotFrame( "Input Curves", null );

        // iterate over all statistic orders
        for ( int i = 0; i < m_regionSize; i++ )
        {
            
            for ( int ii = 0; ii < m_doubleOrderedIndexArray.length; ii++ )
            {

                int index = m_doubleOrderedIndexArray[ii];

                int imageNumber = index / ( m_imageHeight * m_imageWidth );
                int jj = ( index % ( m_imageHeight * m_imageWidth ) ) / m_imageWidth;
                int kk = ( index % ( m_imageHeight * m_imageWidth ) ) % m_imageWidth;

                histogram[ii] = m_orderedImageData[i][imageNumber][jj][kk];
                xaxis[ii]= ii;
            }
            
            
            //XYPlotFrame plotInputCurve = new XYPlotFrame( "Input Curve[" + i + "]", null );
            //plotInputCurve.addCurve( new Curve( xaxis, histogram , Color.BLUE ), true );

            IntegerSignalHistogramCalculator histCalc = new IntegerSignalHistogramCalculator( histogram, 0, histogram.length, IntegerSignalHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_SIGNAL );

            Curve hist = histCalc.calculateHistogram( );

            float[] signalY = hist.getYvalues( );

            for ( int kk = 0 ; kk < signalY.length; kk++ )
            {
                if ( signalY[kk] > 13000 )
                {
                    signalY[kk] = 13000;
                }
            }

            hist.setYValues( signalY );
            
            if (i==1)
            {
                hist.setColor( Color.BLACK );
            }
            if (i==2)
            {
                hist.setColor( Color.BLUE );
            }
            if (i==3)
            {
                hist.setColor( Color.GREEN );
            }
            if (i==4)
            {
                hist.setColor( Color.RED );
            }
            if (i==5)
            {
                hist.setColor( Color.YELLOW );
            }
            if (i==6)
            {
                hist.setColor( Color.DARK_GRAY );
            }
            if (i==7)
            {
                hist.setColor( Color.ORANGE );
            }
            if (i==8)
            {
                hist.setColor( Color.MAGENTA );
            }


            // XYPlotFrame plotMean = new XYPlotFrame( "Histogram Order[" + i + "]", null );

            plotHistogram.addCurve( hist, true );

        }

    }

    /**
     * Displays the histograms of all OSF statistical orders.
     */
    public void displayRankOrderSpectraOfDefinedROIs( )
    {

        int[] xaxis =  new int[ m_numberOfImages * m_imageWidth * m_imageHeight ];
        int[] histogram = new int[ m_numberOfImages * m_imageWidth * m_imageHeight ];

        for ( int i = 0; i < m_numberOfImages * m_imageWidth * m_imageHeight; i++ )
        {
            xaxis[i] = i;
        }
            
        XYPlotFrame plotRankOrderSpectra = new XYPlotFrame( "Rank Order Spectra of ROIs", null );
        
        
        // First get the list of all defined ROIs in the slice stack ..
        Vector< LinkedList <RegioPolygon> > m_allDefinedPolygons = m_dicomCanvas.getPolygonsVector( );
        Vector< LinkedList<int[]> > m_rankOrderSpectra = new Vector< LinkedList< int[]> >( );
        
        
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
             LinkedList<RegioPolygon> polygons = m_allDefinedPolygons.get( i );
             
             // create the LinkedList to store the results into
             LinkedList<int[]> spectraLinkedList = new LinkedList<int[]>( );
             
             for ( int j = 0; j < polygons.size( ); j++ )
             {
                 RegioPolygon rpol = polygons.get( j );
                 
                 // identify the rank orders of pixels within the ROI ..
                 int[] ranks = calculateROIRankOrders( i, rpol );
                 spectraLinkedList.add( ranks );
             }
             m_rankOrderSpectra.add( spectraLinkedList );
        }
        
        
        // Iterate over all and display the spectra of currently 
        // selected image number.
        int selectedImageNumber = m_dicomCanvas.getCurrentlyDisplayedImageNumber();
        
        LinkedList<int[]> currentRankOrderSpectra = m_rankOrderSpectra.get( selectedImageNumber );
        
        int numberOfROIsInSlice = m_dicomCanvas.getNumberOfPolygons( selectedImageNumber );
        
        for ( int i = 0; i < numberOfROIsInSlice; i++ )
        {
            
            int[] signal = currentRankOrderSpectra.get( i );
            
            RankSpectrumHistogramCalculator histCalc = new RankSpectrumHistogramCalculator( signal, 0, m_numberOfImages * m_imageWidth * m_imageHeight, RankSpectrumHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_SIGNAL );

            Curve hist = histCalc.calculateRankOrderHistogram( );

            if ( i<=1 )
            {
                hist.setColor( Color.BLACK );
            }
            if ( i==2 )
            {
                hist.setColor( Color.BLUE );
            }
            if ( i==3 )
            {
                hist.setColor( Color.GREEN );
            }
            if ( i==4 )
            {
                hist.setColor( Color.RED );
            }
            if ( i==5 )
            {
                hist.setColor( Color.YELLOW );
            }
            if ( i==6 )
            {
                hist.setColor( Color.DARK_GRAY );
            }
            if ( i==7 )
            {
                hist.setColor( Color.ORANGE );
            }
            if ( i>8 )
            {
                hist.setColor( Color.MAGENTA );
            }

            // XYPlotFrame plotMean = new XYPlotFrame( "Histogram Order[" + i + "]", null );

            plotRankOrderSpectra.addCurve( hist, true );

        }

    }
    
    
    /**
     *  Calculates the average pixel value and standard deviation of the
     *  polygon number (polygon_number) which encloses a ROI).
     */
    public int[] calculateROIRankOrders( int imageNumber, RegioPolygon rpol )
    { 
        int[] ranks = null;
        // get a reference on the selected polygon        
	
        float minimum = Float.MAX_VALUE;
        float maximum = -Float.MIN_VALUE;
        
        
        int dataOffset = m_imageWidth * m_imageHeight * imageNumber;
        
        float zoom = 1f / m_dicomCanvas.getZoom( );
        Polygon roi = (Polygon)rpol.scaled( zoom );
        
        if ( roi != null )
        {
            Rectangle bb = roi.getBounds( );
            if ( bb.x < 0 )
            {    
                bb.x = 0;
            }
            if ( bb.x + bb.width >= m_imageWidth ) 
            {
                bb.width = m_imageWidth - bb.x - 1;
            }
            if ( bb.y < 0 )
            {
                bb.y = 0;
            }
        
            if ( bb.y + bb.height >= m_imageHeight ) 
            {
                bb.height = m_imageHeight - bb.y - 1;
            }
        
            int imageSize = m_imageWidth * m_imageHeight;
            
            int numberOfPixelsWithinROI = 0;
            
            // Iterate over allpixels within the ROI contour ..
            for ( int y = bb.y; y <= bb.y + bb.height; y++ ) 
            {
                for ( int x = bb.x; x <= bb.x + bb.width; x++ ) 
                {
                    if ( roi.contains( x, y ) ) 
                    {            
                        numberOfPixelsWithinROI++;
                    }
                }
            }
            
            ranks = new int[numberOfPixelsWithinROI];
            
            int cntr = 0;
            
            // Iterate over allpixels within the ROI contour ..
            for ( int y = bb.y; y <= bb.y + bb.height; y++ ) 
            {
                
                for ( int x = bb.x; x <= bb.x + bb.width; x++ ) 
                {
                    if ( roi.contains( x, y ) ) 
                    {               
                        ranks[cntr] = this.m_doubleOrderedIndexArray[ imageNumber * imageSize + x + m_imageWidth * y ];
                        cntr++;         
                    }
                }
            }
        }
        return ranks;
    }
    
    /**
     *
     * Returns whether the control is initialized.
     *
     * @return
     */
    public boolean isInitialized( )
    {
        return m_isInitialized;
    }

    // Attributes

     // Listeners to changes in the segmentation gui control
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );

    private DicomSliceCanvasPolyDraw m_dicomCanvas;
    private int[][][] m_imageStackPixelData;
    private int m_regionSize = -1;
    
    // dimensions of the pixel data structures
    private int m_imageWidth, m_imageHeight, m_numberOfImages;
    private float m_zoom;

    private int[][][][] m_orderedImageData;

    // data structures needed for double ordering
    private String[] m_doubleOrderedData;
    private int[]    m_doubleOrderedIndexArray;


    private float[][][] m_allNorms;
    private float[][][] m_allAnglesWithIdentityVector;

    // available region size metrics
    public final static int USE_4_POINT_METRIC = 4;
    public final static int USE_8_POINT_METRIC = 8;
    public final static int USE_20_POINT_METRIC = 20;


    // EVENTS NOTIFICATION Strings

    public final static int OSF_INITIALIZATION_READY_EVENT = -1;
    public final static int OSF_SEGMENTATION_PERFORMED_EVENT = 0;
    public final static int NOTIFY_OSF_PROGRESS_EVENT = 1;

    public final static int START_RECALCULATION_OSF_SEGMENTATION_EVENT = 2;

    public final static int NOTIFY_OSF_PROGRESS_EVENT_PASS_1 = 10;
    public final static int NOTIFY_OSF_PROGRESS_EVENT_PASS_2 = 20;
    public final static int NOTIFY_OSF_PROGRESS_EVENT_PASS_3 = 30;
    public final static int NOTIFY_OSF_PROGRESS_EVENT_PASS_4 = 40;
    public final static int NOTIFY_OSF_PROGRESS_EVENT_PASS_5 = 50;

    // Additional attributes for simple thressholding segmentation
    private JFrame m_parent;

    private boolean m_performPixelValueThresholdingBeforeSegmentation = false;
    private boolean m_performOrderStatisticROIThressholdingSegmentation = false;
    
    private boolean[][][] m_binaryPixelIntensityThressholdOverlay;
    
    // below the data object that contains the final overlay information,
    // after simple thressholding AND OSF segmentation
    private boolean[][][] m_binaryPixelIntensityOSFThressholdOverlay;

    int m_simpleThressholdingMinimumValue;
    int m_simpleThressholdingMaximumValue;

    int m_OSFThressholdingMinimumValue;
    int m_OSFThressholdingMaximumValue;
    
    private RegioPolygon m_segmentationWindow;

    // indicates the progress of the segmentation process
    private int m_progress = 0;

    private ArrayList< LinkedList< boolean[][] > > m_imageSegmentsLinkedList = new ArrayList< LinkedList< boolean[][] > >( );

    private SimpleOrderStatisticImageSegmentationCalculator m_simpleProbCalc;
    
    private Color[] m_segmentationColors;

    private int m_probeSizeX;
    private int m_probeSizeY;

    private int m_numberOfProbingPixels;
    private int m_pixelRangeComputationType;
    private int m_pixelRangeMultiplier;
    private int m_permutations;

    private boolean m_performMorphologicalFiltering = false;
    private boolean m_isInitialized = false;

    private int m_osfSortingType = OrderStatisticsProbingRegionPropertiesPanel.MEDIAN_ORDER_MINIMUM_2_MEDIAN_2_MAXIMUM_SORTING;
    private float m_nordicScalingFactor = 1f;
    
    // Variable required for OSF-output regression analysis
    private double[] m_slope;
    private double[] m_intercept;
    
    private boolean[][][] m_leftOverlay;
    private boolean[][][] m_rightOverlay;
    
}