/*
 * File: $Id: RandomMinimizer.java,v 1.2 2006/04/20 09:29:29 hans Exp $
 * History
 * Date: 18.04.2006, 10:58  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2006 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
package org.spectrim.signalprocessing;

import org.spectrim.lib.FloatRange;
import java.util.concurrent.ThreadLocalRandom;

/**
 * Modula-2 comment:
 * (**************************************************************************)
 * (*** This program minimizes an P dimensional function ***) (*** J(p1,p2,..)
 * in a random way, i.e. by most likehood distributions ***) (*** Test version
 * 1.0 by H. Slotboom / april 30th, 1993 ***)
 * (**************************************************************************)
 *
 * Recoded in Java by hs on 18-04-2006
 *
 * @author hans
 */
public abstract class RandomMinimizerFloat {

    /**
     * Creates a new instance of RandomMinimizer
     */
    public RandomMinimizerFloat() {
        m_n_iter_max = 100;
        m_number_of_random_trials = 100;
    }

    /**
     * Computes the function values of the function to minimize
     */  
    public abstract float fnc(float[] par);

    /**
     * Initializes the parameter vector
     */
    public void setStartParameters(float[] par) {

        m_par = new float[par.length];
        System.arraycopy(par, 0, m_par, 0, par.length);
    }

    /**
     * Initializes the parameter ranges vector
     */
    public void setParameterRanges(FloatRange[] par_range) {
        m_parameterRanges = new FloatRange[par_range.length];

        for (int i = 0; i < par_range.length; i++) {
            FloatRange parRange = new FloatRange(par_range[i].getMinimum(), par_range[i].getMaximum());
            m_parameterRanges[i] = parRange;
        }
    }

    /**
     * Checks the starting value parameters on their validity
     */
    private boolean CheckParameters() {

        boolean returnFlag = true;

        for (int i = 0; i < m_par.length; i++) {
            if ((m_par[i] >= m_parameterRanges[i].getMinimum()) && (m_par[i] <= m_parameterRanges[i].getMaximum())) {
                ; // everything is ok
            } else {
                returnFlag = false;
            }
        }

        if (returnFlag == true) {
            if (m_verbose == true) {
                System.out.println("All initial parameters are O.K. ");
                System.out.println("");
            }
        } else {
            System.out.println(" *** > Corrupt initial parameter array in class SCANalyze.SignalProcessing.RandomMinimizer: ");
            System.out.println(" Initial parameter values do not satisfy the parameter boundary conditions ");
        }

        return returnFlag;
    }

    /**
     * Initializes the random search algorithm
     */
    private void InitRandomSearch() {

        float ran_fval;

        for (int j = 0; j < m_par.length; j++) {
            if (m_parameterRanges[j].getMaximum() == m_parameterRanges[j].getMinimum()) {
                m_par[j] = m_parameterRanges[j].getMinimum();
            } else {
                m_par[j] = 0.5f * (m_parameterRanges[j].getMaximum() + m_parameterRanges[j].getMinimum()) + (m_parameterRanges[j].getMaximum() - m_parameterRanges[j].getMinimum()) * (rnd.nextFloat() - 0.5f);
            }
        }
        m_tmp_min_fnc = fnc(m_par);
        System.arraycopy(m_par, 0, m_parMinimum, 0, m_par.length);

        for (int i = 0; i < m_number_of_random_trials - 1; i++) {

            for (int j = 0; j < m_par.length; j++) {
                if (m_parameterRanges[j].getMaximum() == m_parameterRanges[j].getMinimum()) {
                    m_par[j] = m_parameterRanges[j].getMinimum();
                } else {
                    m_par[j] = 0.5f * (m_parameterRanges[j].getMaximum() + m_parameterRanges[j].getMinimum()) + (m_parameterRanges[j].getMaximum() - m_parameterRanges[j].getMinimum()) * (rnd.nextFloat() - 0.5f);
                }
            }
            ran_fval = fnc(m_par);

            if (ran_fval < m_tmp_min_fnc) {
                m_tmp_min_fnc = ran_fval;

                m_parMinimum = copyVector(m_par);
            }

        }
    }

    /**
     * Directs the values of a float vector to standard output
     */
    public void displayVector(float[] invec) {
        for (int i = 0; i < invec.length; i++) {
            System.out.println("Par[" + i + "] = " + invec[i]);
        }
    }

    /**
     * Returns a copy of float precision inpar arrqay
     */
    private float[] copyVector(float[] inpar) {

        float[] outpar = new float[inpar.length];
        System.arraycopy(inpar, 0, outpar, 0, inpar.length);
        return outpar;
    }

    /**
     * Computes a random parameter vector
     */
    private float[] RandomPar(int k, float[] center, int step) {
        float ranval, ran1, ran2;

//        float[] tmpPar = new float[center.length];
        float[] tmpPar = center;

        for (int i = step * center.length / this.m_nr_steps; i < (step + 1) * (float) center.length /(float) this.m_nr_steps; i++) {
            float tstMin = m_parameterRanges[i].getMinimum();
            float tstMax = m_parameterRanges[i].getMaximum();

            if (tstMin == tstMax) {
                tmpPar[i] = tstMin;
            } else {
                do {
                    ran1 = 2.0f * (rnd.nextFloat() - 0.5f);
                    ran2 = rnd.nextFloat() - 0.5f;
                    if (ran2 != 0.0) {
                        ran2 = ran2 / Math.abs(ran2);
                    } else {
                        ran2 = 1.0f;
                    }

                    ranval = center[i] + (m_parameterRanges[i].getMaximum() - m_parameterRanges[i].getMinimum()) * ran2 * (float) Math.exp(-(float) k * Math.abs(ran1));
                } while ((ranval < tstMin) || (ranval > tstMax));

                // System.out.print( '*' );

                tmpPar[i] = ranval;
            }
        }
        return tmpPar;

    }

    /**
     * Performs a random minimization
     */
    private void randomMinimize() {

        float ran_fval;

//        float[] convmin = new float[m_n_iter_max + 1];

//        float[] xaxis = new float[m_n_iter_max + 1];

        m_tmp_min_fnc = fnc(m_parMinimum);

//        xaxis[0] = 0.0;
//        convmin[0] = m_tmp_min_fnc;

        float[] tmp_par = new float[m_par.length];
        float[] tmp_par_min = new float[m_par.length];


        tmp_par_min = copyVector(m_parMinimum);

        if (m_nr_steps > 1) {
            for (int c = 0; c < cycles; c++) {
                for (int s = 0; s < m_nr_steps; s++) {
                    for (int k = 1; k <= m_n_iter_max; k++) {
                        /* search for a new minimum */
                        for (int i = 0; i < m_number_of_random_trials; i++) {
                            tmp_par = RandomPar(k, m_parMinimum, s);
                            ran_fval = fnc(tmp_par);
                            if (ran_fval < m_tmp_min_fnc) {
                                m_tmp_min_fnc = ran_fval;
                                tmp_par_min = copyVector(tmp_par);
                            }
                        }

                        m_parMinimum = copyVector(tmp_par_min);

                        if (m_verbose == true) {
                            System.out.println("");
                            System.out.println("After the " + k + "-th iteration : ");
                            System.out.println("");
                            System.out.println("  Vector containing minimizing parameters : ");
                            displayVector(m_parMinimum);
                            System.out.println("  Minimum function-value : " + m_tmp_min_fnc);
                            System.out.println("");
                        }

//            convmin[k] = m_tmp_min_fnc;

//            xaxis[k] = (float) k;

                    }
                }
            }
        } else {
            for (int k = 1; k <= m_n_iter_max; k++) {
                /* search for a new minimum */
                for (int i = 0; i < m_number_of_random_trials; i++) {
                    tmp_par = RandomPar(k, m_parMinimum, 0);
                    ran_fval = fnc(tmp_par);
                    if (ran_fval < m_tmp_min_fnc) {
                        m_tmp_min_fnc = ran_fval;
                        tmp_par_min = copyVector(tmp_par);
                    }
                }

                m_parMinimum = copyVector(tmp_par_min);

                if (m_verbose == true) {
                    System.out.println("");
                    System.out.println("After the " + k + "-th iteration : ");
                    System.out.println("");
                    System.out.println("  Vector containing minimizing parameters : ");
                    displayVector(m_parMinimum);
                    System.out.println("  Minimum function-value : " + m_tmp_min_fnc);
                    System.out.println("");
                }

//            convmin[k] = m_tmp_min_fnc;

//            xaxis[k] = (float) k;

            }
        }

//        if (convmin[0] != 0.0) {
//            float factor = convmin[0];
//            for (int i = 0; i < convmin.length; i++) {
//                convmin[i] = convmin[i] / factor;
//            }
//        }
        if (m_verbose == true) {
//            XYPlotFrame convMinPlot = new XYPlotFrame("Minimum function values", null);
//            convMinPlot.addCurve(new Curve(xaxis, convmin), true);
        }

        if (m_verbose == true) {
            System.out.println("Obtained minimum : ");
            System.out.println("");
            System.out.println("");

            System.out.println("  Vector containing minimizing parameters : ");
            System.out.println("");
            displayVector(m_parMinimum);
            System.out.println("  Minimum function-value : " + m_tmp_min_fnc);
            System.out.println("");
        }

    }

    /**
     * ¨
     * Returs the sum of least squares
     *
     * @return
     */
    public float getMinimumFunctionValue() {
        return m_tmp_min_fnc;
    }

    /**
     * Performs the random minimization This minimization/maximization algorithm
     * minimizes and maximizes a given p-dimensional function f(par) on a given
     * limited finite parameter domain. The minimization is carried out
     * randomly. The user has to specify the maximum number of iterations
     * n_itermax, the number of random function evaluations per iteration N, the
     * parameter domain 'par_range' in which the minimum and maximum of the
     * function has to be found and whether an initial parameter- vector is
     * available. In the case that init_val_avail is FALSE the algorithm will
     * produce its own initial guess.
     */
    public boolean performRandomMinimization() {
        boolean flag = true;

        if ((m_par.length > 0) && (m_par.length == m_parameterRanges.length)) {
            if (CheckParameters() == true) {

                m_parMinimum = copyVector(m_par);


                InitRandomSearch();

                randomMinimize();
            } else {
                flag = false;
                System.out.println(" Failure : Check initial parameter values and parameter range. ");
                System.out.println(" And try again ");
            }
        } else {
            System.out.println("Initial Parameter vector and/or parameter ranges vector missing or incomplete ");
        }

        return flag;

    }

    /**
     * Performs the random minimization
     */
    public boolean performRandomMinimization(int numberOfRandomTrials, int niter_max, boolean verbose) {
        boolean flag = true;
        m_verbose = verbose;
        m_number_of_random_trials = numberOfRandomTrials;
        m_n_iter_max = niter_max;
        //  This minimization/maximization algorithm minimizes and maximizes  
        //  a given p-dimensional function f(par) on a given limited finite   
        //  parameter domain. The minimization is carried out randomly. The   
        //  user has to specify the maximum number of iterations n_itermax,   
        //  the number of random function evaluations per iteration N, the    
        //  parameter domain 'par_range' in which the minimum and maximum of  
        //  the function has to be found and whether an initial parameter-    
        //  vector is available. In the case that init_val_avail is FALSE the 
        //  algorithm will produce its own initial guess.                     
        //

        if ((m_par.length > 0) && (m_par.length == m_parameterRanges.length)) {
            if (CheckParameters() == true) {

                m_parMinimum = copyVector(m_par);

                InitRandomSearch();

                randomMinimize();
            } else {
                flag = false;
                System.out.println(" Failure : Check initial parameter values and parameter range. ");
                System.out.println(" And try again ");
            }
        } else {
            System.out.println("Initial Parameter vector and/or paramter ranges vector missing or incomplete ");
        }

        return flag;

    }

    /**
     * Returs the value of the parameter vector m_parMinimum which contains the
     * optimal parameters
     */
    public float[] getMinimizingParameters() {
        return m_parMinimum;
    }

    /**
     * Sets the number of maximum allowed iterations
     */
    public void setMaxNumberOfIterations(int maxIter) {
        this.m_n_iter_max = maxIter;
    }

    /**
     * Sets the number of maximum allowed iterations
     */
    public int getMaxNumberOfIterations() {
        return m_n_iter_max;
    }

    /**
     * Sets the number of maximum allowed iterations
     */
    public void setMaxNumberOfRandomTries(int maxRandomTrials) {
        m_number_of_random_trials = maxRandomTrials;
    }

    /**
     * Sets the number of maximum allowed iterations
     */
    public int getMaxNumberOfRandomTries() {
        return m_number_of_random_trials;
    }

    public void setNumberOfSteps(int nr, int nrcycles) {
        m_nr_steps = nr;
        cycles = nrcycles;
    }
    // Member variables
    private float[] m_par;
    private FloatRange[] m_parameterRanges;
    private ThreadLocalRandom rnd = ThreadLocalRandom.current();
    private int m_nr_steps = 1;
    private int cycles = 1;
    private int m_n_iter_max;
    private int m_number_of_random_trials;
    private float m_tmp_min_fnc;
    private float[] m_parMinimum;
    private boolean m_verbose = true;
}