/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.tdfdfit;

import java.io.EOFException;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import org.spectrim.model.GlobalModel;
import org.spectrim.spectra.Spectra;
import org.spectrim.spectra.Spectrum;

/**
 * This class reads a .mrui file. 
 * This is a small light-weight re-implementation to read .mrui files
 * that contain metabolite base set responses.
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */
public class JMRUIFile 
{
    // Atrributes
    private final String m_mruiFileRef;
    
    // Objects to store the response into
    private float[] m_absorptionChannelFD;
    private float[] m_dispersionChannelFD;
    private float[] m_absorptionChannelTD;
    private float[] m_dispersionChannelTD;
    
    private double  m_setStepTime;
    private double  m_transmitterFreq;
    private boolean m_setEchoes;
    private double  m_B0;
    private double  m_nucleus;
    private double  m_zeroOrderPhase;
    
    // from jmui taken constants
    public final static int SIGNAL        = 0;
    public final static int SIMSPECTRUM   = 1;
    public final static int SIMSIGNAL     = 2;
    public final static int PEAKTABLE     = 3;
    public final static int SIMULATION    = -1;
    
    private double      m_nop;
    private double      m_dataType;
    private double      m_beginTime;
    private double      m_hzref;
    private double      m_ppmref;
    private double      m_echoes;
    private double      m_apodizeView;
    private int         m_zeroFillingView;
    private int         m_numberOfFrames;
    private int         m_leftPointsEcho;

    
    
    /**
     * Constructs a new JMRUIFile-object based on a file reference to 
     * points to a .mrui file. 
     */
    public JMRUIFile( )
    {
        m_mruiFileRef = null;
    }
    
    
    /**
     * Constructs a new JMRUIFile-object based on a file reference to 
     * points to a .mrui file.
     * @param mruiFileRef 
     */
    public JMRUIFile( String mruiFileRef )
    {
        m_mruiFileRef = mruiFileRef;
    }
    
    /**
     * 
     * Attempts to load the mrui file.
     * 
     * @return 
     */
    public boolean loadMRUITypedData( )
    {
        try 
        {
            RandomAccessFile in = new RandomAccessFile( m_mruiFileRef , "r" );
                
            System.out.println( "Loading jMRUI v4 file" );

            // The following is arbitrary:
            m_setStepTime = 1.0;
            m_nucleus = -1;
            m_B0 = -1;
            m_transmitterFreq = 63860000.00;
            m_setEchoes= false;
            // end of arbitrary.

            // the first double in the file specifies the type of data :
            // 0 for a fid
            // 1 for a simulated Spectrum
            // 2 for a simulated Fid
            // 3 for a peak table
            // int choice = SIMSPECTRUM;
            int type   = SIMSPECTRUM;
            
            try 
            {
                type = (int) in.readDouble( ) - 1;
            } 
            catch ( IOException e ) 
            {
                System.out.println( "Could not read from the .mrui file : " + this.m_mruiFileRef );
            }
            switch ( type ) 
            {

                case SIGNAL:
                    type = SIGNAL;
                    break;
                case SIMSIGNAL:
                    type = SIMSIGNAL;
                    break;
                case SIMSPECTRUM:
                      type = SIMSPECTRUM;
                      break;
                case PEAKTABLE:
                      type = PEAKTABLE;
                      System.out.println( "Peak table cannot serve as metabolite baseset information." );
                default:
                      System.out.println( "This method only accepts FID, simulated FID or simulated spectrum." );
            }

            try 
            {
                // Note: first double has already been read from file!
                double nopf = in.readDouble( );
                m_nop = (int) nopf;
                // To initialise the Data object with the information
                // in the header of the file.
                m_setStepTime       = in.readDouble( );
                m_beginTime         = in.readDouble( );
                m_zeroOrderPhase    = in.readDouble( );
                m_transmitterFreq   = in.readDouble( );
                m_B0                = in.readDouble( );
                m_nucleus           = in.readDouble( );
                m_hzref             = in.readDouble( );
                m_ppmref            = in.readDouble( );

                if ( m_hzref != 0 )
                {
                    m_ppmref = m_hzref * 1e6 / m_transmitterFreq;
                }
                else if ( m_ppmref != 0 )
                {
                    m_hzref = m_ppmref * m_transmitterFreq / 1e6;
                }
                double echoes = in.readDouble( );
                if ( echoes != 0 )
                {
                    m_setEchoes = true;
                }
                
                m_apodizeView       = in.readDouble( );
                m_zeroFillingView   = (int)in.readDouble( );
                m_numberOfFrames    = (int)in.readDouble( );
                
                
                if ( m_numberOfFrames == 0 )
                {
                    m_numberOfFrames = 1;  // At least one signal per file
                }
                
                m_leftPointsEcho = (int)in.readDouble( );
                
                // Skip all currently unused fields in the header of the file
                double tempDouble;
                for (int i = 0; i < 64 - 15; i++)
                {
                    tempDouble = in.readDouble( );
                }
                
                // Read the actual signal data of the signal from the file
                for ( int j = 0; j < m_numberOfFrames; j++ ) 
                {
                    System.out.println( "Loading signal frame " + (j + 1) + " of " + m_numberOfFrames + " from file.\r");
                    
                    m_absorptionChannelTD = new float[(int)m_nop];
                    m_dispersionChannelTD = new float[(int)m_nop];
                    
                    try 
                    {
                        for ( int i = 0; i < m_nop; i++ ) 
                        {
                            m_absorptionChannelTD[i] = (float)in.readDouble( );
                            m_dispersionChannelTD[i] = -(float)in.readDouble( );
                        }
                    } 
                    catch ( EOFException eof ) 
                    {
                        System.out.println( "EOF exception occurred in file : " + m_mruiFileRef );
                        return false;
                    }
              
                }
                
                System.out.println( "File read correctly." );
                in.close( );

            } 
            catch ( java.io.IOException ex ) 
            {
                System.out.println( "An IO-error occurred while attempting to load file : " + m_mruiFileRef );
                return false;
            }
        }
        catch ( FileNotFoundException ex ) 
        {
            System.out.println( "File not found : " + m_mruiFileRef );
            return false;
        }
        
        return true;
    }
    
    
     /**
     * 
     * Attempts to save the selected voxels as mrui file.
     * 
     * @param model
     * @param inOneFile if true than multiple responses are stored in one file
     *                  if false than for each instance a separate file is created
     * @return 
     */
    public boolean saveMRUITypedData( GlobalModel model, boolean inOneFile ) throws IOException
                  
    {
        m_nop = model.getSpectraVectorSize( );
        m_dataType = 0; // only FID signals are allowed now
        m_setStepTime = 1000.0 * 1.0/model.getBandwidth( ); // unit in msec
        m_beginTime = 0.0;
        m_zeroOrderPhase = 0.0; // no phase information is exported;
        m_transmitterFreq = model.getResonanceFrequency( );
        m_B0 = model.getB0( );
        m_nucleus = 1.0;
        m_ppmref = -4.7;
        m_hzref = m_ppmref * m_transmitterFreq / 1e6;
        
        m_echoes = 0.0;
        m_apodizeView = 0.0;
        m_zeroFillingView = 0;
        m_numberOfFrames = model.getSelectedVoxels( ).size( );
        m_leftPointsEcho = 0;
        
        if ( inOneFile == true )
        {
            try 
            {
                RandomAccessFile out = new RandomAccessFile( m_mruiFileRef , "rw" );
                
                System.out.println( "Writing jMRUI v4 file" );

                // the first double in the file specifies the type of data :
                // 0 for a fid
                // 1 for a simulated Spectrum
                // 2 for a simulated Fid
                // 3 for a peak table
                // int choice = SIMSPECTRUM;
                // int type   = SIMSPECTRUM;

                try 
                {
                    out.writeDouble( (double)m_dataType );
                } 
                catch ( IOException e ) 
                {
                    System.out.println( "Could not write from the .mrui file : " + this.m_mruiFileRef );
                }
                switch ( (int)m_dataType ) 
                {

                    case SIGNAL:
                                    System.out.println( "Writing signal file to : " + this.m_mruiFileRef );;
                                    break;
                    case SIMSIGNAL:
                                    System.out.println( "Writing simulated signal file to : " + this.m_mruiFileRef );;
                                    break;

                    case SIMSPECTRUM:
                                    System.out.println( "Writing simulated spectrum file to : " + this.m_mruiFileRef );;
                                    break;
                    case PEAKTABLE:
                                    System.out.println( "Writing a peak table file to : " + this.m_mruiFileRef );;
                                    break;          
                    default:
                                    System.out.println( "Warning: Unclear output format!" );
                }

                try 
                {
                    // Note: first double has already been read from file!
                    out.writeDouble( m_nop );

                    // To initialise the Data object with the information
                    // in the header of the file.
                    out.writeDouble( m_setStepTime );
                    out.writeDouble( m_beginTime );
                    out.writeDouble( m_zeroOrderPhase );
                    out.writeDouble( m_transmitterFreq );
                    out.writeDouble( m_B0 );
                    out.writeDouble( m_nucleus );
                    out.writeDouble( m_hzref );
                    out.writeDouble( m_ppmref );

                    out.writeDouble( m_echoes );

                    out.writeDouble( m_apodizeView );
                    out.writeDouble( (double)m_zeroFillingView );
                    out.writeDouble( (double)m_numberOfFrames );
                    out.writeDouble( (double)m_leftPointsEcho );

                    // Skip all currently unused fields in the header of the file
                    double tempDouble = 0.0;
                    for ( int i = 0; i < 64 - 15; i++)
                    {
                        out.writeDouble( tempDouble );
                    }

                    Spectra spectra = model.getSpectra( );


                    ArrayList<String> selVox = model.getSelectedVoxels( );

                    HashMap<String, Spectrum> data =  spectra.getData( );

                    Iterator it = selVox.iterator( );
                    int frameCntr =0;
                    while ( it.hasNext( ) ) 
                    {
                        String voxID = "" + it.next( );
                        float[] absSignal = model.getCSIData( ).getSpectra( ).getSpectrumData( voxID, Spectra.Domain.TD, Spectra.Mode.ABS );
                        float[] dispSignal = model.getCSIData( ).getSpectra( ).getSpectrumData( voxID, Spectra.Domain.TD, Spectra.Mode.DISP );

                        System.out.println( "Writing signal frame " + (frameCntr++) + " of " + m_numberOfFrames + " to file.\r");
                        for ( int i = 0; i < m_nop; i++ ) 
                        {
                            out.writeDouble( absSignal[i] );
                            out.writeDouble( dispSignal[i] );
                        }

                    }  
                    System.out.println( "File written correctly." );
                    out.close( );

                } 
                catch ( java.io.IOException ex ) 
                {
                    System.out.println( "An IO-error occurred while attempting to load file : " + m_mruiFileRef );
                    return false;
                }
            }
            catch ( FileNotFoundException ex ) 
            {
                System.out.println( "File not found : " + m_mruiFileRef );
                return false;
            }
        }
        else
        {
            try 
            {
                
                Spectra spectra = model.getSpectra( );
                ArrayList<String> selVox = model.getSelectedVoxels( );

                HashMap<String, Spectrum> data =  spectra.getData( );

                Iterator it = selVox.iterator( );
                int frameCntr =0;
                
                while ( it.hasNext( ) ) 
                {
                    String voxID = "" + it.next( );
                    
                    String fileNameWOExtenstion = m_mruiFileRef.substring( 0, m_mruiFileRef.lastIndexOf( "." ) );
                    
                    RandomAccessFile out = new RandomAccessFile( fileNameWOExtenstion + "-" + voxID + ".mrui", "rw" ); 
                    System.out.println( "Writing jMRUI v4 file" );

                    // the first double in the file specifies the type of data :
                    // 0 for a fid
                    // 1 for a simulated Spectrum
                    // 2 for a simulated Fid
                    // 3 for a peak table
                    // int choice = SIMSPECTRUM;
                    // int type   = SIMSPECTRUM;

                    try 
                    {
                        out.writeDouble( (double)m_dataType );
                    } 
                    catch ( IOException e ) 
                    {
                        System.out.println( "Could not write from the .mrui file : " + this.m_mruiFileRef );
                    }
                    switch ( (int)m_dataType ) 
                    {

                        case SIGNAL:
                                        System.out.println( "Writing signal file to : " + this.m_mruiFileRef );;
                                        break;
                        case SIMSIGNAL:
                                        System.out.println( "Writing simulated signal file to : " + this.m_mruiFileRef );;
                                        break;

                        case SIMSPECTRUM:
                                        System.out.println( "Writing simulated spectrum file to : " + this.m_mruiFileRef );;
                                        break;
                        case PEAKTABLE:
                                        System.out.println( "Writing a peak table file to : " + this.m_mruiFileRef );;
                                        break;          
                        default:
                                        System.out.println( "Warning: Unclear output format!" );
                    }

                
                    // Note: first double has already been read from file!
                    out.writeDouble( m_nop );

                    // To initialise the Data object with the information
                    // in the header of the file.
                    out.writeDouble( m_setStepTime );
                    out.writeDouble( m_beginTime );
                    out.writeDouble( m_zeroOrderPhase );
                    out.writeDouble( m_transmitterFreq );
                    out.writeDouble( m_B0 );
                    out.writeDouble( m_nucleus );
                    out.writeDouble( m_hzref );
                    out.writeDouble( m_ppmref );

                    out.writeDouble( m_echoes );

                    out.writeDouble( m_apodizeView );
                    out.writeDouble( (double)m_zeroFillingView );
                    out.writeDouble( (double)m_numberOfFrames );
                    out.writeDouble( (double)m_leftPointsEcho );

                    // Skip all currently unused fields in the header of the file
                    double tempDouble = 0.0;
                    for ( int i = 0; i < 64 - 15; i++)
                    {
                        out.writeDouble( tempDouble );
                    }

                    float[] absSignal = model.getCSIData( ).getSpectra( ).getSpectrumData( voxID, Spectra.Domain.TD, Spectra.Mode.ABS );
                    float[] dispSignal = model.getCSIData( ).getSpectra( ).getSpectrumData( voxID, Spectra.Domain.TD, Spectra.Mode.DISP );

                    System.out.println( "Writing signal frame " + (frameCntr++) + " of " + m_numberOfFrames + " to file.\r");
                    for ( int i = 0; i < m_nop; i++ ) 
                    {
                        out.writeDouble( absSignal[i] );
                        out.writeDouble( dispSignal[i] );
                    }
                    out.close( );
                 }  
                 System.out.println( "File(s) written correctly." );
                } 
                catch ( java.io.IOException ex ) 
                {
                    System.out.println( "An IO-error occurred while attempting to load file : " + m_mruiFileRef );
                    return false;
                }
            }
                
        return true;
    }
    
    /**
     * 
     * Attempts to load the mrui file.
     * 
     * @param model
     * @return 
     */
    public boolean saveNumericPatternsOfModelAsMRUITypedData( GlobalModel model )
    {
        
        TDFDFitPrototypeModelSpectrumParameters protoType = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        
        LinkedList<MRNumericPattern> numericPatternsToExport = protoType.getNumericPatterns( ); 
        
        m_nop = model.getSpectraVectorSize( );
        m_dataType = 0; // only FID signals are allowed now
        m_setStepTime = 1000.0 * 1.0/model.getBandwidth( ); // unit in msec
        m_beginTime = 0.0;
        m_zeroOrderPhase = 0.0; // no phase information is exported;
        m_transmitterFreq = model.getResonanceFrequency( );
        m_B0 = model.getB0( );
        m_nucleus = 1.0;
        m_ppmref = -4.7;
        m_hzref = m_ppmref * m_transmitterFreq / 1e6;
        
        m_echoes = 0.0;
        m_apodizeView = 0.0;
        m_zeroFillingView = 0;
        m_numberOfFrames = model.getSelectedVoxels( ).size( );
        m_leftPointsEcho = 0;
        
        Iterator it = numericPatternsToExport.iterator( );
                
        while ( it.hasNext( ) )
        {
            MRNumericPattern numPat = (MRNumericPattern)it.next( );
            String fileReference = numPat.getFileReference( );
            int index = fileReference.lastIndexOf( "." );
            String outputFileReference = fileReference.substring( 0, index );
            try 
            {
                RandomAccessFile out = new RandomAccessFile( outputFileReference + ".mrui" , "rw" );
                
                System.out.println( "Writing jMRUI v4 file" );

                // the first double in the file specifies the type of data :
                // 0 for a fid
                // 1 for a simulated Spectrum
                // 2 for a simulated Fid
                // 3 for a peak table
                // int choice = SIMSPECTRUM;
                // int type   = SIMSPECTRUM;

                try 
                {
                    out.writeDouble( (double)m_dataType );
                } 
                catch ( IOException e ) 
                {
                    System.out.println( "Could not write from the .mrui file : " + outputFileReference + ".mrui" );
                }
                switch ( (int)m_dataType ) 
                {

                    case JMRUIFile.SIGNAL:
                                    System.out.println( "Writing signal file to : " + outputFileReference + ".mrui" );
                                    break;
                    case JMRUIFile.SIMSIGNAL:
                                    System.out.println( "Writing simulated signal file to : " + outputFileReference + ".mrui" );
                                    break;

                    case JMRUIFile.SIMSPECTRUM:
                                    System.out.println( "Writing simulated spectrum file to : " + outputFileReference + ".mrui" );
                                    break;
                    case JMRUIFile.PEAKTABLE:
                                    System.out.println( "Writing a peak table file to : " + outputFileReference + ".mrui" );
                                    break;          
                    default:
                                    System.out.println( "Warning: Unclear output format!" );
                }

                try 
                {
                    // Note: first double has already been read from file!
                    out.writeDouble( m_nop );

                    // To initialise the Data object with the information
                    // in the header of the file.
                    out.writeDouble( m_setStepTime );
                    out.writeDouble( m_beginTime );
                    out.writeDouble( m_zeroOrderPhase );
                    out.writeDouble( m_transmitterFreq );
                    out.writeDouble( m_B0 );
                    out.writeDouble( m_nucleus );
                    out.writeDouble( m_hzref );
                    out.writeDouble( m_ppmref );

                    out.writeDouble( m_echoes );

                    out.writeDouble( m_apodizeView );
                    out.writeDouble( (double)m_zeroFillingView );
                    out.writeDouble( (double)m_numberOfFrames );
                    out.writeDouble( (double)m_leftPointsEcho );

                    // Skip all currently unused fields in the header of the file
                    double tempDouble = 0.0;
                    for ( int i = 0; i < 64 - 15; i++)
                    {
                        out.writeDouble( tempDouble );
                    }

                    MRLineModelSpectralResponse response = numPat.getStaticSpectralResponse( MRSignalProperties.getInstance( ) );
                    
                    float[] abs  = response.getAbsorptionChannelTD( );
                    float[] disp = response.getDispersionChannelTD( );

                    int frameCntr =0;
                       
                    System.out.println( "Writing numeric pattern to : " + outputFileReference + ".mrui" );
                    for ( int i = 0; i < m_nop; i++ ) 
                    {
                        out.writeDouble( abs[i] );
                        out.writeDouble( -disp[i] );
                    }
 
                    System.out.println( "File written correctly." );
                    out.close( );

                } 
                catch ( java.io.IOException ex ) 
                {
                    System.out.println( "An IO-error occurred while attempting to load file : " + m_mruiFileRef );
                    return false;
                }
            }
            catch ( FileNotFoundException ex ) 
            {
                System.out.println( "File not found : " + m_mruiFileRef );
                return false;
            }
        }
        return true;
    }
    
    
    
    
    
    /**
     * Returns the absorption TD response that was loaded from the .mrui file.
     * @return
     */
    public float[] getAbsorbtionTDResponse( )
    {
        return m_absorptionChannelTD;
    }
    
    /**
     * Returns the absorption TD response that was loaded from the .mrui file.
     * @return
     */
    public float[] getDispersionTDResponse( )
    {
        return m_dispersionChannelTD;
    }
    
}
