 /*
 * File: $Id: MRFittingParameters.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 16.07.2003, 7:23  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 

package org.spectrim.tdfdfit;

import java.io.FileNotFoundException;
import java.io.RandomAccessFile;
import java.io.IOException;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.spectrim.xyplot.Curve;
import org.spectrim.xyplot.XYPlotFrame;
import static org.spectrim.tdfdfit.TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME;

/**
 * Container class to store a basic set of TDFDFit fitting parameters
 * 
 * Assumed is here that this such a basic set is related to a Voigt line shape
 * 
 * New (02.2017): the associated response is also computed in the constructor.
 * @author  hans
 */
public class MRFittingParameters implements Cloneable
{
    
    /** 
     * Creates a new instance of MRFittingParameters. 
     */
    public MRFittingParameters( ) 
    {
        
        DefaultFittingParameterRangeValues defs = DefaultFittingParameterRangeValues.getInstance( );
        
        // give this object a UID
        m_name = "Line";
        try 
        {
            m_uid = "" + InetAddress.getLocalHost( ).getHostAddress( ) + System.currentTimeMillis( );
            
            // System.out.println( "uid : " + m_uid );
            
        } catch ( UnknownHostException ex ) 
        {
            Logger.getLogger( MRFittingParameters.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
	m_fitWithReferenceLineShapeFunction = false;
	m_fitWithArbitraryLineShapeFunction = false;

	m_modelType                 = MRLINE_MODEL;
	m_fileReferenceName         = "NILL";	
	m_lineGroup                 = "NILL";			
	m_metabolite                = "NILL";	
	
	m_area                      = 1f;
	m_gaussianWidth             = 0f;
	m_offset                    = 0f;
	m_lorentzianWidth           = 0f;
	m_phase                     = 0f;
	
	m_minimumArea               = 0f;
	m_minimumGaussianWidth      = defs.getGaussianWidthMinValDecimalValue( );
	m_minimumOffset             = defs.getOffsetMinValDecimalValue( );
	m_minimumLorentzianWidth    = defs.getLorentzianWidthMinValDecimalValue( );
	m_minimumPhase              = defs.getPhaseMinValDecimalValue( );

	m_maximumArea               = defs.getAreaMaxValDecimalValue( );
	m_maximumGaussianWidth      = defs.getGaussianWidthMaxValDecimalValue( );
	m_maximumOffset             = defs.getOffsetMaxValDecimalValue( );
	m_maximumLorentzianWidth    = defs.getLorentzianWidthMaxValDecimalValue( );
	m_maximumPhase              = defs.getPhaseMaxValDecimalValue( );

	m_areaReference             = "Free";
	m_offsetReference           = "Free";
	m_gaussianWidthReference    = "Free";
	m_phaseReference            = "Free";
	m_lorentzianWidthReference  = "Free";
        
        m_response = new MRLineModelSpectralResponse( this, MRSignalProperties.getInstance( ) );
    }
    
    /** 
     * Creates a new instance of MRFittingParameters. 
     * @param name
     * @param modelType
     */
    public MRFittingParameters( String name, int modelType ) 
    {
        m_name = name;
        
        // give this object a UID
        try 
        {
            m_uid = "" + InetAddress.getLocalHost( ).getHostAddress( ) + System.currentTimeMillis( );
            // System.out.println( "uid : " + m_uid );
        } 
        catch ( UnknownHostException ex ) 
        {
            Logger.getLogger( MRFittingParameters.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
        
	m_fitWithReferenceLineShapeFunction = false;
	m_fitWithArbitraryLineShapeFunction = false;

	m_modelType                 = modelType;
        
	m_fileReferenceName         = "NILL";	
	m_lineGroup                 = "NILL";			
	m_metabolite                = "NILL";	
	
	m_area                      = 1f;
	m_gaussianWidth             = 0f;
	m_offset                    = 0f;
        
        if ( modelType != MRLINE_MODEL )
        {
            m_lorentzianWidth       = 0f;
        }
        else
        {
            m_lorentzianWidth       = 2f;
        }
        
	m_phase                     = 0f;
	
	m_minimumArea               = 0f;
	m_minimumGaussianWidth      = -1f;
	m_minimumOffset             = -6f;
	m_minimumLorentzianWidth    = 0f;
	m_minimumPhase              = -45f;

	m_maximumArea               = 2f;
	m_maximumGaussianWidth      = 1f;
	m_maximumOffset             = 6f;
	m_maximumLorentzianWidth    = 10f;
	m_maximumPhase              = 45f;

	m_areaReference             = "Free";
	m_offsetReference           = "Free";
	m_gaussianWidthReference    = "Free";
	m_phaseReference            = "Free";
	m_lorentzianWidthReference  = "Free";
        
        m_response = new MRLineModelSpectralResponse( this, MRSignalProperties.getInstance( ) );
        
    }
    
    /**
     * Constructor with known parameter values.
     * @param name              - name of the component
     * @param modelType         - modelType 
     * @param file_reference    - reference to the file system where >'this' is stored
     * @param line_group        - reference to which pattern it belongs
     * @param metabolite        - reference to which metabolite the component belongs
     * @param fitlsf            - whether line shape fit should be performed for this line
     * @param refline           - whether this line is the reference line
     * @param area              - area parameter
     * @param offset            - chemical shift parameter
     * @param g_width           - Gaussian width parameter
     * @param phase             - Phase parameter
     * @param l_width           - Lorentzian width parameter
     * @param area_min          - Minimal area parameter
     * @param offset_min        - Minimal offset parameter
     * @param g_width_min       - Minimal Gaussian width parameter
     * @param phase_min         - Minimal Phase parameter
     * @param l_width_min       - Minimal Lorentzian width parameter
     * @param area_max          - Maximal area parameter
     * @param offset_max        - Maximal offset parameter
     * @param g_width_max       - Maximal Gaussian width parameter
     * @param phase_max         - Maximal Phase parameter
     * @param l_width_max       - Maximal Lorentzian width parameter
     * @param area_prior        - Prior knowledge reference for area parameter
     * @param offset_prior      - Prior knowledge reference for offset parameter
     * @param g_width_prior     - Prior knowledge reference for Gaussian parameter
     * @param phase_prior       - Prior knowledge reference for phase parameter
     * @param l_width_prior     - Prior knowledge reference for Lorentzian parameter
     */
    public MRFittingParameters( String name, int modelType, String file_reference,
                                String line_group, String metabolite, boolean fitlsf, boolean refline,
                                float area, float offset, float g_width, float phase, float l_width,
                                float area_min, float offset_min, float g_width_min, float phase_min, float l_width_min, 
                                float area_max, float offset_max, float g_width_max, float phase_max, float l_width_max,
                                String area_prior, String offset_prior, String g_width_prior, String phase_prior, String l_width_prior )
    {
	m_name = name;
        try 
        {
            m_uid = "" + InetAddress.getLocalHost( ).getHostAddress( ) + System.currentTimeMillis();
        } catch ( UnknownHostException ex ) 
        {
            Logger.getLogger( MRFittingParameters.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
	m_fitWithReferenceLineShapeFunction = refline ; // is response multiplied with lsf function y/n
	m_fitWithArbitraryLineShapeFunction = fitlsf;   // is response multiplied with reference line y/n

	m_modelType                 = modelType;	// line , mod. pattern, num. patterns
	m_fileReferenceName         = file_reference;	// reference to file where pattern can be found
	m_lineGroup                 = line_group;	// group the line belongs to
	m_metabolite                = metabolite;	// metabolite the line belongs to
	
        m_area                      = area;
	m_offset                    = offset;	
	m_gaussianWidth             = g_width;
	m_phase                     = phase;
	m_lorentzianWidth           = l_width;

	m_minimumArea               = area_min;
	m_minimumOffset             = offset_min;	
	m_minimumGaussianWidth      = g_width_min;
	m_minimumPhase              = phase_min;
	m_minimumLorentzianWidth    = l_width_min;

	m_maximumArea               = area_max;
	m_maximumOffset             = offset_max;	
	m_maximumGaussianWidth      = g_width_max;
	m_maximumPhase              = phase_max;
	m_maximumLorentzianWidth    = l_width_max;

	m_areaReference             = area_prior;
	m_offsetReference           = offset_prior;
	m_gaussianWidthReference    = g_width_prior;
	m_phaseReference            = phase_prior;
	m_lorentzianWidthReference  = l_width_prior;
        
        m_response = new MRLineModelSpectralResponse( this, MRSignalProperties.getInstance( ) );
        
    }
    
    /**
     * Sets the name of the reference file.
     * @param fileReference
     */
    public void setFileReferenceName( String fileReference )
    {
	m_fileReferenceName = fileReference;
    }
    
    /**
     * Sets the model type of this line.
     * This can be LINE_MODELTYPE, PATTERN_MODELTYPE or NUMERIC_PATTERN_MODELTYPE
     * @param modelType
     */
    public void setModelType( int modelType )
    {
	m_modelType = modelType;
    }
    
    /**
     * Sets the name of the model.
     * @param name
     */
    public void setName( String name )
    {
	m_name = name;
    }
    
    /**
     * Sets the UID of the model.
     * @param m_uid
     */
    public void set_UID(String m_uid) 
    {
        this.m_uid = m_uid;
    }
    /**
     * Sets the metabolite name the line or pattern belongs to.
     * @param metabolite
     */
    public void setMetabolite( String metabolite )
    {
	m_metabolite = metabolite;
    }
    
    /**
     * Sets the line group the line or pattern belongs to.
     * @param lineGroup
     */
    public void setLineGroup( String lineGroup )
    {
	m_lineGroup = lineGroup;
    }

    
    /**
     * Sets the peak area parameter value.
     * @param area
     */
    public void setArea( float area )
    {
	m_area = area;
        m_response = new MRLineModelSpectralResponse( this, MRSignalProperties.getInstance( ) );
    }

    /**
     * Sets the offset frequency value of peak (resonance frequency).
     * @param offset
     */
    public void setOffset( float offset )
    {
	m_offset = offset;
        m_response = new MRLineModelSpectralResponse( this, MRSignalProperties.getInstance( ) );
    }

//    /**
//     * Sets the offset frequency value in ppm of peak (resonance frequency).
//     * @param offsetInPPM
//     */
//    public void setOffsetInPPM( float offsetInPPM )
//    {
//        MRSignalProperties props = MRSignalProperties.getInstance( );
//        float ppmRange = props.getAcquistitionBandwidth( ) / ( props.getResonanceFrequency( ) / 1000000f );
//        m_offset = - ( props.getNumberOfSamplePoints( ) / ppmRange ) * offsetInPPM + (  props.getNumberOfSamplePoints( ) / ppmRange ) * props.getPPMValueInCenterOfSpectrum( );
//        m_response = new MRLineModelSpectralResponse( this, MRSignalProperties.getInstance( ) );
//    }
//    
//    /**
//     * Sets the minimum offset frequency value in ppm of peak (resonance frequency).
//     * @param minimumOffsetInPPM
//     */
//    public void setMininmumOffsetInPPM( float minimumOffsetInPPM )
//    {
//        MRSignalProperties props = MRSignalProperties.getInstance( );
//        float ppmRange = props.getAcquistitionBandwidth( ) / ( props.getResonanceFrequency( ) / 1000000f );
//        m_minimumOffset = - ( props.getNumberOfSamplePoints( ) / ppmRange ) * minimumOffsetInPPM + (  props.getNumberOfSamplePoints( ) / ppmRange ) * props.getPPMValueInCenterOfSpectrum( );
//        m_response = new MRLineModelSpectralResponse( this, MRSignalProperties.getInstance( ) );
//    }
//    
//    /**
//     * Sets the maximum offset frequency value in ppm of peak (resonance frequency).
//     * @param maximumOffsetInPPM
//     */
//    public void setMaxinmumOffsetInPPM( float maximumOffsetInPPM )
//    {
//        MRSignalProperties props = MRSignalProperties.getInstance( );
//        float ppmRange = props.getAcquistitionBandwidth( ) / ( props.getResonanceFrequency( ) / 1000000f );
//        m_maximumOffset = - ( props.getNumberOfSamplePoints( ) / ppmRange ) * maximumOffsetInPPM + (  props.getNumberOfSamplePoints( ) / ppmRange ) * props.getPPMValueInCenterOfSpectrum( );
//        m_response = new MRLineModelSpectralResponse( this, MRSignalProperties.getInstance( ) );
//    }
//    
    
    
    /** 
     * 
     * Converts the displayed unit to [Hz]-unit, which is the internal units
     * of TDFDFit;
     * Needed to undo the change in value of the display
     * @param offsetValue 
     */
    public void setOffsetFromDisplayUnits( float offsetValue )
    {

        TDFDFittingDefaults fitDef = TDFDFittingDefaults.getInstance( );
        fitDef.load( );
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
        {
            // this is simple, we are done!
            m_offset = offsetValue; 
        }
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
        {
            MRSignalProperties props = MRSignalProperties.getInstance( );
            m_offset = ( offsetValue / props.getAcquistitionBandwidth( ) ) * props.getNumberOfSamplePoints( );
        }
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
        {
            MRSignalProperties props = MRSignalProperties.getInstance( );
            
            float ppmRange = props.getAcquistitionBandwidth( ) / ( props.getResonanceFrequency( ) / 1000000f );
            m_offset = - ( props.getNumberOfSamplePoints( ) / ppmRange ) * offsetValue + (  props.getNumberOfSamplePoints( ) / ppmRange ) * props.getPPMValueInCenterOfSpectrum( );
        }
        
    }
    
    /** 
     * 
     * Converts the displayed unit to [Hz]-unit, which is the internal units
     * of TDFDFit;
     * Needed to undo the change in value of the display
     * @param offsetValue 
     */
    public void setMinimumOffsetFromDisplayUnits( float offsetValue )
    {

        TDFDFittingDefaults fitDef = TDFDFittingDefaults.getInstance( );
        fitDef.load( );
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
        {
            // this is simple, we are done!
            m_minimumOffset = offsetValue; 
        }
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
        {
            MRSignalProperties props = MRSignalProperties.getInstance( );
            m_minimumOffset = ( offsetValue / props.getAcquistitionBandwidth( ) ) * props.getNumberOfSamplePoints( );
        }
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
        {
            MRSignalProperties props = MRSignalProperties.getInstance( );
            
            float ppmRange = props.getAcquistitionBandwidth( ) / ( props.getResonanceFrequency( ) / 1000000f );
            m_minimumOffset = - ( props.getNumberOfSamplePoints( ) / ppmRange ) * offsetValue + (  props.getNumberOfSamplePoints( ) / ppmRange ) * props.getPPMValueInCenterOfSpectrum( );
        }
    } 
    
    /** 
     * 
     * Converts the displayed unit to [Hz]-unit, which is the internal units
     * of TDFDFit;
     * Needed to undo the change in value of the display
     * @param offsetValue 
     */
    public void setMaximumOffsetFromDisplayUnits( float offsetValue )
    {

        TDFDFittingDefaults fitDef = TDFDFittingDefaults.getInstance( );
        fitDef.load( );
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
        {
            // this is simple, we are done!
            m_maximumOffset = offsetValue;
        }
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
        {
            MRSignalProperties props = MRSignalProperties.getInstance( );
            m_maximumOffset = ( offsetValue / props.getAcquistitionBandwidth( ) ) * props.getNumberOfSamplePoints( );
        }
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
        {
            MRSignalProperties props = MRSignalProperties.getInstance( );
            
            float ppmRange = props.getAcquistitionBandwidth( ) / ( props.getResonanceFrequency( ) / 1000000f );
            m_maximumOffset = - ( props.getNumberOfSamplePoints( ) / ppmRange ) * offsetValue + (  props.getNumberOfSamplePoints( ) / ppmRange ) * props.getPPMValueInCenterOfSpectrum( );
        }
    }
    
    /**
     * Sets the Lorentzian width parameter value of the resonance line.
     * @param lorentzianWidth
     */
    public void setLorentzianWidth( float lorentzianWidth )
    {
	m_lorentzianWidth = lorentzianWidth;
        m_response = new MRLineModelSpectralResponse( this, MRSignalProperties.getInstance( ) );
    }

    /**
     * Sets the Lorentzian zero order phase parameter value of 
     * the resonance line.
     * @param phase
     */
    public void setPhase( float phase )
    {
	m_phase = phase;
        m_response = new MRLineModelSpectralResponse( this, MRSignalProperties.getInstance( ) );
    }

    /**
     * Sets the Gaussian width parameter value of the resonance line.
     * @param gaussianWidth
     */
    public void setGaussianWidth( float gaussianWidth )
    {
	m_gaussianWidth = gaussianWidth;
        m_response = new MRLineModelSpectralResponse( this, MRSignalProperties.getInstance( ) );
    }

    /**
     * Sets whether the line shape should also be determined by a separate 
     * line shape function, that is determined during the fit.
     * @param yesorno
     */
    public void setFitWithArbitraryLineShapeFunction( boolean yesorno )
    {
	m_fitWithArbitraryLineShapeFunction = yesorno;
    }

    /**
     * Sets whether the line shape should also be determined with a separate 
     * line shape function, that is obtained from a file and is static 
     * during fitting.
     * @param yesorno
     */
    public void setFitWithReferenceLineShape( boolean yesorno )
    {
	m_fitWithReferenceLineShapeFunction = yesorno;
    }

    /**
     * Sets the minimum allowed value of the area parameter.
     * @param minimumArea
     */
    public void setMinimumArea( float minimumArea )
    {
	m_minimumArea = minimumArea;
    }

    /**
     * Sets the minimum allowed value of the offset frequency parameter.
     * @param minimumOffset
     */
    public void setMinimumOffset( float minimumOffset )
    {
	m_minimumOffset = minimumOffset;
    }
    
    /**
     * Sets the minimum allowed value of the Lorentzian width parameter.
     * @param minimumLorentzianWidth
     */
    public void setMinimumLorentzianWidth( float minimumLorentzianWidth )
    {
	m_minimumLorentzianWidth = minimumLorentzianWidth;
    }

    /**
     * Sets the minimum allowed value of the phase parameter.
     * @param minimumPhase
     */
    public void setMinimumPhase( float minimumPhase )
    {
	m_minimumPhase = minimumPhase;
    }

    /**
     * Sets the minimum allowed value of the Gaussian width parameter.
     * @param minimumGaussianWidth
     */
    public void setMinimumGaussianWidth( float minimumGaussianWidth )
    {
	m_minimumGaussianWidth = minimumGaussianWidth;
    }

    /**
     * Sets the maximum allowed value of the area parameter.
     * @param maximumArea
     */
    public void setMaximumArea( float maximumArea )
    {
	m_maximumArea = maximumArea;
    }

    /**
     * Sets the maximum allowed value of the offset frequency parameter.
     * @param maximumOffset
     */
    public void setMaximumOffset( float maximumOffset )
    {
	m_maximumOffset = maximumOffset;
    }

    /**
     * Sets the maximum allowed value of the Lorentzian line width parameter.
     * @param maximumLorentzianWidth
     */
    public void setMaximumLorentzianWidth( float maximumLorentzianWidth )
    {
	m_maximumLorentzianWidth = maximumLorentzianWidth;
    }

    /**
     * Sets the maximum allowed value of the zero order phase parameter.
     * @param maximumPhase
     */
    public void setMaximumPhase( float maximumPhase )
    {
	m_maximumPhase = maximumPhase;
    }

    /**
     * Sets the maximum allowed value of the Gaussian width parameter.
     * @param maximumGaussianWidth
     */
    public void setMaximumGaussianWidth( float maximumGaussianWidth )
    {
	m_maximumGaussianWidth = maximumGaussianWidth;
    }

    /**
     * Sets the name of the to which the area parameter is fixed
     * (prior knowledge).
     * @param areaReference
     */    
    public void setAreaReference( String areaReference )
    {
	m_areaReference = areaReference;
    }

    /**
     * Sets the name of the line or pattern to which the offset parameter is fixed 
     * (prior knowledge).
     * @param offsetReference
     */  
    public void setOffsetReference( String offsetReference )
    {
	m_offsetReference = offsetReference;
    }
    
    /**
     * Sets the name of the line or pattern to which the phase parameter is fixed 
     * (prior knowledge).
     * @param phaseReference
     */  
    public void setPhaseReference( String phaseReference )
    {
	m_phaseReference = phaseReference;
    }
    
    /**
     * Sets the name of the line or pattern to which the Gaussian width parameter 
     * is fixed (prior knowledge).
     * @param gaussianWidthReference
     */  
    public void setGaussianWidthReference( String gaussianWidthReference )
    {
	m_gaussianWidthReference = gaussianWidthReference;
    }
    
    /**
     * Sets the name of the line or pattern to which the Lorentzian width parameter 
     * is fixed (prior knowledge).
     * @param lorentzianWidthReference
     */
    public void setLorentzianWidthReference( String lorentzianWidthReference )
    {
	m_lorentzianWidthReference = lorentzianWidthReference;
    }
    
    /**
     * Returns the name of the line or pattern.
     * @return 
     */
    public String getName( )
    {
	return m_name;
    }
    
    
    /**
     * Returns the UID of the line or pattern.
     * @return 
     */
    public String get_UID( ) 
    {
        return m_uid;
    }

    /**
     * Returns the name of the reference file name.
     * @return 
     */
    public String getFileReferenceName( )
    {
	return m_fileReferenceName;
    }
    
    /**
     * Returns the name of the metabolite this MRLineModelParameter object
     * belongs to.
     * @return 
     */
    public String getMetabolite( )
    {
	return m_metabolite;
    }

    /**
     * Returns the name of the line group this MRLineModelParameter object
     * belongs to.
     * @return 
     */
    public String getLineGroup( )
    {
	return m_lineGroup;
    }
    
    /**
     * Returns the model type of the line
     * This can be LINE_MODELTYPE, PATTERN_MODELTYPE or NUMERIC_PATTERN_MODELTYPE.
     * @return 
     */
    public int getModelType( )
    {
	return m_modelType;
    }
    
    /**
     * Returns a boolean indicating whether line shape function should be fitted.
     * @return 
     */
    public boolean getFitWithArbitraryLineShapeFunction( )
    {
	return m_fitWithArbitraryLineShapeFunction;
    }

    /**
     * Returns whether the line is fitted multiplied with a (measured) reference 
     * line shape.
     * @return 
     */
    public boolean getFitWithReferenceLineShape( )
    {
	return m_fitWithReferenceLineShapeFunction;
    }

    /**
     * Returns the area parameters.
     * @return 
     */
    public float getArea( )
    {
	return m_area ;
    }

    /**
     * Returns the offset parameter.
     * @return 
     */
    public float getOffset( )
    {
	return m_offset;
    }
    
    /** 
     * 
     * Converts the internal point unit to the user set display unit.
     * 
     * @return 
     */
    public float getOffsetInDisplayUnits( )
    {
        float returnValue =  0f;
        
        TDFDFittingDefaults fitDef = TDFDFittingDefaults.getInstance( );
        fitDef.load( );
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
        { 
            returnValue = m_offset;
        }
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
        {
            MRSignalProperties props = MRSignalProperties.getInstance( );
            returnValue = ( m_offset  /  props.getNumberOfSamplePoints( ) ) * props.getAcquistitionBandwidth( );
        }
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
        {
            MRSignalProperties props = MRSignalProperties.getInstance( );
            float ppmRange =  props.getAcquistitionBandwidth( ) / ( props.getResonanceFrequency( ) / 1000000f );
            returnValue = - m_offset * ppmRange / props.getNumberOfSamplePoints( ) + MRSignalProperties.getInstance( ).getPPMValueInCenterOfSpectrum( );
        }
        
        return returnValue;
    }
    
    /**
     * Returns the Lorentzian line width parameter.
     * @return 
     */
    public float getLorentzianLineWidth( )
    {
	return m_lorentzianWidth;
    }
    
    /**
     * Returns the zero phase parameter.
     * @return 
     */
    public float getPhase( )
    {
	return m_phase;
    }
    
    /**
     * Returns the Lorentzian line width parameters.
     * @return 
     */
    public float getGaussianWidth( )
    {
	return m_gaussianWidth;
    }

    /**
     * Returns the minimum area parameter.
     * @return 
     */
    public float getMinimumArea( )
    {
	return m_minimumArea;
    }

    /** 
     * 
     * Converts the internal point unit to the user set display unit.
     * 
     * @return 
     */
    public float getMinimumOffsetInDisplayUnits( )
    {
        float returnValue =  0f;
        
        TDFDFittingDefaults fitDef = TDFDFittingDefaults.getInstance( );
        fitDef.load( );
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
        { 
            returnValue = m_minimumOffset;
        }
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
        {
            MRSignalProperties props = MRSignalProperties.getInstance( );
            returnValue = ( m_minimumOffset  /  props.getNumberOfSamplePoints( ) ) * props.getAcquistitionBandwidth( );
        }
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
        {
            MRSignalProperties props = MRSignalProperties.getInstance( );
            float ppmRange =  props.getAcquistitionBandwidth( ) / ( props.getResonanceFrequency( ) / 1000000f );
            returnValue = - m_minimumOffset * ppmRange / props.getNumberOfSamplePoints( ) + MRSignalProperties.getInstance( ).getPPMValueInCenterOfSpectrum( );
        }
        
        return returnValue;
    } 
    
    
    /** 
     * 
     * Converts the internal point unit to the user set display unit.
     * 
     * @return 
     */
    public float getMaximumOffsetInDisplayUnits( )
    {
        float returnValue =  0f;
        
        TDFDFittingDefaults fitDef = TDFDFittingDefaults.getInstance( );
        fitDef.load( );
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
        { 
            returnValue = m_maximumOffset;
        }
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
        {
            MRSignalProperties props = MRSignalProperties.getInstance( );
            returnValue = ( m_maximumOffset  /  props.getNumberOfSamplePoints( ) ) * props.getAcquistitionBandwidth( );
        }
        
        if ( fitDef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
        {
            MRSignalProperties props = MRSignalProperties.getInstance( );
            float ppmRange =  props.getAcquistitionBandwidth( ) / ( props.getResonanceFrequency( ) / 1000000f );
            returnValue = - m_maximumOffset * ppmRange / props.getNumberOfSamplePoints( ) + MRSignalProperties.getInstance( ).getPPMValueInCenterOfSpectrum( );
        }
        
        return returnValue;
    } 
    
    
    /**
     * Returns the minimum offset parameter.
     * @return 
     */
    public float getMinimumOffset( )
    {
	return m_minimumOffset;
    }

    /**
     * Returns the minimum Lorentzian width parameter.
     * @return 
     */
    public float getMinimumLorentzianWidth( )
    {
	return m_minimumLorentzianWidth;
    }

    /**
     * Returns the minimum zero order phase parameter.
     * @return 
     */
    public float getMinimumPhase( )
    {
	return m_minimumPhase;
    }

    /**
     * Returns the minimum Gaussian width parameter.
     * @return 
     */    
    public float getMinimumGaussianWidth( )
    {
	return m_minimumGaussianWidth;
    }

    /**
     * Returns the maximum peak area parameter.
     * @return 
     */
    public float getMaximumArea( )
    {
	return m_maximumArea;
    }
    
    /**
     * Returns the maximum peak offset parameter.
     * @return 
     */
    public float getMaximumOffset( )
    {
	return m_maximumOffset;
    }

    /**
     * Returns the maximum Lorentzian width parameter.
     * @return 
     */
    public float getMaximumLorentzianWidth( )
    {
	return m_maximumLorentzianWidth;
    }

    /**
     * Returns the maximum zero order phase parameter.
     * @return 
     */
    public float getMaximumPhase( )
    {
	return m_maximumPhase;
    }

    /**
     * Returns the maximum Gaussian width parameter.
     * @return 
     */
    public float getMaximumGaussianWidth( )
    {
	return m_maximumGaussianWidth;
    }

    /**
     * Returns the name of the to which the area parameter is fixed.
     * @return 
     */
    public String getAreaReference( )
    {
	return m_areaReference;
    }
    
    /**
     * Returns the name of the to which the area parameter is fixed.
     * @return 
     */
    public String getOffsetReference( )
    {
	return m_offsetReference;
    }
    
    /**
     * Returns the name of the to which the Lorentzian parameter is fixed
     * in oder to define prior knowledge. 
     * @return 
     */
    public String getLorentzianLineWidthReference( )
    {
	return m_lorentzianWidthReference;
    }
    
    /**
     * Returns the name of the to which the area parameter is fixed
     * in oder to define prior knowledge. 
     * @return 
     */
    public String getPhaseReference( )
    {
	return m_phaseReference;
    }
    
    /**
     * Returns the name of the to which the area parameter is fixed.
     * @return 
     */
    public String getGaussianWidthReference( )
    {
	return m_gaussianWidthReference;
    }
    
    /**
     * Sets the prior knowledge area value.
     * @param areaReferenceVal
     */
    public void setAreaReferenceVal( float areaReferenceVal )
    {
	m_areaReferenceVal = areaReferenceVal;
    }

    /**
     * Sets the prior knowledge offset value.
     * @param offsetReferenceVal
     */
    public void setOffsetReferenceVal( float offsetReferenceVal )
    {
	m_offsetReferenceVal = offsetReferenceVal;
    }
    
    /**
     * Sets the prior knowledge Lorentzian width value.
     * @param lorentzianWidthReferenceVal
     */
    public void setLorenzianWidthReferenceVal( float lorentzianWidthReferenceVal )
    {
	m_lorentzianWidthReferenceVal = lorentzianWidthReferenceVal;
    }

    public void setPhaseReferenceVal( float phaseReferenceVal )
    {
	m_phaseReferenceVal = phaseReferenceVal;
    }

    public void setGaussianWidthReferenceVal( float gaussianWidthReferenceVal )
    {
	m_gaussianWidthReferenceVal = gaussianWidthReferenceVal;
    }

    /**
     * Returns the area prior knowledge value.
     * @return 
     */
    public float getAreaReferenceVal( )
    {
	return m_areaReferenceVal ;
    }

    /**
     * Returns the offset prior knowledge value.
     * @return 
     */ 
    public float getOffsetReferenceVal( )
    {
	return m_offsetReferenceVal;
    }

    /**
     * Returns the Lorentzian linewidth prior knowledge value.
     * @return 
     */
    public float getLorentzianWidthReferenceVal( )
    {
	return m_lorentzianWidthReferenceVal;
    }

    /**
     * Returns the phase prior knowledge value.
     * @return 
     */
    public float getPhaseReferenceVal( )
    {
        return m_phaseReferenceVal;
    }

    /**
     * Returns the Gaussian Width prior knowledge value.
     * @return 
     */
    public float getGaussianWidthPriorVal( )
    {
        return m_gaussianWidthReferenceVal;
    }
    
    /**
     * Writes the object attributes to a File.
     * @param writer
     */
    public void write( RandomAccessFile writer )
    {
	try
        {
            writer.writeUTF( "<MRFittingParameters>" );
            
            writer.writeUTF( "<MRFittingParametersHeader>" );
            writer.writeUTF( m_name );              // write the name of the component
            writer.writeInt( m_modelType );         // line , mod. pattern, num. patterns
            writer.writeUTF( m_fileReferenceName ); // reference to file where pattern can be found
            writer.writeUTF( m_lineGroup );         // group the line belongs to
            writer.writeUTF( m_metabolite );        // metabolite the line belongs to
        
            writer.writeFloat( m_area );
            writer.writeFloat( m_offset ); 
            writer.writeFloat( m_gaussianWidth ); 
            writer.writeFloat( m_phase ); 
            writer.writeFloat( m_lorentzianWidth );
            
            writer.writeFloat( m_minimumArea ); 
            writer.writeFloat( m_minimumOffset ); 
            writer.writeFloat( m_minimumGaussianWidth ); 
            writer.writeFloat( m_minimumPhase ); 
            writer.writeFloat( m_minimumLorentzianWidth );
            
            writer.writeFloat( m_maximumArea );
            writer.writeFloat( m_maximumOffset ); 
            writer.writeFloat( m_maximumGaussianWidth ); 
            writer.writeFloat( m_maximumPhase ); 
            writer.writeFloat( m_maximumLorentzianWidth );
        
            writer.writeUTF( m_areaReference );
            writer.writeUTF( m_offsetReference );
            writer.writeUTF( m_gaussianWidthReference );
            writer.writeUTF( m_phaseReference );
            writer.writeUTF( m_lorentzianWidthReference );
            
            writer.writeUTF( "</MRFittingParametersHeader>");
            
            m_response.write( writer );
            
            writer.writeUTF( "</MRFittingParameters>" );
            
        }
        catch ( IOException e )
        {
            System.out.println( "IO-exception writing response .." );
        } 
    }

    /**
     * Reads object attributes from file.
     * @param reader
     */
    public void read( RandomAccessFile reader )
    {
        String dummy;
        try
        {
            
            dummy =                     reader.readUTF( );  // <MRFittingParametersHeader>
            // System.out.println( "tag = " + dummy );
            
            m_name =                    reader.readUTF( );  // name of the line or pattern    
            m_modelType =               reader.readInt( );  // line , mod. pattern, num. patterns
            m_fileReferenceName =       reader.readUTF( );  // reference to file where pattern can be found
            m_lineGroup =               reader.readUTF( );  // group the line belongs to
            m_metabolite =              reader.readUTF( );  // metabolite the line belongs to
        
            // Read parameter values
            m_area =                    reader.readFloat( );
            m_offset =                  reader.readFloat( );
            m_gaussianWidth =           reader.readFloat( );
            m_phase =                   reader.readFloat( );
            m_lorentzianWidth =         reader.readFloat( );
        
            // Read minimum allowed parameter values
            m_minimumArea =             reader.readFloat( );
            m_minimumOffset =           reader.readFloat( );
            m_minimumGaussianWidth =    reader.readFloat( );
            m_minimumPhase =            reader.readFloat( );
            m_minimumLorentzianWidth =  reader.readFloat( );
        
            // Read maximum allowed parameter values
            m_maximumArea =             reader.readFloat( );
            m_maximumOffset =           reader.readFloat( );
            m_maximumGaussianWidth =    reader.readFloat( );
            m_maximumPhase =            reader.readFloat( );
            m_maximumLorentzianWidth =  reader.readFloat( );
        
            // Read prior knowledge strings
            m_areaReference =           reader.readUTF( );
            m_offsetReference =         reader.readUTF( );
            m_gaussianWidthReference =  reader.readUTF( );
            m_phaseReference =          reader.readUTF( );
            m_lorentzianWidthReference = reader.readUTF( );
            
            dummy= reader.readUTF( );    //"</MRFittingParametersHeader>" 
            // System.out.println( "tag = " + dummy );
            
            m_response = new MRLineModelSpectralResponse( );
            
            m_response.read( reader, false );
            
            // read the closing tag
            dummy = reader.readUTF( );      //"</MRFittingParameters>"
            // System.out.println( "tag = " + dummy );
            
        }
        catch ( IOException e )
        {
            System.out.println( "IO-exception reading response ..." );
        }
    }
    
    /**
     * Returns currently only the name of the line; this name is needed by 
     * the tree view displaying the line. 
     */
    @Override
    public String toString( )
    {
        return m_name;
    }
    
    /**
     * Returns the current response belonging the the MRFittingParameters 
     * object. 
     * @return 
     */
    public MRLineModelSpectralResponse getResponse( )
    {
       return m_response;
    }
    
    /**
     * Writes the current parametric pattern object as numerical pattern
     * the fitting parameters object of the numerical pattern is set to
     * numerical values -> ( 1.00, 0.00, 0.00, 0.00, 0.00 ).
     */
    public void exportMRFittingParametersAsNumericalPattern( )
    {
        
        TDFDFittingDefaults defaults = TDFDFittingDefaults.getInstance( );
        defaults.load( );
        
        
        String path = defaults.getProperty(DEFAULT_TDFDFIT_FIT_MODEL_NAME );
        String numericPatternDir = System.getProperty( "file.separator" ) + 
                                                       "Numeric_Patterns" + 
                                                       System.getProperty( "file.separator" );
        
        String tdfdfitResponseDir = System.getProperty( "file.separator" ) + 
                                                        "tdfdfit_responses" + 
                                                        System.getProperty( "file.separator" );
        String directory = path + numericPatternDir;
        
        MRNumericPattern numPat = new MRNumericPattern( );
        numPat.setNumberOfPoints( m_response.m_absorptionChannelTD.length );
        numPat.setFileReference( directory + getName( ) + "_AsNumpat.npf" );
        
        // m_response = numPat.setStaticSpectralResponse( MRSignalProperties.getInstance( ) );
        m_response = new MRLineModelSpectralResponse( this, MRSignalProperties.getInstance( ) );
        numPat.setMRLineModelSpectralResponse( m_response );
        numPat.setName( getName( ) + "_num_pat" );
        
        try 
        {
            try ( RandomAccessFile out = new RandomAccessFile( directory + getName( ) + "_AsNumpat.npf", "rw" ) ) 
            {
                numPat.write( out );
            }
        } 
        catch ( FileNotFoundException ex )
        {
            Logger.getLogger( MRParametricPattern.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
        catch ( IOException ex )
        {
            Logger.getLogger( MRParametricPattern.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
        
        writeTDFDFitFormattedResponseFile( path + tdfdfitResponseDir, getName( ) + ".bin" );
    
    }
    
    /**
     * Writes the current file to TDFDFit file format.
     * @param path
     * @param fileName
     */
    public void writeTDFDFitFormattedResponseFile( String path, String fileName )
    {
        
        if ( path != null )
        {
            fileName = path + fileName;
        }
        
        XYPlotFrame xypf = new XYPlotFrame( "MRFittingParameters::writeTDFDFitFormattedResponseFile", null );
        float[] xaxis = new float[m_response.m_absorptionChannelFD.length];
        for ( int i = 0; i < m_response.m_absorptionChannelFD.length; i++ )
        {
            xaxis[i] = (float)i;
        }
        xypf.addCurve( new Curve( xaxis, m_response.m_absorptionChannelFD ), true );  
        
        
        System.out.println( "fileName : " + fileName );
        try
        {     
            try ( RandomAccessFile writer = new RandomAccessFile( fileName, "rw" ) ) 
            {
                ByteBuffer buf, buf1;
                for ( int i = 0; i < m_response.m_absorptionChannelTD.length; i++ )
                {
                    buf = ByteBuffer.allocate( 4 );
                    
                    buf.putFloat( m_response.m_absorptionChannelFD[i] );
                    
                    writer.write( buf.get( 3 ) );
                    writer.write( buf.get( 2 ) );
                    writer.write( buf.get( 1 ) );
                    writer.write( buf.get( 0 ) );
                    
                    System.out.println( "MRNumericPattern::writeTDFDFitFormattedResponseFile : abs[" + i + "] = " + m_response.m_absorptionChannelFD[i] );
                    
                    buf1 = ByteBuffer.allocate( 4 );
                    
                    buf1.putFloat( m_response.m_dispersionChannelFD[i] );
                    
                    writer.write( buf1.get( 3 ) );
                    writer.write( buf1.get( 2 ) );
                    writer.write( buf1.get( 1 ) );
                    writer.write( buf1.get( 0 ) );
                }
            }    
        }
        catch ( IOException e )
        {
            System.out.println( "Cannot write tdfdfit-formatted measured datafile." );
        }
    }
    
    /**
     * 
     * Returns the number to fit of this MRFittingParameters object.
     * 
     * @return 
     */
    public int getNumberOfParametersToFit( )
    {
        m_degreesOfFreedom = 0;
        
        if ( m_areaReferenceVal == 0 )
        {
            m_degreesOfFreedom++;
        }
        if ( m_offsetReferenceVal == 0 )
        {
            m_degreesOfFreedom++;
        }
        if ( m_gaussianWidthReferenceVal == 0 ) 
        {
            m_degreesOfFreedom++;
        }
        if ( m_phaseReferenceVal == 0 )
        {
            m_degreesOfFreedom++;
        }
        if ( m_lorentzianWidthReferenceVal == 0 )
        {
            m_degreesOfFreedom++;
        }
        return m_degreesOfFreedom;
    }
    
    // Attributes
    private int         m_degreesOfFreedom;
    protected boolean   m_fitWithReferenceLineShapeFunction;
    protected boolean   m_fitWithArbitraryLineShapeFunction;

    protected String    m_name;	
    protected String    m_uid;

    protected int       m_modelType;
    protected String    m_fileReferenceName;
	
    protected String    m_lineGroup;     // indicates the group the line or pattern belongs to
    protected String    m_metabolite;    // indicates the metabolite the line or pattern belongs to
    
    protected float     m_area;
    protected float     m_offset;  
    protected float     m_gaussianWidth; 
    protected float     m_phase;
    protected float     m_lorentzianWidth;
	
    protected float     m_minimumArea;
    protected float     m_minimumOffset;
    protected float     m_minimumGaussianWidth; 
    protected float     m_minimumPhase;
    protected float     m_minimumLorentzianWidth;

    protected float     m_maximumArea;
    protected float     m_maximumOffset;
    protected float     m_maximumGaussianWidth; 
    protected float     m_maximumPhase;
    protected float     m_maximumLorentzianWidth;
	
    protected String    m_areaReference;
    protected String    m_offsetReference;
    protected String    m_gaussianWidthReference;
    protected String    m_phaseReference;
    protected String    m_lorentzianWidthReference;
    
    protected float     m_areaReferenceVal;
    protected float     m_offsetReferenceVal;
    protected float     m_gaussianWidthReferenceVal; 
    protected float     m_phaseReferenceVal;
    protected float     m_lorentzianWidthReferenceVal;
    
    private MRLineModelSpectralResponse m_response;
    
    // Public static members: model types!
    public final static int MRLINE_MODEL                = 0;
    public final static int PARAMETRIC_PATTERN_MODEL    = 1;
    public final static int NUMERIC_PATTERN_MODEL       = 2;
    public final static int METABOLITE_MODEL            = 3;
     
}