/*
 * File: $Id: MRMetaboliteModel.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 02.06.2003, 11:33  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.tdfdfit;


import org.spectrim.signalprocessing.FFT;
import java.io.RandomAccessFile;
import java.io.IOException;

import java.util.LinkedList;
import java.util.Iterator;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.nio.ByteBuffer;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.tree.DefaultMutableTreeNode;
import static org.spectrim.tdfdfit.TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME;

/**
 * This class contains all metabolite parameter information:
 *
 * An MRMetaboliteModel may consist of three types of objects
 *      1. <linkedlist>( MRLineModelParameters )
 *      2. <linkedlist>( MRNumericPattern )
 *      3. <linkedlist>( MRParametricPattern )
 *
 * @author  hans
 */

public class MRMetaboliteModel 
{
    
    /** 
     * Creates a new instance of MRMetaboliteModel .
     */
    public MRMetaboliteModel( )
    {
         m_fittingParameters = new MRFittingParameters( m_metaboliteModelName, MRFittingParameters.METABOLITE_MODEL );
    }

    /** 
     * Copy constructor of MRMetaboliteModel.
     * @param copyMetaboliteModel
     */
    public MRMetaboliteModel( MRMetaboliteModel copyMetaboliteModel )
    {
        m_fittingParameters = new MRFittingParameters( copyMetaboliteModel.getName( ), 
                                                       MRFittingParameters.METABOLITE_MODEL );
        setFittingParameters( copyMetaboliteModel.getFittingParameters( ) );
        
        
        // make a deep copy of the simple lines ... 
        LinkedList<MRLineModelParameters> simpleLineList = copyMetaboliteModel.getSimpleLineList( );
        Iterator it1 = simpleLineList.iterator( );
        
        while ( it1.hasNext( ) )
        {
            m_simpleLineList.add( new MRLineModelParameters( (MRLineModelParameters)it1.next( ) ) );
        }
        
        // make a deep copy of the numeric patterns ... 
        LinkedList<MRNumericPattern> numericPatternList = copyMetaboliteModel.getNumericPatternList( );
        Iterator it2 = numericPatternList.iterator( );
        
        while ( it2.hasNext( ) )
        {
            numericPatternList.add( new MRNumericPattern( (MRNumericPattern)it2.next( ) ) );
        }
        
       // make a deep copy of the paramtric patterns ... 
        LinkedList<MRParametricPattern> parametricPatternList = copyMetaboliteModel.getParametricPatternList( );
        Iterator it3 = parametricPatternList.iterator( );
        
        while ( it3.hasNext( ) )
        {
            m_parametricPatternList.add( new MRParametricPattern( (MRParametricPattern)it3.next( ) ) );
        }

    }
    
    /** 
     * Creates a new instance of MRMetaboliteModel. 
     * @param metabName
     */
    public MRMetaboliteModel( String metabName )
    {
         m_metaboliteModelName = metabName;
         m_fittingParameters = new MRFittingParameters( m_metaboliteModelName, MRFittingParameters.METABOLITE_MODEL );
    }
    
    /** 
     * Add a MRLineModelParameters object to the LinkedList of 
     * line_model_name -> MRLineModelParameters.
     */
    public void addMRLineModelParameters( MRLineModelParameters mrLineModelParameters )
    {
        m_simpleLineList.add( mrLineModelParameters );
    }
    
    /**
     * Deletes a MRLineModelParameters object from the LinkedList of 
     * line_model_name -> MRLineModelParameters.
     */
    public void deleteMRLineModelParameters( MRLineModelParameters mrLineModelParameters )
    {
        m_simpleLineList.remove( mrLineModelParameters );
    }
    
    /**
     * Returns currently only the name of the MRMetaboliteModel.
     */
    public String toString( )
    {
        return m_metaboliteModelName;
    }
    
    /**
     * Sets the name of the metabolite model.
     */
     public void setName( String name  )
    {
         m_metaboliteModelName = name;
         m_fittingParameters.setName( name );
    }
    
    /**
     * Returns the name of the metabolite model.
     */
     public String getName( )
    {
         return m_metaboliteModelName;
    }
     
    /**
     * Sets the MRFitting parameter attribute of the numeric pattern.
     */
    public void setFittingParameters( MRFittingParameters fitPars )
    {
        m_fittingParameters.setName( fitPars.getName( ) );
        m_fittingParameters.setModelType( fitPars.getModelType( ) );
        m_fittingParameters.setFileReferenceName( fitPars.getFileReferenceName( ) );
        m_fittingParameters.setLineGroup( fitPars.getLineGroup( ) );
        m_fittingParameters.setMetabolite( fitPars.getMetabolite( ) );
        m_fittingParameters.setFitWithArbitraryLineShapeFunction( fitPars.getFitWithArbitraryLineShapeFunction( ) );
        m_fittingParameters.setFitWithReferenceLineShape( fitPars.getFitWithReferenceLineShape( ) );
        
        // fit parameter values
        m_fittingParameters.setArea( fitPars.getArea( ) );
        m_fittingParameters.setOffset( fitPars.getOffset( ) );
        m_fittingParameters.setGaussianWidth( fitPars.getGaussianWidth( ) );
        m_fittingParameters.setPhase( fitPars.getPhase( ) );
        m_fittingParameters.setLorentzianWidth( fitPars.getLorentzianLineWidth( ) );
        
        // minimum parameter values
        m_fittingParameters.setMinimumArea( fitPars.getMinimumArea( ) );
        m_fittingParameters.setMinimumOffset( fitPars.getMinimumOffset( ) );
        m_fittingParameters.setMinimumGaussianWidth( fitPars.getMinimumGaussianWidth( ) );
        m_fittingParameters.setMinimumPhase( fitPars.getMinimumPhase( ) );
        m_fittingParameters.setMinimumLorentzianWidth( fitPars.getMinimumLorentzianWidth( ) );
        
        // maximum parameter values
        m_fittingParameters.setMaximumArea( fitPars.getMaximumArea( ) );
        m_fittingParameters.setMaximumOffset( fitPars.getMaximumOffset( ) );
        m_fittingParameters.setMaximumGaussianWidth( fitPars.getMaximumGaussianWidth( ) );
        m_fittingParameters.setMaximumPhase( fitPars.getMaximumPhase( ) );
        m_fittingParameters.setMaximumLorentzianWidth( fitPars.getMaximumLorentzianWidth( ) );
        
        // prior knowledge references
        m_fittingParameters.setAreaReference( fitPars.getAreaReference( ) );
        m_fittingParameters.setOffsetReference( fitPars.getOffsetReference( ) );
        m_fittingParameters.setGaussianWidthReference( fitPars.getGaussianWidthReference( ) );
        m_fittingParameters.setPhaseReference( fitPars.getPhaseReference( ) );
        m_fittingParameters.setLorentzianWidthReference( fitPars.getLorentzianLineWidthReference( ) );
    }
    
    /**
     * Returns the MRFittingParameter attribute of the metabolite.
     * @return 
     */
    public MRFittingParameters getFittingParameters( )
    {
        return m_fittingParameters;
    }
     
    /**
     * Returns a reference to the m_parametricPatternList object.
     * @return 
     */
    public LinkedList getSimpleLineList( )
    {
        return m_simpleLineList;
    }     
     
    /**
     * Returns a reference to the m_parametricPatternList object.
     * @return 
     */
    public LinkedList getNumericPatternList( )
    {
        return m_numericPatternList;
    }   

    /**
     * Returns a reference to the m_parametricPatternList object.
     * @return 
     */
    public LinkedList getParametricPatternList( )
    {
        return m_parametricPatternList;
    }
    
    /**
     * Returns a reference to the m_parametricPatternList object.
     * @param key
     * @param linPar
     */
    public void addLineToLatestAddedParametricPattern( String key, MRLineModelParameters linPar )
    {
        Object obj = m_parametricPatternList.getLast( );
        MRParametricPattern numPat = (MRParametricPattern)obj;
        numPat.addSimpleLine( linPar );
    }
    
    /**
     * Adds a parametric pattern to the LinkedList containing MRParametricPattern
     * objects.
     * @param parPat
     */
    public void addParametericPattern( MRParametricPattern parPat )
    {
        m_parametricPatternList.add( parPat );
    }
    
    /**
     * Adds a numeric pattern to the LinkedList containing MRNumericPattern
     * objects.
     * @param key
     * @param numPat
     */
    public void addNumericPattern( String key, MRNumericPattern numPat )
    {
        m_numericPatternList.add( numPat );
    }
    
    /**
     * Reads the attributes from a RandomAccesFile.
     */
    void read( RandomAccessFile in )
    {
        try
        {
            String tag = in.readUTF( );
            // First string after opening tag is the name string of the metabolite model
            m_metaboliteModelName = tag;
            
            
            tag = in.readUTF( );
            m_fittingParameters.read( in );
            
            tag = in.readUTF( );
            while ( tag.compareTo( "</MRMetaboliteModel>" ) != 0 )
            //while ( tag.compareTo( "</MRNumericPattern>" ) != 0 )
            {
                // read a simple line from a metabolite model
                if ( tag.compareTo( "<MRFittingParameters>" ) == 0 )
                {
                    MRLineModelParameters linePars = new MRLineModelParameters( );
                    linePars.read( in );
                    m_simpleLineList.add( linePars ); 
                }
                // read a parametric pattern
                if ( tag.compareTo( "<MRParametricPattern>" ) == 0 )
                {
                    MRParametricPattern parPat = new MRParametricPattern( );
                    parPat.read( in );
                    m_parametricPatternList.add( parPat );
                }
                // read a numeric patterns

                if ( tag.compareTo( "<MRNumericPattern>" ) == 0 )
                {
                    MRNumericPattern numPat = new MRNumericPattern( );
                    numPat.read( in );
                    m_numericPatternList.add( numPat );
                }
                tag = in.readUTF( );
            }
        }
        catch ( IOException e )
        {
            
        }
    }
    
    /**
     * Reads the attributes from a RandomAccesFile and adds components 
     * to the tree.
     */
    void read( RandomAccessFile in, DefaultMutableTreeNode node )
    {
        try
        {
            String tag = in.readUTF( );
            // First string after opening tag is the name string of the metabolite model
            m_metaboliteModelName = tag;
            
            tag = in.readUTF( );
            m_fittingParameters.read( in );
            
            tag = in.readUTF( );
            
            while ( tag.compareTo( "</MRMetaboliteModel>" ) != 0 )
            {
                // read a simple line from a metabolite model
                if ( tag.compareTo( "<MRFittingParameters>") == 0 )
                {
                    MRLineModelParameters linePars = new MRLineModelParameters( );
                    linePars.read( in );
                    DefaultMutableTreeNode childNode = new DefaultMutableTreeNode( linePars );
                    node.add( childNode );
                    m_simpleLineList.add( linePars ); 
                }
                // read a parametric pattern
                if ( tag.compareTo( "<MRParametricPattern>" ) == 0 )
                {
                    MRParametricPattern parPat = new MRParametricPattern( );
                    parPat.read( in );
                    DefaultMutableTreeNode childNode = new DefaultMutableTreeNode( parPat );
                    node.add( childNode );
                    m_parametricPatternList.add( parPat );
                }
                
                // read a numeric patterns
                if ( tag.compareTo( "<MRNumericPattern>") == 0 )
                {
                    MRNumericPattern numPat = new MRNumericPattern( );
                    numPat.read( in );
                    DefaultMutableTreeNode childNode = new DefaultMutableTreeNode( numPat );
                    node.add( childNode );
                    m_numericPatternList.add( numPat );
                }
                tag = in.readUTF( );
            }
        }
        catch ( IOException e )
        {
            e.printStackTrace( );
        }
    }
    
    /**
     * Writes attributes to RandomAccessFile.
     */
    void write( RandomAccessFile out )
    {
        // Write the attributes of this object to a separate file
        writeMRModeModelFile( );
        // Write the attributes to a RandomAccessfile indicated by the 
        // method caller
        try
        {
            out.writeUTF( "<MRMetaboliteModel>" );
            out.writeUTF( m_metaboliteModelName );
            
            m_fittingParameters.write( out );
            
            writeParametricPatterns( out );
            writeNumericPatterns( out );
            writeSimpleLines( out );
            out.writeUTF( "</MRMetaboliteModel>" );
        }
        catch ( IOException e )
        {
        }
    }
    
    /**
     * Writes attributes of this object a RandomAccessFile to a database of
     * MRMetaboliteModel-s; this file is importable into other projects . 
     */
    void writeMRModeModelFile( )
    {
        String fileRef;
        
        TDFDFittingDefaults defaults = TDFDFittingDefaults.getInstance( );
        defaults.load( );
        
        String path = defaults.getProperty( DEFAULT_TDFDFIT_FIT_MODEL_NAME );
        String metaboliteDir = System.getProperty( "file.separator" ) + 
                               "Metabolites";
        String directory = path + metaboliteDir;
        fileRef = path + metaboliteDir + System.getProperty( "file.separator" ) + this.getName( ) + ".mpf";
        
        System.out.println( "Metabolite is written seperately in file : " + fileRef );
        
        try
        {    
            File outfile = new File( fileRef );
            RandomAccessFile writer = new RandomAccessFile( outfile, "rw" );
            
            // Should the fitting reference object be written as well -> yes!!
            writer.writeUTF( "<MRMetaboliteModel>" );
            writer.writeUTF( m_metaboliteModelName );
            m_fittingParameters.write( writer );
            
            writeParametricPatterns( writer );
            writeNumericPatterns( writer );
            writeSimpleLines( writer );
            writer.writeUTF( "</MRMetaboliteModel>" );
        }
        catch ( IOException e )
        {
            System.out.println( "Could not write file : " + fileRef );
        }
    }
    
    /**
     * Writes all parametric patterns to a random access file.
     */
    private void writeParametricPatterns( RandomAccessFile out )
    {
        // get a reference to all the keys of the LinkedList containing 
        // the simple line information
        Iterator iter = m_parametricPatternList.iterator( );
        // Iterate of all elements of the LinkedList
        while ( iter.hasNext( ) )
        {
            MRParametricPattern parPat = (MRParametricPattern)iter.next( );
            parPat.write( out );
        }
    }
    
    /**
     * Writes all numeric patterns to a random access file.
     */
    private void writeNumericPatterns( RandomAccessFile out )
    {
        // get a reference to all the keys of the LinkedList containing 
        // the simple line information
        Iterator iter = m_numericPatternList.iterator( );
        // Iterate of all elements of the LinkedList
        while ( iter.hasNext( ) )
        {
            MRNumericPattern numPat = (MRNumericPattern)iter.next( );
            numPat.write( out );
        }              
    }
    
    /**
     * Writes all simple lines to a random access file.
     */
    private void writeSimpleLines( RandomAccessFile out )
    {
        // get a reference to all the keys of the LinkedList containing 
        // the simple line information
        Iterator iter = m_simpleLineList.iterator( );
        // Iterate of all elements of the LinkedList
        while ( iter.hasNext( ) )
        {
            // String simpleLineKey = ( (MRLineModelParameters)iter.next( ) ).getName( );
            MRLineModelParameters simpleLine = (MRLineModelParameters)iter.next( );
            simpleLine.write( out );
        }
    }
    
    /**
     * Clears all the linked lists in this data structure.
     */
    public void clear( )
    {
        //m_metaboliteModelName = "Default Metabolite name"; 
        
        m_simpleLineList.clear( );
        m_simpleLineList = null;
        m_simpleLineList = new LinkedList<MRLineModelParameters>( );
        
        m_numericPatternList.clear( );
        m_numericPatternList = null;
        m_numericPatternList = new LinkedList<MRNumericPattern>( );
        
        m_parametricPatternList.clear( );
        m_parametricPatternList = null;
        m_parametricPatternList = new LinkedList<MRParametricPattern>( );
    }
    
    /**
     * Writes the current parametric pattern object as numerical pattern
     * the fitting parameters object of the numerical pattern is set to
     * numerical values -> ( 1.00, 0.00, 0.00, 0.00, 0.00 ).
     */
    public void exportMetaboliteModelAsNumericalPattern( )
    {
        MRLineModelSpectralResponse currentResponse = getSpectralResponse( MRSignalProperties.getInstance( ) );
        
        m_absorptionChannelFD  = currentResponse.getAbsorptionChannelFD( );
        m_dispersionChannelFD = currentResponse.getDispersionChannelFD( );
        
        TDFDFittingDefaults defaults = TDFDFittingDefaults.getInstance( );
        defaults.load( );
        
        String path = defaults.getProperty( DEFAULT_TDFDFIT_FIT_MODEL_NAME );
        String parametricPatternDir = System.getProperty( "file.separator" ) + 
                                                          "Numeric_Patterns" + 
                                                          System.getProperty( "file.separator" );
        
        String directory = path + parametricPatternDir;
        
        writeTDFDFitFormattedResponseFile( directory, this.getName( ) + "AsNumPat.bin" );
    
    }
    
    
    
    /**
     * Calculates the spectral representation of this (MRMetaboliteModel)
     * which may consist of lines, numeric and parametric patterns.
     */
    public MRLineModelSpectralResponse getSpectralResponse( MRSignalProperties mrSignalProperties )
    {
        
        // Create a model spectral response per default constructor
        MRLineModelSpectralResponse resp = new MRLineModelSpectralResponse( );
        
        m_lineModelSpectralResponse = null;
        m_lineModelSpectralResponse = new MRLineModelSpectralResponse( mrSignalProperties.getNumberOfSamplePoints( ) );
        m_lineModelSpectralResponse.setMRSignalProperties( mrSignalProperties );
        
        // Iterate over the m_simpleLineList containing the simple lines
        Iterator iter = m_simpleLineList.iterator( );
        while ( iter.hasNext( ) )
        {
            MRLineModelParameters simpleLine = (MRLineModelParameters)iter.next( );
            
            // calculate the corresponding resonance line
            resp = new MRLineModelSpectralResponse( (MRFittingParameters)simpleLine, mrSignalProperties );
            m_lineModelSpectralResponse.addTimeDomainResponse( resp.getAbsorptionChannelTD( ), resp.getDispersionChannelTD( ) );
        }
        
        // Calculate the sum of all ParametricPattern - MRLineModelSpectralResponse
        // objects - if any
        Iterator parPatIter = m_parametricPatternList.iterator( );
        while ( parPatIter.hasNext( ) )
        {
            MRParametricPattern parPat = (MRParametricPattern)parPatIter.next( );
            resp = parPat.getSpectralResponse( mrSignalProperties );
            m_lineModelSpectralResponse.addTimeDomainResponse( resp.getAbsorptionChannelTD( ), resp.getDispersionChannelTD( ) );
        }
        
        m_lineModelSpectralResponse.applyMRLineModelSpectralResponse( m_fittingParameters );
        
        return m_lineModelSpectralResponse;
    }
    
    /**
     * Writes the current parametric pattern object as numerical pattern
     * the fitting parameters object of the numerical pattern is set to
     * numerical values -> ( 1.00, 0.00, 0.00, 0.00, 0.00 ).
     */
    public void exportMetaboliteModel( )
    {
        MRLineModelSpectralResponse currentResponse = getSpectralResponse( MRSignalProperties.getInstance( ) );
        
        m_absorptionChannelFD  = currentResponse.getAbsorptionChannelFD( );
        m_dispersionChannelFD = currentResponse.getDispersionChannelFD( );
        
        TDFDFittingDefaults defaults = TDFDFittingDefaults.getInstance( );
        defaults.load( );
        
        String path = defaults.getProperty( DEFAULT_TDFDFIT_FIT_MODEL_NAME );
        String numericPatternDir = System.getProperty( "file.separator" ) + 
                                                       "Metabolites" + 
                                                       System.getProperty( "file.separator" );
        
        String tdfdfitResponseDir = System.getProperty( "file.separator" ) + 
                                                        "tdfdfit_responses" + 
                                                        System.getProperty( "file.separator" );
        String directory = path + numericPatternDir;
        
        MRMetaboliteModel metabolite = new MRMetaboliteModel( );
        metabolite.setNumberOfPoints( m_absorptionChannelFD.length );
        metabolite.setFileReference( directory + getName( ) + "_AsMetab.mpf" );
        metabolite.setMRLineModelSpectralResponse( currentResponse );
        metabolite.setName( getName( ) + "_metab" );
        
        try 
        {
            try ( RandomAccessFile out = new RandomAccessFile( directory + getName( ) + "_AsNumpat.npf", "rw" ) ) 
            {
                metabolite.write( out );
            }
        } 
        catch ( FileNotFoundException ex )
        {
            Logger.getLogger( MRParametricPattern.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
        catch ( IOException ex )
        {
            Logger.getLogger( MRParametricPattern.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
        
        writeTDFDFitFormattedResponseFile( path + tdfdfitResponseDir, getName( ) + ".bin" );
    
    }
    
    /**
     * Writes the current binary response to a TDFDfit formatted file.
     * @param path
     * @param fileName
     */
    public void writeTDFDFitFormattedResponseFile( String path, String fileName )
    {
        
        if ( path != null )
        {
            fileName = path + System.getProperty( "file.separator" ) + fileName;
        }
        
        System.out.println( "fileName : " + fileName );
        try
        {
            
            RandomAccessFile writer = new RandomAccessFile( fileName, "rw" );
            
            ByteBuffer buf, buf1;
            
            for ( int i = 0; i < m_absorptionChannelFD.length; i++ )
            {
                buf = ByteBuffer.allocate( 4 );
                
                buf.putFloat( m_absorptionChannelFD[i] );
                
                writer.write( buf.get( 3 ) );
                writer.write( buf.get( 2 ) );
                writer.write( buf.get( 1 ) );
                writer.write( buf.get( 0 ) );
                
                buf1 = ByteBuffer.allocate( 4 );
                
                buf1.putFloat( m_dispersionChannelFD[i] );                
                
                writer.write( buf1.get( 3 ) );
                writer.write( buf1.get( 2 ) );
                writer.write( buf1.get( 1 ) );
                writer.write( buf1.get( 0 ) );
                                
            }

            writer.close( );
            
        }
        catch ( IOException e )
        {
            System.out.println( "Cannot write tdfdfit-formatted measured datafile" );
        }
    }
    
    /**
     * Reads a TDFDfit file format binary file and stores its contents in the
     * appropriate member variables.
     */
    public void readTDFDFitFormattedResponseFile( int numberOfComplexPoints, String fileRef )
    {
        
        try
        {          
            RandomAccessFile reader = new RandomAccessFile( fileRef, "rw" );
            
            ByteBuffer buf;
            byte byte0;
            byte byte1;
            byte byte2;
            byte byte3;
            float readFloat;
            
            for ( int i = 0; i < numberOfComplexPoints; i++ )
            {
                
                buf = ByteBuffer.allocate( 4 );
                
                byte0 = reader.readByte( );
                byte1 = reader.readByte( );
                byte2 = reader.readByte( );
                byte3 = reader.readByte( );
                  
                buf.put( byte3 );
                buf.put( byte2 );
                buf.put( byte1 );
                buf.put( byte0 );
                
                readFloat = buf.getFloat( 0 );
                buf.clear( );
                
                m_absorptionChannelFD[i] = readFloat;
                
                buf = ByteBuffer.allocate( 4 );
                
                byte0 = reader.readByte( );
                byte1 = reader.readByte( );
                byte2 = reader.readByte( );
                byte3 = reader.readByte( );
                  
                buf.put( byte3 );
                buf.put( byte2 );
                buf.put( byte1 );
                buf.put( byte0 );
                
                readFloat = buf.getFloat( 0 );
                buf.clear( );
                
                m_dispersionChannelFD[i] = readFloat;
                
            }
            
            // the frequency domain response is read so generate the time 
            // domain response to get full representation
            calculateTDRepresentation( );
            
            reader.close( );
            
        }
        catch ( IOException e )
        {
            e.printStackTrace( );
            System.out.println( "Cannot read tdfdfit-formatted measured datafile" );
        }
    }
    
    /**
     * Calculates the frequency domain representation of 
     * the absorption and dispersion channels.
     */
    private void calculateTDRepresentation( )
    {
        float[][] fftArray = new float[m_absorptionChannelFD.length][2];
        float[] bufAbs = new float[m_absorptionChannelFD.length];
        float[] bufDisp = new float[m_absorptionChannelFD.length];
               
        for ( int i = 0 ; i < m_absorptionChannelFD.length ; i++ )
        {
            fftArray[i][0] = m_absorptionChannelFD[ m_absorptionChannelFD.length - 1 - i ];
            fftArray[i][1] = m_dispersionChannelFD[ m_absorptionChannelFD.length - 1 - i ];
        }
        
        for ( int i = 0 ; i < m_absorptionChannelFD.length ; i++ )
        {
             bufAbs[ i ] = fftArray[i][0];
             bufDisp[ i ] = fftArray[i][1];           
        }
        
        for ( int i = 0 ; i < m_absorptionChannelFD.length ; i++ )
        {
            int d = ( i + 1 + m_absorptionChannelFD.length / 2 ) % m_absorptionChannelFD.length;
            fftArray[d][0] = bufAbs[i];
            fftArray[d][1] = bufDisp[i];
        }
        
        bufAbs = null;
        bufDisp = null;
        
        FFT fft = new FFT( );
        fftArray = fft.fft_1d( fftArray );
        
        m_absorptionChannelTD = new float[m_absorptionChannelFD.length];
        m_dispersionChannelTD = new float[m_dispersionChannelFD.length];

        for ( int i = 0 ; i < m_absorptionChannelFD.length ; i++ )
        {
            //int d = ( i + m_absorptionChannelTD.length / 2 ) % m_absorptionChannelFD.length;
            m_absorptionChannelTD[i] = fftArray[i][0] / m_absorptionChannelTD.length;
            m_dispersionChannelTD[i] = fftArray[i][1] / m_absorptionChannelTD.length;
        }
    
        fftArray = null;
    }
        
    // contains a linkedlist of < simple line >
    private LinkedList<MRLineModelParameters> m_simpleLineList = new LinkedList<MRLineModelParameters>( );
    // contains a linkedlist of < numeric pattern >
    private LinkedList<MRNumericPattern> m_numericPatternList = new LinkedList<MRNumericPattern>( );
    // contains a linkedlist of < parametric pattern >
    private LinkedList<MRParametricPattern> m_parametricPatternList = new LinkedList<MRParametricPattern>( );
    
    private String m_metaboliteModelName = "Metabolite";
    private MRFittingParameters m_fittingParameters;

    // Attribute that holds the sum spectrum of all simple lines,
    // numeric and parametric pattern responses
    private MRLineModelSpectralResponse m_lineModelSpectralResponse;
    
    private int     m_numberOfPoints;
    private String  m_fileReference;
    private float[] m_absorptionChannelFD;
    private float[] m_dispersionChannelFD;
    private float[] m_absorptionChannelTD;
    private float[] m_dispersionChannelTD;
    
     /**
     * Sets the MRLineModelSpectralResonse attribute of the metabolite pattern;
     * Method is used when converting an object to an MRNumericPattern typed
     * object.
     * @param currentResponse.   
     */
    void setMRLineModelSpectralResponse( MRLineModelSpectralResponse spectralResponse ) 
    {
        m_lineModelSpectralResponse = spectralResponse;
        m_absorptionChannelFD = m_lineModelSpectralResponse.getAbsorptionChannelFD( );
        m_dispersionChannelFD = m_lineModelSpectralResponse.getDispersionChannelFD( );
        
        m_absorptionChannelTD = m_lineModelSpectralResponse.getAbsorptionChannelTD( );
        m_dispersionChannelTD = m_lineModelSpectralResponse.getDispersionChannelTD( );
    }
    
    /**
     * Sets the number of points of the response. 
     * @param numberOfPoints 
     */
    public void setNumberOfPoints( int numberOfPoints ) 
    {
        m_numberOfPoints = numberOfPoints;
    }

    /**
     * Sets the file reference attribute; that is the location (path) 
     * where the file is stored on in the file system.
     * 
     * @param fileReference 
     */
    public void setFileReference( String fileReference ) 
    {
        m_fileReference = fileReference;
    }
    
    
    /**
     * Writes the metabolite to a pattern file.
     * @param metabFileName 
     */
    void writeMetaboliteAsTDFDFitDotPatFile( String metabFileName ) 
    {
        
        exportMetaboliteModel( );
        
        TextFileWriter writer = null;
        MRSignalProperties mrSignalProperties = MRSignalProperties.getInstance( );
        
        float frequencyscale = mrSignalProperties.getFrequencyScale( );
        int samplePoints = mrSignalProperties.getNumberOfSamplePoints( );
        
        float bandwidth = samplePoints;
        
        try 
        {    
            writer = new TextFileWriter( new FileWriter( metabFileName ) );
            
            writer.setNumberFormat( "0.00000000E00" );
            writer.writeStringLn( "start_pattern" );
            writer.writeString( "number_of_lines\t" );
            writer.writeInteger( m_simpleLineList.size( ) );
            writer.writeStringLn( "" );
            writer.writeStringLn( "parameters" );
            writer.writeStringLn( "name\t\tline_type\tarea\t\tfrequency\tgauss_width\tzero_phase\tlorentz_width" );
            
            for ( int i = 0; i < m_simpleLineList.size( ); i++ )
            {
                
                MRFittingParameters fitPars = m_simpleLineList.get( i );
                
                writer.writeString( fitPars.getName( ) + "\t\t" );
                
                // set default line model to voigt ..
                // TODO make this user setable in a later stage
                writer.writeString( "voigt" + "\t\t" );
                writer.writeFloat( fitPars.getArea( ) );
                writer.writeString( "\t" );
                writer.writeFloat( (fitPars.getOffset( ) + bandwidth/2f) );
                writer.writeString( "\t" );
                writer.writeFloat( fitPars.getGaussianWidth( ) );
                writer.writeString( "\t" );
                writer.writeFloat( fitPars.getPhase( ) );
                writer.writeString( "\t" );
                writer.writeFloat( fitPars.getLorentzianLineWidth( ) );
                writer.writeStringLn( "" );
            }
            writer.writeStringLn( "end_of_pattern" );
            writer.flush( );
            writer.close( );
        } 
        catch ( IOException ex ) 
        {
            Logger.getLogger( MRParametricPattern.class.getName(  ) ).log( Level.SEVERE, null, ex );
        }
    }
   
}