/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.tdfdfit.quantitativeresults;

import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.LinkedList;
import org.spectrim.tdfdfit.MRFittingParameters;
import org.spectrim.tdfdfit.MRLineModelParameters;
import org.spectrim.tdfdfit.MRSignalProperties;
import org.spectrim.tdfdfit.TDFDFitPrototypeModelSpectrumParameters;

/**
 * This container class collects all data necessary to compute the metabolite
 * concentrations of a single voxel spectrum.
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */
public class QuantitativeSVSContextDataContainer 
{

    private QuantitativeSVSContextDataContainer( )
    {
        // nothing to do here.
    }
    
     /**
     * Returns the one and only instance of QuantitativeSVSContextDataContainer.
     * @return 
     */
    public static QuantitativeSVSContextDataContainer getInstance( )
    {
        if ( theInstance == null )
        {
            theInstance = new QuantitativeSVSContextDataContainer( );
        }
        return theInstance;
    }
    
    
    /**
     * Sets the 2 compartment water reference signals (time domain) as been measured.
     * @param numberOf2CompartentWaterResponses
     * @param real
     * @param imag
     */
    public void set2CompartmentModelSignals( int numberOf2CompartentWaterResponses, 
                                             double[][] real, 
                                             double[][] imag )
    {
        m_waterRef2CompReal = new double[numberOf2CompartentWaterResponses][real[0].length];
        m_waterRef2CompImag = new double[numberOf2CompartentWaterResponses][imag[0].length];

        for ( int j = 0; j < numberOf2CompartentWaterResponses; j++ )
        {
            for ( int i = 0; i < real[0].length; i++ )
            {
                m_waterRef2CompReal[j][i] = real[j][i];
                m_waterRef2CompImag[j][i] = imag[j][i];
            }
        }
        m_numberOf2CompModelResponses = numberOf2CompartentWaterResponses;
        m_twoCompartmentModelSet = true;
    }
    
    
    /**
     * Sets the list of file references of the spectroscopy related data.
     * @param list     
     */
    public void setSpectroscopyDicomWaterFileReferences( LinkedList<String> list )
    {
        m_spectroscopyDicomWaterFileReference = list;
    }
    
    /**
     * Boolean defining whether a two compartment model is available for this
     * SVS dataset.
     * @param isSet 
     */
    public void setTwoCompartmentModelSet( boolean isSet )
    {
        m_twoCompartmentModelSet = isSet;
    }
    
    /**
     * Boolean defining whether a two compartment model is available for this
     * SVS dataset.
     * @param isSet 
     */
    public void setSimpleWaterRefefenceSignalSet( boolean isSet )
    {
        m_setSimpleWaterRefefenceSignalSet = isSet;
    }
    
    /**
     * Boolean defining whether a two compartment model is available for this
     * SVS dataset. 
     * @return 
     */
    public boolean getSimpleWaterRefefenceSignalSet( )
    {
        return m_setSimpleWaterRefefenceSignalSet;
    }
    
    /**
     * Boolean defining whether the water reference model is fitted.
     * @param isFitted 
     */
    public void setTwoCompartmentModelFitted( boolean isFitted )
    {
        m_twoCompartmentModelFitted = isFitted;
    }
    
    /**
     * Returns boolean defining whether a two compartment model is available 
     * for this SVS dataset. 
     * @return 
     */
    public boolean getTwoCompartmentModelSet( )
    {
        return m_twoCompartmentModelSet;
    }
    
    /**
     * Returns boolean defining whether the water reference model is fitted. 
     * @return 
     */
    public boolean getTwoCompartmentModelFitted( )
    {
        return m_twoCompartmentModelFitted;
    }
    
    /**
     * Sets boolean whether the single water reference signal is set.
     * @param waterRefIsSet
     */
    public void setSingleWaterReferenceDefined( boolean waterRefIsSet )
    {
        m_setSimpleWaterRefefenceSignalSet = waterRefIsSet;
    }
    
    /**
     * Returns boolean whether the single water reference signal is set.
     * @return 
     */
    public boolean getSingleWaterReferenceDefined( )
    {
        return m_setSimpleWaterRefefenceSignalSet;
    }
    
    /**
     * Sets the reference water defined file name (single response!).
     * @param rawWaterReferenceSpectrosopyDicomFileName
     */
    public void setReferenceWaterDefinedFileName( String rawWaterReferenceSpectrosopyDicomFileName )
    {
        m_simpleWaterReferenceFileName = rawWaterReferenceSpectrosopyDicomFileName;
    }
    
    /**
     * Sets the reference water defined file name (single response!).
     * @return 
     */
    public String getReferenceWaterDefinedFileName( )
    {
        return m_simpleWaterReferenceFileName;
    }
    
    /**
     * Sets the reference 2 compartment water references file names.
     * @param fittedResponsesFileNames
     */
    public void setFittedSpectrumFileNames( LinkedList<String> fittedResponsesFileNames )
    {
        m_fittedResponseFileNames = fittedResponsesFileNames;
    }
    
    /**
     * Returns the reference 2 compartment water references file names.
     * @return 
     */
    public LinkedList<String> getMeasuredSpectraFileNames( )
    {
        return m_fittedResponseFileNames;
    }
    
    /**
     * Sets the names of all metabolites (or spectral components of the model.
     * @param metaboliteNames 
     */
    public void setMetaboliteNames( LinkedList<String> metaboliteNames )
    {
        m_metaboliteNames = metaboliteNames;
    }
    
    /**
     * Returns the names of all metabolites (or spectral components) of 
     * the model. 
     * @return 
     */
    public LinkedList<String> getMetaboliteNames( )
    {
        return m_metaboliteNames;
    }
    
    /**
     * Returns the 2D array of peak area values.
     * @return 
     */
    public double[][] getPeakAreas( )
    {
        return m_peakAreas;
    }

    /**
     * Sets the 2D array of peak area values. 
     * @param peakAreas 
     */
    public void setPeakAreas( double[][] peakAreas ) 
    {
        m_peakAreas = peakAreas;
    }
    
    /**
     * Returns the error in the peak areas.
     * @return 
     */
    public double[][] getErrorPeakAreas( ) 
    {
        return m_errorPeakAreas;
    }

    /**
     * Sets the error in the peak areas. 
     * @param errorPeakAreas
     */
    public void setErrorPeakAreas( double[][] errorPeakAreas ) 
    {
        m_errorPeakAreas = errorPeakAreas;
    }

    /**
     * Returns the fitted chemical shift values.
     * @return 
     */
    public double[][] getChemShift( ) 
    {
        return m_chemShift;
    }
    
    /**
     * Sets the fitted chemical shift values. 
     * @param chemShift
     */
    public void setChemShift( double[][] chemShift )
    {
        m_chemShift = chemShift;
    }
    
    /**
     * Returns the error in the chemical shift parameters.
     * @return 
     */
    public double[][] getErrorsChemShift( ) 
    {
        return m_errorsChemShift;
    }
    
    /**
     * Sets the error in the chemical shift parameters. 
     * @param errorsChemShift
     */
    public void setErrorsChemShift( double[][] errorsChemShift ) 
    {
        m_errorsChemShift = errorsChemShift;
    }

    /**
     * Returns the error in the phase shift parameters.
     * @return 
     */
    public double[][] getPhase( ) 
    {
        return m_phases;
    }

    /**
     * Sets the error in the phase shift parameters. 
     * @param phase
     */
    public void setPhase( double[][] phase ) 
    {
        m_phases = phase;
    }

    /**
     * Returns the error in the phase parameters.
     * @return 
     */
    public double[][] getErrorsPhase( ) 
    {
        return m_errorsPhase;
    }

    /**
     * Sets the error in the phase parameters.
     * @param errorsPhase
     */
    public void setErrorsPhase( double[][] errorsPhase ) 
    {
        m_errorsPhase = errorsPhase;
    }

    /**
     * Returns the Lorentz width values.
     * @return 
     */
    public double[][] getLorentzWidth( ) 
    {
        return m_lorentzWidth;
    }

    /**
     * Sets the Lorentz width values.
     * @param lorentzWidth
     */
    public void setLorentzWidth( double[][] lorentzWidth )
    {
        m_lorentzWidth = lorentzWidth;
    }

    /**
     * Returns the errors in the Lorentzian width parameter.
     * @return 
     */
    public double[][] getErrorsLorentzWidth( ) 
    {
        return m_errorsLorentzWidth;
    }

    /**
     * Set the errors in the Lorentzian width parameters. 
     * @param errorsLorentzWidth 
     */
    public void setErrorsLorentzWidth( double[][] errorsLorentzWidth ) 
    {
        m_errorsLorentzWidth = errorsLorentzWidth;
    }

    /**
     * Returns the LorentzianR width.
     * @return 
     */
    public double[][] getLorentzRWidth( ) 
    {
        return m_lorentzRWidth;
    }
    
    /**
     * Sets the LorentzianR width. 
     * @param lorentzRWidth
     */
    public void setLorentzRWidth( double[][] lorentzRWidth ) 
    {
        m_lorentzRWidth = lorentzRWidth;
    }

    /**
     * Returns the LorentzianR width.
     * @return 
     */
    public double[][] getErrorsLorentzRWidth( ) 
    {
        return m_errorsLorentzRWidth;
    }

    /**
     * Sets the errors in the LorentzianR parameters.
     * @param errorsLorentzRWidth 
     */
    public void setErrorsLorentzRWidth( double[][] errorsLorentzRWidth ) 
    {
        m_errorsLorentzRWidth = errorsLorentzRWidth;
    }
    
    /**
     * Returns the number of acquired water responses that were acquired for
     * the 2 compartment model.
     * @return 
     */
    public int getNumberOfFittedResponses( ) 
    {
        return m_fittedResponseFileNames.size();
    }

    /**
     * Sets the number of acquired water responses that were acquired for
     * the 2 compartment model. 
     * @param numberOf2CompWaterRefResponses
     */
    public void setNumberOf2CompWaterRefResponses( int numberOf2CompWaterRefResponses ) 
    {
        m_numberOfWaterReferenceResponses = numberOf2CompWaterRefResponses;
    }

    /**
     * Returns the amplitude of the first time constant of the bi-exponential
     * water decay.
     * @return 
     */
    public double getAmp1_TCM( ) 
    {
        return m_amp1_TCM;
    }

    /**
     * Sets the amplitude of the first time constant of the bi-exponential
     * water decay.
     * @param amp1_TCM
     */
    public void setAmp1_TCM( double  amp1_TCM ) 
    {
        m_amp1_TCM = amp1_TCM;
    }

    /**
     * Returns the amplitude of the second time constant of the bi-exponential
     * water decay.
     * @return 
     */
    public double getAmp2_TCM( ) 
    {
        return m_amp2_TCM;
    }
    
    /**
     * Sets the amplitude of the second time constant of the bi-exponential
     * water decay.
     * @param amp2_TCM
     */
    public void setAmp2_TCM( double amp2_TCM ) 
    {
        m_amp2_TCM = amp2_TCM;
    }

    /**
     * Returns the time constant of the first exponential decay.
     * water decay.
     * @return 
     */
    public double getTC1_TCM( ) 
    {
        return m_TC1_TCM;
    }

    /**
     * Sets the time constant of the first exponential decay.
     * @param TC1_TCM 
     */
    public void setTC1_TCM( double TC1_TCM ) 
    {
        m_TC1_TCM = TC1_TCM;
    }
    
    /**
     * Returns the time constant of the second exponential decay
     * @return 
     */
    public double getTC2_TCM( ) 
    {
        return m_TC2_TCM;
    }

    /**
     * Sets the time constant of the first exponential decay.
     * @param TC2_TCM 
     */
    public void setTC2_TCM( double TC2_TCM ) 
    {
        m_TC2_TCM = TC2_TCM;
    }

    /**
     * Returns the error in amplitude 1 of the bi-exponential decay.
     * @return 
     */
    public double getErrAmp1_TCM( ) 
    {
        return m_err_amp1_TCM;
    }

    /**
     * Sets the error in amplitude 1 of the bi-exponential decay.
     * @param err_amp1_TCM 
     */
    public void setErrAmp1_TCM( double err_amp1_TCM ) 
    {
        m_err_amp1_TCM = err_amp1_TCM;
    }

    /**
     * Returns the error in amplitude 2 of the bi-exponential decay.
     * @return
     */
    public double getErrAmp2_TCM( ) 
    {
        return m_err_amp2_TCM;
    }

    /**
     * Sets the error in amplitude 2 of the bi-exponential decay.
     * @param errAmp2_TCM 
     */
    public void setErrAmp2_TCM( double errAmp2_TCM ) 
    {
        m_err_amp2_TCM = errAmp2_TCM;
    }

    /**
     * Returns the error in TC1 of the first exponential decay.
     * @return 
     */
    public double getErr_TC1_TCM( ) 
    {
        return m_err_TC1_TCM;
    }
    
    /**
     * Sets the error in TC1 of the first exponential decay. 
     * @param err_TC1_TCM
     */
    public void setErrTC1_TCM( double err_TC1_TCM ) 
    {
        m_err_TC1_TCM = err_TC1_TCM;
    }

    /**
     * Returns the error in TC1 of the second exponential decay.
     * @return 
     */
    public double getErr_TC2_TCM( ) 
    {
        return m_err_TC2_TCM;
    }

    /**
     * Sets the error in TC2 of the second exponential decay. 
     * @param err_TC2_TCM
     */
    public void setErrTC2_TCM( double err_TC2_TCM ) 
    {
        m_err_TC2_TCM = err_TC2_TCM;
    }
    
    /**
     * Sets the real part of the reference water response.
     * @param refWaterReal 
     */
    public void setWaterReferenceSignalReal( double[] refWaterReal )
    {
        m_waterRefReal = refWaterReal;
    }
    
    /**
     * Sets the imaginary part of the reference water response.
     * @param refWaterImag 
     */
    public void setWaterReferenceSignalImag( double[] refWaterImag )
    {
        m_waterRefImag = refWaterImag;
    }
    
    /**
     * Returns the real part of the reference water response. 
     * @return  
     */
    public double[] getWaterReferenceSignalReal( )
    {
        return m_waterRefReal;
    }
    
    /**
     * Returns the imaginary part of the reference water response. 
     * @return  
     */
    public double[] getWaterReferenceSignalImag( )
    {
        return m_waterRefImag;
    }
    
    /**
     * Sets the JMRUI specific (private) Dicom tag.
     * @return 
     */
    public SVS_JMRUI_DICOM_Tag_FAST getSVSJMRUIDicomTag( ) 
    {
        return m_SVSJMRUIDicomTag;
    }
    
    /**
     * Returns the JMRUI specific (private) Dicom tag. 
     * @param SVSJMRUIDicomTag
     */
    public void setSVSJMRUIDicomTag( SVS_JMRUI_DICOM_Tag_FAST SVSJMRUIDicomTag ) 
    {
        m_SVSJMRUIDicomTag = SVSJMRUIDicomTag;
    }

    /**
     * Returns the file reference of reference image 0.
     * @return 
     */
    public String getReferenceImage0( ) 
    {
        return m_referenceImage0;
    }

    /**
     * Sets the file reference of reference image 0.
     * @param referenceImage0 
     */
    public void setReferenceImage0( String referenceImage0 ) 
    {
        m_referenceImage0 = referenceImage0;
    }
    
    /**
     * Returns the file reference of reference image 1.
     * @return 
     */
    public String getReferenceImage1( ) 
    {
        return m_referenceImage1;
    }
    
    /**
     * Sets the file reference of reference image 0. 
     * @param referenceImage1
     */
    public void setReferenceImage1( String referenceImage1 ) 
    {
        m_referenceImage1 = referenceImage1;
    }
    
    /**
     * Returns the file reference of reference image 2.
     * @return 
     */
    public String getReferenceImage2( ) 
    {
        return m_referenceImage2;
    }

    /**
     * Sets the file reference of reference image 2. 
     * @param referenceImage2
     */
    public void setReferenceImage2( String referenceImage2 ) 
    {
        m_referenceImage2 = referenceImage2;
    }

    /**
     * Returns the real part of measured signal arrays.
     * @return 
     */
    public double[][] getMeasuredReal( ) 
    {
        return m_measuredReal;
    }

    /**
     * Sets the real part of the measured signal arrays. 
     * @param measuredReal
     */
    public void setMeasuredReal( double[][] measuredReal ) 
    {
        m_measuredReal = measuredReal;
    }
    
    /**
     * Returns the imaginary part of measured signal arrays.
     * @return 
     */     
    public double[][] getMeasuredImag( ) 
    {
        return m_measuredImag;
    }
    
    /**
     * Sets the imaginary part of measured signal arrays. 
     * @param measuredImag
     */
    public void setMeasuredImag( double[][] measuredImag ) 
    {
        m_measuredImag = measuredImag;
    }
    
    /**
     * Returns the real part of the fitted signal arrays.
     * @return 
     */  
    public double[][] getFittedReal( ) 
    {
        return m_fittedReal;
    }

    /**
     * Sets the real part of the fitted signal arrays. 
     * @param fittedReal
     */
    public void setFittedReal( double[][] fittedReal ) 
    {
        m_fittedReal = fittedReal;
    }

     /**
     * Returns the imaginary  part of the fitted signal arrays.
     * @return 
     */  
    public double[][] getFittedImag( ) 
    {
        return m_fittedImag;
    }
    
    /**
     * Sets the imaginary part of the fitted signal arrays. 
     * @param fittedImag
     */
    public void setFittedImag( double[][] fittedImag ) 
    {
        m_fittedImag = fittedImag;
    }
    
    /**
     * Returns the number of signals fitted.
     * @return 
     */
    public int getNumberOfMeasuredSignalsFitted( )
    {
        return this.getNumberOfFittedResponses( );
    }
    
    /**
     * Returns the number of components fitted by TDFDFit.
     * @return 
     */
    public int getNumberOfComponentsFitted( )
    {
        TDFDFitPrototypeModelSpectrumParameters model = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        return model.getNumberOfSpectralComponentsToFit( );
    }
    
    /**
     * This method reads the fit results from file and updates the 
     * MRFittingParameter objects.
     * @param fileToParse
     */
    public void readFitresultsToDataStructures( String fileToParse )
    {

        TDFDFitPrototypeModelSpectrumParameters model = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        
        int totalNumberOfComponents = model.getNumberOfSpectralComponentsToFit( );
        
        // allocate memmory for member variables
        m_areas = new double[1][totalNumberOfComponents];
        m_resonanceFrequencies = new double[1][totalNumberOfComponents];
        m_phases = new double[1][totalNumberOfComponents];
        m_lorentzWidths = new double[1][totalNumberOfComponents];
        m_gaussianWidths = new double[1][totalNumberOfComponents];
    
        m_areaErrors = new double[1][totalNumberOfComponents];
        m_resonanceFrequenciesErrors = new double[1][totalNumberOfComponents];
        m_phasesErrors = new double[1][totalNumberOfComponents];
        m_lorentzWidthsErrors = new double[1][totalNumberOfComponents];
        m_gaussianWidthsErrors = new double[1][totalNumberOfComponents];
        
        int cntr = 0;
        MRFittingParameters fitPar = new MRLineModelParameters( );
        
        try
        {            
            RandomAccessFile reader = new RandomAccessFile( fileToParse, "rw" );
            
            String str = reader.readLine( );
            System.out.println( "string read " + str );

            while ( str.trim( ).compareTo( "tdfdfit_starting_values" ) != 0 )
            {
                str = reader.readLine( );
            }
            
            // read the following blank line
            reader.readLine( );
            
            // 1. Iterate over simple lines!
            for ( int i = 0; i < model.getNumberOfSimpleLines( ); i++ )
            {          
                str = reader.readLine( );
                System.out.println( str );
                int ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                int length = str.indexOf( " " );
                String firstToken = str.substring( 0, length );
                str = str.substring(length);
                
                ////////////////////////////////////////////////////////////////
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                
                length = str.indexOf( " " );
                String secondToken = str.substring( 0, length );
                str = str.substring( length );
                Float area = new Float( secondToken );
                
                ////////////////////////////////////////////////////////////////
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );                
                length = str.indexOf( " " );           
                String thirdToken = str.substring( 0, length );
                str = str.substring( length );
                Float offset = new Float( thirdToken );
                
                ////////////////////////////////////////////////////////////////
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                
                length = str.indexOf( " " );
                String fourthToken = str.substring( 0, length );
                str = str.substring( length );
                Float gauss = new Float( fourthToken );
                
                ////////////////////////////////////////////////////////////////
                
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                length = str.indexOf( " " );
                String fifthToken = str.substring( 0, length );
                str = str.substring( length );
                Float phase = new Float( fifthToken );
                
                ////////////////////////////////////////////////////////////////
                
                str = str.trim( );                
                Float lorentz = new Float( str );

                System.out.println( "simple line : " +  
                                    area.toString( ) + " " + 
                                    offset.toString( ) + " " + 
                                    gauss.toString( ) + " " + 
                                    phase.toString( ) + " " + 
                                    lorentz.toString( ) );
 
                m_areas[0][cntr] = area / (float)MRSignalProperties.getInstance( ).getNumberOfSamplePoints( );
                m_resonanceFrequencies[0][cntr] = offset - ( (float)MRSignalProperties.getInstance( ).getNumberOfSamplePoints( ) ) / 2.0f;
                m_phases[0][cntr] = phase;
                m_lorentzWidths[0][cntr] = lorentz;
                m_gaussianWidths[0][cntr] = gauss;
    
                m_areaErrors[0][cntr] = 0.0;
                m_resonanceFrequenciesErrors[0][cntr] = 0.0;
                m_phasesErrors[0][cntr] = 0.0;
                m_lorentzWidthsErrors[0][cntr] = 0.0;
                m_gaussianWidthsErrors[0][cntr] = 0.0;
                cntr++;
            }
            
            // 2. Iterate over parametric patterns!
            for ( int i = 0; i < model.getNumberOfParametricPatterns( ); i++ )
            {          
                str = reader.readLine( );
                System.out.println( str );
                int ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                int length = str.indexOf( " " );
                String firstToken = str.substring( 0, length );
                str = str.substring(length);
                
                ////////////////////////////////////////////////////////////////
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                
                length = str.indexOf( " " );
                String secondToken = str.substring( 0, length );
                str = str.substring( length );
                Float area = new Float( secondToken );
                
                ////////////////////////////////////////////////////////////////
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );                
                length = str.indexOf( " " );           
                String thirdToken = str.substring( 0, length );
                str = str.substring( length );
                Float offset = new Float( thirdToken );
                
                ////////////////////////////////////////////////////////////////
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                
                length = str.indexOf( " " );
                String fourthToken = str.substring( 0, length );
                str = str.substring( length );
                Float gauss = new Float( fourthToken );
                
                ////////////////////////////////////////////////////////////////
                
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                length = str.indexOf( " " );
                String fifthToken = str.substring( 0, length );
                str = str.substring( length );
                Float phase = new Float( fifthToken );
                
                ////////////////////////////////////////////////////////////////
                
                str = str.trim( );                
                Float lorentz = new Float( str );

                System.out.println( "parametric pat : " + 
                                    area.toString( ) + " " + 
                                    offset.toString( ) + " " + 
                                    gauss.toString( ) + " " + 
                                    phase.toString( ) + " " + 
                                    lorentz.toString( ) );
    
                // update the members
                m_areas[0][cntr] = area / (float)MRSignalProperties.getInstance( ).getNumberOfSamplePoints( );
                m_resonanceFrequencies[0][cntr] = offset;
                m_phases[0][cntr] = phase;
                m_lorentzWidths[0][cntr] = lorentz;
                m_gaussianWidths[0][cntr] = gauss;
    
                m_areaErrors[0][cntr] = 0.0;
                m_resonanceFrequenciesErrors[0][cntr] = 0.0;
                m_phasesErrors[0][cntr] = 0.0;
                m_lorentzWidthsErrors[0][cntr] = 0.0;
                m_gaussianWidthsErrors[0][cntr] = 0.0;
                cntr++; 
            }
            
            // 3. Iterate over numeric patterns!
            for ( int i = 0; i < model.getNumberOfNumericPatterns( ); i++ )
            {          
                str = reader.readLine( );
                System.out.println( str );
                int ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                int length = str.indexOf( " " );
                String firstToken = str.substring( 0, length );
                str = str.substring(length);
                
                ////////////////////////////////////////////////////////////////
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                
                length = str.indexOf( " " );
                String secondToken = str.substring( 0, length );
                str = str.substring( length );
                Float area = new Float( secondToken );
                
                ////////////////////////////////////////////////////////////////
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );                
                length = str.indexOf( " " );           
                String thirdToken = str.substring( 0, length );
                str = str.substring( length );
                Float offset = new Float( thirdToken );
                
                ////////////////////////////////////////////////////////////////
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                
                length = str.indexOf( " " );
                String fourthToken = str.substring( 0, length );
                str = str.substring( length );
                Float gauss = new Float( fourthToken );
                
                ////////////////////////////////////////////////////////////////
                
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                length = str.indexOf( " " );
                String fifthToken = str.substring( 0, length );
                str = str.substring( length );
                Float phase = new Float( fifthToken );
                
                ////////////////////////////////////////////////////////////////
                
                str = str.trim( );                
                Float lorentz = new Float( str );

                System.out.println( "numeric pat : " + 
                                    area.toString( ) + " " + 
                                    offset.toString( ) + " " + 
                                    gauss.toString( ) + " " + 
                                    phase.toString( ) + " " + 
                                    lorentz.toString( ) );
    
                // update the members
                m_areas[0][cntr] = area / (float)MRSignalProperties.getInstance( ).getNumberOfSamplePoints( );
                m_resonanceFrequencies[0][cntr] = offset;
                m_phases[0][cntr] = phase;
                m_lorentzWidths[0][cntr] = lorentz;
                m_gaussianWidths[0][cntr] = gauss;
    
                m_areaErrors[0][cntr] = 0.0;
                m_resonanceFrequenciesErrors[0][cntr] = 0.0;
                m_phasesErrors[0][cntr] = 0.0;
                m_lorentzWidthsErrors[0][cntr] = 0.0;
                m_gaussianWidthsErrors[0][cntr] = 0.0;
                cntr++; 
            }
            
            // 4. Iterate over metabolites!
            for ( int i = 0; i < model.getNumberOfMetabolitePatterns( ); i++ )
            {          
                str = reader.readLine( );
                System.out.println( str );
                int ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                int length = str.indexOf( " " );
                String firstToken = str.substring( 0, length );
                str = str.substring(length);
                
                ////////////////////////////////////////////////////////////////
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                
                length = str.indexOf( " " );
                String secondToken = str.substring( 0, length );
                str = str.substring( length );
                Float area = new Float( secondToken );
                
                ////////////////////////////////////////////////////////////////
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );                
                length = str.indexOf( " " );           
                String thirdToken = str.substring( 0, length );
                str = str.substring( length );
                Float offset = new Float( thirdToken );
                
                ////////////////////////////////////////////////////////////////
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                
                length = str.indexOf( " " );
                String fourthToken = str.substring( 0, length );
                str = str.substring( length );
                Float gauss = new Float( fourthToken );
                
                ////////////////////////////////////////////////////////////////
                
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                length = str.indexOf( " " );
                String fifthToken = str.substring( 0, length );
                str = str.substring( length );
                Float phase = new Float( fifthToken );
                
                ////////////////////////////////////////////////////////////////
                
                str = str.trim( );                
                Float lorentz = new Float( str );

                System.out.println( "metabolite pat : " + 
                                    area.toString( ) + " " + 
                                    offset.toString( ) + " " + 
                                    gauss.toString( ) + " " + 
                                    phase.toString( ) + " " + 
                                    lorentz.toString( ) );
    
                // Update the members
                m_areas[0][cntr] = area / (float)MRSignalProperties.getInstance( ).getNumberOfSamplePoints( );
                m_resonanceFrequencies[0][cntr] = offset;
                m_phases[0][cntr] = phase;
                m_lorentzWidths[0][cntr] = lorentz;
                m_gaussianWidths[0][cntr] = gauss;
    
                m_areaErrors[0][cntr] = 0.0;
                m_resonanceFrequenciesErrors[0][cntr] = 0.0;
                m_phasesErrors[0][cntr] = 0.0;
                m_lorentzWidthsErrors[0][cntr] = 0.0;
                m_gaussianWidthsErrors[0][cntr] = 0.0;
                cntr++; 
            }
        }
        catch ( IOException e )
        {
            System.out.println( "IOException in reading >>filename<<_verbose.par" );
        }
    }
    
    
    /**
     * Returns the area parameters. 
     * @return 
     */
    public double[][] getAreas( ) 
    {
        return m_areas;
    }

    /**
     * Sets the area parameters. 
     * @param areas
     */
    public void setAreas( double[] areas ) 
    {
        for ( int i=0; i < areas.length; i++ )
        {
            m_areas[0][i] = areas[i];
        }
    }
    
    /**
     * Returns the resonance frequency parameters. 
     * @return 
     */
    public double[][] getResonanceFrequencies( ) 
    {
        return m_resonanceFrequencies;
    }
    
    /**
     * Sets the resonance frequency parameters. 
     * @param resonanceFrequencies
     */
    public void setResonanceFrequencies( double[] resonanceFrequencies ) 
    {
        for ( int i=0; i < resonanceFrequencies.length; i++ )
        {
            m_resonanceFrequencies[0][i] = resonanceFrequencies[i];
        }
    }
    
    /**
     * Returns the phases parameters. 
     * @return 
     */
    public double[][] getPhases( ) 
    {
        return m_phases;
    }
    
    /**
     * Sets the phases parameters. 
     * @param phases
     */
    public void setPhases( double[] phases ) 
    {
        for ( int i=0; i < phases.length; i++ )
        {
            m_phases[0][i] = phases[i];
        }
    }
    
    /**
     * Returns the Lorentzian width parameters.  
     * @return 
     */
    public double[][] getLorentzWidths( ) 
    {
        return m_lorentzWidths;
    }
    
    /**
     * Returns the Lorentzian width parameters. 
     * @param lorentzWidths
     */
    public void setLorentzWidths( double[] lorentzWidths ) 
    {
        for ( int i=0; i < lorentzWidths.length; i++ )
        {
            m_lorentzWidths[0][i] = lorentzWidths[i];
        }
    }
    
    /**
     * Returns the Gaussian width parameters.  
     * @return 
     */
    public double[][] getGaussianWidths( )
    {
        return m_gaussianWidths;
    }

    /**
     * Sets the Gaussian width parameters.  
     * @param gaussianWidths 
     */
    public void setGaussianWidths( double[] gaussianWidths ) 
    {
        for ( int i=0; i < gaussianWidths.length; i++ )
        {
            m_gaussianWidths[0][i] = gaussianWidths[i];
        }
    }

    /**
     * Sets the CR-MVB errors of the area parameters.
     * @return 
     */
    public double[][] getAreaErrors( ) 
    {
        return m_areaErrors;
    }

    /**
     * Sets the CR-MVB errors of the area parameters.
     * @param areaErrors
     */
    public void setAreaErrors(double[] areaErrors) 
    {
        for ( int i=0; i < areaErrors.length; i++ )
        {
            m_areaErrors[0][i] = areaErrors[i];
        }
    }
    /**
     * Sets the CR-MVB errors of the frequency parameters.
     * @return 
     */
    public double[][] getResonanceFrequenciesErrors( ) 
    {
        return m_resonanceFrequenciesErrors;
    }

    /**
     * Returns the CR-MVB errors of the frequency parameters.
     * @param resonanceFrequenciesErrors 
     */
    public void setResonanceFrequenciesErrors( double[] resonanceFrequenciesErrors ) 
    {
        for ( int i=0; i < resonanceFrequenciesErrors.length; i++ )
        {
            m_resonanceFrequenciesErrors[0][i] = resonanceFrequenciesErrors[i];
        }
    }

    /**
     * Returns the CR-MVB errors of the zero order phase parameters. 
     * @return 
     */
    public double[][] getPhasesErrors( ) 
    {
        return m_phasesErrors;
    }

    /**
     * Sets the CR-MVB errors of the zero order phase parameters.  
     * @param phasesErrors 
     */
    public void setPhasesErrors( double[] phasesErrors ) 
    {
        for ( int i=0; i < phasesErrors.length; i++ )
        {
            m_phasesErrors[0][i] = phasesErrors[i];
        }
    }
    
    /**
     * Returns the CR-MVB errors of the Lorentz width parameters. 
     * @return 
     */
    public double[][] getLorentzWidthsErrors( ) 
    {
        return m_lorentzWidthsErrors;
    }

    /**
     * Sets the CR-MVB errors of the Lorentz width parameters. 
     * @param lorentzWidthsErrors
     */
    public void setLorentzWidthsErrors( double[] lorentzWidthsErrors ) 
    {
        for ( int i=0; i < lorentzWidthsErrors.length; i++ )
        {
            m_lorentzWidthsErrors[0][i] = lorentzWidthsErrors[i];
        }
    }
    
    /**
     * Returns the CR-MVB errors of the Gaussian width parameters.  
     * @return 
     */
    public double[][] getGaussianWidthsErrors( ) 
    {
        return m_gaussianWidthsErrors;
    }
    
    /**
     * Sets the CR-MVB errors of the Gaussian width parameters. 
     * @param gaussianWidthsErrors
     */
    public void setGaussianWidthsErrors( double[] gaussianWidthsErrors ) 
    {
        for ( int i=0; i < gaussianWidthsErrors.length; i++ )
        {
            m_gaussianWidthsErrors[0][i] = gaussianWidthsErrors[i];
        }
    }
    
    // Attributes 
    private String m_fileReference;
    
    private double[][] m_areas;
    private double[][] m_resonanceFrequencies;
    private double[][] m_phases;
    private double[][] m_lorentzWidths;
    private double[][] m_gaussianWidths;
    
    private double[][] m_areaErrors;
    private double[][] m_resonanceFrequenciesErrors;
    private double[][] m_phasesErrors;
    private double[][] m_lorentzWidthsErrors;
    private double[][] m_gaussianWidthsErrors;
    
    // Static private attribute is the instance itself (Singleton!)
    private static  QuantitativeSVSContextDataContainer theInstance = null;
    
    // Further attributes
    private LinkedList<String> m_spectroscopyDicomWaterFileReference = new LinkedList<>( ) ;
    private boolean     m_twoCompartmentModelSet;
    private boolean     m_setSimpleWaterRefefenceSignalSet;
    private boolean     m_twoCompartmentModelFitted;
    private boolean     m_singleWaterReferenceSignalDefined;
    
    private String      m_simpleWaterReferenceFileName;
    private LinkedList<String> m_fittedResponseFileNames = new LinkedList<String>( );
    
    private LinkedList<String> m_metaboliteNames = new LinkedList<>( );
    
    private double[][]  m_peakAreas;
    private double[][]  m_errorPeakAreas;
    
    private double[][]  m_chemShift;
    private double[][]  m_errorsChemShift;
    // private double[][]  m_phase;
    private double[][]  m_errorsPhase;
    private double[][]  m_lorentzWidth;
    private double[][]  m_errorsLorentzWidth;
    private double[][]  m_lorentzRWidth;
    private double[][]  m_errorsLorentzRWidth;

    private int         m_numberOfWaterReferenceResponses;
    private double      m_amp1_TCM;
    private double      m_amp2_TCM;
    private double      m_TC1_TCM;
    private double      m_TC2_TCM;
    private double      m_err_amp1_TCM;
    private double      m_err_amp2_TCM;
    private double      m_err_TC1_TCM;
    private double      m_err_TC2_TCM;

    private double[]    m_waterRefReal;
    private double[]    m_waterRefImag;  
        
    private SVS_JMRUI_DICOM_Tag_FAST m_SVSJMRUIDicomTag;
    
    private String      m_referenceImage0;
    private String      m_referenceImage1;
    private String      m_referenceImage2;
    
    private double[][]  m_measuredReal;
    private double[][]  m_measuredImag;
    private double[][]  m_fittedReal;
    private double[][]  m_fittedImag;
   
    private double[][]  m_waterRef2CompReal;
    private double[][]  m_waterRef2CompImag;
    private int         m_numberOf2CompModelResponses;
    
}